//! layout test for scenario shown to get stuck compacting.
//! The original of this set of files is querying a catalog of a partition stuck doing
//! non-productive compactions (which needs veritical splitting to resolve the impasse).
//!
//! See [crate::layout] module for detailed documentation

use data_types::CompactionLevel;
use iox_time::Time;
use std::time::Duration;

use crate::layouts::{layout_setup_builder, parquet_builder, run_layout_scenario, ONE_MB};
const MAX_DESIRED_FILE_SIZE: u64 = 100 * ONE_MB;

#[tokio::test]
async fn stuck_l0() {
    test_helpers::maybe_start_logging();

    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(20)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(100000))
        .with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853019000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930563065100652))
                .with_file_size_bytes(149933875),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686845579000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(103205619),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853319000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935546047601759))
                .with_file_size_bytes(150536767),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686871559000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686936871554969451))
                .with_file_size_bytes(102393626),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686854759000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935947459465643))
                .with_file_size_bytes(87151809),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686845579000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5682010),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852839000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935742511199929))
                .with_file_size_bytes(75607192),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686855419000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935151542899174))
                .with_file_size_bytes(87166408),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686855059000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929965334855957))
                .with_file_size_bytes(88035623),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686855659000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931893702512591))
                .with_file_size_bytes(90543489),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852899000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934966479515832))
                .with_file_size_bytes(75851382),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853079000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931336078719452))
                .with_file_size_bytes(149692663),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853319000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929421018268948))
                .with_file_size_bytes(150619037),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853379000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930780953922120))
                .with_file_size_bytes(58021414),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852839000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929712329555892))
                .with_file_size_bytes(75536272),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853019000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933271571861107))
                .with_file_size_bytes(149014949),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852899000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931600579333716))
                .with_file_size_bytes(72914229),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852959000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933528170895870))
                .with_file_size_bytes(74896171),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686855119000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933830062404735))
                .with_file_size_bytes(89245536),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686852119000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934254955029762))
                .with_file_size_bytes(105905115),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686849719000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932458050354802))
                .with_file_size_bytes(104819243),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686853679000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934759745855254))
                .with_file_size_bytes(150386578),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686841379000000000)
                .with_max_time(1686854219000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932677391046778))
                .with_file_size_bytes(67069745),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686845639000000000)
                .with_max_time(1686849779000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5526463),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686845639000000000)
                .with_max_time(1686849779000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(101878097),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686849779000000000)
                .with_max_time(1686858119000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932458050354802))
                .with_file_size_bytes(104808702),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686849839000000000)
                .with_max_time(1686850559000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(21186155),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686849839000000000)
                .with_max_time(1686850559000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(998505),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686850619000000000)
                .with_max_time(1686854819000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5580685),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686850619000000000)
                .with_max_time(1686854819000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(103246896),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686852179000000000)
                .with_max_time(1686862859000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934254955029762))
                .with_file_size_bytes(105513447),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686852899000000000)
                .with_max_time(1686864359000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935742511199929))
                .with_file_size_bytes(139541880),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686852899000000000)
                .with_max_time(1686864359000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929712329555892))
                .with_file_size_bytes(139400211),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686852959000000000)
                .with_max_time(1686864419000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931600579333716))
                .with_file_size_bytes(136888003),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686852959000000000)
                .with_max_time(1686864419000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934966479515832))
                .with_file_size_bytes(139953230),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853019000000000)
                .with_max_time(1686864599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933528170895870))
                .with_file_size_bytes(138845602),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853079000000000)
                .with_max_time(1686864659000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933271571861107))
                .with_file_size_bytes(84174642),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853079000000000)
                .with_max_time(1686864659000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930563065100652))
                .with_file_size_bytes(83486810),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853139000000000)
                .with_max_time(1686864839000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931336078719452))
                .with_file_size_bytes(83035926),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853379000000000)
                .with_max_time(1686865259000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929421018268948))
                .with_file_size_bytes(80749475),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853379000000000)
                .with_max_time(1686865259000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935546047601759))
                .with_file_size_bytes(80622284),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853439000000000)
                .with_max_time(1686865439000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930780953922120))
                .with_file_size_bytes(130471302),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686853739000000000)
                .with_max_time(1686866039000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934759745855254))
                .with_file_size_bytes(76518641),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686854279000000000)
                .with_max_time(1686867059000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932677391046778))
                .with_file_size_bytes(81222708),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686854819000000000)
                .with_max_time(1686868199000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935947459465643))
                .with_file_size_bytes(93828618),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686854879000000000)
                .with_max_time(1686859019000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(101899966),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686854879000000000)
                .with_max_time(1686859019000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5444939),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686855119000000000)
                .with_max_time(1686868739000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929965334855957))
                .with_file_size_bytes(97364742),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686855179000000000)
                .with_max_time(1686868859000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933830062404735))
                .with_file_size_bytes(96919046),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686855479000000000)
                .with_max_time(1686869519000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935151542899174))
                .with_file_size_bytes(101734904),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686855719000000000)
                .with_max_time(1686869939000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931893702512591))
                .with_file_size_bytes(100008012),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686858179000000000)
                .with_max_time(1686865979000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932458050354802))
                .with_file_size_bytes(98556380),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686859079000000000)
                .with_max_time(1686859499000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(593319),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686859079000000000)
                .with_max_time(1686859499000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(14403989),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686859559000000000)
                .with_max_time(1686863699000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5423734),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686859559000000000)
                .with_max_time(1686863699000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(101893482),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686862919000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934254955029762))
                .with_file_size_bytes(102580493),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686863759000000000)
                .with_max_time(1686867659000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(5026731),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686863759000000000)
                .with_max_time(1686867839000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(100495018),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864419000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929712329555892))
                .with_file_size_bytes(78503529),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864419000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935742511199929))
                .with_file_size_bytes(78149265),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864479000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934966479515832))
                .with_file_size_bytes(77391966),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864479000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931600579333716))
                .with_file_size_bytes(83215868),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864659000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933528170895870))
                .with_file_size_bytes(76904008),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864719000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930563065100652))
                .with_file_size_bytes(56776838),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864719000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933271571861107))
                .with_file_size_bytes(56708180),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686864899000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931336078719452))
                .with_file_size_bytes(55114047),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686865319000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929421018268948))
                .with_file_size_bytes(51263308),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686865319000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935546047601759))
                .with_file_size_bytes(51157926),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686865499000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686930780953922120))
                .with_file_size_bytes(92510190),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686866099000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686934759745855254))
                .with_file_size_bytes(46749740),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686867119000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686932677391046778))
                .with_file_size_bytes(114531826),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686867719000000000)
                .with_max_time(1686867839000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(229903),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686867899000000000)
                .with_max_time(1686868319000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928811433793899))
                .with_file_size_bytes(14513946),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686867899000000000)
                .with_max_time(1686868319000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(602054),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686868259000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935947459465643))
                .with_file_size_bytes(70522099),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686868379000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928854574095806))
                .with_file_size_bytes(93408439),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686868379000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928118432114258))
                .with_file_size_bytes(41089381),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686868799000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686929965334855957))
                .with_file_size_bytes(61094135),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686868919000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686933830062404735))
                .with_file_size_bytes(59466261),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686869579000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686935151542899174))
                .with_file_size_bytes(51024344),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686869999000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686931893702512591))
                .with_file_size_bytes(45632935),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686871619000000000)
                .with_max_time(1686873599000000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686936871554969451))
                .with_file_size_bytes(9380799),
        )
        .await;

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.1[1686841379000000000,1686853019000000000] 1686930563.07s 143mb|-------------L0.1-------------|                                                          "
    - "L0.3[1686841379000000000,1686853319000000000] 1686935546.05s 144mb|-------------L0.3--------------|                                                         "
    - "L0.4[1686841379000000000,1686871559000000000] 1686936871.55s 98mb|---------------------------------------L0.4---------------------------------------|      "
    - "L0.5[1686841379000000000,1686854759000000000] 1686935947.46s 83mb|---------------L0.5----------------|                                                     "
    - "L0.7[1686841379000000000,1686852839000000000] 1686935742.51s 72mb|-------------L0.7-------------|                                                          "
    - "L0.8[1686841379000000000,1686855419000000000] 1686935151.54s 83mb|----------------L0.8-----------------|                                                   "
    - "L0.9[1686841379000000000,1686855059000000000] 1686929965.33s 84mb|----------------L0.9----------------|                                                    "
    - "L0.10[1686841379000000000,1686855659000000000] 1686931893.7s 86mb|----------------L0.10----------------|                                                   "
    - "L0.11[1686841379000000000,1686852899000000000] 1686934966.48s 72mb|------------L0.11-------------|                                                          "
    - "L0.12[1686841379000000000,1686853079000000000] 1686931336.08s 143mb|------------L0.12-------------|                                                          "
    - "L0.13[1686841379000000000,1686853319000000000] 1686929421.02s 144mb|-------------L0.13-------------|                                                         "
    - "L0.14[1686841379000000000,1686853379000000000] 1686930780.95s 55mb|-------------L0.14-------------|                                                         "
    - "L0.15[1686841379000000000,1686852839000000000] 1686929712.33s 72mb|------------L0.15-------------|                                                          "
    - "L0.16[1686841379000000000,1686853019000000000] 1686933271.57s 142mb|------------L0.16-------------|                                                          "
    - "L0.17[1686841379000000000,1686852899000000000] 1686931600.58s 70mb|------------L0.17-------------|                                                          "
    - "L0.18[1686841379000000000,1686852959000000000] 1686933528.17s 71mb|------------L0.18-------------|                                                          "
    - "L0.19[1686841379000000000,1686855119000000000] 1686933830.06s 85mb|---------------L0.19----------------|                                                    "
    - "L0.20[1686841379000000000,1686852119000000000] 1686934254.96s 101mb|-----------L0.20------------|                                                            "
    - "L0.21[1686841379000000000,1686849719000000000] 1686932458.05s 100mb|--------L0.21--------|                                                                   "
    - "L0.22[1686841379000000000,1686853679000000000] 1686934759.75s 143mb|-------------L0.22--------------|                                                        "
    - "L0.23[1686841379000000000,1686854219000000000] 1686932677.39s 64mb|--------------L0.23--------------|                                                       "
    - "L0.26[1686849779000000000,1686858119000000000] 1686932458.05s 100mb                       |--------L0.26--------|                                            "
    - "L0.31[1686852179000000000,1686862859000000000] 1686934254.96s 101mb                              |-----------L0.31-----------|                               "
    - "L0.32[1686852899000000000,1686864359000000000] 1686935742.51s 133mb                                |------------L0.32-------------|                          "
    - "L0.33[1686852899000000000,1686864359000000000] 1686929712.33s 133mb                                |------------L0.33-------------|                          "
    - "L0.34[1686852959000000000,1686864419000000000] 1686931600.58s 131mb                                |------------L0.34-------------|                          "
    - "L0.35[1686852959000000000,1686864419000000000] 1686934966.48s 133mb                                |------------L0.35-------------|                          "
    - "L0.36[1686853019000000000,1686864599000000000] 1686933528.17s 132mb                                |------------L0.36-------------|                          "
    - "L0.37[1686853079000000000,1686864659000000000] 1686933271.57s 80mb                                |------------L0.37-------------|                          "
    - "L0.38[1686853079000000000,1686864659000000000] 1686930563.07s 80mb                                |------------L0.38-------------|                          "
    - "L0.39[1686853139000000000,1686864839000000000] 1686931336.08s 79mb                                |------------L0.39-------------|                          "
    - "L0.40[1686853379000000000,1686865259000000000] 1686929421.02s 77mb                                 |-------------L0.40-------------|                        "
    - "L0.41[1686853379000000000,1686865259000000000] 1686935546.05s 77mb                                 |-------------L0.41-------------|                        "
    - "L0.42[1686853439000000000,1686865439000000000] 1686930780.95s 124mb                                 |-------------L0.42-------------|                        "
    - "L0.43[1686853739000000000,1686866039000000000] 1686934759.75s 73mb                                  |-------------L0.43--------------|                      "
    - "L0.44[1686854279000000000,1686867059000000000] 1686932677.39s 77mb                                    |--------------L0.44--------------|                   "
    - "L0.45[1686854819000000000,1686868199000000000] 1686935947.46s 89mb                                     |---------------L0.45---------------|                "
    - "L0.48[1686855119000000000,1686868739000000000] 1686929965.33s 93mb                                      |---------------L0.48----------------|              "
    - "L0.49[1686855179000000000,1686868859000000000] 1686933830.06s 92mb                                      |---------------L0.49----------------|              "
    - "L0.50[1686855479000000000,1686869519000000000] 1686935151.54s 97mb                                       |----------------L0.50----------------|            "
    - "L0.51[1686855719000000000,1686869939000000000] 1686931893.7s 95mb                                        |----------------L0.51----------------|           "
    - "L0.52[1686858179000000000,1686865979000000000] 1686932458.05s 94mb                                              |-------L0.52-------|                       "
    - "L0.57[1686862919000000000,1686873599000000000] 1686934254.96s 98mb                                                            |-----------L0.57-----------| "
    - "L0.60[1686864419000000000,1686873599000000000] 1686929712.33s 75mb                                                                |---------L0.60---------| "
    - "L0.61[1686864419000000000,1686873599000000000] 1686935742.51s 75mb                                                                |---------L0.61---------| "
    - "L0.62[1686864479000000000,1686873599000000000] 1686934966.48s 74mb                                                                |---------L0.62---------| "
    - "L0.63[1686864479000000000,1686873599000000000] 1686931600.58s 79mb                                                                |---------L0.63---------| "
    - "L0.64[1686864659000000000,1686873599000000000] 1686933528.17s 73mb                                                                 |--------L0.64---------| "
    - "L0.65[1686864719000000000,1686873599000000000] 1686930563.07s 54mb                                                                 |--------L0.65---------| "
    - "L0.66[1686864719000000000,1686873599000000000] 1686933271.57s 54mb                                                                 |--------L0.66---------| "
    - "L0.67[1686864899000000000,1686873599000000000] 1686931336.08s 53mb                                                                 |--------L0.67---------| "
    - "L0.68[1686865319000000000,1686873599000000000] 1686929421.02s 49mb                                                                  |--------L0.68--------| "
    - "L0.69[1686865319000000000,1686873599000000000] 1686935546.05s 49mb                                                                  |--------L0.69--------| "
    - "L0.70[1686865499000000000,1686873599000000000] 1686930780.95s 88mb                                                                   |-------L0.70--------| "
    - "L0.71[1686866099000000000,1686873599000000000] 1686934759.75s 45mb                                                                     |------L0.71-------| "
    - "L0.72[1686867119000000000,1686873599000000000] 1686932677.39s 109mb                                                                       |-----L0.72------| "
    - "L0.76[1686868259000000000,1686873599000000000] 1686935947.46s 67mb                                                                           |---L0.76----| "
    - "L0.79[1686868799000000000,1686873599000000000] 1686929965.33s 58mb                                                                            |---L0.79---| "
    - "L0.80[1686868919000000000,1686873599000000000] 1686933830.06s 57mb                                                                            |---L0.80---| "
    - "L0.81[1686869579000000000,1686873599000000000] 1686935151.54s 49mb                                                                              |--L0.81--| "
    - "L0.82[1686869999000000000,1686873599000000000] 1686931893.7s 44mb                                                                               |-L0.82--| "
    - "L0.83[1686871619000000000,1686873599000000000] 1686936871.55s 9mb                                                                                    |L0.83|"
    - "L1                                                                                                                 "
    - "L1.6[1686841379000000000,1686845579000000000] 1686928854.57s 5mb|--L1.6---|                                                                               "
    - "L1.24[1686845639000000000,1686849779000000000] 1686928854.57s 5mb           |--L1.24--|                                                                    "
    - "L1.28[1686849839000000000,1686850559000000000] 1686928854.57s 975kb                       |L1.28|                                                            "
    - "L1.29[1686850619000000000,1686854819000000000] 1686928854.57s 5mb                         |--L1.29--|                                                      "
    - "L1.47[1686854879000000000,1686859019000000000] 1686928854.57s 5mb                                     |--L1.47--|                                          "
    - "L1.53[1686859079000000000,1686859499000000000] 1686928854.57s 579kb                                                 |L1.53|                                  "
    - "L1.55[1686859559000000000,1686863699000000000] 1686928854.57s 5mb                                                  |--L1.55--|                             "
    - "L1.58[1686863759000000000,1686867659000000000] 1686928854.57s 5mb                                                              |-L1.58--|                  "
    - "L1.73[1686867719000000000,1686867839000000000] 1686928854.57s 225kb                                                                         |L1.73|          "
    - "L1.75[1686867899000000000,1686868319000000000] 1686928854.57s 588kb                                                                          |L1.75|         "
    - "L1.77[1686868379000000000,1686873599000000000] 1686928854.57s 89mb                                                                           |---L1.77----| "
    - "L2                                                                                                                 "
    - "L2.2[1686841379000000000,1686845579000000000] 1686928811.43s 98mb|--L2.2---|                                                                               "
    - "L2.25[1686845639000000000,1686849779000000000] 1686928811.43s 97mb           |--L2.25--|                                                                    "
    - "L2.27[1686849839000000000,1686850559000000000] 1686928811.43s 20mb                       |L2.27|                                                            "
    - "L2.30[1686850619000000000,1686854819000000000] 1686928811.43s 98mb                         |--L2.30--|                                                      "
    - "L2.46[1686854879000000000,1686859019000000000] 1686928811.43s 97mb                                     |--L2.46--|                                          "
    - "L2.54[1686859079000000000,1686859499000000000] 1686928811.43s 14mb                                                 |L2.54|                                  "
    - "L2.56[1686859559000000000,1686863699000000000] 1686928811.43s 97mb                                                  |--L2.56--|                             "
    - "L2.59[1686863759000000000,1686867839000000000] 1686928811.43s 96mb                                                              |--L2.59--|                 "
    - "L2.74[1686867899000000000,1686868319000000000] 1686928811.43s 14mb                                                                          |L2.74|         "
    - "L2.78[1686868379000000000,1686873599000000000] 1686928118.43s 39mb                                                                           |---L2.78----| "
    - "**** Final Output Files (41.44gb written)"
    - "L2                                                                                                                 "
    - "L2.1506[1686841379000000000,1686841830146391412] 1686936871.55s 100mb|L2.1506|                                                                                 "
    - "L2.1615[1686841830146391413,1686842272734754598] 1686936871.55s 100mb |L2.1615|                                                                                "
    - "L2.1616[1686842272734754599,1686842715323117783] 1686936871.55s 100mb  |L2.1616|                                                                               "
    - "L2.1617[1686842715323117784,1686843089433841661] 1686936871.55s 85mb   |L2.1617|                                                                              "
    - "L2.1631[1686843089433841662,1686843544819538616] 1686936871.55s 100mb    |L2.1631|                                                                             "
    - "L2.1632[1686843544819538617,1686844000205235570] 1686936871.55s 100mb      |L2.1632|                                                                           "
    - "L2.1633[1686844000205235571,1686844006223321786] 1686936871.55s 1mb       |L2.1633|                                                                          "
    - "L2.1634[1686844006223321787,1686844558565925583] 1686936871.55s 100mb       |L2.1634|                                                                          "
    - "L2.1635[1686844558565925584,1686845110908529379] 1686936871.55s 100mb        |L2.1635|                                                                         "
    - "L2.1636[1686845110908529380,1686845399754314022] 1686936871.55s 52mb          |L2.1636|                                                                       "
    - "L2.1637[1686845399754314023,1686845828734675041] 1686936871.55s 100mb           |L2.1637|                                                                      "
    - "L2.1638[1686845828734675042,1686846257715036059] 1686936871.55s 100mb            |L2.1638|                                                                     "
    - "L2.1639[1686846257715036060,1686846333021885171] 1686936871.55s 18mb             |L2.1639|                                                                    "
    - "L2.1640[1686846333021885172,1686846691134013454] 1686936871.55s 100mb             |L2.1640|                                                                    "
    - "L2.1641[1686846691134013455,1686847049246141736] 1686936871.55s 100mb              |L2.1641|                                                                   "
    - "L2.1642[1686847049246141737,1686847215718005883] 1686936871.55s 46mb               |L2.1642|                                                                  "
    - "L2.1643[1686847215718005884,1686847702600239065] 1686936871.55s 100mb                |L2.1643|                                                                 "
    - "L2.1644[1686847702600239066,1686848189482472246] 1686936871.55s 100mb                 |L2.1644|                                                                "
    - "L2.1645[1686848189482472247,1686848513620965616] 1686936871.55s 67mb                   |L2.1645|                                                              "
    - "L2.1656[1686848513620965617,1686849040083884614] 1686936871.55s 100mb                   |L2.1656|                                                              "
    - "L2.1657[1686849040083884615,1686849561540115022] 1686936871.55s 99mb                     |L2.1657|                                                            "
    - "L2.1658[1686849561540115023,1686849962539887485] 1686936871.55s 100mb                      |L2.1658|                                                           "
    - "L2.1659[1686849962539887486,1686850363539659947] 1686936871.55s 100mb                       |L2.1659|                                                          "
    - "L2.1660[1686850363539659948,1686850403000000000] 1686936871.55s 10mb                         |L2.1660|                                                        "
    - "L2.1683[1686850403000000001,1686850839595751321] 1686936871.55s 100mb                         |L2.1683|                                                        "
    - "L2.1684[1686850839595751322,1686851276191502641] 1686936871.55s 100mb                          |L2.1684|                                                       "
    - "L2.1685[1686851276191502642,1686851529568101662] 1686936871.55s 58mb                           |L2.1685|                                                      "
    - "L2.1692[1686851529568101663,1686852034790162698] 1686936871.55s 100mb                            |L2.1692|                                                     "
    - "L2.1693[1686852034790162699,1686852540012223733] 1686936871.55s 100mb                             |L2.1693|                                                    "
    - "L2.1694[1686852540012223734,1686852699540540531] 1686936871.55s 32mb                               |L2.1694|                                                  "
    - "L2.1707[1686852699540540532,1686853178683862218] 1686936871.55s 100mb                               |L2.1707|                                                  "
    - "L2.1708[1686853178683862219,1686853657827183904] 1686936871.55s 100mb                                |L2.1708|                                                 "
    - "L2.1709[1686853657827183905,1686853937177035656] 1686936871.55s 58mb                                  |L2.1709|                                               "
    - "L2.1710[1686853937177035657,1686854376975623947] 1686936871.55s 100mb                                   |L2.1710|                                              "
    - "L2.1711[1686854376975623948,1686854816774212237] 1686936871.55s 100mb                                    |L2.1711|                                             "
    - "L2.1712[1686854816774212238,1686854951811503765] 1686936871.55s 31mb                                     |L2.1712|                                            "
    - "L2.1713[1686854951811503766,1686855334994518642] 1686936871.55s 100mb                                     |L2.1713|                                            "
    - "L2.1714[1686855334994518643,1686855718177533518] 1686936871.55s 100mb                                      |L2.1714|                                           "
    - "L2.1715[1686855718177533519,1686855814952951422] 1686936871.55s 25mb                                        |L2.1715|                                         "
    - "L2.1716[1686855814952951423,1686856242828120943] 1686936871.55s 100mb                                        |L2.1716|                                         "
    - "L2.1717[1686856242828120944,1686856670703290463] 1686936871.55s 100mb                                         |L2.1717|                                        "
    - "L2.1718[1686856670703290464,1686857061484420497] 1686936871.55s 91mb                                          |L2.1718|                                       "
    - "L2.1721[1686857061484420498,1686857533170368523] 1686936871.55s 100mb                                           |L2.1721|                                      "
    - "L2.1722[1686857533170368524,1686858004856316548] 1686936871.55s 100mb                                             |L2.1722|                                    "
    - "L2.1723[1686858004856316549,1686858110845666271] 1686936871.55s 22mb                                              |L2.1723|                                   "
    - "L2.1724[1686858110845666272,1686858549633346816] 1686936871.55s 100mb                                              |L2.1724|                                   "
    - "L2.1725[1686858549633346817,1686858988421027360] 1686936871.55s 100mb                                               |L2.1725|                                  "
    - "L2.1726[1686858988421027361,1686859260138002023] 1686936871.55s 62mb                                                 |L2.1726|                                "
    - "L2.1727[1686859260138002024,1686859794383312214] 1686936871.55s 100mb                                                 |L2.1727|                                "
    - "L2.1728[1686859794383312215,1686860328628622404] 1686936871.55s 100mb                                                   |L2.1728|                              "
    - "L2.1729[1686860328628622405,1686860846662278383] 1686936871.55s 97mb                                                    |L2.1729|                             "
    - "L2.1734[1686860846662278384,1686861228052662060] 1686936871.55s 100mb                                                      |L2.1734|                           "
    - "L2.1735[1686861228052662061,1686861609443045736] 1686936871.55s 100mb                                                       |L2.1735|                          "
    - "L2.1736[1686861609443045737,1686861812108728437] 1686936871.55s 53mb                                                        |L2.1736|                         "
    - "L2.1737[1686861812108728438,1686862208555549766] 1686936871.55s 100mb                                                         |L2.1737|                        "
    - "L2.1738[1686862208555549767,1686862605002371094] 1686936871.55s 100mb                                                          |L2.1738|                       "
    - "L2.1739[1686862605002371095,1686862618002078989] 1686936871.55s 3mb                                                           |L2.1739|                      "
    - "L2.1740[1686862618002078990,1686863086619857890] 1686936871.55s 100mb                                                           |L2.1740|                      "
    - "L2.1741[1686863086619857891,1686863555237636790] 1686936871.55s 100mb                                                            |L2.1741|                     "
    - "L2.1742[1686863555237636791,1686863699000000000] 1686936871.55s 31mb                                                             |L2.1742|                    "
    - "L2.1743[1686863699000000001,1686865135092420619] 1686936871.55s 100mb                                                              |L2.1743|                   "
    - "L2.1744[1686865135092420620,1686866571184841237] 1686936871.55s 100mb                                                                  |L2.1744|               "
    - "L2.1745[1686866571184841238,1686867839000000000] 1686936871.55s 88mb                                                                      |L2.1745|           "
    - "L2.1746[1686867839000000001,1686868223000000000] 1686936871.55s 30mb                                                                         |L2.1746|        "
    - "L2.1747[1686868223000000001,1686868319000000000] 1686936871.55s 8mb                                                                          |L2.1747|       "
    - "L2.1782[1686868319000000001,1686869127171211860] 1686936871.55s 100mb                                                                           |L2.1782|      "
    - "L2.1783[1686869127171211861,1686869935342423719] 1686936871.55s 100mb                                                                             |L2.1783|    "
    - "L2.1784[1686869935342423720,1686870115756756731] 1686936871.55s 22mb                                                                               |L2.1784|  "
    - "L2.1785[1686870115756756732,1686870608246428569] 1686936871.55s 100mb                                                                                |L2.1785| "
    - "L2.1786[1686870608246428570,1686871100736100406] 1686936871.55s 100mb                                                                                 |L2.1786|"
    - "L2.1787[1686871100736100407,1686871497752641904] 1686936871.55s 81mb                                                                                   |L2.1787|"
    - "L2.1788[1686871497752641905,1686871988074203157] 1686936871.55s 100mb                                                                                    |L2.1788|"
    - "L2.1789[1686871988074203158,1686872478395764409] 1686936871.55s 100mb                                                                                     |L2.1789|"
    - "L2.1790[1686872478395764410,1686872728189189161] 1686936871.55s 51mb                                                                                      |L2.1790|"
    - "L2.1791[1686872728189189162,1686873217700530422] 1686936871.55s 100mb                                                                                       |L2.1791|"
    - "L2.1792[1686873217700530423,1686873599000000000] 1686936871.55s 78mb                                                                                        |L2.1792|"
    - "**** Breakdown of where bytes were written"
    - 1.66gb written by split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))
    - 18.49gb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 5.22gb written by compact(ManySmallFiles)
    - 524mb written by split(StartLevelOverlapsTooBig)
    - 7.29gb written by split(ReduceOverlap)
    - 8.26gb written by split(HighL0OverlapTotalBacklog)
    "###
    );
}

#[tokio::test]
async fn stuck_l1() {
    test_helpers::maybe_start_logging();

    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(20)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(100))
        .with_writes_breakdown()
        .build()
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686873630000000000)
                .with_max_time(1686879712000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(104071379),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686873630000000000)
                .with_max_time(1686920683000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928116935534089))
                .with_file_size_bytes(74761432),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686879750000000000)
                .with_max_time(1686885832000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(104046636),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686885870000000000)
                .with_max_time(1686888172000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(39504848),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686888210000000000)
                .with_max_time(1686894292000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(104068640),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686894330000000000)
                .with_max_time(1686900412000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(104024462),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686900450000000000)
                .with_max_time(1686901072000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(12847477),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686901110000000000)
                .with_max_time(1686907132000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(103082698),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686907170000000000)
                .with_max_time(1686910072000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686927078592450239))
                .with_file_size_bytes(51292692),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686910110000000000)
                .with_max_time(1686919792000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686926864318936602))
                .with_file_size_bytes(105671599),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686919830000000000)
                .with_max_time(1686926803000000000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686926864318936602))
                .with_file_size_bytes(71282156),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1686920730000000000)
                .with_max_time(1686926803000000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1686928116935534089))
                .with_file_size_bytes(38566243),
        )
        .await;

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L1                                                                                                                 "
    - "L1.2[1686873630000000000,1686920683000000000] 1686928116.94s 71mb|------------------------------------L1.2-------------------------------------|           "
    - "L1.12[1686920730000000000,1686926803000000000] 1686928116.94s 37mb                                                                               |-L1.12--| "
    - "L2                                                                                                                 "
    - "L2.1[1686873630000000000,1686879712000000000] 1686927078.59s 99mb|--L2.1--|                                                                                "
    - "L2.3[1686879750000000000,1686885832000000000] 1686927078.59s 99mb          |--L2.3--|                                                                      "
    - "L2.4[1686885870000000000,1686888172000000000] 1686927078.59s 38mb                    |L2.4|                                                                "
    - "L2.5[1686888210000000000,1686894292000000000] 1686927078.59s 99mb                        |--L2.5--|                                                        "
    - "L2.6[1686894330000000000,1686900412000000000] 1686927078.59s 99mb                                   |--L2.6--|                                             "
    - "L2.7[1686900450000000000,1686901072000000000] 1686927078.59s 12mb                                             |L2.7|                                       "
    - "L2.8[1686901110000000000,1686907132000000000] 1686927078.59s 98mb                                              |--L2.8--|                                  "
    - "L2.9[1686907170000000000,1686910072000000000] 1686927078.59s 49mb                                                        |L2.9|                            "
    - "L2.10[1686910110000000000,1686919792000000000] 1686926864.32s 101mb                                                             |----L2.10-----|             "
    - "L2.11[1686919830000000000,1686926803000000000] 1686926864.32s 68mb                                                                              |--L2.11--| "
    - "**** Simulation run 0, type=split(ReduceOverlap)(split_times=[1686879712000000000, 1686885832000000000, 1686888172000000000, 1686894292000000000, 1686900412000000000, 1686901072000000000, 1686907132000000000, 1686910072000000000, 1686919792000000000]). 1 Input Files, 71mb total:"
    - "L1, all files 71mb                                                                                                 "
    - "L1.2[1686873630000000000,1686920683000000000] 1686928116.94s|------------------------------------------L1.2------------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 71mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1686873630000000000,1686879712000000000] 1686928116.94s 9mb|--L1.?---|                                                                               "
    - "L1.?[1686879712000000001,1686885832000000000] 1686928116.94s 9mb           |--L1.?---|                                                                    "
    - "L1.?[1686885832000000001,1686888172000000000] 1686928116.94s 4mb                       |L1.?|                                                             "
    - "L1.?[1686888172000000001,1686894292000000000] 1686928116.94s 9mb                           |--L1.?---|                                                    "
    - "L1.?[1686894292000000001,1686900412000000000] 1686928116.94s 9mb                                       |--L1.?---|                                        "
    - "L1.?[1686900412000000001,1686901072000000000] 1686928116.94s 1mb                                                   |L1.?|                                 "
    - "L1.?[1686901072000000001,1686907132000000000] 1686928116.94s 9mb                                                    |--L1.?---|                           "
    - "L1.?[1686907132000000001,1686910072000000000] 1686928116.94s 4mb                                                                |L1.?|                    "
    - "L1.?[1686910072000000001,1686919792000000000] 1686928116.94s 15mb                                                                     |------L1.?------|   "
    - "L1.?[1686919792000000001,1686920683000000000] 1686928116.94s 1mb                                                                                        |L1.?|"
    - "Committing partition 1:"
    - "  Soft Deleting 1 files: L1.2"
    - "  Creating 10 files"
    - "**** Simulation run 1, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1686879262359644750, 1686884894719289500]). 6 Input Files, 258mb total:"
    - "L1                                                                                                                 "
    - "L1.13[1686873630000000000,1686879712000000000] 1686928116.94s 9mb|---------------L1.13---------------|                                                     "
    - "L1.14[1686879712000000001,1686885832000000000] 1686928116.94s 9mb                                     |---------------L1.14---------------|                "
    - "L1.15[1686885832000000001,1686888172000000000] 1686928116.94s 4mb                                                                           |---L1.15----| "
    - "L2                                                                                                                 "
    - "L2.1[1686873630000000000,1686879712000000000] 1686927078.59s 99mb|---------------L2.1----------------|                                                     "
    - "L2.3[1686879750000000000,1686885832000000000] 1686927078.59s 99mb                                     |---------------L2.3----------------|                "
    - "L2.4[1686885870000000000,1686888172000000000] 1686927078.59s 38mb                                                                           |----L2.4----| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 258mb total:"
    - "L2                                                                                                                 "
    - "L2.?[1686873630000000000,1686879262359644750] 1686928116.94s 100mb|--------------L2.?--------------|                                                        "
    - "L2.?[1686879262359644751,1686884894719289500] 1686928116.94s 100mb                                  |--------------L2.?--------------|                      "
    - "L2.?[1686884894719289501,1686888172000000000] 1686928116.94s 58mb                                                                     |-------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L2.1, L2.3, L2.4, L1.13, L1.14, L1.15"
    - "  Creating 3 files"
    - "**** Final Output Files (329mb written)"
    - "L1                                                                                                                 "
    - "L1.12[1686920730000000000,1686926803000000000] 1686928116.94s 37mb                                                                               |-L1.12--| "
    - "L1.16[1686888172000000001,1686894292000000000] 1686928116.94s 9mb                        |-L1.16--|                                                        "
    - "L1.17[1686894292000000001,1686900412000000000] 1686928116.94s 9mb                                  |-L1.17--|                                              "
    - "L1.18[1686900412000000001,1686901072000000000] 1686928116.94s 1mb                                             |L1.18|                                      "
    - "L1.19[1686901072000000001,1686907132000000000] 1686928116.94s 9mb                                              |-L1.19--|                                  "
    - "L1.20[1686907132000000001,1686910072000000000] 1686928116.94s 4mb                                                        |L1.20|                           "
    - "L1.21[1686910072000000001,1686919792000000000] 1686928116.94s 15mb                                                             |----L1.21-----|             "
    - "L1.22[1686919792000000001,1686920683000000000] 1686928116.94s 1mb                                                                              |L1.22|     "
    - "L2                                                                                                                 "
    - "L2.5[1686888210000000000,1686894292000000000] 1686927078.59s 99mb                        |--L2.5--|                                                        "
    - "L2.6[1686894330000000000,1686900412000000000] 1686927078.59s 99mb                                   |--L2.6--|                                             "
    - "L2.7[1686900450000000000,1686901072000000000] 1686927078.59s 12mb                                             |L2.7|                                       "
    - "L2.8[1686901110000000000,1686907132000000000] 1686927078.59s 98mb                                              |--L2.8--|                                  "
    - "L2.9[1686907170000000000,1686910072000000000] 1686927078.59s 49mb                                                        |L2.9|                            "
    - "L2.10[1686910110000000000,1686919792000000000] 1686926864.32s 101mb                                                             |----L2.10-----|             "
    - "L2.11[1686919830000000000,1686926803000000000] 1686926864.32s 68mb                                                                              |--L2.11--| "
    - "L2.23[1686873630000000000,1686879262359644750] 1686928116.94s 100mb|-L2.23-|                                                                                 "
    - "L2.24[1686879262359644751,1686884894719289500] 1686928116.94s 100mb         |-L2.24-|                                                                        "
    - "L2.25[1686884894719289501,1686888172000000000] 1686928116.94s 58mb                   |L2.25|                                                                "
    - "**** Breakdown of where bytes were written"
    - 258mb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 71mb written by split(ReduceOverlap)
    "###
    );
    // TODO(maybe): see matching comment in files_to_compact.rs/limit_files_to_compact
    // The L1s left above are less than ideal, but maybe not bad.  This scenario initially just barely met the criteria for compaction and started with splits
    // to remove overlaps between L1 and L2 (that's good).  Due to the grouping of files they didn't all get to compact in the first round (that's ok).
    // But the first few that got compacted in the first round were enough to make the partition no longer meet the criteria for compaction, so the rest
    // are left sitting there ready to compact with their L2s, but not quite getting to.
    // The critical point is that this case doesn't loop forever anymore.
}

#[tokio::test]
async fn stuck_l0_large_l0s() {
    test_helpers::maybe_start_logging();

    let max_files = 20;
    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(max_files)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(100000))
        //.with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    // This will be a big set of overlapping L0s which lands in a single chain, forcing sorting by max_l0_created_at
    // within the chain.  We'll use that forced sorting by max_l0_created_at to try to force the first 20 files (which
    // are max sized), to be repeatedly compacted with themselves.

    // Three things to notice about the first set of files:
    // 1) Everything in this test will be forced into one overlapping chain, which forces sorting by max_lo_created_at
    //    within the chain.
    // 2) The first/earliest/left-most (by max_l0_created_at) 20 files are already max sized so L0->L0 compaction won't
    //    change them.  Compacting the first 20 will be an unproductive compaction (20 files -> 20 files).
    // 3) All of these files are time range 1->2000.  Compacting these then together will give us 20 max sized files
    //    that cover 100 ns each. All of those alternately split files will still overlap the rest of the chain, so
    //    compacting the first set doesn't kick anything out of the chain, and doesn't accomplish anything.
    for i in 0..max_files {
        setup
            .partition
            .create_parquet_file(
                parquet_builder()
                    .with_min_time(1)
                    .with_max_time(2000)
                    .with_compaction_level(CompactionLevel::Initial)
                    .with_max_l0_created_at(Time::from_timestamp_nanos(i as i64))
                    .with_file_size_bytes(MAX_DESIRED_FILE_SIZE),
            )
            .await;
    }
    // Things to notice about the second set of files:
    // 1) We're adding enough smaller files to bring down the average size for the chain so it looks like we need
    //    ManySmallFiles compactions.
    // 2) These files overlap the problem set of big files above (by min_time), but their max_l0_created_at puts
    //    them all after the problem set of big files.
    for i in max_files..(max_files * 10) {
        setup
            .partition
            .create_parquet_file(
                parquet_builder()
                    .with_min_time(i as i64)
                    .with_max_time(i as i64 * 10000)
                    .with_compaction_level(CompactionLevel::Initial)
                    .with_max_l0_created_at(Time::from_timestamp_nanos(i as i64))
                    .with_file_size_bytes(10),
            )
            .await;
    }
    // In Summary:
    // Without special handling, this scenario is catagorized as ManySmallFiles, and repeatedly & unproductively compacts
    // the first set of files, which never accomplishes anything.
    // This test demonstrates the need for `file_classification_for_many_files` skipping oversized chunks of files.
    // Without that clause, this test loops forever with unproductive compactions.
    // With that clause, the first set of large files gets set aside during ManySmallFiles mode, then gets later compacted
    // into L1s when the rest of the L0s are reduced to fewer/larger files.

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.1[1,2000] 0ns 100mb   |L0.1|                                                                                    "
    - "L0.2[1,2000] 1ns 100mb   |L0.2|                                                                                    "
    - "L0.3[1,2000] 2ns 100mb   |L0.3|                                                                                    "
    - "L0.4[1,2000] 3ns 100mb   |L0.4|                                                                                    "
    - "L0.5[1,2000] 4ns 100mb   |L0.5|                                                                                    "
    - "L0.6[1,2000] 5ns 100mb   |L0.6|                                                                                    "
    - "L0.7[1,2000] 6ns 100mb   |L0.7|                                                                                    "
    - "L0.8[1,2000] 7ns 100mb   |L0.8|                                                                                    "
    - "L0.9[1,2000] 8ns 100mb   |L0.9|                                                                                    "
    - "L0.10[1,2000] 9ns 100mb  |L0.10|                                                                                   "
    - "L0.11[1,2000] 10ns 100mb |L0.11|                                                                                   "
    - "L0.12[1,2000] 11ns 100mb |L0.12|                                                                                   "
    - "L0.13[1,2000] 12ns 100mb |L0.13|                                                                                   "
    - "L0.14[1,2000] 13ns 100mb |L0.14|                                                                                   "
    - "L0.15[1,2000] 14ns 100mb |L0.15|                                                                                   "
    - "L0.16[1,2000] 15ns 100mb |L0.16|                                                                                   "
    - "L0.17[1,2000] 16ns 100mb |L0.17|                                                                                   "
    - "L0.18[1,2000] 17ns 100mb |L0.18|                                                                                   "
    - "L0.19[1,2000] 18ns 100mb |L0.19|                                                                                   "
    - "L0.20[1,2000] 19ns 100mb |L0.20|                                                                                   "
    - "L0.21[20,200000] 20ns 10b|-L0.21-|                                                                                 "
    - "L0.22[21,210000] 21ns 10b|-L0.22-|                                                                                 "
    - "L0.23[22,220000] 22ns 10b|-L0.23-|                                                                                 "
    - "L0.24[23,230000] 23ns 10b|-L0.24--|                                                                                "
    - "L0.25[24,240000] 24ns 10b|-L0.25--|                                                                                "
    - "L0.26[25,250000] 25ns 10b|--L0.26--|                                                                               "
    - "L0.27[26,260000] 26ns 10b|--L0.27--|                                                                               "
    - "L0.28[27,270000] 27ns 10b|--L0.28---|                                                                              "
    - "L0.29[28,280000] 28ns 10b|--L0.29---|                                                                              "
    - "L0.30[29,290000] 29ns 10b|---L0.30---|                                                                             "
    - "L0.31[30,300000] 30ns 10b|---L0.31---|                                                                             "
    - "L0.32[31,310000] 31ns 10b|---L0.32----|                                                                            "
    - "L0.33[32,320000] 32ns 10b|---L0.33----|                                                                            "
    - "L0.34[33,330000] 33ns 10b|---L0.34----|                                                                            "
    - "L0.35[34,340000] 34ns 10b|----L0.35----|                                                                           "
    - "L0.36[35,350000] 35ns 10b|----L0.36----|                                                                           "
    - "L0.37[36,360000] 36ns 10b|----L0.37-----|                                                                          "
    - "L0.38[37,370000] 37ns 10b|----L0.38-----|                                                                          "
    - "L0.39[38,380000] 38ns 10b|-----L0.39-----|                                                                         "
    - "L0.40[39,390000] 39ns 10b|-----L0.40-----|                                                                         "
    - "L0.41[40,400000] 40ns 10b|-----L0.41------|                                                                        "
    - "L0.42[41,410000] 41ns 10b|-----L0.42------|                                                                        "
    - "L0.43[42,420000] 42ns 10b|-----L0.43------|                                                                        "
    - "L0.44[43,430000] 43ns 10b|------L0.44------|                                                                       "
    - "L0.45[44,440000] 44ns 10b|------L0.45------|                                                                       "
    - "L0.46[45,450000] 45ns 10b|------L0.46-------|                                                                      "
    - "L0.47[46,460000] 46ns 10b|------L0.47-------|                                                                      "
    - "L0.48[47,470000] 47ns 10b|-------L0.48-------|                                                                     "
    - "L0.49[48,480000] 48ns 10b|-------L0.49-------|                                                                     "
    - "L0.50[49,490000] 49ns 10b|-------L0.50--------|                                                                    "
    - "L0.51[50,500000] 50ns 10b|-------L0.51--------|                                                                    "
    - "L0.52[51,510000] 51ns 10b|--------L0.52--------|                                                                   "
    - "L0.53[52,520000] 52ns 10b|--------L0.53--------|                                                                   "
    - "L0.54[53,530000] 53ns 10b|--------L0.54--------|                                                                   "
    - "L0.55[54,540000] 54ns 10b|--------L0.55---------|                                                                  "
    - "L0.56[55,550000] 55ns 10b|--------L0.56---------|                                                                  "
    - "L0.57[56,560000] 56ns 10b|---------L0.57---------|                                                                 "
    - "L0.58[57,570000] 57ns 10b|---------L0.58---------|                                                                 "
    - "L0.59[58,580000] 58ns 10b|---------L0.59----------|                                                                "
    - "L0.60[59,590000] 59ns 10b|---------L0.60----------|                                                                "
    - "L0.61[60,600000] 60ns 10b|----------L0.61----------|                                                               "
    - "L0.62[61,610000] 61ns 10b|----------L0.62----------|                                                               "
    - "L0.63[62,620000] 62ns 10b|----------L0.63-----------|                                                              "
    - "L0.64[63,630000] 63ns 10b|----------L0.64-----------|                                                              "
    - "L0.65[64,640000] 64ns 10b|----------L0.65-----------|                                                              "
    - "L0.66[65,650000] 65ns 10b|-----------L0.66-----------|                                                             "
    - "L0.67[66,660000] 66ns 10b|-----------L0.67-----------|                                                             "
    - "L0.68[67,670000] 67ns 10b|-----------L0.68------------|                                                            "
    - "L0.69[68,680000] 68ns 10b|-----------L0.69------------|                                                            "
    - "L0.70[69,690000] 69ns 10b|------------L0.70------------|                                                           "
    - "L0.71[70,700000] 70ns 10b|------------L0.71------------|                                                           "
    - "L0.72[71,710000] 71ns 10b|------------L0.72-------------|                                                          "
    - "L0.73[72,720000] 72ns 10b|------------L0.73-------------|                                                          "
    - "L0.74[73,730000] 73ns 10b|-------------L0.74-------------|                                                         "
    - "L0.75[74,740000] 74ns 10b|-------------L0.75-------------|                                                         "
    - "L0.76[75,750000] 75ns 10b|-------------L0.76-------------|                                                         "
    - "L0.77[76,760000] 76ns 10b|-------------L0.77--------------|                                                        "
    - "L0.78[77,770000] 77ns 10b|-------------L0.78--------------|                                                        "
    - "L0.79[78,780000] 78ns 10b|--------------L0.79--------------|                                                       "
    - "L0.80[79,790000] 79ns 10b|--------------L0.80--------------|                                                       "
    - "L0.81[80,800000] 80ns 10b|--------------L0.81---------------|                                                      "
    - "L0.82[81,810000] 81ns 10b|--------------L0.82---------------|                                                      "
    - "L0.83[82,820000] 82ns 10b|---------------L0.83---------------|                                                     "
    - "L0.84[83,830000] 83ns 10b|---------------L0.84---------------|                                                     "
    - "L0.85[84,840000] 84ns 10b|---------------L0.85---------------|                                                     "
    - "L0.86[85,850000] 85ns 10b|---------------L0.86----------------|                                                    "
    - "L0.87[86,860000] 86ns 10b|---------------L0.87----------------|                                                    "
    - "L0.88[87,870000] 87ns 10b|----------------L0.88----------------|                                                   "
    - "L0.89[88,880000] 88ns 10b|----------------L0.89----------------|                                                   "
    - "L0.90[89,890000] 89ns 10b|----------------L0.90-----------------|                                                  "
    - "L0.91[90,900000] 90ns 10b|----------------L0.91-----------------|                                                  "
    - "L0.92[91,910000] 91ns 10b|-----------------L0.92-----------------|                                                 "
    - "L0.93[92,920000] 92ns 10b|-----------------L0.93-----------------|                                                 "
    - "L0.94[93,930000] 93ns 10b|-----------------L0.94------------------|                                                "
    - "L0.95[94,940000] 94ns 10b|-----------------L0.95------------------|                                                "
    - "L0.96[95,950000] 95ns 10b|-----------------L0.96------------------|                                                "
    - "L0.97[96,960000] 96ns 10b|------------------L0.97------------------|                                               "
    - "L0.98[97,970000] 97ns 10b|------------------L0.98------------------|                                               "
    - "L0.99[98,980000] 98ns 10b|------------------L0.99-------------------|                                              "
    - "L0.100[99,990000] 99ns 10b|------------------L0.100------------------|                                              "
    - "L0.101[100,1000000] 100ns 10b|------------------L0.101-------------------|                                             "
    - "L0.102[101,1010000] 101ns 10b|------------------L0.102-------------------|                                             "
    - "L0.103[102,1020000] 102ns 10b|-------------------L0.103-------------------|                                            "
    - "L0.104[103,1030000] 103ns 10b|-------------------L0.104-------------------|                                            "
    - "L0.105[104,1040000] 104ns 10b|-------------------L0.105--------------------|                                           "
    - "L0.106[105,1050000] 105ns 10b|-------------------L0.106--------------------|                                           "
    - "L0.107[106,1060000] 106ns 10b|-------------------L0.107--------------------|                                           "
    - "L0.108[107,1070000] 107ns 10b|--------------------L0.108--------------------|                                          "
    - "L0.109[108,1080000] 108ns 10b|--------------------L0.109--------------------|                                          "
    - "L0.110[109,1090000] 109ns 10b|--------------------L0.110---------------------|                                         "
    - "L0.111[110,1100000] 110ns 10b|--------------------L0.111---------------------|                                         "
    - "L0.112[111,1110000] 111ns 10b|---------------------L0.112---------------------|                                        "
    - "L0.113[112,1120000] 112ns 10b|---------------------L0.113---------------------|                                        "
    - "L0.114[113,1130000] 113ns 10b|---------------------L0.114----------------------|                                       "
    - "L0.115[114,1140000] 114ns 10b|---------------------L0.115----------------------|                                       "
    - "L0.116[115,1150000] 115ns 10b|----------------------L0.116----------------------|                                      "
    - "L0.117[116,1160000] 116ns 10b|----------------------L0.117----------------------|                                      "
    - "L0.118[117,1170000] 117ns 10b|----------------------L0.118----------------------|                                      "
    - "L0.119[118,1180000] 118ns 10b|----------------------L0.119-----------------------|                                     "
    - "L0.120[119,1190000] 119ns 10b|----------------------L0.120-----------------------|                                     "
    - "L0.121[120,1200000] 120ns 10b|-----------------------L0.121-----------------------|                                    "
    - "L0.122[121,1210000] 121ns 10b|-----------------------L0.122-----------------------|                                    "
    - "L0.123[122,1220000] 122ns 10b|-----------------------L0.123------------------------|                                   "
    - "L0.124[123,1230000] 123ns 10b|-----------------------L0.124------------------------|                                   "
    - "L0.125[124,1240000] 124ns 10b|------------------------L0.125------------------------|                                  "
    - "L0.126[125,1250000] 125ns 10b|------------------------L0.126------------------------|                                  "
    - "L0.127[126,1260000] 126ns 10b|------------------------L0.127------------------------|                                  "
    - "L0.128[127,1270000] 127ns 10b|------------------------L0.128-------------------------|                                 "
    - "L0.129[128,1280000] 128ns 10b|------------------------L0.129-------------------------|                                 "
    - "L0.130[129,1290000] 129ns 10b|-------------------------L0.130-------------------------|                                "
    - "L0.131[130,1300000] 130ns 10b|-------------------------L0.131-------------------------|                                "
    - "L0.132[131,1310000] 131ns 10b|-------------------------L0.132--------------------------|                               "
    - "L0.133[132,1320000] 132ns 10b|-------------------------L0.133--------------------------|                               "
    - "L0.134[133,1330000] 133ns 10b|--------------------------L0.134--------------------------|                              "
    - "L0.135[134,1340000] 134ns 10b|--------------------------L0.135--------------------------|                              "
    - "L0.136[135,1350000] 135ns 10b|--------------------------L0.136---------------------------|                             "
    - "L0.137[136,1360000] 136ns 10b|--------------------------L0.137---------------------------|                             "
    - "L0.138[137,1370000] 137ns 10b|--------------------------L0.138---------------------------|                             "
    - "L0.139[138,1380000] 138ns 10b|---------------------------L0.139---------------------------|                            "
    - "L0.140[139,1390000] 139ns 10b|---------------------------L0.140---------------------------|                            "
    - "L0.141[140,1400000] 140ns 10b|---------------------------L0.141----------------------------|                           "
    - "L0.142[141,1410000] 141ns 10b|---------------------------L0.142----------------------------|                           "
    - "L0.143[142,1420000] 142ns 10b|----------------------------L0.143----------------------------|                          "
    - "L0.144[143,1430000] 143ns 10b|----------------------------L0.144----------------------------|                          "
    - "L0.145[144,1440000] 144ns 10b|----------------------------L0.145-----------------------------|                         "
    - "L0.146[145,1450000] 145ns 10b|----------------------------L0.146-----------------------------|                         "
    - "L0.147[146,1460000] 146ns 10b|-----------------------------L0.147-----------------------------|                        "
    - "L0.148[147,1470000] 147ns 10b|-----------------------------L0.148-----------------------------|                        "
    - "L0.149[148,1480000] 148ns 10b|-----------------------------L0.149-----------------------------|                        "
    - "L0.150[149,1490000] 149ns 10b|-----------------------------L0.150------------------------------|                       "
    - "L0.151[150,1500000] 150ns 10b|-----------------------------L0.151------------------------------|                       "
    - "L0.152[151,1510000] 151ns 10b|------------------------------L0.152------------------------------|                      "
    - "L0.153[152,1520000] 152ns 10b|------------------------------L0.153------------------------------|                      "
    - "L0.154[153,1530000] 153ns 10b|------------------------------L0.154-------------------------------|                     "
    - "L0.155[154,1540000] 154ns 10b|------------------------------L0.155-------------------------------|                     "
    - "L0.156[155,1550000] 155ns 10b|-------------------------------L0.156-------------------------------|                    "
    - "L0.157[156,1560000] 156ns 10b|-------------------------------L0.157-------------------------------|                    "
    - "L0.158[157,1570000] 157ns 10b|-------------------------------L0.158-------------------------------|                    "
    - "L0.159[158,1580000] 158ns 10b|-------------------------------L0.159--------------------------------|                   "
    - "L0.160[159,1590000] 159ns 10b|-------------------------------L0.160--------------------------------|                   "
    - "L0.161[160,1600000] 160ns 10b|--------------------------------L0.161--------------------------------|                  "
    - "L0.162[161,1610000] 161ns 10b|--------------------------------L0.162--------------------------------|                  "
    - "L0.163[162,1620000] 162ns 10b|--------------------------------L0.163---------------------------------|                 "
    - "L0.164[163,1630000] 163ns 10b|--------------------------------L0.164---------------------------------|                 "
    - "L0.165[164,1640000] 164ns 10b|---------------------------------L0.165---------------------------------|                "
    - "L0.166[165,1650000] 165ns 10b|---------------------------------L0.166---------------------------------|                "
    - "L0.167[166,1660000] 166ns 10b|---------------------------------L0.167----------------------------------|               "
    - "L0.168[167,1670000] 167ns 10b|---------------------------------L0.168----------------------------------|               "
    - "L0.169[168,1680000] 168ns 10b|---------------------------------L0.169----------------------------------|               "
    - "L0.170[169,1690000] 169ns 10b|----------------------------------L0.170----------------------------------|              "
    - "L0.171[170,1700000] 170ns 10b|----------------------------------L0.171----------------------------------|              "
    - "L0.172[171,1710000] 171ns 10b|----------------------------------L0.172-----------------------------------|             "
    - "L0.173[172,1720000] 172ns 10b|----------------------------------L0.173-----------------------------------|             "
    - "L0.174[173,1730000] 173ns 10b|-----------------------------------L0.174-----------------------------------|            "
    - "L0.175[174,1740000] 174ns 10b|-----------------------------------L0.175-----------------------------------|            "
    - "L0.176[175,1750000] 175ns 10b|-----------------------------------L0.176------------------------------------|           "
    - "L0.177[176,1760000] 176ns 10b|-----------------------------------L0.177------------------------------------|           "
    - "L0.178[177,1770000] 177ns 10b|------------------------------------L0.178------------------------------------|          "
    - "L0.179[178,1780000] 178ns 10b|------------------------------------L0.179------------------------------------|          "
    - "L0.180[179,1790000] 179ns 10b|------------------------------------L0.180------------------------------------|          "
    - "L0.181[180,1800000] 180ns 10b|------------------------------------L0.181-------------------------------------|         "
    - "L0.182[181,1810000] 181ns 10b|------------------------------------L0.182-------------------------------------|         "
    - "L0.183[182,1820000] 182ns 10b|-------------------------------------L0.183-------------------------------------|        "
    - "L0.184[183,1830000] 183ns 10b|-------------------------------------L0.184-------------------------------------|        "
    - "L0.185[184,1840000] 184ns 10b|-------------------------------------L0.185--------------------------------------|       "
    - "L0.186[185,1850000] 185ns 10b|-------------------------------------L0.186--------------------------------------|       "
    - "L0.187[186,1860000] 186ns 10b|--------------------------------------L0.187--------------------------------------|      "
    - "L0.188[187,1870000] 187ns 10b|--------------------------------------L0.188--------------------------------------|      "
    - "L0.189[188,1880000] 188ns 10b|--------------------------------------L0.189---------------------------------------|     "
    - "L0.190[189,1890000] 189ns 10b|--------------------------------------L0.190---------------------------------------|     "
    - "L0.191[190,1900000] 190ns 10b|--------------------------------------L0.191---------------------------------------|     "
    - "L0.192[191,1910000] 191ns 10b|---------------------------------------L0.192---------------------------------------|    "
    - "L0.193[192,1920000] 192ns 10b|---------------------------------------L0.193---------------------------------------|    "
    - "L0.194[193,1930000] 193ns 10b|---------------------------------------L0.194----------------------------------------|   "
    - "L0.195[194,1940000] 194ns 10b|---------------------------------------L0.195----------------------------------------|   "
    - "L0.196[195,1950000] 195ns 10b|----------------------------------------L0.196----------------------------------------|  "
    - "L0.197[196,1960000] 196ns 10b|----------------------------------------L0.197----------------------------------------|  "
    - "L0.198[197,1970000] 197ns 10b|----------------------------------------L0.198-----------------------------------------| "
    - "L0.199[198,1980000] 198ns 10b|----------------------------------------L0.199-----------------------------------------| "
    - "L0.200[199,1990000] 199ns 10b|----------------------------------------L0.200-----------------------------------------| "
    - "**** Simulation run 0, type=compact(ManySmallFiles). 20 Input Files, 200b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.21[20,200000] 20ns    |-------------------L0.21--------------------|                                            "
    - "L0.22[21,210000] 21ns    |--------------------L0.22---------------------|                                          "
    - "L0.23[22,220000] 22ns    |---------------------L0.23----------------------|                                        "
    - "L0.24[23,230000] 23ns    |-----------------------L0.24-----------------------|                                     "
    - "L0.25[24,240000] 24ns    |------------------------L0.25------------------------|                                   "
    - "L0.26[25,250000] 25ns    |-------------------------L0.26-------------------------|                                 "
    - "L0.27[26,260000] 26ns    |--------------------------L0.27--------------------------|                               "
    - "L0.28[27,270000] 27ns    |---------------------------L0.28----------------------------|                            "
    - "L0.29[28,280000] 28ns    |----------------------------L0.29-----------------------------|                          "
    - "L0.30[29,290000] 29ns    |-----------------------------L0.30------------------------------|                        "
    - "L0.31[30,300000] 30ns    |-------------------------------L0.31-------------------------------|                     "
    - "L0.32[31,310000] 31ns    |--------------------------------L0.32--------------------------------|                   "
    - "L0.33[32,320000] 32ns    |---------------------------------L0.33---------------------------------|                 "
    - "L0.34[33,330000] 33ns    |----------------------------------L0.34-----------------------------------|              "
    - "L0.35[34,340000] 34ns    |-----------------------------------L0.35------------------------------------|            "
    - "L0.36[35,350000] 35ns    |------------------------------------L0.36-------------------------------------|          "
    - "L0.37[36,360000] 36ns    |--------------------------------------L0.37--------------------------------------|       "
    - "L0.38[37,370000] 37ns    |---------------------------------------L0.38---------------------------------------|     "
    - "L0.39[38,380000] 38ns    |----------------------------------------L0.39----------------------------------------|   "
    - "L0.40[39,390000] 39ns    |-----------------------------------------L0.40-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 200b total:"
    - "L0, all files 200b                                                                                                 "
    - "L0.?[20,390000] 39ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.21, L0.22, L0.23, L0.24, L0.25, L0.26, L0.27, L0.28, L0.29, L0.30, L0.31, L0.32, L0.33, L0.34, L0.35, L0.36, L0.37, L0.38, L0.39, L0.40"
    - "  Creating 1 files"
    - "**** Simulation run 1, type=compact(ManySmallFiles). 20 Input Files, 390b total:"
    - "L0                                                                                                                 "
    - "L0.201[20,390000] 39ns 200b|--------------------------L0.201--------------------------|                              "
    - "L0.41[40,400000] 40ns 10b|---------------------------L0.41----------------------------|                            "
    - "L0.42[41,410000] 41ns 10b|----------------------------L0.42----------------------------|                           "
    - "L0.43[42,420000] 42ns 10b|-----------------------------L0.43-----------------------------|                         "
    - "L0.44[43,430000] 43ns 10b|-----------------------------L0.44------------------------------|                        "
    - "L0.45[44,440000] 44ns 10b|------------------------------L0.45-------------------------------|                      "
    - "L0.46[45,450000] 45ns 10b|-------------------------------L0.46-------------------------------|                     "
    - "L0.47[46,460000] 46ns 10b|--------------------------------L0.47--------------------------------|                   "
    - "L0.48[47,470000] 47ns 10b|--------------------------------L0.48---------------------------------|                  "
    - "L0.49[48,480000] 48ns 10b|---------------------------------L0.49----------------------------------|                "
    - "L0.50[49,490000] 49ns 10b|----------------------------------L0.50-----------------------------------|              "
    - "L0.51[50,500000] 50ns 10b|-----------------------------------L0.51-----------------------------------|             "
    - "L0.52[51,510000] 51ns 10b|------------------------------------L0.52------------------------------------|           "
    - "L0.53[52,520000] 52ns 10b|------------------------------------L0.53-------------------------------------|          "
    - "L0.54[53,530000] 53ns 10b|-------------------------------------L0.54--------------------------------------|        "
    - "L0.55[54,540000] 54ns 10b|--------------------------------------L0.55--------------------------------------|       "
    - "L0.56[55,550000] 55ns 10b|---------------------------------------L0.56---------------------------------------|     "
    - "L0.57[56,560000] 56ns 10b|---------------------------------------L0.57----------------------------------------|    "
    - "L0.58[57,570000] 57ns 10b|----------------------------------------L0.58-----------------------------------------|  "
    - "L0.59[58,580000] 58ns 10b|-----------------------------------------L0.59-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 390b total:"
    - "L0, all files 390b                                                                                                 "
    - "L0.?[20,580000] 58ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.41, L0.42, L0.43, L0.44, L0.45, L0.46, L0.47, L0.48, L0.49, L0.50, L0.51, L0.52, L0.53, L0.54, L0.55, L0.56, L0.57, L0.58, L0.59, L0.201"
    - "  Creating 1 files"
    - "**** Simulation run 2, type=compact(ManySmallFiles). 20 Input Files, 580b total:"
    - "L0                                                                                                                 "
    - "L0.202[20,580000] 58ns 390b|-----------------------------L0.202------------------------------|                       "
    - "L0.60[59,590000] 59ns 10b|------------------------------L0.60-------------------------------|                      "
    - "L0.61[60,600000] 60ns 10b|-------------------------------L0.61--------------------------------|                    "
    - "L0.62[61,610000] 61ns 10b|--------------------------------L0.62--------------------------------|                   "
    - "L0.63[62,620000] 62ns 10b|--------------------------------L0.63---------------------------------|                  "
    - "L0.64[63,630000] 63ns 10b|---------------------------------L0.64---------------------------------|                 "
    - "L0.65[64,640000] 64ns 10b|---------------------------------L0.65----------------------------------|                "
    - "L0.66[65,650000] 65ns 10b|----------------------------------L0.66----------------------------------|               "
    - "L0.67[66,660000] 66ns 10b|-----------------------------------L0.67-----------------------------------|             "
    - "L0.68[67,670000] 67ns 10b|-----------------------------------L0.68------------------------------------|            "
    - "L0.69[68,680000] 68ns 10b|------------------------------------L0.69------------------------------------|           "
    - "L0.70[69,690000] 69ns 10b|------------------------------------L0.70-------------------------------------|          "
    - "L0.71[70,700000] 70ns 10b|-------------------------------------L0.71-------------------------------------|         "
    - "L0.72[71,710000] 71ns 10b|-------------------------------------L0.72--------------------------------------|        "
    - "L0.73[72,720000] 72ns 10b|--------------------------------------L0.73---------------------------------------|      "
    - "L0.74[73,730000] 73ns 10b|---------------------------------------L0.74---------------------------------------|     "
    - "L0.75[74,740000] 74ns 10b|---------------------------------------L0.75----------------------------------------|    "
    - "L0.76[75,750000] 75ns 10b|----------------------------------------L0.76----------------------------------------|   "
    - "L0.77[76,760000] 76ns 10b|----------------------------------------L0.77-----------------------------------------|  "
    - "L0.78[77,770000] 77ns 10b|-----------------------------------------L0.78-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 580b total:"
    - "L0, all files 580b                                                                                                 "
    - "L0.?[20,770000] 77ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.60, L0.61, L0.62, L0.63, L0.64, L0.65, L0.66, L0.67, L0.68, L0.69, L0.70, L0.71, L0.72, L0.73, L0.74, L0.75, L0.76, L0.77, L0.78, L0.202"
    - "  Creating 1 files"
    - "**** Simulation run 3, type=compact(ManySmallFiles). 20 Input Files, 770b total:"
    - "L0                                                                                                                 "
    - "L0.203[20,770000] 77ns 580b|--------------------------------L0.203--------------------------------|                  "
    - "L0.79[78,780000] 78ns 10b|---------------------------------L0.79---------------------------------|                 "
    - "L0.80[79,790000] 79ns 10b|---------------------------------L0.80----------------------------------|                "
    - "L0.81[80,800000] 80ns 10b|---------------------------------L0.81----------------------------------|                "
    - "L0.82[81,810000] 81ns 10b|----------------------------------L0.82----------------------------------|               "
    - "L0.83[82,820000] 82ns 10b|----------------------------------L0.83-----------------------------------|              "
    - "L0.84[83,830000] 83ns 10b|-----------------------------------L0.84-----------------------------------|             "
    - "L0.85[84,840000] 84ns 10b|-----------------------------------L0.85------------------------------------|            "
    - "L0.86[85,850000] 85ns 10b|------------------------------------L0.86------------------------------------|           "
    - "L0.87[86,860000] 86ns 10b|------------------------------------L0.87-------------------------------------|          "
    - "L0.88[87,870000] 87ns 10b|-------------------------------------L0.88-------------------------------------|         "
    - "L0.89[88,880000] 88ns 10b|-------------------------------------L0.89--------------------------------------|        "
    - "L0.90[89,890000] 89ns 10b|--------------------------------------L0.90--------------------------------------|       "
    - "L0.91[90,900000] 90ns 10b|--------------------------------------L0.91---------------------------------------|      "
    - "L0.92[91,910000] 91ns 10b|---------------------------------------L0.92---------------------------------------|     "
    - "L0.93[92,920000] 92ns 10b|---------------------------------------L0.93----------------------------------------|    "
    - "L0.94[93,930000] 93ns 10b|----------------------------------------L0.94----------------------------------------|   "
    - "L0.95[94,940000] 94ns 10b|----------------------------------------L0.95-----------------------------------------|  "
    - "L0.96[95,950000] 95ns 10b|-----------------------------------------L0.96-----------------------------------------| "
    - "L0.97[96,960000] 96ns 10b|-----------------------------------------L0.97-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 770b total:"
    - "L0, all files 770b                                                                                                 "
    - "L0.?[20,960000] 96ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.79, L0.80, L0.81, L0.82, L0.83, L0.84, L0.85, L0.86, L0.87, L0.88, L0.89, L0.90, L0.91, L0.92, L0.93, L0.94, L0.95, L0.96, L0.97, L0.203"
    - "  Creating 1 files"
    - "**** Simulation run 4, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 770b total:"
    - "L0, all files 770b                                                                                                 "
    - "L0.204[20,960000] 96ns   |-----------------------------------------L0.204-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 770b total:"
    - "L0                                                                                                                 "
    - "L0.?[20,153077] 96ns 122b|----L0.?----|                                                                            "
    - "L0.?[153078,306153] 96ns 122b              |----L0.?----|                                                              "
    - "L0.?[306154,459229] 96ns 122b                            |----L0.?----|                                                "
    - "L0.?[459230,612305] 96ns 122b                                           |----L0.?----|                                 "
    - "L0.?[612306,765381] 96ns 122b                                                         |----L0.?----|                   "
    - "L0.?[765382,918457] 96ns 122b                                                                       |----L0.?----|     "
    - "L0.?[918458,960000] 96ns 38b                                                                                      |L0.?|"
    - "**** Simulation run 5, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.98[97,970000] 97ns    |-----------------------------------------L0.98------------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[97,153077] 97ns 1b  |----L0.?----|                                                                            "
    - "L0.?[153078,306153] 97ns 1b              |----L0.?----|                                                              "
    - "L0.?[306154,459229] 97ns 1b                            |----L0.?----|                                                "
    - "L0.?[459230,612305] 97ns 1b                                          |----L0.?----|                                  "
    - "L0.?[612306,765381] 97ns 1b                                                        |----L0.?----|                    "
    - "L0.?[765382,918457] 97ns 1b                                                                       |----L0.?----|     "
    - "L0.?[918458,970000] 97ns 4b                                                                                     |L0.?|"
    - "**** Simulation run 6, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.99[98,980000] 98ns    |-----------------------------------------L0.99------------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[98,153077] 98ns 1b  |----L0.?----|                                                                            "
    - "L0.?[153078,306153] 98ns 1b              |----L0.?----|                                                              "
    - "L0.?[306154,459229] 98ns 1b                            |----L0.?----|                                                "
    - "L0.?[459230,612305] 98ns 1b                                          |----L0.?----|                                  "
    - "L0.?[612306,765381] 98ns 1b                                                        |----L0.?----|                    "
    - "L0.?[765382,918457] 98ns 1b                                                                      |----L0.?----|      "
    - "L0.?[918458,980000] 98ns 4b                                                                                    |L0.?|"
    - "**** Simulation run 7, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.100[99,990000] 99ns   |-----------------------------------------L0.100-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[99,153077] 99ns 1b  |---L0.?----|                                                                             "
    - "L0.?[153078,306153] 99ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 99ns 1b                           |---L0.?----|                                                  "
    - "L0.?[459230,612305] 99ns 1b                                         |---L0.?----|                                    "
    - "L0.?[612306,765381] 99ns 1b                                                       |---L0.?----|                      "
    - "L0.?[765382,918457] 99ns 1b                                                                     |---L0.?----|        "
    - "L0.?[918458,990000] 99ns 4b                                                                                   |L0.?| "
    - "**** Simulation run 8, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.101[100,1000000] 100ns|-----------------------------------------L0.101-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[100,153077] 100ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 100ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 100ns 1b                           |---L0.?----|                                                  "
    - "L0.?[459230,612305] 100ns 1b                                         |---L0.?----|                                    "
    - "L0.?[612306,765381] 100ns 1b                                                       |---L0.?----|                      "
    - "L0.?[765382,918457] 100ns 1b                                                                    |---L0.?----|         "
    - "L0.?[918458,1000000] 100ns 4b                                                                                  |L0.?-| "
    - "**** Simulation run 9, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.102[101,1010000] 101ns|-----------------------------------------L0.102-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[101,153077] 101ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 101ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 101ns 1b                           |---L0.?----|                                                  "
    - "L0.?[459230,612305] 101ns 1b                                        |---L0.?----|                                     "
    - "L0.?[612306,765381] 101ns 1b                                                      |---L0.?----|                       "
    - "L0.?[765382,918457] 101ns 1b                                                                    |---L0.?----|         "
    - "L0.?[918458,1010000] 101ns 4b                                                                                 |-L0.?-| "
    - "**** Simulation run 10, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.103[102,1020000] 102ns|-----------------------------------------L0.103-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[102,153077] 102ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 102ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 102ns 1b                           |---L0.?----|                                                  "
    - "L0.?[459230,612305] 102ns 1b                                        |---L0.?----|                                     "
    - "L0.?[612306,765381] 102ns 1b                                                      |---L0.?----|                       "
    - "L0.?[765382,918457] 102ns 1b                                                                   |---L0.?----|          "
    - "L0.?[918458,1020000] 102ns 4b                                                                                 |-L0.?-| "
    - "**** Simulation run 11, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.104[103,1030000] 103ns|-----------------------------------------L0.104-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[103,153077] 103ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 103ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 103ns 1b                          |---L0.?----|                                                   "
    - "L0.?[459230,612305] 103ns 1b                                        |---L0.?----|                                     "
    - "L0.?[612306,765381] 103ns 1b                                                     |---L0.?----|                        "
    - "L0.?[765382,918457] 103ns 1b                                                                  |---L0.?----|           "
    - "L0.?[918458,1030000] 103ns 4b                                                                                |-L0.?--| "
    - "**** Simulation run 12, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.105[104,1040000] 104ns|-----------------------------------------L0.105-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[104,153077] 104ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 104ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 104ns 1b                          |---L0.?----|                                                   "
    - "L0.?[459230,612305] 104ns 1b                                       |---L0.?----|                                      "
    - "L0.?[612306,765381] 104ns 1b                                                    |---L0.?----|                         "
    - "L0.?[765382,918457] 104ns 1b                                                                  |---L0.?----|           "
    - "L0.?[918458,1040000] 104ns 4b                                                                               |--L0.?--| "
    - "**** Simulation run 13, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.106[105,1050000] 105ns|-----------------------------------------L0.106-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[105,153077] 105ns 1b|---L0.?----|                                                                             "
    - "L0.?[153078,306153] 105ns 1b             |---L0.?----|                                                                "
    - "L0.?[306154,459229] 105ns 1b                          |---L0.?----|                                                   "
    - "L0.?[459230,612305] 105ns 1b                                       |---L0.?----|                                      "
    - "L0.?[612306,765381] 105ns 1b                                                    |---L0.?----|                         "
    - "L0.?[765382,918457] 105ns 1b                                                                 |---L0.?----|            "
    - "L0.?[918458,1050000] 105ns 4b                                                                              |--L0.?---| "
    - "**** Simulation run 14, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.107[106,1060000] 106ns|-----------------------------------------L0.107-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[106,153077] 106ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 106ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 106ns 1b                         |---L0.?---|                                                     "
    - "L0.?[459230,612305] 106ns 1b                                      |---L0.?---|                                        "
    - "L0.?[612306,765381] 106ns 1b                                                   |---L0.?---|                           "
    - "L0.?[765382,918457] 106ns 1b                                                                |---L0.?---|              "
    - "L0.?[918458,1060000] 106ns 4b                                                                             |---L0.?---| "
    - "**** Simulation run 15, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.108[107,1070000] 107ns|-----------------------------------------L0.108-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[107,153077] 107ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 107ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 107ns 1b                         |---L0.?---|                                                     "
    - "L0.?[459230,612305] 107ns 1b                                      |---L0.?---|                                        "
    - "L0.?[612306,765381] 107ns 1b                                                   |---L0.?---|                           "
    - "L0.?[765382,918457] 107ns 1b                                                                |---L0.?---|              "
    - "L0.?[918458,1070000] 107ns 4b                                                                             |---L0.?---| "
    - "**** Simulation run 16, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.109[108,1080000] 108ns|-----------------------------------------L0.109-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[108,153077] 108ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 108ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 108ns 1b                         |---L0.?---|                                                     "
    - "L0.?[459230,612305] 108ns 1b                                      |---L0.?---|                                        "
    - "L0.?[612306,765381] 108ns 1b                                                   |---L0.?---|                           "
    - "L0.?[765382,918457] 108ns 1b                                                               |---L0.?---|               "
    - "L0.?[918458,1071533] 108ns 1b                                                                            |---L0.?---|  "
    - "L0.?[1071534,1080000] 108ns 3b                                                                                         |L0.?|"
    - "**** Simulation run 17, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.110[109,1090000] 109ns|-----------------------------------------L0.110-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[109,153077] 109ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 109ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 109ns 1b                         |---L0.?---|                                                     "
    - "L0.?[459230,612305] 109ns 1b                                     |---L0.?---|                                         "
    - "L0.?[612306,765381] 109ns 1b                                                  |---L0.?---|                            "
    - "L0.?[765382,918457] 109ns 1b                                                               |---L0.?---|               "
    - "L0.?[918458,1071533] 109ns 1b                                                                           |---L0.?---|   "
    - "L0.?[1071534,1090000] 109ns 3b                                                                                        |L0.?|"
    - "**** Simulation run 18, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.111[110,1100000] 110ns|-----------------------------------------L0.111-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[110,153077] 110ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 110ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 110ns 1b                         |---L0.?---|                                                     "
    - "L0.?[459230,612305] 110ns 1b                                     |---L0.?---|                                         "
    - "L0.?[612306,765381] 110ns 1b                                                  |---L0.?---|                            "
    - "L0.?[765382,918457] 110ns 1b                                                              |---L0.?---|                "
    - "L0.?[918458,1071533] 110ns 1b                                                                           |---L0.?---|   "
    - "L0.?[1071534,1100000] 110ns 3b                                                                                       |L0.?|"
    - "**** Simulation run 19, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.112[111,1110000] 111ns|-----------------------------------------L0.112-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[111,153077] 111ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 111ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 111ns 1b                        |---L0.?---|                                                      "
    - "L0.?[459230,612305] 111ns 1b                                     |---L0.?---|                                         "
    - "L0.?[612306,765381] 111ns 1b                                                 |---L0.?---|                             "
    - "L0.?[765382,918457] 111ns 1b                                                              |---L0.?---|                "
    - "L0.?[918458,1071533] 111ns 1b                                                                          |---L0.?---|    "
    - "L0.?[1071534,1110000] 111ns 3b                                                                                      |L0.?|"
    - "**** Simulation run 20, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.113[112,1120000] 112ns|-----------------------------------------L0.113-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[112,153077] 112ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 112ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 112ns 1b                        |---L0.?---|                                                      "
    - "L0.?[459230,612305] 112ns 1b                                    |---L0.?---|                                          "
    - "L0.?[612306,765381] 112ns 1b                                                 |---L0.?---|                             "
    - "L0.?[765382,918457] 112ns 1b                                                             |---L0.?---|                 "
    - "L0.?[918458,1071533] 112ns 1b                                                                         |---L0.?---|     "
    - "L0.?[1071534,1120000] 112ns 3b                                                                                      |L0.?|"
    - "**** Simulation run 21, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.114[113,1130000] 113ns|-----------------------------------------L0.114-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[113,153077] 113ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 113ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 113ns 1b                        |---L0.?---|                                                      "
    - "L0.?[459230,612305] 113ns 1b                                    |---L0.?---|                                          "
    - "L0.?[612306,765381] 113ns 1b                                                |---L0.?---|                              "
    - "L0.?[765382,918457] 113ns 1b                                                            |---L0.?---|                  "
    - "L0.?[918458,1071533] 113ns 1b                                                                         |---L0.?---|     "
    - "L0.?[1071534,1130000] 113ns 3b                                                                                     |L0.?|"
    - "**** Simulation run 22, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.115[114,1140000] 114ns|-----------------------------------------L0.115-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[114,153077] 114ns 1b|---L0.?---|                                                                              "
    - "L0.?[153078,306153] 114ns 1b            |---L0.?---|                                                                  "
    - "L0.?[306154,459229] 114ns 1b                        |---L0.?---|                                                      "
    - "L0.?[459230,612305] 114ns 1b                                    |---L0.?---|                                          "
    - "L0.?[612306,765381] 114ns 1b                                                |---L0.?---|                              "
    - "L0.?[765382,918457] 114ns 1b                                                            |---L0.?---|                  "
    - "L0.?[918458,1071533] 114ns 1b                                                                        |---L0.?---|      "
    - "L0.?[1071534,1140000] 114ns 3b                                                                                    |L0.?|"
    - "**** Simulation run 23, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.116[115,1150000] 115ns|-----------------------------------------L0.116-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[115,153077] 115ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 115ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 115ns 1b                       |--L0.?---|                                                        "
    - "L0.?[459230,612305] 115ns 1b                                   |--L0.?---|                                            "
    - "L0.?[612306,765381] 115ns 1b                                               |--L0.?---|                                "
    - "L0.?[765382,918457] 115ns 1b                                                           |--L0.?---|                    "
    - "L0.?[918458,1071533] 115ns 1b                                                                       |--L0.?---|        "
    - "L0.?[1071534,1150000] 115ns 3b                                                                                   |L0.?| "
    - "**** Simulation run 24, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.117[116,1160000] 116ns|-----------------------------------------L0.117-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[116,153077] 116ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 116ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 116ns 1b                       |--L0.?---|                                                        "
    - "L0.?[459230,612305] 116ns 1b                                   |--L0.?---|                                            "
    - "L0.?[612306,765381] 116ns 1b                                               |--L0.?---|                                "
    - "L0.?[765382,918457] 116ns 1b                                                           |--L0.?---|                    "
    - "L0.?[918458,1071533] 116ns 1b                                                                       |--L0.?---|        "
    - "L0.?[1071534,1160000] 116ns 3b                                                                                   |L0.?| "
    - "**** Simulation run 25, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.118[117,1170000] 117ns|-----------------------------------------L0.118-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[117,153077] 117ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 117ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 117ns 1b                       |--L0.?---|                                                        "
    - "L0.?[459230,612305] 117ns 1b                                   |--L0.?---|                                            "
    - "L0.?[612306,765381] 117ns 1b                                               |--L0.?---|                                "
    - "L0.?[765382,918457] 117ns 1b                                                          |--L0.?---|                     "
    - "L0.?[918458,1071533] 117ns 1b                                                                      |--L0.?---|         "
    - "L0.?[1071534,1170000] 117ns 3b                                                                                  |L0.?-| "
    - "**** Simulation run 26, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.119[118,1180000] 118ns|-----------------------------------------L0.119-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[118,153077] 118ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 118ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 118ns 1b                       |--L0.?---|                                                        "
    - "L0.?[459230,612305] 118ns 1b                                   |--L0.?---|                                            "
    - "L0.?[612306,765381] 118ns 1b                                              |--L0.?---|                                 "
    - "L0.?[765382,918457] 118ns 1b                                                          |--L0.?---|                     "
    - "L0.?[918458,1071533] 118ns 1b                                                                      |--L0.?---|         "
    - "L0.?[1071534,1180000] 118ns 3b                                                                                 |-L0.?-| "
    - "**** Simulation run 27, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.120[119,1190000] 119ns|-----------------------------------------L0.120-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[119,153077] 119ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 119ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 119ns 1b                       |--L0.?---|                                                        "
    - "L0.?[459230,612305] 119ns 1b                                  |--L0.?---|                                             "
    - "L0.?[612306,765381] 119ns 1b                                              |--L0.?---|                                 "
    - "L0.?[765382,918457] 119ns 1b                                                         |--L0.?---|                      "
    - "L0.?[918458,1071533] 119ns 1b                                                                     |--L0.?---|          "
    - "L0.?[1071534,1190000] 119ns 3b                                                                                 |-L0.?-| "
    - "**** Simulation run 28, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.121[120,1200000] 120ns|-----------------------------------------L0.121-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[120,153077] 120ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 120ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 120ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 120ns 1b                                  |--L0.?---|                                             "
    - "L0.?[612306,765381] 120ns 1b                                             |--L0.?---|                                  "
    - "L0.?[765382,918457] 120ns 1b                                                         |--L0.?---|                      "
    - "L0.?[918458,1071533] 120ns 1b                                                                    |--L0.?---|           "
    - "L0.?[1071534,1200000] 120ns 3b                                                                                |-L0.?--| "
    - "**** Simulation run 29, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.122[121,1210000] 121ns|-----------------------------------------L0.122-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[121,153077] 121ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 121ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 121ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 121ns 1b                                  |--L0.?---|                                             "
    - "L0.?[612306,765381] 121ns 1b                                             |--L0.?---|                                  "
    - "L0.?[765382,918457] 121ns 1b                                                        |--L0.?---|                       "
    - "L0.?[918458,1071533] 121ns 1b                                                                    |--L0.?---|           "
    - "L0.?[1071534,1210000] 121ns 3b                                                                               |--L0.?--| "
    - "**** Simulation run 30, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.123[122,1220000] 122ns|-----------------------------------------L0.123-----------------------------------------|"
    - "**** 8 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[122,153077] 122ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 122ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 122ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 122ns 1b                                 |--L0.?---|                                              "
    - "L0.?[612306,765381] 122ns 1b                                             |--L0.?---|                                  "
    - "L0.?[765382,918457] 122ns 1b                                                        |--L0.?---|                       "
    - "L0.?[918458,1071533] 122ns 1b                                                                   |--L0.?---|            "
    - "L0.?[1071534,1220000] 122ns 3b                                                                               |--L0.?--| "
    - "**** Simulation run 31, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.124[123,1230000] 123ns|-----------------------------------------L0.124-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[123,153077] 123ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 123ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 123ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 123ns 1b                                 |--L0.?---|                                              "
    - "L0.?[612306,765381] 123ns 1b                                            |--L0.?---|                                   "
    - "L0.?[765382,918457] 123ns 1b                                                        |--L0.?---|                       "
    - "L0.?[918458,1071533] 123ns 1b                                                                   |--L0.?---|            "
    - "L0.?[1071534,1224609] 123ns 1b                                                                              |--L0.?---| "
    - "L0.?[1224610,1230000] 123ns 2b                                                                                         |L0.?|"
    - "**** Simulation run 32, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.125[124,1240000] 124ns|-----------------------------------------L0.125-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[124,153077] 124ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 124ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 124ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 124ns 1b                                 |--L0.?---|                                              "
    - "L0.?[612306,765381] 124ns 1b                                            |--L0.?---|                                   "
    - "L0.?[765382,918457] 124ns 1b                                                       |--L0.?---|                        "
    - "L0.?[918458,1071533] 124ns 1b                                                                  |--L0.?---|             "
    - "L0.?[1071534,1224609] 124ns 1b                                                                             |--L0.?---|  "
    - "L0.?[1224610,1240000] 124ns 2b                                                                                        |L0.?|"
    - "**** Simulation run 33, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.126[125,1250000] 125ns|-----------------------------------------L0.126-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[125,153077] 125ns 1b|--L0.?---|                                                                               "
    - "L0.?[153078,306153] 125ns 1b           |--L0.?---|                                                                    "
    - "L0.?[306154,459229] 125ns 1b                      |--L0.?---|                                                         "
    - "L0.?[459230,612305] 125ns 1b                                 |--L0.?---|                                              "
    - "L0.?[612306,765381] 125ns 1b                                            |--L0.?---|                                   "
    - "L0.?[765382,918457] 125ns 1b                                                       |--L0.?---|                        "
    - "L0.?[918458,1071533] 125ns 1b                                                                  |--L0.?---|             "
    - "L0.?[1071534,1224609] 125ns 1b                                                                             |--L0.?---|  "
    - "L0.?[1224610,1250000] 125ns 2b                                                                                        |L0.?|"
    - "**** Simulation run 34, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.127[126,1260000] 126ns|-----------------------------------------L0.127-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[126,153077] 126ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 126ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 126ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 126ns 1b                                |--L0.?--|                                                "
    - "L0.?[612306,765381] 126ns 1b                                           |--L0.?--|                                     "
    - "L0.?[765382,918457] 126ns 1b                                                      |--L0.?--|                          "
    - "L0.?[918458,1071533] 126ns 1b                                                                 |--L0.?--|               "
    - "L0.?[1071534,1224609] 126ns 1b                                                                            |--L0.?--|    "
    - "L0.?[1224610,1260000] 126ns 2b                                                                                       |L0.?|"
    - "**** Simulation run 35, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.128[127,1270000] 127ns|-----------------------------------------L0.128-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[127,153077] 127ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 127ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 127ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 127ns 1b                                |--L0.?--|                                                "
    - "L0.?[612306,765381] 127ns 1b                                           |--L0.?--|                                     "
    - "L0.?[765382,918457] 127ns 1b                                                      |--L0.?--|                          "
    - "L0.?[918458,1071533] 127ns 1b                                                                 |--L0.?--|               "
    - "L0.?[1071534,1224609] 127ns 1b                                                                           |--L0.?--|     "
    - "L0.?[1224610,1270000] 127ns 2b                                                                                      |L0.?|"
    - "**** Simulation run 36, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.129[128,1280000] 128ns|-----------------------------------------L0.129-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[128,153077] 128ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 128ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 128ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 128ns 1b                                |--L0.?--|                                                "
    - "L0.?[612306,765381] 128ns 1b                                           |--L0.?--|                                     "
    - "L0.?[765382,918457] 128ns 1b                                                     |--L0.?--|                           "
    - "L0.?[918458,1071533] 128ns 1b                                                                |--L0.?--|                "
    - "L0.?[1071534,1224609] 128ns 1b                                                                           |--L0.?--|     "
    - "L0.?[1224610,1280000] 128ns 2b                                                                                      |L0.?|"
    - "**** Simulation run 37, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.130[129,1290000] 129ns|-----------------------------------------L0.130-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[129,153077] 129ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 129ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 129ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 129ns 1b                                |--L0.?--|                                                "
    - "L0.?[612306,765381] 129ns 1b                                          |--L0.?--|                                      "
    - "L0.?[765382,918457] 129ns 1b                                                     |--L0.?--|                           "
    - "L0.?[918458,1071533] 129ns 1b                                                                |--L0.?--|                "
    - "L0.?[1071534,1224609] 129ns 1b                                                                          |--L0.?--|      "
    - "L0.?[1224610,1290000] 129ns 2b                                                                                     |L0.?|"
    - "**** Simulation run 38, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.131[130,1300000] 130ns|-----------------------------------------L0.131-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[130,153077] 130ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 130ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 130ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 130ns 1b                               |--L0.?--|                                                 "
    - "L0.?[612306,765381] 130ns 1b                                          |--L0.?--|                                      "
    - "L0.?[765382,918457] 130ns 1b                                                    |--L0.?--|                            "
    - "L0.?[918458,1071533] 130ns 1b                                                               |--L0.?--|                 "
    - "L0.?[1071534,1224609] 130ns 1b                                                                          |--L0.?--|      "
    - "L0.?[1224610,1300000] 130ns 2b                                                                                    |L0.?|"
    - "**** Simulation run 39, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.132[131,1310000] 131ns|-----------------------------------------L0.132-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[131,153077] 131ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 131ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 131ns 1b                     |--L0.?--|                                                           "
    - "L0.?[459230,612305] 131ns 1b                               |--L0.?--|                                                 "
    - "L0.?[612306,765381] 131ns 1b                                          |--L0.?--|                                      "
    - "L0.?[765382,918457] 131ns 1b                                                    |--L0.?--|                            "
    - "L0.?[918458,1071533] 131ns 1b                                                               |--L0.?--|                 "
    - "L0.?[1071534,1224609] 131ns 1b                                                                         |--L0.?--|       "
    - "L0.?[1224610,1310000] 131ns 2b                                                                                    |L0.?|"
    - "**** Simulation run 40, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.133[132,1320000] 132ns|-----------------------------------------L0.133-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[132,153077] 132ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 132ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 132ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 132ns 1b                               |--L0.?--|                                                 "
    - "L0.?[612306,765381] 132ns 1b                                         |--L0.?--|                                       "
    - "L0.?[765382,918457] 132ns 1b                                                    |--L0.?--|                            "
    - "L0.?[918458,1071533] 132ns 1b                                                              |--L0.?--|                  "
    - "L0.?[1071534,1224609] 132ns 1b                                                                         |--L0.?--|       "
    - "L0.?[1224610,1320000] 132ns 2b                                                                                   |L0.?| "
    - "**** Simulation run 41, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.134[133,1330000] 133ns|-----------------------------------------L0.134-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[133,153077] 133ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 133ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 133ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 133ns 1b                               |--L0.?--|                                                 "
    - "L0.?[612306,765381] 133ns 1b                                         |--L0.?--|                                       "
    - "L0.?[765382,918457] 133ns 1b                                                   |--L0.?--|                             "
    - "L0.?[918458,1071533] 133ns 1b                                                              |--L0.?--|                  "
    - "L0.?[1071534,1224609] 133ns 1b                                                                        |--L0.?--|        "
    - "L0.?[1224610,1330000] 133ns 2b                                                                                  |L0.?-| "
    - "**** Simulation run 42, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.135[134,1340000] 134ns|-----------------------------------------L0.135-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[134,153077] 134ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 134ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 134ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 134ns 1b                              |--L0.?--|                                                  "
    - "L0.?[612306,765381] 134ns 1b                                         |--L0.?--|                                       "
    - "L0.?[765382,918457] 134ns 1b                                                   |--L0.?--|                             "
    - "L0.?[918458,1071533] 134ns 1b                                                             |--L0.?--|                   "
    - "L0.?[1071534,1224609] 134ns 1b                                                                       |--L0.?--|         "
    - "L0.?[1224610,1340000] 134ns 2b                                                                                  |L0.?-| "
    - "**** Simulation run 43, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.136[135,1350000] 135ns|-----------------------------------------L0.136-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[135,153077] 135ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 135ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 135ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 135ns 1b                              |--L0.?--|                                                  "
    - "L0.?[612306,765381] 135ns 1b                                        |--L0.?--|                                        "
    - "L0.?[765382,918457] 135ns 1b                                                   |--L0.?--|                             "
    - "L0.?[918458,1071533] 135ns 1b                                                             |--L0.?--|                   "
    - "L0.?[1071534,1224609] 135ns 1b                                                                       |--L0.?--|         "
    - "L0.?[1224610,1350000] 135ns 2b                                                                                 |-L0.?-| "
    - "Committing partition 1:"
    - "  Soft Deleting 40 files: L0.98, L0.99, L0.100, L0.101, L0.102, L0.103, L0.104, L0.105, L0.106, L0.107, L0.108, L0.109, L0.110, L0.111, L0.112, L0.113, L0.114, L0.115, L0.116, L0.117, L0.118, L0.119, L0.120, L0.121, L0.122, L0.123, L0.124, L0.125, L0.126, L0.127, L0.128, L0.129, L0.130, L0.131, L0.132, L0.133, L0.134, L0.135, L0.136, L0.204"
    - "  Creating 321 files"
    - "**** Simulation run 44, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.137[136,1360000] 136ns|-----------------------------------------L0.137-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[136,153077] 136ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 136ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 136ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 136ns 1b                              |--L0.?--|                                                  "
    - "L0.?[612306,765381] 136ns 1b                                        |--L0.?--|                                        "
    - "L0.?[765382,918457] 136ns 1b                                                  |--L0.?--|                              "
    - "L0.?[918458,1071533] 136ns 1b                                                            |--L0.?--|                    "
    - "L0.?[1071534,1224609] 136ns 1b                                                                      |--L0.?--|          "
    - "L0.?[1224610,1360000] 136ns 2b                                                                                 |-L0.?-| "
    - "**** Simulation run 45, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.138[137,1370000] 137ns|-----------------------------------------L0.138-----------------------------------------|"
    - "**** 9 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[137,153077] 137ns 1b|--L0.?--|                                                                                "
    - "L0.?[153078,306153] 137ns 1b          |--L0.?--|                                                                      "
    - "L0.?[306154,459229] 137ns 1b                    |--L0.?--|                                                            "
    - "L0.?[459230,612305] 137ns 1b                              |--L0.?--|                                                  "
    - "L0.?[612306,765381] 137ns 1b                                        |--L0.?--|                                        "
    - "L0.?[765382,918457] 137ns 1b                                                  |--L0.?--|                              "
    - "L0.?[918458,1071533] 137ns 1b                                                            |--L0.?--|                    "
    - "L0.?[1071534,1224609] 137ns 1b                                                                      |--L0.?--|          "
    - "L0.?[1224610,1370000] 137ns 2b                                                                                |-L0.?--| "
    - "**** Simulation run 46, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.139[138,1380000] 138ns|-----------------------------------------L0.139-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[138,153077] 138ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 138ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 138ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 138ns                             |-L0.?--|                                                    "
    - "L0.?[612306,765381] 138ns                                       |-L0.?--|                                          "
    - "L0.?[765382,918457] 138ns                                                 |-L0.?--|                                "
    - "L0.?[918458,1071533] 138ns                                                           |-L0.?--|                      "
    - "L0.?[1071534,1224609] 138ns                                                                     |-L0.?--|            "
    - "L0.?[1224610,1377685] 138ns                                                                               |-L0.?--|  "
    - "L0.?[1377686,1380000] 138ns                                                                                         |L0.?|"
    - "**** Simulation run 47, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.140[139,1390000] 139ns|-----------------------------------------L0.140-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[139,153077] 139ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 139ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 139ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 139ns                             |-L0.?--|                                                    "
    - "L0.?[612306,765381] 139ns                                       |-L0.?--|                                          "
    - "L0.?[765382,918457] 139ns                                                 |-L0.?--|                                "
    - "L0.?[918458,1071533] 139ns                                                           |-L0.?--|                      "
    - "L0.?[1071534,1224609] 139ns                                                                     |-L0.?--|            "
    - "L0.?[1224610,1377685] 139ns                                                                               |-L0.?--|  "
    - "L0.?[1377686,1390000] 139ns                                                                                         |L0.?|"
    - "**** Simulation run 48, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.141[140,1400000] 140ns|-----------------------------------------L0.141-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[140,153077] 140ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 140ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 140ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 140ns                             |-L0.?--|                                                    "
    - "L0.?[612306,765381] 140ns                                       |-L0.?--|                                          "
    - "L0.?[765382,918457] 140ns                                                 |-L0.?--|                                "
    - "L0.?[918458,1071533] 140ns                                                           |-L0.?--|                      "
    - "L0.?[1071534,1224609] 140ns                                                                    |-L0.?--|             "
    - "L0.?[1224610,1377685] 140ns                                                                              |-L0.?--|   "
    - "L0.?[1377686,1400000] 140ns                                                                                        |L0.?|"
    - "**** Simulation run 49, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.142[141,1410000] 141ns|-----------------------------------------L0.142-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[141,153077] 141ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 141ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 141ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 141ns                             |-L0.?--|                                                    "
    - "L0.?[612306,765381] 141ns                                       |-L0.?--|                                          "
    - "L0.?[765382,918457] 141ns                                                |-L0.?--|                                 "
    - "L0.?[918458,1071533] 141ns                                                          |-L0.?--|                       "
    - "L0.?[1071534,1224609] 141ns                                                                    |-L0.?--|             "
    - "L0.?[1224610,1377685] 141ns                                                                              |-L0.?--|   "
    - "L0.?[1377686,1410000] 141ns                                                                                       |L0.?|"
    - "**** Simulation run 50, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.143[142,1420000] 142ns|-----------------------------------------L0.143-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[142,153077] 142ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 142ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 142ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 142ns                             |-L0.?--|                                                    "
    - "L0.?[612306,765381] 142ns                                      |-L0.?--|                                           "
    - "L0.?[765382,918457] 142ns                                                |-L0.?--|                                 "
    - "L0.?[918458,1071533] 142ns                                                          |-L0.?--|                       "
    - "L0.?[1071534,1224609] 142ns                                                                   |-L0.?--|              "
    - "L0.?[1224610,1377685] 142ns                                                                             |-L0.?--|    "
    - "L0.?[1377686,1420000] 142ns                                                                                       |L0.?|"
    - "**** Simulation run 51, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.144[143,1430000] 143ns|-----------------------------------------L0.144-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[143,153077] 143ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 143ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 143ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 143ns                            |-L0.?--|                                                     "
    - "L0.?[612306,765381] 143ns                                      |-L0.?--|                                           "
    - "L0.?[765382,918457] 143ns                                                |-L0.?--|                                 "
    - "L0.?[918458,1071533] 143ns                                                         |-L0.?--|                        "
    - "L0.?[1071534,1224609] 143ns                                                                   |-L0.?--|              "
    - "L0.?[1224610,1377685] 143ns                                                                             |-L0.?--|    "
    - "L0.?[1377686,1430000] 143ns                                                                                      |L0.?|"
    - "**** Simulation run 52, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.145[144,1440000] 144ns|-----------------------------------------L0.145-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[144,153077] 144ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 144ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 144ns                   |-L0.?--|                                                              "
    - "L0.?[459230,612305] 144ns                            |-L0.?--|                                                     "
    - "L0.?[612306,765381] 144ns                                      |-L0.?--|                                           "
    - "L0.?[765382,918457] 144ns                                               |-L0.?--|                                  "
    - "L0.?[918458,1071533] 144ns                                                         |-L0.?--|                        "
    - "L0.?[1071534,1224609] 144ns                                                                  |-L0.?--|               "
    - "L0.?[1224610,1377685] 144ns                                                                            |-L0.?--|     "
    - "L0.?[1377686,1440000] 144ns                                                                                      |L0.?|"
    - "**** Simulation run 53, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.146[145,1450000] 145ns|-----------------------------------------L0.146-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[145,153077] 145ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 145ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 145ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 145ns                            |-L0.?--|                                                     "
    - "L0.?[612306,765381] 145ns                                      |-L0.?--|                                           "
    - "L0.?[765382,918457] 145ns                                               |-L0.?--|                                  "
    - "L0.?[918458,1071533] 145ns                                                         |-L0.?--|                        "
    - "L0.?[1071534,1224609] 145ns                                                                  |-L0.?--|               "
    - "L0.?[1224610,1377685] 145ns                                                                            |-L0.?--|     "
    - "L0.?[1377686,1450000] 145ns                                                                                     |L0.?|"
    - "**** Simulation run 54, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.147[146,1460000] 146ns|-----------------------------------------L0.147-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[146,153077] 146ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 146ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 146ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 146ns                            |-L0.?--|                                                     "
    - "L0.?[612306,765381] 146ns                                     |-L0.?--|                                            "
    - "L0.?[765382,918457] 146ns                                               |-L0.?--|                                  "
    - "L0.?[918458,1071533] 146ns                                                        |-L0.?--|                         "
    - "L0.?[1071534,1224609] 146ns                                                                  |-L0.?--|               "
    - "L0.?[1224610,1377685] 146ns                                                                           |-L0.?--|      "
    - "L0.?[1377686,1460000] 146ns                                                                                    |L0.?|"
    - "**** Simulation run 55, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.148[147,1470000] 147ns|-----------------------------------------L0.148-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[147,153077] 147ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 147ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 147ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 147ns                            |-L0.?--|                                                     "
    - "L0.?[612306,765381] 147ns                                     |-L0.?--|                                            "
    - "L0.?[765382,918457] 147ns                                              |-L0.?--|                                   "
    - "L0.?[918458,1071533] 147ns                                                        |-L0.?--|                         "
    - "L0.?[1071534,1224609] 147ns                                                                 |-L0.?--|                "
    - "L0.?[1224610,1377685] 147ns                                                                          |-L0.?--|       "
    - "L0.?[1377686,1470000] 147ns                                                                                    |L0.?|"
    - "**** Simulation run 56, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.149[148,1480000] 148ns|-----------------------------------------L0.149-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[148,153077] 148ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 148ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 148ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 148ns                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 148ns                                     |-L0.?--|                                            "
    - "L0.?[765382,918457] 148ns                                              |-L0.?--|                                   "
    - "L0.?[918458,1071533] 148ns                                                       |-L0.?--|                          "
    - "L0.?[1071534,1224609] 148ns                                                                 |-L0.?--|                "
    - "L0.?[1224610,1377685] 148ns                                                                          |-L0.?--|       "
    - "L0.?[1377686,1480000] 148ns                                                                                   |L0.?| "
    - "**** Simulation run 57, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.150[149,1490000] 149ns|----------------------------------------L0.150-----------------------------------------| "
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[149,153077] 149ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 149ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 149ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 149ns                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 149ns                                    |-L0.?--|                                             "
    - "L0.?[765382,918457] 149ns                                              |-L0.?--|                                   "
    - "L0.?[918458,1071533] 149ns                                                       |-L0.?--|                          "
    - "L0.?[1071534,1224609] 149ns                                                                |-L0.?--|                 "
    - "L0.?[1224610,1377685] 149ns                                                                         |-L0.?--|        "
    - "L0.?[1377686,1490000] 149ns                                                                                   |L0.?| "
    - "**** Simulation run 58, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.151[150,1500000] 150ns|-----------------------------------------L0.151-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[150,153077] 150ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 150ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 150ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 150ns                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 150ns                                    |-L0.?--|                                             "
    - "L0.?[765382,918457] 150ns                                             |-L0.?--|                                    "
    - "L0.?[918458,1071533] 150ns                                                       |-L0.?--|                          "
    - "L0.?[1071534,1224609] 150ns                                                                |-L0.?--|                 "
    - "L0.?[1224610,1377685] 150ns                                                                         |-L0.?--|        "
    - "L0.?[1377686,1500000] 150ns                                                                                  |L0.?-| "
    - "**** Simulation run 59, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.152[151,1510000] 151ns|-----------------------------------------L0.152-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[151,153077] 151ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 151ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 151ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 151ns                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 151ns                                    |-L0.?--|                                             "
    - "L0.?[765382,918457] 151ns                                             |-L0.?--|                                    "
    - "L0.?[918458,1071533] 151ns                                                      |-L0.?--|                           "
    - "L0.?[1071534,1224609] 151ns                                                               |-L0.?--|                  "
    - "L0.?[1224610,1377685] 151ns                                                                        |-L0.?--|         "
    - "L0.?[1377686,1510000] 151ns                                                                                  |L0.?-| "
    - "**** Simulation run 60, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.153[152,1520000] 152ns|-----------------------------------------L0.153-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.?[152,153077] 152ns   |-L0.?--|                                                                                 "
    - "L0.?[153078,306153] 152ns         |-L0.?--|                                                                        "
    - "L0.?[306154,459229] 152ns                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 152ns                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 152ns                                    |-L0.?--|                                             "
    - "L0.?[765382,918457] 152ns                                             |-L0.?--|                                    "
    - "L0.?[918458,1071533] 152ns                                                      |-L0.?--|                           "
    - "L0.?[1071534,1224609] 152ns                                                               |-L0.?--|                  "
    - "L0.?[1224610,1377685] 152ns                                                                        |-L0.?--|         "
    - "L0.?[1377686,1520000] 152ns                                                                                 |-L0.?-| "
    - "**** Simulation run 61, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.154[153,1530000] 153ns|-----------------------------------------L0.154-----------------------------------------|"
    - "**** 10 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[153,153077] 153ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 153ns 1b        |-L0.?--|                                                                         "
    - "L0.?[306154,459229] 153ns 1b                  |-L0.?--|                                                               "
    - "L0.?[459230,612305] 153ns 1b                           |-L0.?--|                                                      "
    - "L0.?[612306,765381] 153ns 1b                                    |-L0.?--|                                             "
    - "L0.?[765382,918457] 153ns 1b                                             |-L0.?--|                                    "
    - "L0.?[918458,1071533] 153ns 1b                                                      |-L0.?--|                           "
    - "L0.?[1071534,1224609] 153ns 1b                                                               |-L0.?--|                  "
    - "L0.?[1224610,1377685] 153ns 1b                                                                        |-L0.?--|         "
    - "L0.?[1377686,1530000] 153ns 2b                                                                                 |-L0.?-| "
    - "**** Simulation run 62, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.155[154,1540000] 154ns|----------------------------------------L0.155-----------------------------------------| "
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[154,153077] 154ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 154ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 154ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 154ns 0b                          |-L0.?-|                                                        "
    - "L0.?[612306,765381] 154ns 0b                                   |-L0.?-|                                               "
    - "L0.?[765382,918457] 154ns 0b                                            |-L0.?-|                                      "
    - "L0.?[918458,1071533] 154ns 0b                                                     |-L0.?-|                             "
    - "L0.?[1071534,1224609] 154ns 0b                                                              |-L0.?-|                    "
    - "L0.?[1224610,1377685] 154ns 0b                                                                       |-L0.?-|           "
    - "L0.?[1377686,1530761] 154ns 0b                                                                                |-L0.?-|  "
    - "L0.?[1530762,1540000] 154ns 10b                                                                                         |L0.?|"
    - "**** Simulation run 63, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.156[155,1550000] 155ns|-----------------------------------------L0.156-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[155,153077] 155ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 155ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 155ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 155ns 0b                          |-L0.?-|                                                        "
    - "L0.?[612306,765381] 155ns 0b                                   |-L0.?-|                                               "
    - "L0.?[765382,918457] 155ns 0b                                            |-L0.?-|                                      "
    - "L0.?[918458,1071533] 155ns 0b                                                     |-L0.?-|                             "
    - "L0.?[1071534,1224609] 155ns 0b                                                              |-L0.?-|                    "
    - "L0.?[1224610,1377685] 155ns 0b                                                                       |-L0.?-|           "
    - "L0.?[1377686,1530761] 155ns 0b                                                                               |-L0.?-|   "
    - "L0.?[1530762,1550000] 155ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 64, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.157[156,1560000] 156ns|-----------------------------------------L0.157-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[156,153077] 156ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 156ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 156ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 156ns 0b                          |-L0.?-|                                                        "
    - "L0.?[612306,765381] 156ns 0b                                   |-L0.?-|                                               "
    - "L0.?[765382,918457] 156ns 0b                                            |-L0.?-|                                      "
    - "L0.?[918458,1071533] 156ns 0b                                                    |-L0.?-|                              "
    - "L0.?[1071534,1224609] 156ns 0b                                                             |-L0.?-|                     "
    - "L0.?[1224610,1377685] 156ns 0b                                                                      |-L0.?-|            "
    - "L0.?[1377686,1530761] 156ns 0b                                                                               |-L0.?-|   "
    - "L0.?[1530762,1560000] 156ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 65, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.158[157,1570000] 157ns|-----------------------------------------L0.158-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[157,153077] 157ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 157ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 157ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 157ns 0b                          |-L0.?-|                                                        "
    - "L0.?[612306,765381] 157ns 0b                                   |-L0.?-|                                               "
    - "L0.?[765382,918457] 157ns 0b                                           |-L0.?-|                                       "
    - "L0.?[918458,1071533] 157ns 0b                                                    |-L0.?-|                              "
    - "L0.?[1071534,1224609] 157ns 0b                                                             |-L0.?-|                     "
    - "L0.?[1224610,1377685] 157ns 0b                                                                      |-L0.?-|            "
    - "L0.?[1377686,1530761] 157ns 0b                                                                              |-L0.?-|    "
    - "L0.?[1530762,1570000] 157ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 66, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.159[158,1580000] 158ns|-----------------------------------------L0.159-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[158,153077] 158ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 158ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 158ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 158ns 0b                          |-L0.?-|                                                        "
    - "L0.?[612306,765381] 158ns 0b                                  |-L0.?-|                                                "
    - "L0.?[765382,918457] 158ns 0b                                           |-L0.?-|                                       "
    - "L0.?[918458,1071533] 158ns 0b                                                    |-L0.?-|                              "
    - "L0.?[1071534,1224609] 158ns 0b                                                             |-L0.?-|                     "
    - "L0.?[1224610,1377685] 158ns 0b                                                                     |-L0.?-|             "
    - "L0.?[1377686,1530761] 158ns 0b                                                                              |-L0.?-|    "
    - "L0.?[1530762,1580000] 158ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 67, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.160[159,1590000] 159ns|----------------------------------------L0.160-----------------------------------------| "
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[159,153077] 159ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 159ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 159ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 159ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 159ns 0b                                  |-L0.?-|                                                "
    - "L0.?[765382,918457] 159ns 0b                                           |-L0.?-|                                       "
    - "L0.?[918458,1071533] 159ns 0b                                                   |-L0.?-|                               "
    - "L0.?[1071534,1224609] 159ns 0b                                                            |-L0.?-|                      "
    - "L0.?[1224610,1377685] 159ns 0b                                                                     |-L0.?-|             "
    - "L0.?[1377686,1530761] 159ns 0b                                                                             |-L0.?-|     "
    - "L0.?[1530762,1590000] 159ns 10b                                                                                      |L0.?|"
    - "**** Simulation run 68, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.161[160,1600000] 160ns|-----------------------------------------L0.161-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[160,153077] 160ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 160ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 160ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 160ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 160ns 0b                                  |-L0.?-|                                                "
    - "L0.?[765382,918457] 160ns 0b                                           |-L0.?-|                                       "
    - "L0.?[918458,1071533] 160ns 0b                                                   |-L0.?-|                               "
    - "L0.?[1071534,1224609] 160ns 0b                                                            |-L0.?-|                      "
    - "L0.?[1224610,1377685] 160ns 0b                                                                    |-L0.?-|              "
    - "L0.?[1377686,1530761] 160ns 0b                                                                             |-L0.?-|     "
    - "L0.?[1530762,1600000] 160ns 10b                                                                                      |L0.?|"
    - "**** Simulation run 69, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.162[161,1610000] 161ns|-----------------------------------------L0.162-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[161,153077] 161ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 161ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 161ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 161ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 161ns 0b                                  |-L0.?-|                                                "
    - "L0.?[765382,918457] 161ns 0b                                          |-L0.?-|                                        "
    - "L0.?[918458,1071533] 161ns 0b                                                   |-L0.?-|                               "
    - "L0.?[1071534,1224609] 161ns 0b                                                           |-L0.?-|                       "
    - "L0.?[1224610,1377685] 161ns 0b                                                                    |-L0.?-|              "
    - "L0.?[1377686,1530761] 161ns 0b                                                                             |-L0.?-|     "
    - "L0.?[1530762,1610000] 161ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 70, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.163[162,1620000] 162ns|-----------------------------------------L0.163-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[162,153077] 162ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 162ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 162ns 0b                 |-L0.?-|                                                                 "
    - "L0.?[459230,612305] 162ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 162ns 0b                                  |-L0.?-|                                                "
    - "L0.?[765382,918457] 162ns 0b                                          |-L0.?-|                                        "
    - "L0.?[918458,1071533] 162ns 0b                                                   |-L0.?-|                               "
    - "L0.?[1071534,1224609] 162ns 0b                                                           |-L0.?-|                       "
    - "L0.?[1224610,1377685] 162ns 0b                                                                    |-L0.?-|              "
    - "L0.?[1377686,1530761] 162ns 0b                                                                            |-L0.?-|      "
    - "L0.?[1530762,1620000] 162ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 71, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.164[163,1630000] 163ns|-----------------------------------------L0.164-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[163,153077] 163ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 163ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 163ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 163ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 163ns 0b                                 |-L0.?-|                                                 "
    - "L0.?[765382,918457] 163ns 0b                                          |-L0.?-|                                        "
    - "L0.?[918458,1071533] 163ns 0b                                                  |-L0.?-|                                "
    - "L0.?[1071534,1224609] 163ns 0b                                                           |-L0.?-|                       "
    - "L0.?[1224610,1377685] 163ns 0b                                                                   |-L0.?-|               "
    - "L0.?[1377686,1530761] 163ns 0b                                                                            |-L0.?-|      "
    - "L0.?[1530762,1630000] 163ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 72, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.165[164,1640000] 164ns|----------------------------------------L0.165-----------------------------------------| "
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[164,153077] 164ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 164ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 164ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 164ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 164ns 0b                                 |-L0.?-|                                                 "
    - "L0.?[765382,918457] 164ns 0b                                         |-L0.?-|                                         "
    - "L0.?[918458,1071533] 164ns 0b                                                  |-L0.?-|                                "
    - "L0.?[1071534,1224609] 164ns 0b                                                          |-L0.?-|                        "
    - "L0.?[1224610,1377685] 164ns 0b                                                                   |-L0.?-|               "
    - "L0.?[1377686,1530761] 164ns 0b                                                                           |-L0.?-|       "
    - "L0.?[1530762,1640000] 164ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 73, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.166[165,1650000] 165ns|-----------------------------------------L0.166-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[165,153077] 165ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 165ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 165ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 165ns 0b                         |-L0.?-|                                                         "
    - "L0.?[612306,765381] 165ns 0b                                 |-L0.?-|                                                 "
    - "L0.?[765382,918457] 165ns 0b                                         |-L0.?-|                                         "
    - "L0.?[918458,1071533] 165ns 0b                                                  |-L0.?-|                                "
    - "L0.?[1071534,1224609] 165ns 0b                                                          |-L0.?-|                        "
    - "L0.?[1224610,1377685] 165ns 0b                                                                  |-L0.?-|                "
    - "L0.?[1377686,1530761] 165ns 0b                                                                           |-L0.?-|       "
    - "L0.?[1530762,1650000] 165ns 10b                                                                                   |L0.?| "
    - "**** Simulation run 74, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.167[166,1660000] 166ns|-----------------------------------------L0.167-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[166,153077] 166ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 166ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 166ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 166ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 166ns 0b                                 |-L0.?-|                                                 "
    - "L0.?[765382,918457] 166ns 0b                                         |-L0.?-|                                         "
    - "L0.?[918458,1071533] 166ns 0b                                                 |-L0.?-|                                 "
    - "L0.?[1071534,1224609] 166ns 0b                                                          |-L0.?-|                        "
    - "L0.?[1224610,1377685] 166ns 0b                                                                  |-L0.?-|                "
    - "L0.?[1377686,1530761] 166ns 0b                                                                          |-L0.?-|        "
    - "L0.?[1530762,1660000] 166ns 10b                                                                                  |L0.?-| "
    - "**** Simulation run 75, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.168[167,1670000] 167ns|-----------------------------------------L0.168-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[167,153077] 167ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 167ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 167ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 167ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 167ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 167ns 0b                                         |-L0.?-|                                         "
    - "L0.?[918458,1071533] 167ns 0b                                                 |-L0.?-|                                 "
    - "L0.?[1071534,1224609] 167ns 0b                                                         |-L0.?-|                         "
    - "L0.?[1224610,1377685] 167ns 0b                                                                 |-L0.?-|                 "
    - "L0.?[1377686,1530761] 167ns 0b                                                                          |-L0.?-|        "
    - "L0.?[1530762,1670000] 167ns 10b                                                                                  |L0.?-| "
    - "**** Simulation run 76, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.169[168,1680000] 168ns|-----------------------------------------L0.169-----------------------------------------|"
    - "**** 11 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[168,153077] 168ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 168ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 168ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 168ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 168ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 168ns 0b                                        |-L0.?-|                                          "
    - "L0.?[918458,1071533] 168ns 0b                                                 |-L0.?-|                                 "
    - "L0.?[1071534,1224609] 168ns 0b                                                         |-L0.?-|                         "
    - "L0.?[1224610,1377685] 168ns 0b                                                                 |-L0.?-|                 "
    - "L0.?[1377686,1530761] 168ns 0b                                                                         |-L0.?-|         "
    - "L0.?[1530762,1680000] 168ns 10b                                                                                  |L0.?-| "
    - "**** Simulation run 77, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.170[169,1690000] 169ns|-----------------------------------------L0.170-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[169,153077] 169ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 169ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 169ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 169ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 169ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 169ns 0b                                        |-L0.?-|                                          "
    - "L0.?[918458,1071533] 169ns 0b                                                |-L0.?-|                                  "
    - "L0.?[1071534,1224609] 169ns 0b                                                         |-L0.?-|                         "
    - "L0.?[1224610,1377685] 169ns 0b                                                                 |-L0.?-|                 "
    - "L0.?[1377686,1530761] 169ns 0b                                                                         |-L0.?-|         "
    - "L0.?[1530762,1683837] 169ns 0b                                                                                 |-L0.?-| "
    - "L0.?[1683838,1690000] 169ns 10b                                                                                         |L0.?|"
    - "**** Simulation run 78, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.171[170,1700000] 170ns|-----------------------------------------L0.171-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[170,153077] 170ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 170ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 170ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 170ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 170ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 170ns 0b                                        |-L0.?-|                                          "
    - "L0.?[918458,1071533] 170ns 0b                                                |-L0.?-|                                  "
    - "L0.?[1071534,1224609] 170ns 0b                                                        |-L0.?-|                          "
    - "L0.?[1224610,1377685] 170ns 0b                                                                |-L0.?-|                  "
    - "L0.?[1377686,1530761] 170ns 0b                                                                        |-L0.?-|          "
    - "L0.?[1530762,1683837] 170ns 0b                                                                                 |-L0.?-| "
    - "L0.?[1683838,1700000] 170ns 10b                                                                                         |L0.?|"
    - "**** Simulation run 79, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.172[171,1710000] 171ns|-----------------------------------------L0.172-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[171,153077] 171ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 171ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 171ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 171ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 171ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 171ns 0b                                        |-L0.?-|                                          "
    - "L0.?[918458,1071533] 171ns 0b                                                |-L0.?-|                                  "
    - "L0.?[1071534,1224609] 171ns 0b                                                        |-L0.?-|                          "
    - "L0.?[1224610,1377685] 171ns 0b                                                                |-L0.?-|                  "
    - "L0.?[1377686,1530761] 171ns 0b                                                                        |-L0.?-|          "
    - "L0.?[1530762,1683837] 171ns 0b                                                                                |-L0.?-|  "
    - "L0.?[1683838,1710000] 171ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 80, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.173[172,1720000] 172ns|-----------------------------------------L0.173-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[172,153077] 172ns 0b|-L0.?-|                                                                                  "
    - "L0.?[153078,306153] 172ns 0b        |-L0.?-|                                                                          "
    - "L0.?[306154,459229] 172ns 0b                |-L0.?-|                                                                  "
    - "L0.?[459230,612305] 172ns 0b                        |-L0.?-|                                                          "
    - "L0.?[612306,765381] 172ns 0b                                |-L0.?-|                                                  "
    - "L0.?[765382,918457] 172ns 0b                                        |-L0.?-|                                          "
    - "L0.?[918458,1071533] 172ns 0b                                                |-L0.?-|                                  "
    - "L0.?[1071534,1224609] 172ns 0b                                                        |-L0.?-|                          "
    - "L0.?[1224610,1377685] 172ns 0b                                                                |-L0.?-|                  "
    - "L0.?[1377686,1530761] 172ns 0b                                                                        |-L0.?-|          "
    - "L0.?[1530762,1683837] 172ns 0b                                                                                |-L0.?-|  "
    - "L0.?[1683838,1720000] 172ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 81, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.174[173,1730000] 173ns|-----------------------------------------L0.174-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[173,153077] 173ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 173ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 173ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 173ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 173ns 0b                               |L0.?-|                                                    "
    - "L0.?[765382,918457] 173ns 0b                                       |L0.?-|                                            "
    - "L0.?[918458,1071533] 173ns 0b                                               |L0.?-|                                    "
    - "L0.?[1071534,1224609] 173ns 0b                                                       |L0.?-|                            "
    - "L0.?[1224610,1377685] 173ns 0b                                                               |L0.?-|                    "
    - "L0.?[1377686,1530761] 173ns 0b                                                                       |L0.?-|            "
    - "L0.?[1530762,1683837] 173ns 0b                                                                               |L0.?-|    "
    - "L0.?[1683838,1730000] 173ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 82, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.175[174,1740000] 174ns|-----------------------------------------L0.175-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[174,153077] 174ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 174ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 174ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 174ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 174ns 0b                               |L0.?-|                                                    "
    - "L0.?[765382,918457] 174ns 0b                                       |L0.?-|                                            "
    - "L0.?[918458,1071533] 174ns 0b                                               |L0.?-|                                    "
    - "L0.?[1071534,1224609] 174ns 0b                                                       |L0.?-|                            "
    - "L0.?[1224610,1377685] 174ns 0b                                                               |L0.?-|                    "
    - "L0.?[1377686,1530761] 174ns 0b                                                                       |L0.?-|            "
    - "L0.?[1530762,1683837] 174ns 0b                                                                               |L0.?-|    "
    - "L0.?[1683838,1740000] 174ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 83, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.176[175,1750000] 175ns|-----------------------------------------L0.176-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[175,153077] 175ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 175ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 175ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 175ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 175ns 0b                               |L0.?-|                                                    "
    - "L0.?[765382,918457] 175ns 0b                                       |L0.?-|                                            "
    - "L0.?[918458,1071533] 175ns 0b                                               |L0.?-|                                    "
    - "L0.?[1071534,1224609] 175ns 0b                                                       |L0.?-|                            "
    - "L0.?[1224610,1377685] 175ns 0b                                                              |L0.?-|                     "
    - "L0.?[1377686,1530761] 175ns 0b                                                                      |L0.?-|             "
    - "L0.?[1530762,1683837] 175ns 0b                                                                              |L0.?-|     "
    - "L0.?[1683838,1750000] 175ns 10b                                                                                      |L0.?|"
    - "Committing partition 1:"
    - "  Soft Deleting 40 files: L0.137, L0.138, L0.139, L0.140, L0.141, L0.142, L0.143, L0.144, L0.145, L0.146, L0.147, L0.148, L0.149, L0.150, L0.151, L0.152, L0.153, L0.154, L0.155, L0.156, L0.157, L0.158, L0.159, L0.160, L0.161, L0.162, L0.163, L0.164, L0.165, L0.166, L0.167, L0.168, L0.169, L0.170, L0.171, L0.172, L0.173, L0.174, L0.175, L0.176"
    - "  Creating 427 files"
    - "**** Simulation run 84, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.177[176,1760000] 176ns|-----------------------------------------L0.177-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[176,153077] 176ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 176ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 176ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 176ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 176ns 0b                               |L0.?-|                                                    "
    - "L0.?[765382,918457] 176ns 0b                                       |L0.?-|                                            "
    - "L0.?[918458,1071533] 176ns 0b                                              |L0.?-|                                     "
    - "L0.?[1071534,1224609] 176ns 0b                                                      |L0.?-|                             "
    - "L0.?[1224610,1377685] 176ns 0b                                                              |L0.?-|                     "
    - "L0.?[1377686,1530761] 176ns 0b                                                                      |L0.?-|             "
    - "L0.?[1530762,1683837] 176ns 0b                                                                              |L0.?-|     "
    - "L0.?[1683838,1760000] 176ns 10b                                                                                      |L0.?|"
    - "**** Simulation run 85, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.178[177,1770000] 177ns|-----------------------------------------L0.178-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[177,153077] 177ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 177ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 177ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 177ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 177ns 0b                               |L0.?-|                                                    "
    - "L0.?[765382,918457] 177ns 0b                                      |L0.?-|                                             "
    - "L0.?[918458,1071533] 177ns 0b                                              |L0.?-|                                     "
    - "L0.?[1071534,1224609] 177ns 0b                                                      |L0.?-|                             "
    - "L0.?[1224610,1377685] 177ns 0b                                                              |L0.?-|                     "
    - "L0.?[1377686,1530761] 177ns 0b                                                                      |L0.?-|             "
    - "L0.?[1530762,1683837] 177ns 0b                                                                             |L0.?-|      "
    - "L0.?[1683838,1770000] 177ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 86, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.179[178,1780000] 178ns|-----------------------------------------L0.179-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[178,153077] 178ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 178ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 178ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 178ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 178ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 178ns 0b                                      |L0.?-|                                             "
    - "L0.?[918458,1071533] 178ns 0b                                              |L0.?-|                                     "
    - "L0.?[1071534,1224609] 178ns 0b                                                      |L0.?-|                             "
    - "L0.?[1224610,1377685] 178ns 0b                                                             |L0.?-|                      "
    - "L0.?[1377686,1530761] 178ns 0b                                                                     |L0.?-|              "
    - "L0.?[1530762,1683837] 178ns 0b                                                                             |L0.?-|      "
    - "L0.?[1683838,1780000] 178ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 87, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.180[179,1790000] 179ns|-----------------------------------------L0.180-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[179,153077] 179ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 179ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 179ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 179ns 0b                       |L0.?-|                                                            "
    - "L0.?[612306,765381] 179ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 179ns 0b                                      |L0.?-|                                             "
    - "L0.?[918458,1071533] 179ns 0b                                              |L0.?-|                                     "
    - "L0.?[1071534,1224609] 179ns 0b                                                     |L0.?-|                              "
    - "L0.?[1224610,1377685] 179ns 0b                                                             |L0.?-|                      "
    - "L0.?[1377686,1530761] 179ns 0b                                                                     |L0.?-|              "
    - "L0.?[1530762,1683837] 179ns 0b                                                                            |L0.?-|       "
    - "L0.?[1683838,1790000] 179ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 88, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.181[180,1800000] 180ns|-----------------------------------------L0.181-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[180,153077] 180ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 180ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 180ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 180ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 180ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 180ns 0b                                      |L0.?-|                                             "
    - "L0.?[918458,1071533] 180ns 0b                                             |L0.?-|                                      "
    - "L0.?[1071534,1224609] 180ns 0b                                                     |L0.?-|                              "
    - "L0.?[1224610,1377685] 180ns 0b                                                             |L0.?-|                      "
    - "L0.?[1377686,1530761] 180ns 0b                                                                    |L0.?-|               "
    - "L0.?[1530762,1683837] 180ns 0b                                                                            |L0.?-|       "
    - "L0.?[1683838,1800000] 180ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 89, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.182[181,1810000] 181ns|-----------------------------------------L0.182-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[181,153077] 181ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 181ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 181ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 181ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 181ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 181ns 0b                                      |L0.?-|                                             "
    - "L0.?[918458,1071533] 181ns 0b                                             |L0.?-|                                      "
    - "L0.?[1071534,1224609] 181ns 0b                                                     |L0.?-|                              "
    - "L0.?[1224610,1377685] 181ns 0b                                                            |L0.?-|                       "
    - "L0.?[1377686,1530761] 181ns 0b                                                                    |L0.?-|               "
    - "L0.?[1530762,1683837] 181ns 0b                                                                            |L0.?-|       "
    - "L0.?[1683838,1810000] 181ns 10b                                                                                   |L0.?| "
    - "**** Simulation run 90, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.183[182,1820000] 182ns|-----------------------------------------L0.183-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[182,153077] 182ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 182ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 182ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 182ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 182ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 182ns 0b                                     |L0.?-|                                              "
    - "L0.?[918458,1071533] 182ns 0b                                             |L0.?-|                                      "
    - "L0.?[1071534,1224609] 182ns 0b                                                    |L0.?-|                               "
    - "L0.?[1224610,1377685] 182ns 0b                                                            |L0.?-|                       "
    - "L0.?[1377686,1530761] 182ns 0b                                                                    |L0.?-|               "
    - "L0.?[1530762,1683837] 182ns 0b                                                                           |L0.?-|        "
    - "L0.?[1683838,1820000] 182ns 10b                                                                                   |L0.?| "
    - "**** Simulation run 91, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.184[183,1830000] 183ns|-----------------------------------------L0.184-----------------------------------------|"
    - "**** 12 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[183,153077] 183ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 183ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 183ns 0b               |L0.?-|                                                                    "
    - "L0.?[459230,612305] 183ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 183ns 0b                              |L0.?-|                                                     "
    - "L0.?[765382,918457] 183ns 0b                                     |L0.?-|                                              "
    - "L0.?[918458,1071533] 183ns 0b                                             |L0.?-|                                      "
    - "L0.?[1071534,1224609] 183ns 0b                                                    |L0.?-|                               "
    - "L0.?[1224610,1377685] 183ns 0b                                                            |L0.?-|                       "
    - "L0.?[1377686,1530761] 183ns 0b                                                                   |L0.?-|                "
    - "L0.?[1530762,1683837] 183ns 0b                                                                           |L0.?-|        "
    - "L0.?[1683838,1830000] 183ns 10b                                                                                  |L0.?-| "
    - "**** Simulation run 92, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.185[184,1840000] 184ns|-----------------------------------------L0.185-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[184,153077] 184ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 184ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 184ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 184ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 184ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 184ns 0b                                     |L0.?-|                                              "
    - "L0.?[918458,1071533] 184ns 0b                                            |L0.?-|                                       "
    - "L0.?[1071534,1224609] 184ns 0b                                                    |L0.?-|                               "
    - "L0.?[1224610,1377685] 184ns 0b                                                           |L0.?-|                        "
    - "L0.?[1377686,1530761] 184ns 0b                                                                   |L0.?-|                "
    - "L0.?[1530762,1683837] 184ns 0b                                                                          |L0.?-|         "
    - "L0.?[1683838,1836913] 184ns 0b                                                                                  |L0.?-| "
    - "L0.?[1836914,1840000] 184ns 10b                                                                                         |L0.?|"
    - "**** Simulation run 93, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.186[185,1850000] 185ns|-----------------------------------------L0.186-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[185,153077] 185ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 185ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 185ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 185ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 185ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 185ns 0b                                     |L0.?-|                                              "
    - "L0.?[918458,1071533] 185ns 0b                                            |L0.?-|                                       "
    - "L0.?[1071534,1224609] 185ns 0b                                                    |L0.?-|                               "
    - "L0.?[1224610,1377685] 185ns 0b                                                           |L0.?-|                        "
    - "L0.?[1377686,1530761] 185ns 0b                                                                   |L0.?-|                "
    - "L0.?[1530762,1683837] 185ns 0b                                                                          |L0.?-|         "
    - "L0.?[1683838,1836913] 185ns 0b                                                                                 |L0.?-|  "
    - "L0.?[1836914,1850000] 185ns 10b                                                                                         |L0.?|"
    - "**** Simulation run 94, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.187[186,1860000] 186ns|-----------------------------------------L0.187-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[186,153077] 186ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 186ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 186ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 186ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 186ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 186ns 0b                                     |L0.?-|                                              "
    - "L0.?[918458,1071533] 186ns 0b                                            |L0.?-|                                       "
    - "L0.?[1071534,1224609] 186ns 0b                                                   |L0.?-|                                "
    - "L0.?[1224610,1377685] 186ns 0b                                                           |L0.?-|                        "
    - "L0.?[1377686,1530761] 186ns 0b                                                                  |L0.?-|                 "
    - "L0.?[1530762,1683837] 186ns 0b                                                                          |L0.?-|         "
    - "L0.?[1683838,1836913] 186ns 0b                                                                                 |L0.?-|  "
    - "L0.?[1836914,1860000] 186ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 95, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.188[187,1870000] 187ns|-----------------------------------------L0.188-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[187,153077] 187ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 187ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 187ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 187ns 0b                      |L0.?-|                                                             "
    - "L0.?[612306,765381] 187ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 187ns 0b                                    |L0.?-|                                               "
    - "L0.?[918458,1071533] 187ns 0b                                            |L0.?-|                                       "
    - "L0.?[1071534,1224609] 187ns 0b                                                   |L0.?-|                                "
    - "L0.?[1224610,1377685] 187ns 0b                                                          |L0.?-|                         "
    - "L0.?[1377686,1530761] 187ns 0b                                                                  |L0.?-|                 "
    - "L0.?[1530762,1683837] 187ns 0b                                                                         |L0.?-|          "
    - "L0.?[1683838,1836913] 187ns 0b                                                                                 |L0.?-|  "
    - "L0.?[1836914,1870000] 187ns 10b                                                                                        |L0.?|"
    - "**** Simulation run 96, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.189[188,1880000] 188ns|-----------------------------------------L0.189-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[188,153077] 188ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 188ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 188ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 188ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 188ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 188ns 0b                                    |L0.?-|                                               "
    - "L0.?[918458,1071533] 188ns 0b                                           |L0.?-|                                        "
    - "L0.?[1071534,1224609] 188ns 0b                                                   |L0.?-|                                "
    - "L0.?[1224610,1377685] 188ns 0b                                                          |L0.?-|                         "
    - "L0.?[1377686,1530761] 188ns 0b                                                                 |L0.?-|                  "
    - "L0.?[1530762,1683837] 188ns 0b                                                                         |L0.?-|          "
    - "L0.?[1683838,1836913] 188ns 0b                                                                                |L0.?-|   "
    - "L0.?[1836914,1880000] 188ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 97, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.190[189,1890000] 189ns|-----------------------------------------L0.190-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[189,153077] 189ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 189ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 189ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 189ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 189ns 0b                             |L0.?-|                                                      "
    - "L0.?[765382,918457] 189ns 0b                                    |L0.?-|                                               "
    - "L0.?[918458,1071533] 189ns 0b                                           |L0.?-|                                        "
    - "L0.?[1071534,1224609] 189ns 0b                                                   |L0.?-|                                "
    - "L0.?[1224610,1377685] 189ns 0b                                                          |L0.?-|                         "
    - "L0.?[1377686,1530761] 189ns 0b                                                                 |L0.?-|                  "
    - "L0.?[1530762,1683837] 189ns 0b                                                                        |L0.?-|           "
    - "L0.?[1683838,1836913] 189ns 0b                                                                                |L0.?-|   "
    - "L0.?[1836914,1890000] 189ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 98, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.191[190,1900000] 190ns|-----------------------------------------L0.191-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[190,153077] 190ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 190ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 190ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 190ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 190ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 190ns 0b                                    |L0.?-|                                               "
    - "L0.?[918458,1071533] 190ns 0b                                           |L0.?-|                                        "
    - "L0.?[1071534,1224609] 190ns 0b                                                  |L0.?-|                                 "
    - "L0.?[1224610,1377685] 190ns 0b                                                          |L0.?-|                         "
    - "L0.?[1377686,1530761] 190ns 0b                                                                 |L0.?-|                  "
    - "L0.?[1530762,1683837] 190ns 0b                                                                        |L0.?-|           "
    - "L0.?[1683838,1836913] 190ns 0b                                                                               |L0.?-|    "
    - "L0.?[1836914,1900000] 190ns 10b                                                                                       |L0.?|"
    - "**** Simulation run 99, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.192[191,1910000] 191ns|-----------------------------------------L0.192-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[191,153077] 191ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 191ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 191ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 191ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 191ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 191ns 0b                                    |L0.?-|                                               "
    - "L0.?[918458,1071533] 191ns 0b                                           |L0.?-|                                        "
    - "L0.?[1071534,1224609] 191ns 0b                                                  |L0.?-|                                 "
    - "L0.?[1224610,1377685] 191ns 0b                                                         |L0.?-|                          "
    - "L0.?[1377686,1530761] 191ns 0b                                                                |L0.?-|                   "
    - "L0.?[1530762,1683837] 191ns 0b                                                                        |L0.?-|           "
    - "L0.?[1683838,1836913] 191ns 0b                                                                               |L0.?-|    "
    - "L0.?[1836914,1910000] 191ns 10b                                                                                      |L0.?|"
    - "**** Simulation run 100, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.193[192,1920000] 192ns|-----------------------------------------L0.193-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[192,153077] 192ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 192ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 192ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 192ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 192ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 192ns 0b                                   |L0.?-|                                                "
    - "L0.?[918458,1071533] 192ns 0b                                           |L0.?-|                                        "
    - "L0.?[1071534,1224609] 192ns 0b                                                  |L0.?-|                                 "
    - "L0.?[1224610,1377685] 192ns 0b                                                         |L0.?-|                          "
    - "L0.?[1377686,1530761] 192ns 0b                                                                |L0.?-|                   "
    - "L0.?[1530762,1683837] 192ns 0b                                                                       |L0.?-|            "
    - "L0.?[1683838,1836913] 192ns 0b                                                                              |L0.?-|     "
    - "L0.?[1836914,1920000] 192ns 10b                                                                                      |L0.?|"
    - "**** Simulation run 101, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.194[193,1930000] 193ns|-----------------------------------------L0.194-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[193,153077] 193ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 193ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 193ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 193ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 193ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 193ns 0b                                   |L0.?-|                                                "
    - "L0.?[918458,1071533] 193ns 0b                                          |L0.?-|                                         "
    - "L0.?[1071534,1224609] 193ns 0b                                                 |L0.?-|                                  "
    - "L0.?[1224610,1377685] 193ns 0b                                                         |L0.?-|                          "
    - "L0.?[1377686,1530761] 193ns 0b                                                                |L0.?-|                   "
    - "L0.?[1530762,1683837] 193ns 0b                                                                       |L0.?-|            "
    - "L0.?[1683838,1836913] 193ns 0b                                                                              |L0.?-|     "
    - "L0.?[1836914,1930000] 193ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 102, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.195[194,1940000] 194ns|-----------------------------------------L0.195-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[194,153077] 194ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 194ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 194ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 194ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 194ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 194ns 0b                                   |L0.?-|                                                "
    - "L0.?[918458,1071533] 194ns 0b                                          |L0.?-|                                         "
    - "L0.?[1071534,1224609] 194ns 0b                                                 |L0.?-|                                  "
    - "L0.?[1224610,1377685] 194ns 0b                                                        |L0.?-|                           "
    - "L0.?[1377686,1530761] 194ns 0b                                                               |L0.?-|                    "
    - "L0.?[1530762,1683837] 194ns 0b                                                                       |L0.?-|            "
    - "L0.?[1683838,1836913] 194ns 0b                                                                              |L0.?-|     "
    - "L0.?[1836914,1940000] 194ns 10b                                                                                     |L0.?|"
    - "**** Simulation run 103, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.196[195,1950000] 195ns|-----------------------------------------L0.196-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[195,153077] 195ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 195ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 195ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 195ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 195ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 195ns 0b                                   |L0.?-|                                                "
    - "L0.?[918458,1071533] 195ns 0b                                          |L0.?-|                                         "
    - "L0.?[1071534,1224609] 195ns 0b                                                 |L0.?-|                                  "
    - "L0.?[1224610,1377685] 195ns 0b                                                        |L0.?-|                           "
    - "L0.?[1377686,1530761] 195ns 0b                                                               |L0.?-|                    "
    - "L0.?[1530762,1683837] 195ns 0b                                                                      |L0.?-|             "
    - "L0.?[1683838,1836913] 195ns 0b                                                                             |L0.?-|      "
    - "L0.?[1836914,1950000] 195ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 104, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.197[196,1960000] 196ns|-----------------------------------------L0.197-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[196,153077] 196ns 0b|L0.?-|                                                                                   "
    - "L0.?[153078,306153] 196ns 0b       |L0.?-|                                                                            "
    - "L0.?[306154,459229] 196ns 0b              |L0.?-|                                                                     "
    - "L0.?[459230,612305] 196ns 0b                     |L0.?-|                                                              "
    - "L0.?[612306,765381] 196ns 0b                            |L0.?-|                                                       "
    - "L0.?[765382,918457] 196ns 0b                                   |L0.?-|                                                "
    - "L0.?[918458,1071533] 196ns 0b                                          |L0.?-|                                         "
    - "L0.?[1071534,1224609] 196ns 0b                                                 |L0.?-|                                  "
    - "L0.?[1224610,1377685] 196ns 0b                                                        |L0.?-|                           "
    - "L0.?[1377686,1530761] 196ns 0b                                                               |L0.?-|                    "
    - "L0.?[1530762,1683837] 196ns 0b                                                                      |L0.?-|             "
    - "L0.?[1683838,1836913] 196ns 0b                                                                             |L0.?-|      "
    - "L0.?[1836914,1960000] 196ns 10b                                                                                    |L0.?|"
    - "**** Simulation run 105, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.198[197,1970000] 197ns|-----------------------------------------L0.198-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[197,153077] 197ns 0b|L0.?|                                                                                    "
    - "L0.?[153078,306153] 197ns 0b      |L0.?|                                                                              "
    - "L0.?[306154,459229] 197ns 0b             |L0.?|                                                                       "
    - "L0.?[459230,612305] 197ns 0b                    |L0.?|                                                                "
    - "L0.?[612306,765381] 197ns 0b                           |L0.?|                                                         "
    - "L0.?[765382,918457] 197ns 0b                                  |L0.?|                                                  "
    - "L0.?[918458,1071533] 197ns 0b                                         |L0.?|                                           "
    - "L0.?[1071534,1224609] 197ns 0b                                                |L0.?|                                    "
    - "L0.?[1224610,1377685] 197ns 0b                                                       |L0.?|                             "
    - "L0.?[1377686,1530761] 197ns 0b                                                              |L0.?|                      "
    - "L0.?[1530762,1683837] 197ns 0b                                                                     |L0.?|               "
    - "L0.?[1683838,1836913] 197ns 0b                                                                            |L0.?|        "
    - "L0.?[1836914,1970000] 197ns 10b                                                                                   |L0.?| "
    - "**** Simulation run 106, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.199[198,1980000] 198ns|-----------------------------------------L0.199-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[198,153077] 198ns 0b|L0.?|                                                                                    "
    - "L0.?[153078,306153] 198ns 0b      |L0.?|                                                                              "
    - "L0.?[306154,459229] 198ns 0b             |L0.?|                                                                       "
    - "L0.?[459230,612305] 198ns 0b                    |L0.?|                                                                "
    - "L0.?[612306,765381] 198ns 0b                           |L0.?|                                                         "
    - "L0.?[765382,918457] 198ns 0b                                  |L0.?|                                                  "
    - "L0.?[918458,1071533] 198ns 0b                                         |L0.?|                                           "
    - "L0.?[1071534,1224609] 198ns 0b                                                |L0.?|                                    "
    - "L0.?[1224610,1377685] 198ns 0b                                                       |L0.?|                             "
    - "L0.?[1377686,1530761] 198ns 0b                                                              |L0.?|                      "
    - "L0.?[1530762,1683837] 198ns 0b                                                                     |L0.?|               "
    - "L0.?[1683838,1836913] 198ns 0b                                                                            |L0.?|        "
    - "L0.?[1836914,1980000] 198ns 10b                                                                                   |L0.?| "
    - "**** Simulation run 107, type=split(HighL0OverlapTotalBacklog)(split_times=[153077, 306153, 459229, 612305, 765381, 918457, 1071533, 1224609, 1377685, 1530761, 1683837, 1836913]). 1 Input Files, 10b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.200[199,1990000] 199ns|-----------------------------------------L0.200-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 10b total:"
    - "L0                                                                                                                 "
    - "L0.?[199,153077] 199ns 0b|L0.?|                                                                                    "
    - "L0.?[153078,306153] 199ns 0b      |L0.?|                                                                              "
    - "L0.?[306154,459229] 199ns 0b             |L0.?|                                                                       "
    - "L0.?[459230,612305] 199ns 0b                    |L0.?|                                                                "
    - "L0.?[612306,765381] 199ns 0b                           |L0.?|                                                         "
    - "L0.?[765382,918457] 199ns 0b                                  |L0.?|                                                  "
    - "L0.?[918458,1071533] 199ns 0b                                         |L0.?|                                           "
    - "L0.?[1071534,1224609] 199ns 0b                                                |L0.?|                                    "
    - "L0.?[1224610,1377685] 199ns 0b                                                       |L0.?|                             "
    - "L0.?[1377686,1530761] 199ns 0b                                                              |L0.?|                      "
    - "L0.?[1530762,1683837] 199ns 0b                                                                     |L0.?|               "
    - "L0.?[1683838,1836913] 199ns 0b                                                                            |L0.?|        "
    - "L0.?[1836914,1990000] 199ns 10b                                                                                   |L0.?| "
    - "Committing partition 1:"
    - "  Soft Deleting 24 files: L0.177, L0.178, L0.179, L0.180, L0.181, L0.182, L0.183, L0.184, L0.185, L0.186, L0.187, L0.188, L0.189, L0.190, L0.191, L0.192, L0.193, L0.194, L0.195, L0.196, L0.197, L0.198, L0.199, L0.200"
    - "  Creating 304 files"
    - "**** Simulation run 108, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.205[20,153077] 96ns 122b|-----------------------------------------L0.205-----------------------------------------|"
    - "L0.212[97,153077] 97ns 1b|----------------------------------------L0.212-----------------------------------------| "
    - "L0.219[98,153077] 98ns 1b|----------------------------------------L0.219-----------------------------------------| "
    - "L0.226[99,153077] 99ns 1b|----------------------------------------L0.226-----------------------------------------| "
    - "L0.233[100,153077] 100ns 1b|----------------------------------------L0.233-----------------------------------------| "
    - "L0.240[101,153077] 101ns 1b|----------------------------------------L0.240-----------------------------------------| "
    - "L0.247[102,153077] 102ns 1b|----------------------------------------L0.247-----------------------------------------| "
    - "L0.254[103,153077] 103ns 1b|----------------------------------------L0.254-----------------------------------------| "
    - "L0.261[104,153077] 104ns 1b|----------------------------------------L0.261-----------------------------------------| "
    - "L0.268[105,153077] 105ns 1b|----------------------------------------L0.268-----------------------------------------| "
    - "L0.275[106,153077] 106ns 1b|----------------------------------------L0.275-----------------------------------------| "
    - "L0.282[107,153077] 107ns 1b|----------------------------------------L0.282-----------------------------------------| "
    - "L0.289[108,153077] 108ns 1b|----------------------------------------L0.289-----------------------------------------| "
    - "L0.297[109,153077] 109ns 1b|----------------------------------------L0.297-----------------------------------------| "
    - "L0.305[110,153077] 110ns 1b|----------------------------------------L0.305-----------------------------------------| "
    - "L0.313[111,153077] 111ns 1b|----------------------------------------L0.313-----------------------------------------| "
    - "L0.321[112,153077] 112ns 1b|----------------------------------------L0.321-----------------------------------------| "
    - "L0.329[113,153077] 113ns 1b|----------------------------------------L0.329-----------------------------------------| "
    - "L0.337[114,153077] 114ns 1b|----------------------------------------L0.337-----------------------------------------| "
    - "L0.345[115,153077] 115ns 1b|----------------------------------------L0.345-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[20,153077] 115ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.205, L0.212, L0.219, L0.226, L0.233, L0.240, L0.247, L0.254, L0.261, L0.268, L0.275, L0.282, L0.289, L0.297, L0.305, L0.313, L0.321, L0.329, L0.337, L0.345"
    - "  Creating 1 files"
    - "**** Simulation run 109, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.206[153078,306153] 96ns 122b|-----------------------------------------L0.206-----------------------------------------|"
    - "L0.213[153078,306153] 97ns 1b|-----------------------------------------L0.213-----------------------------------------|"
    - "L0.220[153078,306153] 98ns 1b|-----------------------------------------L0.220-----------------------------------------|"
    - "L0.227[153078,306153] 99ns 1b|-----------------------------------------L0.227-----------------------------------------|"
    - "L0.234[153078,306153] 100ns 1b|-----------------------------------------L0.234-----------------------------------------|"
    - "L0.241[153078,306153] 101ns 1b|-----------------------------------------L0.241-----------------------------------------|"
    - "L0.248[153078,306153] 102ns 1b|-----------------------------------------L0.248-----------------------------------------|"
    - "L0.255[153078,306153] 103ns 1b|-----------------------------------------L0.255-----------------------------------------|"
    - "L0.262[153078,306153] 104ns 1b|-----------------------------------------L0.262-----------------------------------------|"
    - "L0.269[153078,306153] 105ns 1b|-----------------------------------------L0.269-----------------------------------------|"
    - "L0.276[153078,306153] 106ns 1b|-----------------------------------------L0.276-----------------------------------------|"
    - "L0.283[153078,306153] 107ns 1b|-----------------------------------------L0.283-----------------------------------------|"
    - "L0.290[153078,306153] 108ns 1b|-----------------------------------------L0.290-----------------------------------------|"
    - "L0.298[153078,306153] 109ns 1b|-----------------------------------------L0.298-----------------------------------------|"
    - "L0.306[153078,306153] 110ns 1b|-----------------------------------------L0.306-----------------------------------------|"
    - "L0.314[153078,306153] 111ns 1b|-----------------------------------------L0.314-----------------------------------------|"
    - "L0.322[153078,306153] 112ns 1b|-----------------------------------------L0.322-----------------------------------------|"
    - "L0.330[153078,306153] 113ns 1b|-----------------------------------------L0.330-----------------------------------------|"
    - "L0.338[153078,306153] 114ns 1b|-----------------------------------------L0.338-----------------------------------------|"
    - "L0.346[153078,306153] 115ns 1b|-----------------------------------------L0.346-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[153078,306153] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.206, L0.213, L0.220, L0.227, L0.234, L0.241, L0.248, L0.255, L0.262, L0.269, L0.276, L0.283, L0.290, L0.298, L0.306, L0.314, L0.322, L0.330, L0.338, L0.346"
    - "  Creating 1 files"
    - "**** Simulation run 110, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.354[153078,306153] 116ns|-----------------------------------------L0.354-----------------------------------------|"
    - "L0.362[153078,306153] 117ns|-----------------------------------------L0.362-----------------------------------------|"
    - "L0.370[153078,306153] 118ns|-----------------------------------------L0.370-----------------------------------------|"
    - "L0.378[153078,306153] 119ns|-----------------------------------------L0.378-----------------------------------------|"
    - "L0.386[153078,306153] 120ns|-----------------------------------------L0.386-----------------------------------------|"
    - "L0.394[153078,306153] 121ns|-----------------------------------------L0.394-----------------------------------------|"
    - "L0.402[153078,306153] 122ns|-----------------------------------------L0.402-----------------------------------------|"
    - "L0.410[153078,306153] 123ns|-----------------------------------------L0.410-----------------------------------------|"
    - "L0.419[153078,306153] 124ns|-----------------------------------------L0.419-----------------------------------------|"
    - "L0.428[153078,306153] 125ns|-----------------------------------------L0.428-----------------------------------------|"
    - "L0.437[153078,306153] 126ns|-----------------------------------------L0.437-----------------------------------------|"
    - "L0.446[153078,306153] 127ns|-----------------------------------------L0.446-----------------------------------------|"
    - "L0.455[153078,306153] 128ns|-----------------------------------------L0.455-----------------------------------------|"
    - "L0.464[153078,306153] 129ns|-----------------------------------------L0.464-----------------------------------------|"
    - "L0.473[153078,306153] 130ns|-----------------------------------------L0.473-----------------------------------------|"
    - "L0.482[153078,306153] 131ns|-----------------------------------------L0.482-----------------------------------------|"
    - "L0.491[153078,306153] 132ns|-----------------------------------------L0.491-----------------------------------------|"
    - "L0.500[153078,306153] 133ns|-----------------------------------------L0.500-----------------------------------------|"
    - "L0.509[153078,306153] 134ns|-----------------------------------------L0.509-----------------------------------------|"
    - "L0.518[153078,306153] 135ns|-----------------------------------------L0.518-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[153078,306153] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "**** Simulation run 111, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.727[153078,306153] 156ns|-----------------------------------------L0.727-----------------------------------------|"
    - "L0.738[153078,306153] 157ns|-----------------------------------------L0.738-----------------------------------------|"
    - "L0.749[153078,306153] 158ns|-----------------------------------------L0.749-----------------------------------------|"
    - "L0.760[153078,306153] 159ns|-----------------------------------------L0.760-----------------------------------------|"
    - "L0.771[153078,306153] 160ns|-----------------------------------------L0.771-----------------------------------------|"
    - "L0.782[153078,306153] 161ns|-----------------------------------------L0.782-----------------------------------------|"
    - "L0.793[153078,306153] 162ns|-----------------------------------------L0.793-----------------------------------------|"
    - "L0.804[153078,306153] 163ns|-----------------------------------------L0.804-----------------------------------------|"
    - "L0.815[153078,306153] 164ns|-----------------------------------------L0.815-----------------------------------------|"
    - "L0.826[153078,306153] 165ns|-----------------------------------------L0.826-----------------------------------------|"
    - "L0.837[153078,306153] 166ns|-----------------------------------------L0.837-----------------------------------------|"
    - "L0.848[153078,306153] 167ns|-----------------------------------------L0.848-----------------------------------------|"
    - "L0.859[153078,306153] 168ns|-----------------------------------------L0.859-----------------------------------------|"
    - "L0.870[153078,306153] 169ns|-----------------------------------------L0.870-----------------------------------------|"
    - "L0.882[153078,306153] 170ns|-----------------------------------------L0.882-----------------------------------------|"
    - "L0.894[153078,306153] 171ns|-----------------------------------------L0.894-----------------------------------------|"
    - "L0.906[153078,306153] 172ns|-----------------------------------------L0.906-----------------------------------------|"
    - "L0.918[153078,306153] 173ns|-----------------------------------------L0.918-----------------------------------------|"
    - "L0.930[153078,306153] 174ns|-----------------------------------------L0.930-----------------------------------------|"
    - "L0.942[153078,306153] 175ns|-----------------------------------------L0.942-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[153078,306153] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.727, L0.738, L0.749, L0.760, L0.771, L0.782, L0.793, L0.804, L0.815, L0.826, L0.837, L0.848, L0.859, L0.870, L0.882, L0.894, L0.906, L0.918, L0.930, L0.942"
    - "  Creating 1 files"
    - "**** Simulation run 112, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.954[153078,306153] 176ns|-----------------------------------------L0.954-----------------------------------------|"
    - "L0.966[153078,306153] 177ns|-----------------------------------------L0.966-----------------------------------------|"
    - "L0.978[153078,306153] 178ns|-----------------------------------------L0.978-----------------------------------------|"
    - "L0.990[153078,306153] 179ns|-----------------------------------------L0.990-----------------------------------------|"
    - "L0.1002[153078,306153] 180ns|----------------------------------------L0.1002-----------------------------------------|"
    - "L0.1014[153078,306153] 181ns|----------------------------------------L0.1014-----------------------------------------|"
    - "L0.1026[153078,306153] 182ns|----------------------------------------L0.1026-----------------------------------------|"
    - "L0.1038[153078,306153] 183ns|----------------------------------------L0.1038-----------------------------------------|"
    - "L0.1050[153078,306153] 184ns|----------------------------------------L0.1050-----------------------------------------|"
    - "L0.1063[153078,306153] 185ns|----------------------------------------L0.1063-----------------------------------------|"
    - "L0.1076[153078,306153] 186ns|----------------------------------------L0.1076-----------------------------------------|"
    - "L0.1089[153078,306153] 187ns|----------------------------------------L0.1089-----------------------------------------|"
    - "L0.1102[153078,306153] 188ns|----------------------------------------L0.1102-----------------------------------------|"
    - "L0.1115[153078,306153] 189ns|----------------------------------------L0.1115-----------------------------------------|"
    - "L0.1128[153078,306153] 190ns|----------------------------------------L0.1128-----------------------------------------|"
    - "L0.1141[153078,306153] 191ns|----------------------------------------L0.1141-----------------------------------------|"
    - "L0.1154[153078,306153] 192ns|----------------------------------------L0.1154-----------------------------------------|"
    - "L0.1167[153078,306153] 193ns|----------------------------------------L0.1167-----------------------------------------|"
    - "L0.1180[153078,306153] 194ns|----------------------------------------L0.1180-----------------------------------------|"
    - "L0.1193[153078,306153] 195ns|----------------------------------------L0.1193-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[153078,306153] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.954, L0.966, L0.978, L0.990, L0.1002, L0.1014, L0.1026, L0.1038, L0.1050, L0.1063, L0.1076, L0.1089, L0.1102, L0.1115, L0.1128, L0.1141, L0.1154, L0.1167, L0.1180, L0.1193"
    - "  Creating 1 files"
    - "**** Simulation run 113, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1206[153078,306153] 196ns|----------------------------------------L0.1206-----------------------------------------|"
    - "L0.1219[153078,306153] 197ns|----------------------------------------L0.1219-----------------------------------------|"
    - "L0.1232[153078,306153] 198ns|----------------------------------------L0.1232-----------------------------------------|"
    - "L0.1245[153078,306153] 199ns|----------------------------------------L0.1245-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[153078,306153] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1206, L0.1219, L0.1232, L0.1245"
    - "  Creating 1 files"
    - "**** Simulation run 114, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.207[306154,459229] 96ns 122b|-----------------------------------------L0.207-----------------------------------------|"
    - "L0.214[306154,459229] 97ns 1b|-----------------------------------------L0.214-----------------------------------------|"
    - "L0.221[306154,459229] 98ns 1b|-----------------------------------------L0.221-----------------------------------------|"
    - "L0.228[306154,459229] 99ns 1b|-----------------------------------------L0.228-----------------------------------------|"
    - "L0.235[306154,459229] 100ns 1b|-----------------------------------------L0.235-----------------------------------------|"
    - "L0.242[306154,459229] 101ns 1b|-----------------------------------------L0.242-----------------------------------------|"
    - "L0.249[306154,459229] 102ns 1b|-----------------------------------------L0.249-----------------------------------------|"
    - "L0.256[306154,459229] 103ns 1b|-----------------------------------------L0.256-----------------------------------------|"
    - "L0.263[306154,459229] 104ns 1b|-----------------------------------------L0.263-----------------------------------------|"
    - "L0.270[306154,459229] 105ns 1b|-----------------------------------------L0.270-----------------------------------------|"
    - "L0.277[306154,459229] 106ns 1b|-----------------------------------------L0.277-----------------------------------------|"
    - "L0.284[306154,459229] 107ns 1b|-----------------------------------------L0.284-----------------------------------------|"
    - "L0.291[306154,459229] 108ns 1b|-----------------------------------------L0.291-----------------------------------------|"
    - "L0.299[306154,459229] 109ns 1b|-----------------------------------------L0.299-----------------------------------------|"
    - "L0.307[306154,459229] 110ns 1b|-----------------------------------------L0.307-----------------------------------------|"
    - "L0.315[306154,459229] 111ns 1b|-----------------------------------------L0.315-----------------------------------------|"
    - "L0.323[306154,459229] 112ns 1b|-----------------------------------------L0.323-----------------------------------------|"
    - "L0.331[306154,459229] 113ns 1b|-----------------------------------------L0.331-----------------------------------------|"
    - "L0.339[306154,459229] 114ns 1b|-----------------------------------------L0.339-----------------------------------------|"
    - "L0.347[306154,459229] 115ns 1b|-----------------------------------------L0.347-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[306154,459229] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.207, L0.214, L0.221, L0.228, L0.235, L0.242, L0.249, L0.256, L0.263, L0.270, L0.277, L0.284, L0.291, L0.299, L0.307, L0.315, L0.323, L0.331, L0.339, L0.347"
    - "  Creating 1 files"
    - "**** Simulation run 115, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.355[306154,459229] 116ns|-----------------------------------------L0.355-----------------------------------------|"
    - "L0.363[306154,459229] 117ns|-----------------------------------------L0.363-----------------------------------------|"
    - "L0.371[306154,459229] 118ns|-----------------------------------------L0.371-----------------------------------------|"
    - "L0.379[306154,459229] 119ns|-----------------------------------------L0.379-----------------------------------------|"
    - "L0.387[306154,459229] 120ns|-----------------------------------------L0.387-----------------------------------------|"
    - "L0.395[306154,459229] 121ns|-----------------------------------------L0.395-----------------------------------------|"
    - "L0.403[306154,459229] 122ns|-----------------------------------------L0.403-----------------------------------------|"
    - "L0.411[306154,459229] 123ns|-----------------------------------------L0.411-----------------------------------------|"
    - "L0.420[306154,459229] 124ns|-----------------------------------------L0.420-----------------------------------------|"
    - "L0.429[306154,459229] 125ns|-----------------------------------------L0.429-----------------------------------------|"
    - "L0.438[306154,459229] 126ns|-----------------------------------------L0.438-----------------------------------------|"
    - "L0.447[306154,459229] 127ns|-----------------------------------------L0.447-----------------------------------------|"
    - "L0.456[306154,459229] 128ns|-----------------------------------------L0.456-----------------------------------------|"
    - "L0.465[306154,459229] 129ns|-----------------------------------------L0.465-----------------------------------------|"
    - "L0.474[306154,459229] 130ns|-----------------------------------------L0.474-----------------------------------------|"
    - "L0.483[306154,459229] 131ns|-----------------------------------------L0.483-----------------------------------------|"
    - "L0.492[306154,459229] 132ns|-----------------------------------------L0.492-----------------------------------------|"
    - "L0.501[306154,459229] 133ns|-----------------------------------------L0.501-----------------------------------------|"
    - "L0.510[306154,459229] 134ns|-----------------------------------------L0.510-----------------------------------------|"
    - "L0.519[306154,459229] 135ns|-----------------------------------------L0.519-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[306154,459229] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.355, L0.363, L0.371, L0.379, L0.387, L0.395, L0.403, L0.411, L0.420, L0.429, L0.438, L0.447, L0.456, L0.465, L0.474, L0.483, L0.492, L0.501, L0.510, L0.519"
    - "  Creating 1 files"
    - "**** Simulation run 116, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.528[306154,459229] 136ns 1b|-----------------------------------------L0.528-----------------------------------------|"
    - "L0.537[306154,459229] 137ns 1b|-----------------------------------------L0.537-----------------------------------------|"
    - "L0.546[306154,459229] 138ns 1b|-----------------------------------------L0.546-----------------------------------------|"
    - "L0.556[306154,459229] 139ns 1b|-----------------------------------------L0.556-----------------------------------------|"
    - "L0.566[306154,459229] 140ns 1b|-----------------------------------------L0.566-----------------------------------------|"
    - "L0.576[306154,459229] 141ns 1b|-----------------------------------------L0.576-----------------------------------------|"
    - "L0.586[306154,459229] 142ns 1b|-----------------------------------------L0.586-----------------------------------------|"
    - "L0.596[306154,459229] 143ns 1b|-----------------------------------------L0.596-----------------------------------------|"
    - "L0.606[306154,459229] 144ns 1b|-----------------------------------------L0.606-----------------------------------------|"
    - "L0.616[306154,459229] 145ns 1b|-----------------------------------------L0.616-----------------------------------------|"
    - "L0.626[306154,459229] 146ns 1b|-----------------------------------------L0.626-----------------------------------------|"
    - "L0.636[306154,459229] 147ns 1b|-----------------------------------------L0.636-----------------------------------------|"
    - "L0.646[306154,459229] 148ns 1b|-----------------------------------------L0.646-----------------------------------------|"
    - "L0.656[306154,459229] 149ns 1b|-----------------------------------------L0.656-----------------------------------------|"
    - "L0.666[306154,459229] 150ns 1b|-----------------------------------------L0.666-----------------------------------------|"
    - "L0.676[306154,459229] 151ns 1b|-----------------------------------------L0.676-----------------------------------------|"
    - "L0.686[306154,459229] 152ns 1b|-----------------------------------------L0.686-----------------------------------------|"
    - "L0.696[306154,459229] 153ns 1b|-----------------------------------------L0.696-----------------------------------------|"
    - "L0.706[306154,459229] 154ns 0b|-----------------------------------------L0.706-----------------------------------------|"
    - "L0.717[306154,459229] 155ns 0b|-----------------------------------------L0.717-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[306154,459229] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.528, L0.537, L0.546, L0.556, L0.566, L0.576, L0.586, L0.596, L0.606, L0.616, L0.626, L0.636, L0.646, L0.656, L0.666, L0.676, L0.686, L0.696, L0.706, L0.717"
    - "  Creating 1 files"
    - "**** Simulation run 117, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.728[306154,459229] 156ns|-----------------------------------------L0.728-----------------------------------------|"
    - "L0.739[306154,459229] 157ns|-----------------------------------------L0.739-----------------------------------------|"
    - "L0.750[306154,459229] 158ns|-----------------------------------------L0.750-----------------------------------------|"
    - "L0.761[306154,459229] 159ns|-----------------------------------------L0.761-----------------------------------------|"
    - "L0.772[306154,459229] 160ns|-----------------------------------------L0.772-----------------------------------------|"
    - "L0.783[306154,459229] 161ns|-----------------------------------------L0.783-----------------------------------------|"
    - "L0.794[306154,459229] 162ns|-----------------------------------------L0.794-----------------------------------------|"
    - "L0.805[306154,459229] 163ns|-----------------------------------------L0.805-----------------------------------------|"
    - "L0.816[306154,459229] 164ns|-----------------------------------------L0.816-----------------------------------------|"
    - "L0.827[306154,459229] 165ns|-----------------------------------------L0.827-----------------------------------------|"
    - "L0.838[306154,459229] 166ns|-----------------------------------------L0.838-----------------------------------------|"
    - "L0.849[306154,459229] 167ns|-----------------------------------------L0.849-----------------------------------------|"
    - "L0.860[306154,459229] 168ns|-----------------------------------------L0.860-----------------------------------------|"
    - "L0.871[306154,459229] 169ns|-----------------------------------------L0.871-----------------------------------------|"
    - "L0.883[306154,459229] 170ns|-----------------------------------------L0.883-----------------------------------------|"
    - "L0.895[306154,459229] 171ns|-----------------------------------------L0.895-----------------------------------------|"
    - "L0.907[306154,459229] 172ns|-----------------------------------------L0.907-----------------------------------------|"
    - "L0.919[306154,459229] 173ns|-----------------------------------------L0.919-----------------------------------------|"
    - "L0.931[306154,459229] 174ns|-----------------------------------------L0.931-----------------------------------------|"
    - "L0.943[306154,459229] 175ns|-----------------------------------------L0.943-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[306154,459229] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.728, L0.739, L0.750, L0.761, L0.772, L0.783, L0.794, L0.805, L0.816, L0.827, L0.838, L0.849, L0.860, L0.871, L0.883, L0.895, L0.907, L0.919, L0.931, L0.943"
    - "  Creating 1 files"
    - "**** Simulation run 118, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.955[306154,459229] 176ns|-----------------------------------------L0.955-----------------------------------------|"
    - "L0.967[306154,459229] 177ns|-----------------------------------------L0.967-----------------------------------------|"
    - "L0.979[306154,459229] 178ns|-----------------------------------------L0.979-----------------------------------------|"
    - "L0.991[306154,459229] 179ns|-----------------------------------------L0.991-----------------------------------------|"
    - "L0.1003[306154,459229] 180ns|----------------------------------------L0.1003-----------------------------------------|"
    - "L0.1015[306154,459229] 181ns|----------------------------------------L0.1015-----------------------------------------|"
    - "L0.1027[306154,459229] 182ns|----------------------------------------L0.1027-----------------------------------------|"
    - "L0.1039[306154,459229] 183ns|----------------------------------------L0.1039-----------------------------------------|"
    - "L0.1051[306154,459229] 184ns|----------------------------------------L0.1051-----------------------------------------|"
    - "L0.1064[306154,459229] 185ns|----------------------------------------L0.1064-----------------------------------------|"
    - "L0.1077[306154,459229] 186ns|----------------------------------------L0.1077-----------------------------------------|"
    - "L0.1090[306154,459229] 187ns|----------------------------------------L0.1090-----------------------------------------|"
    - "L0.1103[306154,459229] 188ns|----------------------------------------L0.1103-----------------------------------------|"
    - "L0.1116[306154,459229] 189ns|----------------------------------------L0.1116-----------------------------------------|"
    - "L0.1129[306154,459229] 190ns|----------------------------------------L0.1129-----------------------------------------|"
    - "L0.1142[306154,459229] 191ns|----------------------------------------L0.1142-----------------------------------------|"
    - "L0.1155[306154,459229] 192ns|----------------------------------------L0.1155-----------------------------------------|"
    - "L0.1168[306154,459229] 193ns|----------------------------------------L0.1168-----------------------------------------|"
    - "L0.1181[306154,459229] 194ns|----------------------------------------L0.1181-----------------------------------------|"
    - "L0.1194[306154,459229] 195ns|----------------------------------------L0.1194-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[306154,459229] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.955, L0.967, L0.979, L0.991, L0.1003, L0.1015, L0.1027, L0.1039, L0.1051, L0.1064, L0.1077, L0.1090, L0.1103, L0.1116, L0.1129, L0.1142, L0.1155, L0.1168, L0.1181, L0.1194"
    - "  Creating 1 files"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.354, L0.362, L0.370, L0.378, L0.386, L0.394, L0.402, L0.410, L0.419, L0.428, L0.437, L0.446, L0.455, L0.464, L0.473, L0.482, L0.491, L0.500, L0.509, L0.518"
    - "  Creating 1 files"
    - "**** Simulation run 119, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.527[153078,306153] 136ns 1b|-----------------------------------------L0.527-----------------------------------------|"
    - "L0.536[153078,306153] 137ns 1b|-----------------------------------------L0.536-----------------------------------------|"
    - "L0.545[153078,306153] 138ns 1b|-----------------------------------------L0.545-----------------------------------------|"
    - "L0.555[153078,306153] 139ns 1b|-----------------------------------------L0.555-----------------------------------------|"
    - "L0.565[153078,306153] 140ns 1b|-----------------------------------------L0.565-----------------------------------------|"
    - "L0.575[153078,306153] 141ns 1b|-----------------------------------------L0.575-----------------------------------------|"
    - "L0.585[153078,306153] 142ns 1b|-----------------------------------------L0.585-----------------------------------------|"
    - "L0.595[153078,306153] 143ns 1b|-----------------------------------------L0.595-----------------------------------------|"
    - "L0.605[153078,306153] 144ns 1b|-----------------------------------------L0.605-----------------------------------------|"
    - "L0.615[153078,306153] 145ns 1b|-----------------------------------------L0.615-----------------------------------------|"
    - "L0.625[153078,306153] 146ns 1b|-----------------------------------------L0.625-----------------------------------------|"
    - "L0.635[153078,306153] 147ns 1b|-----------------------------------------L0.635-----------------------------------------|"
    - "L0.645[153078,306153] 148ns 1b|-----------------------------------------L0.645-----------------------------------------|"
    - "L0.655[153078,306153] 149ns 1b|-----------------------------------------L0.655-----------------------------------------|"
    - "L0.665[153078,306153] 150ns 1b|-----------------------------------------L0.665-----------------------------------------|"
    - "L0.675[153078,306153] 151ns 1b|-----------------------------------------L0.675-----------------------------------------|"
    - "L0.685[153078,306153] 152ns 1b|-----------------------------------------L0.685-----------------------------------------|"
    - "L0.695[153078,306153] 153ns 1b|-----------------------------------------L0.695-----------------------------------------|"
    - "L0.705[153078,306153] 154ns 0b|-----------------------------------------L0.705-----------------------------------------|"
    - "L0.716[153078,306153] 155ns 0b|-----------------------------------------L0.716-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[153078,306153] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.527, L0.536, L0.545, L0.555, L0.565, L0.575, L0.585, L0.595, L0.605, L0.615, L0.625, L0.635, L0.645, L0.655, L0.665, L0.675, L0.685, L0.695, L0.705, L0.716"
    - "  Creating 1 files"
    - "**** Simulation run 120, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1207[306154,459229] 196ns|----------------------------------------L0.1207-----------------------------------------|"
    - "L0.1220[306154,459229] 197ns|----------------------------------------L0.1220-----------------------------------------|"
    - "L0.1233[306154,459229] 198ns|----------------------------------------L0.1233-----------------------------------------|"
    - "L0.1246[306154,459229] 199ns|----------------------------------------L0.1246-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[306154,459229] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1207, L0.1220, L0.1233, L0.1246"
    - "  Creating 1 files"
    - "**** Simulation run 121, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.208[459230,612305] 96ns 122b|-----------------------------------------L0.208-----------------------------------------|"
    - "L0.215[459230,612305] 97ns 1b|-----------------------------------------L0.215-----------------------------------------|"
    - "L0.222[459230,612305] 98ns 1b|-----------------------------------------L0.222-----------------------------------------|"
    - "L0.229[459230,612305] 99ns 1b|-----------------------------------------L0.229-----------------------------------------|"
    - "L0.236[459230,612305] 100ns 1b|-----------------------------------------L0.236-----------------------------------------|"
    - "L0.243[459230,612305] 101ns 1b|-----------------------------------------L0.243-----------------------------------------|"
    - "L0.250[459230,612305] 102ns 1b|-----------------------------------------L0.250-----------------------------------------|"
    - "L0.257[459230,612305] 103ns 1b|-----------------------------------------L0.257-----------------------------------------|"
    - "L0.264[459230,612305] 104ns 1b|-----------------------------------------L0.264-----------------------------------------|"
    - "L0.271[459230,612305] 105ns 1b|-----------------------------------------L0.271-----------------------------------------|"
    - "L0.278[459230,612305] 106ns 1b|-----------------------------------------L0.278-----------------------------------------|"
    - "L0.285[459230,612305] 107ns 1b|-----------------------------------------L0.285-----------------------------------------|"
    - "L0.292[459230,612305] 108ns 1b|-----------------------------------------L0.292-----------------------------------------|"
    - "L0.300[459230,612305] 109ns 1b|-----------------------------------------L0.300-----------------------------------------|"
    - "L0.308[459230,612305] 110ns 1b|-----------------------------------------L0.308-----------------------------------------|"
    - "L0.316[459230,612305] 111ns 1b|-----------------------------------------L0.316-----------------------------------------|"
    - "L0.324[459230,612305] 112ns 1b|-----------------------------------------L0.324-----------------------------------------|"
    - "L0.332[459230,612305] 113ns 1b|-----------------------------------------L0.332-----------------------------------------|"
    - "L0.340[459230,612305] 114ns 1b|-----------------------------------------L0.340-----------------------------------------|"
    - "L0.348[459230,612305] 115ns 1b|-----------------------------------------L0.348-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[459230,612305] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.208, L0.215, L0.222, L0.229, L0.236, L0.243, L0.250, L0.257, L0.264, L0.271, L0.278, L0.285, L0.292, L0.300, L0.308, L0.316, L0.324, L0.332, L0.340, L0.348"
    - "  Creating 1 files"
    - "**** Simulation run 122, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.356[459230,612305] 116ns|-----------------------------------------L0.356-----------------------------------------|"
    - "L0.364[459230,612305] 117ns|-----------------------------------------L0.364-----------------------------------------|"
    - "L0.372[459230,612305] 118ns|-----------------------------------------L0.372-----------------------------------------|"
    - "L0.380[459230,612305] 119ns|-----------------------------------------L0.380-----------------------------------------|"
    - "L0.388[459230,612305] 120ns|-----------------------------------------L0.388-----------------------------------------|"
    - "L0.396[459230,612305] 121ns|-----------------------------------------L0.396-----------------------------------------|"
    - "L0.404[459230,612305] 122ns|-----------------------------------------L0.404-----------------------------------------|"
    - "L0.412[459230,612305] 123ns|-----------------------------------------L0.412-----------------------------------------|"
    - "L0.421[459230,612305] 124ns|-----------------------------------------L0.421-----------------------------------------|"
    - "L0.430[459230,612305] 125ns|-----------------------------------------L0.430-----------------------------------------|"
    - "L0.439[459230,612305] 126ns|-----------------------------------------L0.439-----------------------------------------|"
    - "L0.448[459230,612305] 127ns|-----------------------------------------L0.448-----------------------------------------|"
    - "L0.457[459230,612305] 128ns|-----------------------------------------L0.457-----------------------------------------|"
    - "L0.466[459230,612305] 129ns|-----------------------------------------L0.466-----------------------------------------|"
    - "L0.475[459230,612305] 130ns|-----------------------------------------L0.475-----------------------------------------|"
    - "L0.484[459230,612305] 131ns|-----------------------------------------L0.484-----------------------------------------|"
    - "L0.493[459230,612305] 132ns|-----------------------------------------L0.493-----------------------------------------|"
    - "L0.502[459230,612305] 133ns|-----------------------------------------L0.502-----------------------------------------|"
    - "L0.511[459230,612305] 134ns|-----------------------------------------L0.511-----------------------------------------|"
    - "L0.520[459230,612305] 135ns|-----------------------------------------L0.520-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[459230,612305] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.356, L0.364, L0.372, L0.380, L0.388, L0.396, L0.404, L0.412, L0.421, L0.430, L0.439, L0.448, L0.457, L0.466, L0.475, L0.484, L0.493, L0.502, L0.511, L0.520"
    - "  Creating 1 files"
    - "**** Simulation run 123, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.529[459230,612305] 136ns 1b|-----------------------------------------L0.529-----------------------------------------|"
    - "L0.538[459230,612305] 137ns 1b|-----------------------------------------L0.538-----------------------------------------|"
    - "L0.547[459230,612305] 138ns 1b|-----------------------------------------L0.547-----------------------------------------|"
    - "L0.557[459230,612305] 139ns 1b|-----------------------------------------L0.557-----------------------------------------|"
    - "L0.567[459230,612305] 140ns 1b|-----------------------------------------L0.567-----------------------------------------|"
    - "L0.577[459230,612305] 141ns 1b|-----------------------------------------L0.577-----------------------------------------|"
    - "L0.587[459230,612305] 142ns 1b|-----------------------------------------L0.587-----------------------------------------|"
    - "L0.597[459230,612305] 143ns 1b|-----------------------------------------L0.597-----------------------------------------|"
    - "L0.607[459230,612305] 144ns 1b|-----------------------------------------L0.607-----------------------------------------|"
    - "L0.617[459230,612305] 145ns 1b|-----------------------------------------L0.617-----------------------------------------|"
    - "L0.627[459230,612305] 146ns 1b|-----------------------------------------L0.627-----------------------------------------|"
    - "L0.637[459230,612305] 147ns 1b|-----------------------------------------L0.637-----------------------------------------|"
    - "L0.647[459230,612305] 148ns 1b|-----------------------------------------L0.647-----------------------------------------|"
    - "L0.657[459230,612305] 149ns 1b|-----------------------------------------L0.657-----------------------------------------|"
    - "L0.667[459230,612305] 150ns 1b|-----------------------------------------L0.667-----------------------------------------|"
    - "L0.677[459230,612305] 151ns 1b|-----------------------------------------L0.677-----------------------------------------|"
    - "L0.687[459230,612305] 152ns 1b|-----------------------------------------L0.687-----------------------------------------|"
    - "L0.697[459230,612305] 153ns 1b|-----------------------------------------L0.697-----------------------------------------|"
    - "L0.707[459230,612305] 154ns 0b|-----------------------------------------L0.707-----------------------------------------|"
    - "L0.718[459230,612305] 155ns 0b|-----------------------------------------L0.718-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[459230,612305] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.529, L0.538, L0.547, L0.557, L0.567, L0.577, L0.587, L0.597, L0.607, L0.617, L0.627, L0.637, L0.647, L0.657, L0.667, L0.677, L0.687, L0.697, L0.707, L0.718"
    - "  Creating 1 files"
    - "**** Simulation run 124, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.729[459230,612305] 156ns|-----------------------------------------L0.729-----------------------------------------|"
    - "L0.740[459230,612305] 157ns|-----------------------------------------L0.740-----------------------------------------|"
    - "L0.751[459230,612305] 158ns|-----------------------------------------L0.751-----------------------------------------|"
    - "L0.762[459230,612305] 159ns|-----------------------------------------L0.762-----------------------------------------|"
    - "L0.773[459230,612305] 160ns|-----------------------------------------L0.773-----------------------------------------|"
    - "L0.784[459230,612305] 161ns|-----------------------------------------L0.784-----------------------------------------|"
    - "L0.795[459230,612305] 162ns|-----------------------------------------L0.795-----------------------------------------|"
    - "L0.806[459230,612305] 163ns|-----------------------------------------L0.806-----------------------------------------|"
    - "L0.817[459230,612305] 164ns|-----------------------------------------L0.817-----------------------------------------|"
    - "L0.828[459230,612305] 165ns|-----------------------------------------L0.828-----------------------------------------|"
    - "L0.839[459230,612305] 166ns|-----------------------------------------L0.839-----------------------------------------|"
    - "L0.850[459230,612305] 167ns|-----------------------------------------L0.850-----------------------------------------|"
    - "L0.861[459230,612305] 168ns|-----------------------------------------L0.861-----------------------------------------|"
    - "L0.872[459230,612305] 169ns|-----------------------------------------L0.872-----------------------------------------|"
    - "L0.884[459230,612305] 170ns|-----------------------------------------L0.884-----------------------------------------|"
    - "L0.896[459230,612305] 171ns|-----------------------------------------L0.896-----------------------------------------|"
    - "L0.908[459230,612305] 172ns|-----------------------------------------L0.908-----------------------------------------|"
    - "L0.920[459230,612305] 173ns|-----------------------------------------L0.920-----------------------------------------|"
    - "L0.932[459230,612305] 174ns|-----------------------------------------L0.932-----------------------------------------|"
    - "L0.944[459230,612305] 175ns|-----------------------------------------L0.944-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[459230,612305] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.729, L0.740, L0.751, L0.762, L0.773, L0.784, L0.795, L0.806, L0.817, L0.828, L0.839, L0.850, L0.861, L0.872, L0.884, L0.896, L0.908, L0.920, L0.932, L0.944"
    - "  Creating 1 files"
    - "**** Simulation run 125, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.956[459230,612305] 176ns|-----------------------------------------L0.956-----------------------------------------|"
    - "L0.968[459230,612305] 177ns|-----------------------------------------L0.968-----------------------------------------|"
    - "L0.980[459230,612305] 178ns|-----------------------------------------L0.980-----------------------------------------|"
    - "L0.992[459230,612305] 179ns|-----------------------------------------L0.992-----------------------------------------|"
    - "L0.1004[459230,612305] 180ns|----------------------------------------L0.1004-----------------------------------------|"
    - "L0.1016[459230,612305] 181ns|----------------------------------------L0.1016-----------------------------------------|"
    - "L0.1028[459230,612305] 182ns|----------------------------------------L0.1028-----------------------------------------|"
    - "L0.1040[459230,612305] 183ns|----------------------------------------L0.1040-----------------------------------------|"
    - "L0.1052[459230,612305] 184ns|----------------------------------------L0.1052-----------------------------------------|"
    - "L0.1065[459230,612305] 185ns|----------------------------------------L0.1065-----------------------------------------|"
    - "L0.1078[459230,612305] 186ns|----------------------------------------L0.1078-----------------------------------------|"
    - "L0.1091[459230,612305] 187ns|----------------------------------------L0.1091-----------------------------------------|"
    - "L0.1104[459230,612305] 188ns|----------------------------------------L0.1104-----------------------------------------|"
    - "L0.1117[459230,612305] 189ns|----------------------------------------L0.1117-----------------------------------------|"
    - "L0.1130[459230,612305] 190ns|----------------------------------------L0.1130-----------------------------------------|"
    - "L0.1143[459230,612305] 191ns|----------------------------------------L0.1143-----------------------------------------|"
    - "L0.1156[459230,612305] 192ns|----------------------------------------L0.1156-----------------------------------------|"
    - "L0.1169[459230,612305] 193ns|----------------------------------------L0.1169-----------------------------------------|"
    - "L0.1182[459230,612305] 194ns|----------------------------------------L0.1182-----------------------------------------|"
    - "L0.1195[459230,612305] 195ns|----------------------------------------L0.1195-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[459230,612305] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.956, L0.968, L0.980, L0.992, L0.1004, L0.1016, L0.1028, L0.1040, L0.1052, L0.1065, L0.1078, L0.1091, L0.1104, L0.1117, L0.1130, L0.1143, L0.1156, L0.1169, L0.1182, L0.1195"
    - "  Creating 1 files"
    - "**** Simulation run 126, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1208[459230,612305] 196ns|----------------------------------------L0.1208-----------------------------------------|"
    - "L0.1221[459230,612305] 197ns|----------------------------------------L0.1221-----------------------------------------|"
    - "L0.1234[459230,612305] 198ns|----------------------------------------L0.1234-----------------------------------------|"
    - "L0.1247[459230,612305] 199ns|----------------------------------------L0.1247-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[459230,612305] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1208, L0.1221, L0.1234, L0.1247"
    - "  Creating 1 files"
    - "**** Simulation run 127, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.209[612306,765381] 96ns 122b|-----------------------------------------L0.209-----------------------------------------|"
    - "L0.216[612306,765381] 97ns 1b|-----------------------------------------L0.216-----------------------------------------|"
    - "L0.223[612306,765381] 98ns 1b|-----------------------------------------L0.223-----------------------------------------|"
    - "L0.230[612306,765381] 99ns 1b|-----------------------------------------L0.230-----------------------------------------|"
    - "L0.237[612306,765381] 100ns 1b|-----------------------------------------L0.237-----------------------------------------|"
    - "L0.244[612306,765381] 101ns 1b|-----------------------------------------L0.244-----------------------------------------|"
    - "L0.251[612306,765381] 102ns 1b|-----------------------------------------L0.251-----------------------------------------|"
    - "L0.258[612306,765381] 103ns 1b|-----------------------------------------L0.258-----------------------------------------|"
    - "L0.265[612306,765381] 104ns 1b|-----------------------------------------L0.265-----------------------------------------|"
    - "L0.272[612306,765381] 105ns 1b|-----------------------------------------L0.272-----------------------------------------|"
    - "L0.279[612306,765381] 106ns 1b|-----------------------------------------L0.279-----------------------------------------|"
    - "L0.286[612306,765381] 107ns 1b|-----------------------------------------L0.286-----------------------------------------|"
    - "L0.293[612306,765381] 108ns 1b|-----------------------------------------L0.293-----------------------------------------|"
    - "L0.301[612306,765381] 109ns 1b|-----------------------------------------L0.301-----------------------------------------|"
    - "L0.309[612306,765381] 110ns 1b|-----------------------------------------L0.309-----------------------------------------|"
    - "L0.317[612306,765381] 111ns 1b|-----------------------------------------L0.317-----------------------------------------|"
    - "L0.325[612306,765381] 112ns 1b|-----------------------------------------L0.325-----------------------------------------|"
    - "L0.333[612306,765381] 113ns 1b|-----------------------------------------L0.333-----------------------------------------|"
    - "L0.341[612306,765381] 114ns 1b|-----------------------------------------L0.341-----------------------------------------|"
    - "L0.349[612306,765381] 115ns 1b|-----------------------------------------L0.349-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[612306,765381] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.209, L0.216, L0.223, L0.230, L0.237, L0.244, L0.251, L0.258, L0.265, L0.272, L0.279, L0.286, L0.293, L0.301, L0.309, L0.317, L0.325, L0.333, L0.341, L0.349"
    - "  Creating 1 files"
    - "**** Simulation run 128, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.357[612306,765381] 116ns|-----------------------------------------L0.357-----------------------------------------|"
    - "L0.365[612306,765381] 117ns|-----------------------------------------L0.365-----------------------------------------|"
    - "L0.373[612306,765381] 118ns|-----------------------------------------L0.373-----------------------------------------|"
    - "L0.381[612306,765381] 119ns|-----------------------------------------L0.381-----------------------------------------|"
    - "L0.389[612306,765381] 120ns|-----------------------------------------L0.389-----------------------------------------|"
    - "L0.397[612306,765381] 121ns|-----------------------------------------L0.397-----------------------------------------|"
    - "L0.405[612306,765381] 122ns|-----------------------------------------L0.405-----------------------------------------|"
    - "L0.413[612306,765381] 123ns|-----------------------------------------L0.413-----------------------------------------|"
    - "L0.422[612306,765381] 124ns|-----------------------------------------L0.422-----------------------------------------|"
    - "L0.431[612306,765381] 125ns|-----------------------------------------L0.431-----------------------------------------|"
    - "L0.440[612306,765381] 126ns|-----------------------------------------L0.440-----------------------------------------|"
    - "L0.449[612306,765381] 127ns|-----------------------------------------L0.449-----------------------------------------|"
    - "L0.458[612306,765381] 128ns|-----------------------------------------L0.458-----------------------------------------|"
    - "L0.467[612306,765381] 129ns|-----------------------------------------L0.467-----------------------------------------|"
    - "L0.476[612306,765381] 130ns|-----------------------------------------L0.476-----------------------------------------|"
    - "L0.485[612306,765381] 131ns|-----------------------------------------L0.485-----------------------------------------|"
    - "L0.494[612306,765381] 132ns|-----------------------------------------L0.494-----------------------------------------|"
    - "L0.503[612306,765381] 133ns|-----------------------------------------L0.503-----------------------------------------|"
    - "L0.512[612306,765381] 134ns|-----------------------------------------L0.512-----------------------------------------|"
    - "L0.521[612306,765381] 135ns|-----------------------------------------L0.521-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[612306,765381] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.357, L0.365, L0.373, L0.381, L0.389, L0.397, L0.405, L0.413, L0.422, L0.431, L0.440, L0.449, L0.458, L0.467, L0.476, L0.485, L0.494, L0.503, L0.512, L0.521"
    - "  Creating 1 files"
    - "**** Simulation run 129, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.530[612306,765381] 136ns 1b|-----------------------------------------L0.530-----------------------------------------|"
    - "L0.539[612306,765381] 137ns 1b|-----------------------------------------L0.539-----------------------------------------|"
    - "L0.548[612306,765381] 138ns 1b|-----------------------------------------L0.548-----------------------------------------|"
    - "L0.558[612306,765381] 139ns 1b|-----------------------------------------L0.558-----------------------------------------|"
    - "L0.568[612306,765381] 140ns 1b|-----------------------------------------L0.568-----------------------------------------|"
    - "L0.578[612306,765381] 141ns 1b|-----------------------------------------L0.578-----------------------------------------|"
    - "L0.588[612306,765381] 142ns 1b|-----------------------------------------L0.588-----------------------------------------|"
    - "L0.598[612306,765381] 143ns 1b|-----------------------------------------L0.598-----------------------------------------|"
    - "L0.608[612306,765381] 144ns 1b|-----------------------------------------L0.608-----------------------------------------|"
    - "L0.618[612306,765381] 145ns 1b|-----------------------------------------L0.618-----------------------------------------|"
    - "L0.628[612306,765381] 146ns 1b|-----------------------------------------L0.628-----------------------------------------|"
    - "L0.638[612306,765381] 147ns 1b|-----------------------------------------L0.638-----------------------------------------|"
    - "L0.648[612306,765381] 148ns 1b|-----------------------------------------L0.648-----------------------------------------|"
    - "L0.658[612306,765381] 149ns 1b|-----------------------------------------L0.658-----------------------------------------|"
    - "L0.668[612306,765381] 150ns 1b|-----------------------------------------L0.668-----------------------------------------|"
    - "L0.678[612306,765381] 151ns 1b|-----------------------------------------L0.678-----------------------------------------|"
    - "L0.688[612306,765381] 152ns 1b|-----------------------------------------L0.688-----------------------------------------|"
    - "L0.698[612306,765381] 153ns 1b|-----------------------------------------L0.698-----------------------------------------|"
    - "L0.708[612306,765381] 154ns 0b|-----------------------------------------L0.708-----------------------------------------|"
    - "L0.719[612306,765381] 155ns 0b|-----------------------------------------L0.719-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[612306,765381] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.530, L0.539, L0.548, L0.558, L0.568, L0.578, L0.588, L0.598, L0.608, L0.618, L0.628, L0.638, L0.648, L0.658, L0.668, L0.678, L0.688, L0.698, L0.708, L0.719"
    - "  Creating 1 files"
    - "**** Simulation run 130, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.730[612306,765381] 156ns|-----------------------------------------L0.730-----------------------------------------|"
    - "L0.741[612306,765381] 157ns|-----------------------------------------L0.741-----------------------------------------|"
    - "L0.752[612306,765381] 158ns|-----------------------------------------L0.752-----------------------------------------|"
    - "L0.763[612306,765381] 159ns|-----------------------------------------L0.763-----------------------------------------|"
    - "L0.774[612306,765381] 160ns|-----------------------------------------L0.774-----------------------------------------|"
    - "L0.785[612306,765381] 161ns|-----------------------------------------L0.785-----------------------------------------|"
    - "L0.796[612306,765381] 162ns|-----------------------------------------L0.796-----------------------------------------|"
    - "L0.807[612306,765381] 163ns|-----------------------------------------L0.807-----------------------------------------|"
    - "L0.818[612306,765381] 164ns|-----------------------------------------L0.818-----------------------------------------|"
    - "L0.829[612306,765381] 165ns|-----------------------------------------L0.829-----------------------------------------|"
    - "L0.840[612306,765381] 166ns|-----------------------------------------L0.840-----------------------------------------|"
    - "L0.851[612306,765381] 167ns|-----------------------------------------L0.851-----------------------------------------|"
    - "L0.862[612306,765381] 168ns|-----------------------------------------L0.862-----------------------------------------|"
    - "L0.873[612306,765381] 169ns|-----------------------------------------L0.873-----------------------------------------|"
    - "L0.885[612306,765381] 170ns|-----------------------------------------L0.885-----------------------------------------|"
    - "L0.897[612306,765381] 171ns|-----------------------------------------L0.897-----------------------------------------|"
    - "L0.909[612306,765381] 172ns|-----------------------------------------L0.909-----------------------------------------|"
    - "L0.921[612306,765381] 173ns|-----------------------------------------L0.921-----------------------------------------|"
    - "L0.933[612306,765381] 174ns|-----------------------------------------L0.933-----------------------------------------|"
    - "L0.945[612306,765381] 175ns|-----------------------------------------L0.945-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[612306,765381] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.730, L0.741, L0.752, L0.763, L0.774, L0.785, L0.796, L0.807, L0.818, L0.829, L0.840, L0.851, L0.862, L0.873, L0.885, L0.897, L0.909, L0.921, L0.933, L0.945"
    - "  Creating 1 files"
    - "**** Simulation run 131, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.957[612306,765381] 176ns|-----------------------------------------L0.957-----------------------------------------|"
    - "L0.969[612306,765381] 177ns|-----------------------------------------L0.969-----------------------------------------|"
    - "L0.981[612306,765381] 178ns|-----------------------------------------L0.981-----------------------------------------|"
    - "L0.993[612306,765381] 179ns|-----------------------------------------L0.993-----------------------------------------|"
    - "L0.1005[612306,765381] 180ns|----------------------------------------L0.1005-----------------------------------------|"
    - "L0.1017[612306,765381] 181ns|----------------------------------------L0.1017-----------------------------------------|"
    - "L0.1029[612306,765381] 182ns|----------------------------------------L0.1029-----------------------------------------|"
    - "L0.1041[612306,765381] 183ns|----------------------------------------L0.1041-----------------------------------------|"
    - "L0.1053[612306,765381] 184ns|----------------------------------------L0.1053-----------------------------------------|"
    - "L0.1066[612306,765381] 185ns|----------------------------------------L0.1066-----------------------------------------|"
    - "L0.1079[612306,765381] 186ns|----------------------------------------L0.1079-----------------------------------------|"
    - "L0.1092[612306,765381] 187ns|----------------------------------------L0.1092-----------------------------------------|"
    - "L0.1105[612306,765381] 188ns|----------------------------------------L0.1105-----------------------------------------|"
    - "L0.1118[612306,765381] 189ns|----------------------------------------L0.1118-----------------------------------------|"
    - "L0.1131[612306,765381] 190ns|----------------------------------------L0.1131-----------------------------------------|"
    - "L0.1144[612306,765381] 191ns|----------------------------------------L0.1144-----------------------------------------|"
    - "L0.1157[612306,765381] 192ns|----------------------------------------L0.1157-----------------------------------------|"
    - "L0.1170[612306,765381] 193ns|----------------------------------------L0.1170-----------------------------------------|"
    - "L0.1183[612306,765381] 194ns|----------------------------------------L0.1183-----------------------------------------|"
    - "L0.1196[612306,765381] 195ns|----------------------------------------L0.1196-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[612306,765381] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.957, L0.969, L0.981, L0.993, L0.1005, L0.1017, L0.1029, L0.1041, L0.1053, L0.1066, L0.1079, L0.1092, L0.1105, L0.1118, L0.1131, L0.1144, L0.1157, L0.1170, L0.1183, L0.1196"
    - "  Creating 1 files"
    - "**** Simulation run 132, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1209[612306,765381] 196ns|----------------------------------------L0.1209-----------------------------------------|"
    - "L0.1222[612306,765381] 197ns|----------------------------------------L0.1222-----------------------------------------|"
    - "L0.1235[612306,765381] 198ns|----------------------------------------L0.1235-----------------------------------------|"
    - "L0.1248[612306,765381] 199ns|----------------------------------------L0.1248-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[612306,765381] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1209, L0.1222, L0.1235, L0.1248"
    - "  Creating 1 files"
    - "**** Simulation run 133, type=compact(ManySmallFiles). 20 Input Files, 141b total:"
    - "L0                                                                                                                 "
    - "L0.210[765382,918457] 96ns 122b|-----------------------------------------L0.210-----------------------------------------|"
    - "L0.217[765382,918457] 97ns 1b|-----------------------------------------L0.217-----------------------------------------|"
    - "L0.224[765382,918457] 98ns 1b|-----------------------------------------L0.224-----------------------------------------|"
    - "L0.231[765382,918457] 99ns 1b|-----------------------------------------L0.231-----------------------------------------|"
    - "L0.238[765382,918457] 100ns 1b|-----------------------------------------L0.238-----------------------------------------|"
    - "L0.245[765382,918457] 101ns 1b|-----------------------------------------L0.245-----------------------------------------|"
    - "L0.252[765382,918457] 102ns 1b|-----------------------------------------L0.252-----------------------------------------|"
    - "L0.259[765382,918457] 103ns 1b|-----------------------------------------L0.259-----------------------------------------|"
    - "L0.266[765382,918457] 104ns 1b|-----------------------------------------L0.266-----------------------------------------|"
    - "L0.273[765382,918457] 105ns 1b|-----------------------------------------L0.273-----------------------------------------|"
    - "L0.280[765382,918457] 106ns 1b|-----------------------------------------L0.280-----------------------------------------|"
    - "L0.287[765382,918457] 107ns 1b|-----------------------------------------L0.287-----------------------------------------|"
    - "L0.294[765382,918457] 108ns 1b|-----------------------------------------L0.294-----------------------------------------|"
    - "L0.302[765382,918457] 109ns 1b|-----------------------------------------L0.302-----------------------------------------|"
    - "L0.310[765382,918457] 110ns 1b|-----------------------------------------L0.310-----------------------------------------|"
    - "L0.318[765382,918457] 111ns 1b|-----------------------------------------L0.318-----------------------------------------|"
    - "L0.326[765382,918457] 112ns 1b|-----------------------------------------L0.326-----------------------------------------|"
    - "L0.334[765382,918457] 113ns 1b|-----------------------------------------L0.334-----------------------------------------|"
    - "L0.342[765382,918457] 114ns 1b|-----------------------------------------L0.342-----------------------------------------|"
    - "L0.350[765382,918457] 115ns 1b|-----------------------------------------L0.350-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 141b total:"
    - "L0, all files 141b                                                                                                 "
    - "L0.?[765382,918457] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.210, L0.217, L0.224, L0.231, L0.238, L0.245, L0.252, L0.259, L0.266, L0.273, L0.280, L0.287, L0.294, L0.302, L0.310, L0.318, L0.326, L0.334, L0.342, L0.350"
    - "  Creating 1 files"
    - "**** Simulation run 134, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.358[765382,918457] 116ns|-----------------------------------------L0.358-----------------------------------------|"
    - "L0.366[765382,918457] 117ns|-----------------------------------------L0.366-----------------------------------------|"
    - "L0.374[765382,918457] 118ns|-----------------------------------------L0.374-----------------------------------------|"
    - "L0.382[765382,918457] 119ns|-----------------------------------------L0.382-----------------------------------------|"
    - "L0.390[765382,918457] 120ns|-----------------------------------------L0.390-----------------------------------------|"
    - "L0.398[765382,918457] 121ns|-----------------------------------------L0.398-----------------------------------------|"
    - "L0.406[765382,918457] 122ns|-----------------------------------------L0.406-----------------------------------------|"
    - "L0.414[765382,918457] 123ns|-----------------------------------------L0.414-----------------------------------------|"
    - "L0.423[765382,918457] 124ns|-----------------------------------------L0.423-----------------------------------------|"
    - "L0.432[765382,918457] 125ns|-----------------------------------------L0.432-----------------------------------------|"
    - "L0.441[765382,918457] 126ns|-----------------------------------------L0.441-----------------------------------------|"
    - "L0.450[765382,918457] 127ns|-----------------------------------------L0.450-----------------------------------------|"
    - "L0.459[765382,918457] 128ns|-----------------------------------------L0.459-----------------------------------------|"
    - "L0.468[765382,918457] 129ns|-----------------------------------------L0.468-----------------------------------------|"
    - "L0.477[765382,918457] 130ns|-----------------------------------------L0.477-----------------------------------------|"
    - "L0.486[765382,918457] 131ns|-----------------------------------------L0.486-----------------------------------------|"
    - "L0.495[765382,918457] 132ns|-----------------------------------------L0.495-----------------------------------------|"
    - "L0.504[765382,918457] 133ns|-----------------------------------------L0.504-----------------------------------------|"
    - "L0.513[765382,918457] 134ns|-----------------------------------------L0.513-----------------------------------------|"
    - "L0.522[765382,918457] 135ns|-----------------------------------------L0.522-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[765382,918457] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.358, L0.366, L0.374, L0.382, L0.390, L0.398, L0.406, L0.414, L0.423, L0.432, L0.441, L0.450, L0.459, L0.468, L0.477, L0.486, L0.495, L0.504, L0.513, L0.522"
    - "  Creating 1 files"
    - "**** Simulation run 135, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.531[765382,918457] 136ns 1b|-----------------------------------------L0.531-----------------------------------------|"
    - "L0.540[765382,918457] 137ns 1b|-----------------------------------------L0.540-----------------------------------------|"
    - "L0.549[765382,918457] 138ns 1b|-----------------------------------------L0.549-----------------------------------------|"
    - "L0.559[765382,918457] 139ns 1b|-----------------------------------------L0.559-----------------------------------------|"
    - "L0.569[765382,918457] 140ns 1b|-----------------------------------------L0.569-----------------------------------------|"
    - "L0.579[765382,918457] 141ns 1b|-----------------------------------------L0.579-----------------------------------------|"
    - "L0.589[765382,918457] 142ns 1b|-----------------------------------------L0.589-----------------------------------------|"
    - "L0.599[765382,918457] 143ns 1b|-----------------------------------------L0.599-----------------------------------------|"
    - "L0.609[765382,918457] 144ns 1b|-----------------------------------------L0.609-----------------------------------------|"
    - "L0.619[765382,918457] 145ns 1b|-----------------------------------------L0.619-----------------------------------------|"
    - "L0.629[765382,918457] 146ns 1b|-----------------------------------------L0.629-----------------------------------------|"
    - "L0.639[765382,918457] 147ns 1b|-----------------------------------------L0.639-----------------------------------------|"
    - "L0.649[765382,918457] 148ns 1b|-----------------------------------------L0.649-----------------------------------------|"
    - "L0.659[765382,918457] 149ns 1b|-----------------------------------------L0.659-----------------------------------------|"
    - "L0.669[765382,918457] 150ns 1b|-----------------------------------------L0.669-----------------------------------------|"
    - "L0.679[765382,918457] 151ns 1b|-----------------------------------------L0.679-----------------------------------------|"
    - "L0.689[765382,918457] 152ns 1b|-----------------------------------------L0.689-----------------------------------------|"
    - "L0.699[765382,918457] 153ns 1b|-----------------------------------------L0.699-----------------------------------------|"
    - "L0.709[765382,918457] 154ns 0b|-----------------------------------------L0.709-----------------------------------------|"
    - "L0.720[765382,918457] 155ns 0b|-----------------------------------------L0.720-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[765382,918457] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.531, L0.540, L0.549, L0.559, L0.569, L0.579, L0.589, L0.599, L0.609, L0.619, L0.629, L0.639, L0.649, L0.659, L0.669, L0.679, L0.689, L0.699, L0.709, L0.720"
    - "  Creating 1 files"
    - "**** Simulation run 136, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.958[765382,918457] 176ns|-----------------------------------------L0.958-----------------------------------------|"
    - "L0.970[765382,918457] 177ns|-----------------------------------------L0.970-----------------------------------------|"
    - "L0.982[765382,918457] 178ns|-----------------------------------------L0.982-----------------------------------------|"
    - "L0.994[765382,918457] 179ns|-----------------------------------------L0.994-----------------------------------------|"
    - "L0.1006[765382,918457] 180ns|----------------------------------------L0.1006-----------------------------------------|"
    - "L0.1018[765382,918457] 181ns|----------------------------------------L0.1018-----------------------------------------|"
    - "L0.1030[765382,918457] 182ns|----------------------------------------L0.1030-----------------------------------------|"
    - "L0.1042[765382,918457] 183ns|----------------------------------------L0.1042-----------------------------------------|"
    - "L0.1054[765382,918457] 184ns|----------------------------------------L0.1054-----------------------------------------|"
    - "L0.1067[765382,918457] 185ns|----------------------------------------L0.1067-----------------------------------------|"
    - "L0.1080[765382,918457] 186ns|----------------------------------------L0.1080-----------------------------------------|"
    - "L0.1093[765382,918457] 187ns|----------------------------------------L0.1093-----------------------------------------|"
    - "L0.1106[765382,918457] 188ns|----------------------------------------L0.1106-----------------------------------------|"
    - "L0.1119[765382,918457] 189ns|----------------------------------------L0.1119-----------------------------------------|"
    - "L0.1132[765382,918457] 190ns|----------------------------------------L0.1132-----------------------------------------|"
    - "L0.1145[765382,918457] 191ns|----------------------------------------L0.1145-----------------------------------------|"
    - "L0.1158[765382,918457] 192ns|----------------------------------------L0.1158-----------------------------------------|"
    - "L0.1171[765382,918457] 193ns|----------------------------------------L0.1171-----------------------------------------|"
    - "L0.1184[765382,918457] 194ns|----------------------------------------L0.1184-----------------------------------------|"
    - "L0.1197[765382,918457] 195ns|----------------------------------------L0.1197-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[765382,918457] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.958, L0.970, L0.982, L0.994, L0.1006, L0.1018, L0.1030, L0.1042, L0.1054, L0.1067, L0.1080, L0.1093, L0.1106, L0.1119, L0.1132, L0.1145, L0.1158, L0.1171, L0.1184, L0.1197"
    - "  Creating 1 files"
    - "**** Simulation run 137, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1210[765382,918457] 196ns|----------------------------------------L0.1210-----------------------------------------|"
    - "L0.1223[765382,918457] 197ns|----------------------------------------L0.1223-----------------------------------------|"
    - "L0.1236[765382,918457] 198ns|----------------------------------------L0.1236-----------------------------------------|"
    - "L0.1249[765382,918457] 199ns|----------------------------------------L0.1249-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[765382,918457] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1210, L0.1223, L0.1236, L0.1249"
    - "  Creating 1 files"
    - "**** Simulation run 138, type=compact(ManySmallFiles). 20 Input Files, 90b total:"
    - "L0                                                                                                                 "
    - "L0.211[918458,960000] 96ns 38b|--------L0.211--------|                                                                  "
    - "L0.218[918458,970000] 97ns 4b|-----------L0.218-----------|                                                            "
    - "L0.225[918458,980000] 98ns 4b|--------------L0.225--------------|                                                      "
    - "L0.232[918458,990000] 99ns 4b|-----------------L0.232-----------------|                                                "
    - "L0.239[918458,1000000] 100ns 4b|-------------------L0.239--------------------|                                           "
    - "L0.246[918458,1010000] 101ns 4b|----------------------L0.246-----------------------|                                     "
    - "L0.253[918458,1020000] 102ns 4b|-------------------------L0.253--------------------------|                               "
    - "L0.260[918458,1030000] 103ns 4b|----------------------------L0.260-----------------------------|                         "
    - "L0.267[918458,1040000] 104ns 4b|-------------------------------L0.267--------------------------------|                   "
    - "L0.274[918458,1050000] 105ns 4b|----------------------------------L0.274-----------------------------------|             "
    - "L0.281[918458,1060000] 106ns 4b|-------------------------------------L0.281--------------------------------------|       "
    - "L0.288[918458,1070000] 107ns 4b|----------------------------------------L0.288-----------------------------------------| "
    - "L0.295[918458,1071533] 108ns 1b|-----------------------------------------L0.295-----------------------------------------|"
    - "L0.303[918458,1071533] 109ns 1b|-----------------------------------------L0.303-----------------------------------------|"
    - "L0.311[918458,1071533] 110ns 1b|-----------------------------------------L0.311-----------------------------------------|"
    - "L0.319[918458,1071533] 111ns 1b|-----------------------------------------L0.319-----------------------------------------|"
    - "L0.327[918458,1071533] 112ns 1b|-----------------------------------------L0.327-----------------------------------------|"
    - "L0.335[918458,1071533] 113ns 1b|-----------------------------------------L0.335-----------------------------------------|"
    - "L0.343[918458,1071533] 114ns 1b|-----------------------------------------L0.343-----------------------------------------|"
    - "L0.351[918458,1071533] 115ns 1b|-----------------------------------------L0.351-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 90b total:"
    - "L0, all files 90b                                                                                                  "
    - "L0.?[918458,1071533] 115ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.211, L0.218, L0.225, L0.232, L0.239, L0.246, L0.253, L0.260, L0.267, L0.274, L0.281, L0.288, L0.295, L0.303, L0.311, L0.319, L0.327, L0.335, L0.343, L0.351"
    - "  Creating 1 files"
    - "**** Simulation run 139, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.359[918458,1071533] 116ns|-----------------------------------------L0.359-----------------------------------------|"
    - "L0.367[918458,1071533] 117ns|-----------------------------------------L0.367-----------------------------------------|"
    - "L0.375[918458,1071533] 118ns|-----------------------------------------L0.375-----------------------------------------|"
    - "L0.383[918458,1071533] 119ns|-----------------------------------------L0.383-----------------------------------------|"
    - "L0.391[918458,1071533] 120ns|-----------------------------------------L0.391-----------------------------------------|"
    - "L0.399[918458,1071533] 121ns|-----------------------------------------L0.399-----------------------------------------|"
    - "L0.407[918458,1071533] 122ns|-----------------------------------------L0.407-----------------------------------------|"
    - "L0.415[918458,1071533] 123ns|-----------------------------------------L0.415-----------------------------------------|"
    - "L0.424[918458,1071533] 124ns|-----------------------------------------L0.424-----------------------------------------|"
    - "L0.433[918458,1071533] 125ns|-----------------------------------------L0.433-----------------------------------------|"
    - "L0.442[918458,1071533] 126ns|-----------------------------------------L0.442-----------------------------------------|"
    - "L0.451[918458,1071533] 127ns|-----------------------------------------L0.451-----------------------------------------|"
    - "L0.460[918458,1071533] 128ns|-----------------------------------------L0.460-----------------------------------------|"
    - "L0.469[918458,1071533] 129ns|-----------------------------------------L0.469-----------------------------------------|"
    - "L0.478[918458,1071533] 130ns|-----------------------------------------L0.478-----------------------------------------|"
    - "L0.487[918458,1071533] 131ns|-----------------------------------------L0.487-----------------------------------------|"
    - "L0.496[918458,1071533] 132ns|-----------------------------------------L0.496-----------------------------------------|"
    - "L0.505[918458,1071533] 133ns|-----------------------------------------L0.505-----------------------------------------|"
    - "L0.514[918458,1071533] 134ns|-----------------------------------------L0.514-----------------------------------------|"
    - "L0.523[918458,1071533] 135ns|-----------------------------------------L0.523-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[918458,1071533] 135ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.359, L0.367, L0.375, L0.383, L0.391, L0.399, L0.407, L0.415, L0.424, L0.433, L0.442, L0.451, L0.460, L0.469, L0.478, L0.487, L0.496, L0.505, L0.514, L0.523"
    - "  Creating 1 files"
    - "**** Simulation run 140, type=compact(ManySmallFiles). 20 Input Files, 18b total:"
    - "L0                                                                                                                 "
    - "L0.532[918458,1071533] 136ns 1b|-----------------------------------------L0.532-----------------------------------------|"
    - "L0.541[918458,1071533] 137ns 1b|-----------------------------------------L0.541-----------------------------------------|"
    - "L0.550[918458,1071533] 138ns 1b|-----------------------------------------L0.550-----------------------------------------|"
    - "L0.560[918458,1071533] 139ns 1b|-----------------------------------------L0.560-----------------------------------------|"
    - "L0.570[918458,1071533] 140ns 1b|-----------------------------------------L0.570-----------------------------------------|"
    - "L0.580[918458,1071533] 141ns 1b|-----------------------------------------L0.580-----------------------------------------|"
    - "L0.590[918458,1071533] 142ns 1b|-----------------------------------------L0.590-----------------------------------------|"
    - "L0.600[918458,1071533] 143ns 1b|-----------------------------------------L0.600-----------------------------------------|"
    - "L0.610[918458,1071533] 144ns 1b|-----------------------------------------L0.610-----------------------------------------|"
    - "L0.620[918458,1071533] 145ns 1b|-----------------------------------------L0.620-----------------------------------------|"
    - "L0.630[918458,1071533] 146ns 1b|-----------------------------------------L0.630-----------------------------------------|"
    - "L0.640[918458,1071533] 147ns 1b|-----------------------------------------L0.640-----------------------------------------|"
    - "L0.650[918458,1071533] 148ns 1b|-----------------------------------------L0.650-----------------------------------------|"
    - "L0.660[918458,1071533] 149ns 1b|-----------------------------------------L0.660-----------------------------------------|"
    - "L0.670[918458,1071533] 150ns 1b|-----------------------------------------L0.670-----------------------------------------|"
    - "L0.680[918458,1071533] 151ns 1b|-----------------------------------------L0.680-----------------------------------------|"
    - "L0.690[918458,1071533] 152ns 1b|-----------------------------------------L0.690-----------------------------------------|"
    - "L0.700[918458,1071533] 153ns 1b|-----------------------------------------L0.700-----------------------------------------|"
    - "L0.710[918458,1071533] 154ns 0b|-----------------------------------------L0.710-----------------------------------------|"
    - "L0.721[918458,1071533] 155ns 0b|-----------------------------------------L0.721-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 18b total:"
    - "L0, all files 18b                                                                                                  "
    - "L0.?[918458,1071533] 155ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.532, L0.541, L0.550, L0.560, L0.570, L0.580, L0.590, L0.600, L0.610, L0.620, L0.630, L0.640, L0.650, L0.660, L0.670, L0.680, L0.690, L0.700, L0.710, L0.721"
    - "  Creating 1 files"
    - "**** Simulation run 141, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.732[918458,1071533] 156ns|-----------------------------------------L0.732-----------------------------------------|"
    - "L0.743[918458,1071533] 157ns|-----------------------------------------L0.743-----------------------------------------|"
    - "L0.754[918458,1071533] 158ns|-----------------------------------------L0.754-----------------------------------------|"
    - "L0.765[918458,1071533] 159ns|-----------------------------------------L0.765-----------------------------------------|"
    - "L0.776[918458,1071533] 160ns|-----------------------------------------L0.776-----------------------------------------|"
    - "L0.787[918458,1071533] 161ns|-----------------------------------------L0.787-----------------------------------------|"
    - "L0.798[918458,1071533] 162ns|-----------------------------------------L0.798-----------------------------------------|"
    - "L0.809[918458,1071533] 163ns|-----------------------------------------L0.809-----------------------------------------|"
    - "L0.820[918458,1071533] 164ns|-----------------------------------------L0.820-----------------------------------------|"
    - "L0.831[918458,1071533] 165ns|-----------------------------------------L0.831-----------------------------------------|"
    - "L0.842[918458,1071533] 166ns|-----------------------------------------L0.842-----------------------------------------|"
    - "L0.853[918458,1071533] 167ns|-----------------------------------------L0.853-----------------------------------------|"
    - "L0.864[918458,1071533] 168ns|-----------------------------------------L0.864-----------------------------------------|"
    - "L0.875[918458,1071533] 169ns|-----------------------------------------L0.875-----------------------------------------|"
    - "L0.887[918458,1071533] 170ns|-----------------------------------------L0.887-----------------------------------------|"
    - "L0.899[918458,1071533] 171ns|-----------------------------------------L0.899-----------------------------------------|"
    - "L0.911[918458,1071533] 172ns|-----------------------------------------L0.911-----------------------------------------|"
    - "L0.923[918458,1071533] 173ns|-----------------------------------------L0.923-----------------------------------------|"
    - "L0.935[918458,1071533] 174ns|-----------------------------------------L0.935-----------------------------------------|"
    - "L0.947[918458,1071533] 175ns|-----------------------------------------L0.947-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[918458,1071533] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.732, L0.743, L0.754, L0.765, L0.776, L0.787, L0.798, L0.809, L0.820, L0.831, L0.842, L0.853, L0.864, L0.875, L0.887, L0.899, L0.911, L0.923, L0.935, L0.947"
    - "  Creating 1 files"
    - "**** Simulation run 142, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.959[918458,1071533] 176ns|-----------------------------------------L0.959-----------------------------------------|"
    - "L0.971[918458,1071533] 177ns|-----------------------------------------L0.971-----------------------------------------|"
    - "L0.983[918458,1071533] 178ns|-----------------------------------------L0.983-----------------------------------------|"
    - "L0.995[918458,1071533] 179ns|-----------------------------------------L0.995-----------------------------------------|"
    - "L0.1007[918458,1071533] 180ns|----------------------------------------L0.1007-----------------------------------------|"
    - "L0.1019[918458,1071533] 181ns|----------------------------------------L0.1019-----------------------------------------|"
    - "L0.1031[918458,1071533] 182ns|----------------------------------------L0.1031-----------------------------------------|"
    - "L0.1043[918458,1071533] 183ns|----------------------------------------L0.1043-----------------------------------------|"
    - "L0.1055[918458,1071533] 184ns|----------------------------------------L0.1055-----------------------------------------|"
    - "L0.1068[918458,1071533] 185ns|----------------------------------------L0.1068-----------------------------------------|"
    - "L0.1081[918458,1071533] 186ns|----------------------------------------L0.1081-----------------------------------------|"
    - "L0.1094[918458,1071533] 187ns|----------------------------------------L0.1094-----------------------------------------|"
    - "L0.1107[918458,1071533] 188ns|----------------------------------------L0.1107-----------------------------------------|"
    - "L0.1120[918458,1071533] 189ns|----------------------------------------L0.1120-----------------------------------------|"
    - "L0.1133[918458,1071533] 190ns|----------------------------------------L0.1133-----------------------------------------|"
    - "L0.1146[918458,1071533] 191ns|----------------------------------------L0.1146-----------------------------------------|"
    - "L0.1159[918458,1071533] 192ns|----------------------------------------L0.1159-----------------------------------------|"
    - "L0.1172[918458,1071533] 193ns|----------------------------------------L0.1172-----------------------------------------|"
    - "L0.1185[918458,1071533] 194ns|----------------------------------------L0.1185-----------------------------------------|"
    - "L0.1198[918458,1071533] 195ns|----------------------------------------L0.1198-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[918458,1071533] 195ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.959, L0.971, L0.983, L0.995, L0.1007, L0.1019, L0.1031, L0.1043, L0.1055, L0.1068, L0.1081, L0.1094, L0.1107, L0.1120, L0.1133, L0.1146, L0.1159, L0.1172, L0.1185, L0.1198"
    - "  Creating 1 files"
    - "**** Simulation run 143, type=compact(ManySmallFiles). 4 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1211[918458,1071533] 196ns|----------------------------------------L0.1211-----------------------------------------|"
    - "L0.1224[918458,1071533] 197ns|----------------------------------------L0.1224-----------------------------------------|"
    - "L0.1237[918458,1071533] 198ns|----------------------------------------L0.1237-----------------------------------------|"
    - "L0.1250[918458,1071533] 199ns|----------------------------------------L0.1250-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[918458,1071533] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1211, L0.1224, L0.1237, L0.1250"
    - "  Creating 1 files"
    - "**** Simulation run 144, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.731[765382,918457] 156ns|-----------------------------------------L0.731-----------------------------------------|"
    - "L0.742[765382,918457] 157ns|-----------------------------------------L0.742-----------------------------------------|"
    - "L0.753[765382,918457] 158ns|-----------------------------------------L0.753-----------------------------------------|"
    - "L0.764[765382,918457] 159ns|-----------------------------------------L0.764-----------------------------------------|"
    - "L0.775[765382,918457] 160ns|-----------------------------------------L0.775-----------------------------------------|"
    - "L0.786[765382,918457] 161ns|-----------------------------------------L0.786-----------------------------------------|"
    - "L0.797[765382,918457] 162ns|-----------------------------------------L0.797-----------------------------------------|"
    - "L0.808[765382,918457] 163ns|-----------------------------------------L0.808-----------------------------------------|"
    - "L0.819[765382,918457] 164ns|-----------------------------------------L0.819-----------------------------------------|"
    - "L0.830[765382,918457] 165ns|-----------------------------------------L0.830-----------------------------------------|"
    - "L0.841[765382,918457] 166ns|-----------------------------------------L0.841-----------------------------------------|"
    - "L0.852[765382,918457] 167ns|-----------------------------------------L0.852-----------------------------------------|"
    - "L0.863[765382,918457] 168ns|-----------------------------------------L0.863-----------------------------------------|"
    - "L0.874[765382,918457] 169ns|-----------------------------------------L0.874-----------------------------------------|"
    - "L0.886[765382,918457] 170ns|-----------------------------------------L0.886-----------------------------------------|"
    - "L0.898[765382,918457] 171ns|-----------------------------------------L0.898-----------------------------------------|"
    - "L0.910[765382,918457] 172ns|-----------------------------------------L0.910-----------------------------------------|"
    - "L0.922[765382,918457] 173ns|-----------------------------------------L0.922-----------------------------------------|"
    - "L0.934[765382,918457] 174ns|-----------------------------------------L0.934-----------------------------------------|"
    - "L0.946[765382,918457] 175ns|-----------------------------------------L0.946-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[765382,918457] 175ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.731, L0.742, L0.753, L0.764, L0.775, L0.786, L0.797, L0.808, L0.819, L0.830, L0.841, L0.852, L0.863, L0.874, L0.886, L0.898, L0.910, L0.922, L0.934, L0.946"
    - "  Creating 1 files"
    - "**** Simulation run 145, type=compact(ManySmallFiles). 20 Input Files, 50b total:"
    - "L0                                                                                                                 "
    - "L0.296[1071534,1080000] 108ns 3b|L0.296|                                                                                  "
    - "L0.304[1071534,1090000] 109ns 3b|-L0.304-|                                                                                "
    - "L0.312[1071534,1100000] 110ns 3b|----L0.312----|                                                                          "
    - "L0.320[1071534,1110000] 111ns 3b|-------L0.320-------|                                                                    "
    - "L0.328[1071534,1120000] 112ns 3b|----------L0.328----------|                                                              "
    - "L0.336[1071534,1130000] 113ns 3b|-------------L0.336-------------|                                                        "
    - "L0.344[1071534,1140000] 114ns 3b|----------------L0.344----------------|                                                  "
    - "L0.352[1071534,1150000] 115ns 3b|-------------------L0.352-------------------|                                            "
    - "L0.360[1071534,1160000] 116ns 3b|----------------------L0.360----------------------|                                      "
    - "L0.368[1071534,1170000] 117ns 3b|------------------------L0.368-------------------------|                                 "
    - "L0.376[1071534,1180000] 118ns 3b|---------------------------L0.376----------------------------|                           "
    - "L0.384[1071534,1190000] 119ns 3b|------------------------------L0.384-------------------------------|                     "
    - "L0.392[1071534,1200000] 120ns 3b|---------------------------------L0.392----------------------------------|               "
    - "L0.400[1071534,1210000] 121ns 3b|------------------------------------L0.400-------------------------------------|         "
    - "L0.408[1071534,1220000] 122ns 3b|---------------------------------------L0.408----------------------------------------|   "
    - "L0.416[1071534,1224609] 123ns 1b|-----------------------------------------L0.416-----------------------------------------|"
    - "L0.425[1071534,1224609] 124ns 1b|-----------------------------------------L0.425-----------------------------------------|"
    - "L0.434[1071534,1224609] 125ns 1b|-----------------------------------------L0.434-----------------------------------------|"
    - "L0.443[1071534,1224609] 126ns 1b|-----------------------------------------L0.443-----------------------------------------|"
    - "L0.452[1071534,1224609] 127ns 1b|-----------------------------------------L0.452-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 50b total:"
    - "L0, all files 50b                                                                                                  "
    - "L0.?[1071534,1224609] 127ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.296, L0.304, L0.312, L0.320, L0.328, L0.336, L0.344, L0.352, L0.360, L0.368, L0.376, L0.384, L0.392, L0.400, L0.408, L0.416, L0.425, L0.434, L0.443, L0.452"
    - "  Creating 1 files"
    - "**** Simulation run 146, type=compact(ManySmallFiles). 20 Input Files, 20b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.461[1071534,1224609] 128ns|-----------------------------------------L0.461-----------------------------------------|"
    - "L0.470[1071534,1224609] 129ns|-----------------------------------------L0.470-----------------------------------------|"
    - "L0.479[1071534,1224609] 130ns|-----------------------------------------L0.479-----------------------------------------|"
    - "L0.488[1071534,1224609] 131ns|-----------------------------------------L0.488-----------------------------------------|"
    - "L0.497[1071534,1224609] 132ns|-----------------------------------------L0.497-----------------------------------------|"
    - "L0.506[1071534,1224609] 133ns|-----------------------------------------L0.506-----------------------------------------|"
    - "L0.515[1071534,1224609] 134ns|-----------------------------------------L0.515-----------------------------------------|"
    - "L0.524[1071534,1224609] 135ns|-----------------------------------------L0.524-----------------------------------------|"
    - "L0.533[1071534,1224609] 136ns|-----------------------------------------L0.533-----------------------------------------|"
    - "L0.542[1071534,1224609] 137ns|-----------------------------------------L0.542-----------------------------------------|"
    - "L0.551[1071534,1224609] 138ns|-----------------------------------------L0.551-----------------------------------------|"
    - "L0.561[1071534,1224609] 139ns|-----------------------------------------L0.561-----------------------------------------|"
    - "L0.571[1071534,1224609] 140ns|-----------------------------------------L0.571-----------------------------------------|"
    - "L0.581[1071534,1224609] 141ns|-----------------------------------------L0.581-----------------------------------------|"
    - "L0.591[1071534,1224609] 142ns|-----------------------------------------L0.591-----------------------------------------|"
    - "L0.601[1071534,1224609] 143ns|-----------------------------------------L0.601-----------------------------------------|"
    - "L0.611[1071534,1224609] 144ns|-----------------------------------------L0.611-----------------------------------------|"
    - "L0.621[1071534,1224609] 145ns|-----------------------------------------L0.621-----------------------------------------|"
    - "L0.631[1071534,1224609] 146ns|-----------------------------------------L0.631-----------------------------------------|"
    - "L0.641[1071534,1224609] 147ns|-----------------------------------------L0.641-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 20b total:"
    - "L0, all files 20b                                                                                                  "
    - "L0.?[1071534,1224609] 147ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.461, L0.470, L0.479, L0.488, L0.497, L0.506, L0.515, L0.524, L0.533, L0.542, L0.551, L0.561, L0.571, L0.581, L0.591, L0.601, L0.611, L0.621, L0.631, L0.641"
    - "  Creating 1 files"
    - "**** Simulation run 147, type=compact(ManySmallFiles). 20 Input Files, 6b total:"
    - "L0                                                                                                                 "
    - "L0.651[1071534,1224609] 148ns 1b|-----------------------------------------L0.651-----------------------------------------|"
    - "L0.661[1071534,1224609] 149ns 1b|-----------------------------------------L0.661-----------------------------------------|"
    - "L0.671[1071534,1224609] 150ns 1b|-----------------------------------------L0.671-----------------------------------------|"
    - "L0.681[1071534,1224609] 151ns 1b|-----------------------------------------L0.681-----------------------------------------|"
    - "L0.691[1071534,1224609] 152ns 1b|-----------------------------------------L0.691-----------------------------------------|"
    - "L0.701[1071534,1224609] 153ns 1b|-----------------------------------------L0.701-----------------------------------------|"
    - "L0.711[1071534,1224609] 154ns 0b|-----------------------------------------L0.711-----------------------------------------|"
    - "L0.722[1071534,1224609] 155ns 0b|-----------------------------------------L0.722-----------------------------------------|"
    - "L0.733[1071534,1224609] 156ns 0b|-----------------------------------------L0.733-----------------------------------------|"
    - "L0.744[1071534,1224609] 157ns 0b|-----------------------------------------L0.744-----------------------------------------|"
    - "L0.755[1071534,1224609] 158ns 0b|-----------------------------------------L0.755-----------------------------------------|"
    - "L0.766[1071534,1224609] 159ns 0b|-----------------------------------------L0.766-----------------------------------------|"
    - "L0.777[1071534,1224609] 160ns 0b|-----------------------------------------L0.777-----------------------------------------|"
    - "L0.788[1071534,1224609] 161ns 0b|-----------------------------------------L0.788-----------------------------------------|"
    - "L0.799[1071534,1224609] 162ns 0b|-----------------------------------------L0.799-----------------------------------------|"
    - "L0.810[1071534,1224609] 163ns 0b|-----------------------------------------L0.810-----------------------------------------|"
    - "L0.821[1071534,1224609] 164ns 0b|-----------------------------------------L0.821-----------------------------------------|"
    - "L0.832[1071534,1224609] 165ns 0b|-----------------------------------------L0.832-----------------------------------------|"
    - "L0.843[1071534,1224609] 166ns 0b|-----------------------------------------L0.843-----------------------------------------|"
    - "L0.854[1071534,1224609] 167ns 0b|-----------------------------------------L0.854-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 6b total:"
    - "L0, all files 6b                                                                                                   "
    - "L0.?[1071534,1224609] 167ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.651, L0.661, L0.671, L0.681, L0.691, L0.701, L0.711, L0.722, L0.733, L0.744, L0.755, L0.766, L0.777, L0.788, L0.799, L0.810, L0.821, L0.832, L0.843, L0.854"
    - "  Creating 1 files"
    - "**** Simulation run 148, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.865[1071534,1224609] 168ns|-----------------------------------------L0.865-----------------------------------------|"
    - "L0.876[1071534,1224609] 169ns|-----------------------------------------L0.876-----------------------------------------|"
    - "L0.888[1071534,1224609] 170ns|-----------------------------------------L0.888-----------------------------------------|"
    - "L0.900[1071534,1224609] 171ns|-----------------------------------------L0.900-----------------------------------------|"
    - "L0.912[1071534,1224609] 172ns|-----------------------------------------L0.912-----------------------------------------|"
    - "L0.924[1071534,1224609] 173ns|-----------------------------------------L0.924-----------------------------------------|"
    - "L0.936[1071534,1224609] 174ns|-----------------------------------------L0.936-----------------------------------------|"
    - "L0.948[1071534,1224609] 175ns|-----------------------------------------L0.948-----------------------------------------|"
    - "L0.960[1071534,1224609] 176ns|-----------------------------------------L0.960-----------------------------------------|"
    - "L0.972[1071534,1224609] 177ns|-----------------------------------------L0.972-----------------------------------------|"
    - "L0.984[1071534,1224609] 178ns|-----------------------------------------L0.984-----------------------------------------|"
    - "L0.996[1071534,1224609] 179ns|-----------------------------------------L0.996-----------------------------------------|"
    - "L0.1008[1071534,1224609] 180ns|----------------------------------------L0.1008-----------------------------------------|"
    - "L0.1020[1071534,1224609] 181ns|----------------------------------------L0.1020-----------------------------------------|"
    - "L0.1032[1071534,1224609] 182ns|----------------------------------------L0.1032-----------------------------------------|"
    - "L0.1044[1071534,1224609] 183ns|----------------------------------------L0.1044-----------------------------------------|"
    - "L0.1056[1071534,1224609] 184ns|----------------------------------------L0.1056-----------------------------------------|"
    - "L0.1069[1071534,1224609] 185ns|----------------------------------------L0.1069-----------------------------------------|"
    - "L0.1082[1071534,1224609] 186ns|----------------------------------------L0.1082-----------------------------------------|"
    - "L0.1095[1071534,1224609] 187ns|----------------------------------------L0.1095-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1071534,1224609] 187ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.865, L0.876, L0.888, L0.900, L0.912, L0.924, L0.936, L0.948, L0.960, L0.972, L0.984, L0.996, L0.1008, L0.1020, L0.1032, L0.1044, L0.1056, L0.1069, L0.1082, L0.1095"
    - "  Creating 1 files"
    - "**** Simulation run 149, type=compact(ManySmallFiles). 12 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1108[1071534,1224609] 188ns|----------------------------------------L0.1108-----------------------------------------|"
    - "L0.1121[1071534,1224609] 189ns|----------------------------------------L0.1121-----------------------------------------|"
    - "L0.1134[1071534,1224609] 190ns|----------------------------------------L0.1134-----------------------------------------|"
    - "L0.1147[1071534,1224609] 191ns|----------------------------------------L0.1147-----------------------------------------|"
    - "L0.1160[1071534,1224609] 192ns|----------------------------------------L0.1160-----------------------------------------|"
    - "L0.1173[1071534,1224609] 193ns|----------------------------------------L0.1173-----------------------------------------|"
    - "L0.1186[1071534,1224609] 194ns|----------------------------------------L0.1186-----------------------------------------|"
    - "L0.1199[1071534,1224609] 195ns|----------------------------------------L0.1199-----------------------------------------|"
    - "L0.1212[1071534,1224609] 196ns|----------------------------------------L0.1212-----------------------------------------|"
    - "L0.1225[1071534,1224609] 197ns|----------------------------------------L0.1225-----------------------------------------|"
    - "L0.1238[1071534,1224609] 198ns|----------------------------------------L0.1238-----------------------------------------|"
    - "L0.1251[1071534,1224609] 199ns|----------------------------------------L0.1251-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1071534,1224609] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 12 files: L0.1108, L0.1121, L0.1134, L0.1147, L0.1160, L0.1173, L0.1186, L0.1199, L0.1212, L0.1225, L0.1238, L0.1251"
    - "  Creating 1 files"
    - "**** Simulation run 150, type=compact(ManySmallFiles). 20 Input Files, 35b total:"
    - "L0                                                                                                                 "
    - "L0.417[1224610,1230000] 123ns 2b|L0.417|                                                                                  "
    - "L0.426[1224610,1240000] 124ns 2b|L0.426-|                                                                                 "
    - "L0.435[1224610,1250000] 125ns 2b|---L0.435---|                                                                            "
    - "L0.444[1224610,1260000] 126ns 2b|------L0.444------|                                                                      "
    - "L0.453[1224610,1270000] 127ns 2b|---------L0.453---------|                                                                "
    - "L0.462[1224610,1280000] 128ns 2b|------------L0.462------------|                                                          "
    - "L0.471[1224610,1290000] 129ns 2b|---------------L0.471---------------|                                                    "
    - "L0.480[1224610,1300000] 130ns 2b|------------------L0.480------------------|                                              "
    - "L0.489[1224610,1310000] 131ns 2b|---------------------L0.489---------------------|                                        "
    - "L0.498[1224610,1320000] 132ns 2b|------------------------L0.498------------------------|                                  "
    - "L0.507[1224610,1330000] 133ns 2b|--------------------------L0.507---------------------------|                             "
    - "L0.516[1224610,1340000] 134ns 2b|-----------------------------L0.516------------------------------|                       "
    - "L0.525[1224610,1350000] 135ns 2b|--------------------------------L0.525---------------------------------|                 "
    - "L0.534[1224610,1360000] 136ns 2b|-----------------------------------L0.534------------------------------------|           "
    - "L0.543[1224610,1370000] 137ns 2b|--------------------------------------L0.543---------------------------------------|     "
    - "L0.552[1224610,1377685] 138ns 1b|-----------------------------------------L0.552-----------------------------------------|"
    - "L0.562[1224610,1377685] 139ns 1b|-----------------------------------------L0.562-----------------------------------------|"
    - "L0.572[1224610,1377685] 140ns 1b|-----------------------------------------L0.572-----------------------------------------|"
    - "L0.582[1224610,1377685] 141ns 1b|-----------------------------------------L0.582-----------------------------------------|"
    - "L0.592[1224610,1377685] 142ns 1b|-----------------------------------------L0.592-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 35b total:"
    - "L0, all files 35b                                                                                                  "
    - "L0.?[1224610,1377685] 142ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.417, L0.426, L0.435, L0.444, L0.453, L0.462, L0.471, L0.480, L0.489, L0.498, L0.507, L0.516, L0.525, L0.534, L0.543, L0.552, L0.562, L0.572, L0.582, L0.592"
    - "  Creating 1 files"
    - "**** Simulation run 151, type=compact(ManySmallFiles). 20 Input Files, 11b total:"
    - "L0                                                                                                                 "
    - "L0.602[1224610,1377685] 143ns 1b|-----------------------------------------L0.602-----------------------------------------|"
    - "L0.612[1224610,1377685] 144ns 1b|-----------------------------------------L0.612-----------------------------------------|"
    - "L0.622[1224610,1377685] 145ns 1b|-----------------------------------------L0.622-----------------------------------------|"
    - "L0.632[1224610,1377685] 146ns 1b|-----------------------------------------L0.632-----------------------------------------|"
    - "L0.642[1224610,1377685] 147ns 1b|-----------------------------------------L0.642-----------------------------------------|"
    - "L0.652[1224610,1377685] 148ns 1b|-----------------------------------------L0.652-----------------------------------------|"
    - "L0.662[1224610,1377685] 149ns 1b|-----------------------------------------L0.662-----------------------------------------|"
    - "L0.672[1224610,1377685] 150ns 1b|-----------------------------------------L0.672-----------------------------------------|"
    - "L0.682[1224610,1377685] 151ns 1b|-----------------------------------------L0.682-----------------------------------------|"
    - "L0.692[1224610,1377685] 152ns 1b|-----------------------------------------L0.692-----------------------------------------|"
    - "L0.702[1224610,1377685] 153ns 1b|-----------------------------------------L0.702-----------------------------------------|"
    - "L0.712[1224610,1377685] 154ns 0b|-----------------------------------------L0.712-----------------------------------------|"
    - "L0.723[1224610,1377685] 155ns 0b|-----------------------------------------L0.723-----------------------------------------|"
    - "L0.734[1224610,1377685] 156ns 0b|-----------------------------------------L0.734-----------------------------------------|"
    - "L0.745[1224610,1377685] 157ns 0b|-----------------------------------------L0.745-----------------------------------------|"
    - "L0.756[1224610,1377685] 158ns 0b|-----------------------------------------L0.756-----------------------------------------|"
    - "L0.767[1224610,1377685] 159ns 0b|-----------------------------------------L0.767-----------------------------------------|"
    - "L0.778[1224610,1377685] 160ns 0b|-----------------------------------------L0.778-----------------------------------------|"
    - "L0.789[1224610,1377685] 161ns 0b|-----------------------------------------L0.789-----------------------------------------|"
    - "L0.800[1224610,1377685] 162ns 0b|-----------------------------------------L0.800-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 11b total:"
    - "L0, all files 11b                                                                                                  "
    - "L0.?[1224610,1377685] 162ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.602, L0.612, L0.622, L0.632, L0.642, L0.652, L0.662, L0.672, L0.682, L0.692, L0.702, L0.712, L0.723, L0.734, L0.745, L0.756, L0.767, L0.778, L0.789, L0.800"
    - "  Creating 1 files"
    - "**** Simulation run 152, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.811[1224610,1377685] 163ns|-----------------------------------------L0.811-----------------------------------------|"
    - "L0.822[1224610,1377685] 164ns|-----------------------------------------L0.822-----------------------------------------|"
    - "L0.833[1224610,1377685] 165ns|-----------------------------------------L0.833-----------------------------------------|"
    - "L0.844[1224610,1377685] 166ns|-----------------------------------------L0.844-----------------------------------------|"
    - "L0.855[1224610,1377685] 167ns|-----------------------------------------L0.855-----------------------------------------|"
    - "L0.866[1224610,1377685] 168ns|-----------------------------------------L0.866-----------------------------------------|"
    - "L0.877[1224610,1377685] 169ns|-----------------------------------------L0.877-----------------------------------------|"
    - "L0.889[1224610,1377685] 170ns|-----------------------------------------L0.889-----------------------------------------|"
    - "L0.901[1224610,1377685] 171ns|-----------------------------------------L0.901-----------------------------------------|"
    - "L0.913[1224610,1377685] 172ns|-----------------------------------------L0.913-----------------------------------------|"
    - "L0.925[1224610,1377685] 173ns|-----------------------------------------L0.925-----------------------------------------|"
    - "L0.937[1224610,1377685] 174ns|-----------------------------------------L0.937-----------------------------------------|"
    - "L0.949[1224610,1377685] 175ns|-----------------------------------------L0.949-----------------------------------------|"
    - "L0.961[1224610,1377685] 176ns|-----------------------------------------L0.961-----------------------------------------|"
    - "L0.973[1224610,1377685] 177ns|-----------------------------------------L0.973-----------------------------------------|"
    - "L0.985[1224610,1377685] 178ns|-----------------------------------------L0.985-----------------------------------------|"
    - "L0.997[1224610,1377685] 179ns|-----------------------------------------L0.997-----------------------------------------|"
    - "L0.1009[1224610,1377685] 180ns|----------------------------------------L0.1009-----------------------------------------|"
    - "L0.1021[1224610,1377685] 181ns|----------------------------------------L0.1021-----------------------------------------|"
    - "L0.1033[1224610,1377685] 182ns|----------------------------------------L0.1033-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1224610,1377685] 182ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.811, L0.822, L0.833, L0.844, L0.855, L0.866, L0.877, L0.889, L0.901, L0.913, L0.925, L0.937, L0.949, L0.961, L0.973, L0.985, L0.997, L0.1009, L0.1021, L0.1033"
    - "  Creating 1 files"
    - "**** Simulation run 153, type=compact(ManySmallFiles). 17 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1045[1224610,1377685] 183ns|----------------------------------------L0.1045-----------------------------------------|"
    - "L0.1057[1224610,1377685] 184ns|----------------------------------------L0.1057-----------------------------------------|"
    - "L0.1070[1224610,1377685] 185ns|----------------------------------------L0.1070-----------------------------------------|"
    - "L0.1083[1224610,1377685] 186ns|----------------------------------------L0.1083-----------------------------------------|"
    - "L0.1096[1224610,1377685] 187ns|----------------------------------------L0.1096-----------------------------------------|"
    - "L0.1109[1224610,1377685] 188ns|----------------------------------------L0.1109-----------------------------------------|"
    - "L0.1122[1224610,1377685] 189ns|----------------------------------------L0.1122-----------------------------------------|"
    - "L0.1135[1224610,1377685] 190ns|----------------------------------------L0.1135-----------------------------------------|"
    - "L0.1148[1224610,1377685] 191ns|----------------------------------------L0.1148-----------------------------------------|"
    - "L0.1161[1224610,1377685] 192ns|----------------------------------------L0.1161-----------------------------------------|"
    - "L0.1174[1224610,1377685] 193ns|----------------------------------------L0.1174-----------------------------------------|"
    - "L0.1187[1224610,1377685] 194ns|----------------------------------------L0.1187-----------------------------------------|"
    - "L0.1200[1224610,1377685] 195ns|----------------------------------------L0.1200-----------------------------------------|"
    - "L0.1213[1224610,1377685] 196ns|----------------------------------------L0.1213-----------------------------------------|"
    - "L0.1226[1224610,1377685] 197ns|----------------------------------------L0.1226-----------------------------------------|"
    - "L0.1239[1224610,1377685] 198ns|----------------------------------------L0.1239-----------------------------------------|"
    - "L0.1252[1224610,1377685] 199ns|----------------------------------------L0.1252-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1224610,1377685] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 17 files: L0.1045, L0.1057, L0.1070, L0.1083, L0.1096, L0.1109, L0.1122, L0.1135, L0.1148, L0.1161, L0.1174, L0.1187, L0.1200, L0.1213, L0.1226, L0.1239, L0.1252"
    - "  Creating 1 files"
    - "**** Simulation run 154, type=compact(ManySmallFiles). 20 Input Files, 17b total:"
    - "L0                                                                                                                 "
    - "L0.553[1377686,1380000] 138ns 1b|L0.553|                                                                                  "
    - "L0.563[1377686,1390000] 139ns 1b|L0.563|                                                                                  "
    - "L0.573[1377686,1400000] 140ns 1b|--L0.573---|                                                                             "
    - "L0.583[1377686,1410000] 141ns 1b|-----L0.583-----|                                                                        "
    - "L0.593[1377686,1420000] 142ns 1b|--------L0.593--------|                                                                  "
    - "L0.603[1377686,1430000] 143ns 1b|-----------L0.603-----------|                                                            "
    - "L0.613[1377686,1440000] 144ns 1b|--------------L0.613--------------|                                                      "
    - "L0.623[1377686,1450000] 145ns 1b|-----------------L0.623-----------------|                                                "
    - "L0.633[1377686,1460000] 146ns 1b|--------------------L0.633--------------------|                                          "
    - "L0.643[1377686,1470000] 147ns 1b|-----------------------L0.643-----------------------|                                    "
    - "L0.653[1377686,1480000] 148ns 1b|--------------------------L0.653--------------------------|                              "
    - "L0.663[1377686,1490000] 149ns 1b|-----------------------------L0.663-----------------------------|                        "
    - "L0.673[1377686,1500000] 150ns 1b|-------------------------------L0.673--------------------------------|                   "
    - "L0.683[1377686,1510000] 151ns 1b|----------------------------------L0.683-----------------------------------|             "
    - "L0.693[1377686,1520000] 152ns 1b|-------------------------------------L0.693--------------------------------------|       "
    - "L0.703[1377686,1530000] 153ns 2b|----------------------------------------L0.703-----------------------------------------| "
    - "L0.713[1377686,1530761] 154ns 0b|-----------------------------------------L0.713-----------------------------------------|"
    - "L0.724[1377686,1530761] 155ns 0b|-----------------------------------------L0.724-----------------------------------------|"
    - "L0.735[1377686,1530761] 156ns 0b|-----------------------------------------L0.735-----------------------------------------|"
    - "L0.746[1377686,1530761] 157ns 0b|-----------------------------------------L0.746-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 17b total:"
    - "L0, all files 17b                                                                                                  "
    - "L0.?[1377686,1530761] 157ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.553, L0.563, L0.573, L0.583, L0.593, L0.603, L0.613, L0.623, L0.633, L0.643, L0.653, L0.663, L0.673, L0.683, L0.693, L0.703, L0.713, L0.724, L0.735, L0.746"
    - "  Creating 1 files"
    - "**** Simulation run 155, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.757[1377686,1530761] 158ns|-----------------------------------------L0.757-----------------------------------------|"
    - "L0.768[1377686,1530761] 159ns|-----------------------------------------L0.768-----------------------------------------|"
    - "L0.779[1377686,1530761] 160ns|-----------------------------------------L0.779-----------------------------------------|"
    - "L0.790[1377686,1530761] 161ns|-----------------------------------------L0.790-----------------------------------------|"
    - "L0.801[1377686,1530761] 162ns|-----------------------------------------L0.801-----------------------------------------|"
    - "L0.812[1377686,1530761] 163ns|-----------------------------------------L0.812-----------------------------------------|"
    - "L0.823[1377686,1530761] 164ns|-----------------------------------------L0.823-----------------------------------------|"
    - "L0.834[1377686,1530761] 165ns|-----------------------------------------L0.834-----------------------------------------|"
    - "L0.845[1377686,1530761] 166ns|-----------------------------------------L0.845-----------------------------------------|"
    - "L0.856[1377686,1530761] 167ns|-----------------------------------------L0.856-----------------------------------------|"
    - "L0.867[1377686,1530761] 168ns|-----------------------------------------L0.867-----------------------------------------|"
    - "L0.878[1377686,1530761] 169ns|-----------------------------------------L0.878-----------------------------------------|"
    - "L0.890[1377686,1530761] 170ns|-----------------------------------------L0.890-----------------------------------------|"
    - "L0.902[1377686,1530761] 171ns|-----------------------------------------L0.902-----------------------------------------|"
    - "L0.914[1377686,1530761] 172ns|-----------------------------------------L0.914-----------------------------------------|"
    - "L0.926[1377686,1530761] 173ns|-----------------------------------------L0.926-----------------------------------------|"
    - "L0.938[1377686,1530761] 174ns|-----------------------------------------L0.938-----------------------------------------|"
    - "L0.950[1377686,1530761] 175ns|-----------------------------------------L0.950-----------------------------------------|"
    - "L0.962[1377686,1530761] 176ns|-----------------------------------------L0.962-----------------------------------------|"
    - "L0.974[1377686,1530761] 177ns|-----------------------------------------L0.974-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1377686,1530761] 177ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.757, L0.768, L0.779, L0.790, L0.801, L0.812, L0.823, L0.834, L0.845, L0.856, L0.867, L0.878, L0.890, L0.902, L0.914, L0.926, L0.938, L0.950, L0.962, L0.974"
    - "  Creating 1 files"
    - "**** Simulation run 156, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.986[1377686,1530761] 178ns|-----------------------------------------L0.986-----------------------------------------|"
    - "L0.998[1377686,1530761] 179ns|-----------------------------------------L0.998-----------------------------------------|"
    - "L0.1010[1377686,1530761] 180ns|----------------------------------------L0.1010-----------------------------------------|"
    - "L0.1022[1377686,1530761] 181ns|----------------------------------------L0.1022-----------------------------------------|"
    - "L0.1034[1377686,1530761] 182ns|----------------------------------------L0.1034-----------------------------------------|"
    - "L0.1046[1377686,1530761] 183ns|----------------------------------------L0.1046-----------------------------------------|"
    - "L0.1058[1377686,1530761] 184ns|----------------------------------------L0.1058-----------------------------------------|"
    - "L0.1071[1377686,1530761] 185ns|----------------------------------------L0.1071-----------------------------------------|"
    - "L0.1084[1377686,1530761] 186ns|----------------------------------------L0.1084-----------------------------------------|"
    - "L0.1097[1377686,1530761] 187ns|----------------------------------------L0.1097-----------------------------------------|"
    - "L0.1110[1377686,1530761] 188ns|----------------------------------------L0.1110-----------------------------------------|"
    - "L0.1123[1377686,1530761] 189ns|----------------------------------------L0.1123-----------------------------------------|"
    - "L0.1136[1377686,1530761] 190ns|----------------------------------------L0.1136-----------------------------------------|"
    - "L0.1149[1377686,1530761] 191ns|----------------------------------------L0.1149-----------------------------------------|"
    - "L0.1162[1377686,1530761] 192ns|----------------------------------------L0.1162-----------------------------------------|"
    - "L0.1175[1377686,1530761] 193ns|----------------------------------------L0.1175-----------------------------------------|"
    - "L0.1188[1377686,1530761] 194ns|----------------------------------------L0.1188-----------------------------------------|"
    - "L0.1201[1377686,1530761] 195ns|----------------------------------------L0.1201-----------------------------------------|"
    - "L0.1214[1377686,1530761] 196ns|----------------------------------------L0.1214-----------------------------------------|"
    - "L0.1227[1377686,1530761] 197ns|----------------------------------------L0.1227-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1377686,1530761] 197ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.986, L0.998, L0.1010, L0.1022, L0.1034, L0.1046, L0.1058, L0.1071, L0.1084, L0.1097, L0.1110, L0.1123, L0.1136, L0.1149, L0.1162, L0.1175, L0.1188, L0.1201, L0.1214, L0.1227"
    - "  Creating 1 files"
    - "**** Simulation run 157, type=compact(ManySmallFiles). 2 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1240[1377686,1530761] 198ns|----------------------------------------L0.1240-----------------------------------------|"
    - "L0.1253[1377686,1530761] 199ns|----------------------------------------L0.1253-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1377686,1530761] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.1240, L0.1253"
    - "  Creating 1 files"
    - "**** Simulation run 158, type=compact(ManySmallFiles). 20 Input Files, 150b total:"
    - "L0                                                                                                                 "
    - "L0.714[1530762,1540000] 154ns 10b|L0.714|                                                                                  "
    - "L0.725[1530762,1550000] 155ns 10b|-L0.725--|                                                                               "
    - "L0.736[1530762,1560000] 156ns 10b|----L0.736-----|                                                                         "
    - "L0.747[1530762,1570000] 157ns 10b|-------L0.747--------|                                                                   "
    - "L0.758[1530762,1580000] 158ns 10b|----------L0.758----------|                                                              "
    - "L0.769[1530762,1590000] 159ns 10b|-------------L0.769-------------|                                                        "
    - "L0.780[1530762,1600000] 160ns 10b|----------------L0.780----------------|                                                  "
    - "L0.791[1530762,1610000] 161ns 10b|-------------------L0.791-------------------|                                            "
    - "L0.802[1530762,1620000] 162ns 10b|----------------------L0.802----------------------|                                      "
    - "L0.813[1530762,1630000] 163ns 10b|-------------------------L0.813-------------------------|                                "
    - "L0.824[1530762,1640000] 164ns 10b|----------------------------L0.824----------------------------|                          "
    - "L0.835[1530762,1650000] 165ns 10b|-------------------------------L0.835-------------------------------|                    "
    - "L0.846[1530762,1660000] 166ns 10b|---------------------------------L0.846----------------------------------|               "
    - "L0.857[1530762,1670000] 167ns 10b|------------------------------------L0.857-------------------------------------|         "
    - "L0.868[1530762,1680000] 168ns 10b|---------------------------------------L0.868----------------------------------------|   "
    - "L0.879[1530762,1683837] 169ns 0b|-----------------------------------------L0.879-----------------------------------------|"
    - "L0.891[1530762,1683837] 170ns 0b|-----------------------------------------L0.891-----------------------------------------|"
    - "L0.903[1530762,1683837] 171ns 0b|-----------------------------------------L0.903-----------------------------------------|"
    - "L0.915[1530762,1683837] 172ns 0b|-----------------------------------------L0.915-----------------------------------------|"
    - "L0.927[1530762,1683837] 173ns 0b|-----------------------------------------L0.927-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 150b total:"
    - "L0, all files 150b                                                                                                 "
    - "L0.?[1530762,1683837] 173ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.714, L0.725, L0.736, L0.747, L0.758, L0.769, L0.780, L0.791, L0.802, L0.813, L0.824, L0.835, L0.846, L0.857, L0.868, L0.879, L0.891, L0.903, L0.915, L0.927"
    - "  Creating 1 files"
    - "**** Simulation run 159, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.939[1530762,1683837] 174ns|-----------------------------------------L0.939-----------------------------------------|"
    - "L0.951[1530762,1683837] 175ns|-----------------------------------------L0.951-----------------------------------------|"
    - "L0.963[1530762,1683837] 176ns|-----------------------------------------L0.963-----------------------------------------|"
    - "L0.975[1530762,1683837] 177ns|-----------------------------------------L0.975-----------------------------------------|"
    - "L0.987[1530762,1683837] 178ns|-----------------------------------------L0.987-----------------------------------------|"
    - "L0.999[1530762,1683837] 179ns|-----------------------------------------L0.999-----------------------------------------|"
    - "L0.1011[1530762,1683837] 180ns|----------------------------------------L0.1011-----------------------------------------|"
    - "L0.1023[1530762,1683837] 181ns|----------------------------------------L0.1023-----------------------------------------|"
    - "L0.1035[1530762,1683837] 182ns|----------------------------------------L0.1035-----------------------------------------|"
    - "L0.1047[1530762,1683837] 183ns|----------------------------------------L0.1047-----------------------------------------|"
    - "L0.1059[1530762,1683837] 184ns|----------------------------------------L0.1059-----------------------------------------|"
    - "L0.1072[1530762,1683837] 185ns|----------------------------------------L0.1072-----------------------------------------|"
    - "L0.1085[1530762,1683837] 186ns|----------------------------------------L0.1085-----------------------------------------|"
    - "L0.1098[1530762,1683837] 187ns|----------------------------------------L0.1098-----------------------------------------|"
    - "L0.1111[1530762,1683837] 188ns|----------------------------------------L0.1111-----------------------------------------|"
    - "L0.1124[1530762,1683837] 189ns|----------------------------------------L0.1124-----------------------------------------|"
    - "L0.1137[1530762,1683837] 190ns|----------------------------------------L0.1137-----------------------------------------|"
    - "L0.1150[1530762,1683837] 191ns|----------------------------------------L0.1150-----------------------------------------|"
    - "L0.1163[1530762,1683837] 192ns|----------------------------------------L0.1163-----------------------------------------|"
    - "L0.1176[1530762,1683837] 193ns|----------------------------------------L0.1176-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1530762,1683837] 193ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.939, L0.951, L0.963, L0.975, L0.987, L0.999, L0.1011, L0.1023, L0.1035, L0.1047, L0.1059, L0.1072, L0.1085, L0.1098, L0.1111, L0.1124, L0.1137, L0.1150, L0.1163, L0.1176"
    - "  Creating 1 files"
    - "**** Simulation run 160, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1189[1530762,1683837] 194ns|----------------------------------------L0.1189-----------------------------------------|"
    - "L0.1202[1530762,1683837] 195ns|----------------------------------------L0.1202-----------------------------------------|"
    - "L0.1215[1530762,1683837] 196ns|----------------------------------------L0.1215-----------------------------------------|"
    - "L0.1228[1530762,1683837] 197ns|----------------------------------------L0.1228-----------------------------------------|"
    - "L0.1241[1530762,1683837] 198ns|----------------------------------------L0.1241-----------------------------------------|"
    - "L0.1254[1530762,1683837] 199ns|----------------------------------------L0.1254-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1530762,1683837] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1189, L0.1202, L0.1215, L0.1228, L0.1241, L0.1254"
    - "  Creating 1 files"
    - "**** Simulation run 161, type=compact(ManySmallFiles). 20 Input Files, 150b total:"
    - "L0                                                                                                                 "
    - "L0.880[1683838,1690000] 169ns 10b|L0.880|                                                                                  "
    - "L0.892[1683838,1700000] 170ns 10b|L0.892-|                                                                                 "
    - "L0.904[1683838,1710000] 171ns 10b|---L0.904----|                                                                           "
    - "L0.916[1683838,1720000] 172ns 10b|------L0.916-------|                                                                     "
    - "L0.928[1683838,1730000] 173ns 10b|---------L0.928----------|                                                               "
    - "L0.940[1683838,1740000] 174ns 10b|------------L0.940-------------|                                                         "
    - "L0.952[1683838,1750000] 175ns 10b|---------------L0.952---------------|                                                    "
    - "L0.964[1683838,1760000] 176ns 10b|------------------L0.964------------------|                                              "
    - "L0.976[1683838,1770000] 177ns 10b|---------------------L0.976---------------------|                                        "
    - "L0.988[1683838,1780000] 178ns 10b|------------------------L0.988------------------------|                                  "
    - "L0.1000[1683838,1790000] 179ns 10b|--------------------------L0.1000---------------------------|                            "
    - "L0.1012[1683838,1800000] 180ns 10b|-----------------------------L0.1012------------------------------|                      "
    - "L0.1024[1683838,1810000] 181ns 10b|--------------------------------L0.1024---------------------------------|                "
    - "L0.1036[1683838,1820000] 182ns 10b|-----------------------------------L0.1036------------------------------------|          "
    - "L0.1048[1683838,1830000] 183ns 10b|--------------------------------------L0.1048--------------------------------------|     "
    - "L0.1060[1683838,1836913] 184ns 0b|----------------------------------------L0.1060-----------------------------------------|"
    - "L0.1073[1683838,1836913] 185ns 0b|----------------------------------------L0.1073-----------------------------------------|"
    - "L0.1086[1683838,1836913] 186ns 0b|----------------------------------------L0.1086-----------------------------------------|"
    - "L0.1099[1683838,1836913] 187ns 0b|----------------------------------------L0.1099-----------------------------------------|"
    - "L0.1112[1683838,1836913] 188ns 0b|----------------------------------------L0.1112-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 150b total:"
    - "L0, all files 150b                                                                                                 "
    - "L0.?[1683838,1836913] 188ns|------------------------------------------L0.?------------------------------------------|"
    - "**** Simulation run 162, type=compact(ManySmallFiles). 16 Input Files, 160b total:"
    - "L0, all files 10b                                                                                                  "
    - "L0.1061[1836914,1840000] 184ns|L0.1061|                                                                                 "
    - "L0.1074[1836914,1850000] 185ns|L0.1074|                                                                                 "
    - "L0.1087[1836914,1860000] 186ns|--L0.1087--|                                                                             "
    - "L0.1100[1836914,1870000] 187ns|-----L0.1100-----|                                                                       "
    - "L0.1113[1836914,1880000] 188ns|--------L0.1113--------|                                                                 "
    - "L0.1126[1836914,1890000] 189ns|-----------L0.1126-----------|                                                           "
    - "L0.1139[1836914,1900000] 190ns|--------------L0.1139--------------|                                                     "
    - "L0.1152[1836914,1910000] 191ns|----------------L0.1152-----------------|                                                "
    - "L0.1165[1836914,1920000] 192ns|-------------------L0.1165--------------------|                                          "
    - "L0.1178[1836914,1930000] 193ns|----------------------L0.1178-----------------------|                                    "
    - "L0.1191[1836914,1940000] 194ns|-------------------------L0.1191--------------------------|                              "
    - "L0.1204[1836914,1950000] 195ns|----------------------------L0.1204-----------------------------|                        "
    - "L0.1217[1836914,1960000] 196ns|-------------------------------L0.1217--------------------------------|                  "
    - "L0.1230[1836914,1970000] 197ns|----------------------------------L0.1230-----------------------------------|            "
    - "L0.1243[1836914,1980000] 198ns|-------------------------------------L0.1243--------------------------------------|      "
    - "L0.1256[1836914,1990000] 199ns|----------------------------------------L0.1256-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 160b total:"
    - "L0, all files 160b                                                                                                 "
    - "L0.?[1836914,1990000] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 16 files: L0.1061, L0.1074, L0.1087, L0.1100, L0.1113, L0.1126, L0.1139, L0.1152, L0.1165, L0.1178, L0.1191, L0.1204, L0.1217, L0.1230, L0.1243, L0.1256"
    - "  Creating 1 files"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.880, L0.892, L0.904, L0.916, L0.928, L0.940, L0.952, L0.964, L0.976, L0.988, L0.1000, L0.1012, L0.1024, L0.1036, L0.1048, L0.1060, L0.1073, L0.1086, L0.1099, L0.1112"
    - "  Creating 1 files"
    - "**** Simulation run 163, type=compact(ManySmallFiles). 11 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1125[1683838,1836913] 189ns|----------------------------------------L0.1125-----------------------------------------|"
    - "L0.1138[1683838,1836913] 190ns|----------------------------------------L0.1138-----------------------------------------|"
    - "L0.1151[1683838,1836913] 191ns|----------------------------------------L0.1151-----------------------------------------|"
    - "L0.1164[1683838,1836913] 192ns|----------------------------------------L0.1164-----------------------------------------|"
    - "L0.1177[1683838,1836913] 193ns|----------------------------------------L0.1177-----------------------------------------|"
    - "L0.1190[1683838,1836913] 194ns|----------------------------------------L0.1190-----------------------------------------|"
    - "L0.1203[1683838,1836913] 195ns|----------------------------------------L0.1203-----------------------------------------|"
    - "L0.1216[1683838,1836913] 196ns|----------------------------------------L0.1216-----------------------------------------|"
    - "L0.1229[1683838,1836913] 197ns|----------------------------------------L0.1229-----------------------------------------|"
    - "L0.1242[1683838,1836913] 198ns|----------------------------------------L0.1242-----------------------------------------|"
    - "L0.1255[1683838,1836913] 199ns|----------------------------------------L0.1255-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1683838,1836913] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 11 files: L0.1125, L0.1138, L0.1151, L0.1164, L0.1177, L0.1190, L0.1203, L0.1216, L0.1229, L0.1242, L0.1255"
    - "  Creating 1 files"
    - "**** Simulation run 164, type=compact(ManySmallFiles). 20 Input Files, 160b total:"
    - "L0                                                                                                                 "
    - "L0.1257[20,153077] 115ns 141b|----------------------------------------L0.1257-----------------------------------------|"
    - "L0.353[116,153077] 116ns 1b|----------------------------------------L0.353-----------------------------------------| "
    - "L0.361[117,153077] 117ns 1b|----------------------------------------L0.361-----------------------------------------| "
    - "L0.369[118,153077] 118ns 1b|----------------------------------------L0.369-----------------------------------------| "
    - "L0.377[119,153077] 119ns 1b|----------------------------------------L0.377-----------------------------------------| "
    - "L0.385[120,153077] 120ns 1b|----------------------------------------L0.385-----------------------------------------| "
    - "L0.393[121,153077] 121ns 1b|----------------------------------------L0.393-----------------------------------------| "
    - "L0.401[122,153077] 122ns 1b|----------------------------------------L0.401-----------------------------------------| "
    - "L0.409[123,153077] 123ns 1b|----------------------------------------L0.409-----------------------------------------| "
    - "L0.418[124,153077] 124ns 1b|----------------------------------------L0.418-----------------------------------------| "
    - "L0.427[125,153077] 125ns 1b|----------------------------------------L0.427-----------------------------------------| "
    - "L0.436[126,153077] 126ns 1b|----------------------------------------L0.436-----------------------------------------| "
    - "L0.445[127,153077] 127ns 1b|----------------------------------------L0.445-----------------------------------------| "
    - "L0.454[128,153077] 128ns 1b|----------------------------------------L0.454-----------------------------------------| "
    - "L0.463[129,153077] 129ns 1b|----------------------------------------L0.463-----------------------------------------| "
    - "L0.472[130,153077] 130ns 1b|----------------------------------------L0.472-----------------------------------------| "
    - "L0.481[131,153077] 131ns 1b|----------------------------------------L0.481-----------------------------------------| "
    - "L0.490[132,153077] 132ns 1b|----------------------------------------L0.490-----------------------------------------| "
    - "L0.499[133,153077] 133ns 1b|----------------------------------------L0.499-----------------------------------------| "
    - "L0.508[134,153077] 134ns 1b|----------------------------------------L0.508-----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 160b total:"
    - "L0, all files 160b                                                                                                 "
    - "L0.?[20,153077] 134ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.353, L0.361, L0.369, L0.377, L0.385, L0.393, L0.401, L0.409, L0.418, L0.427, L0.436, L0.445, L0.454, L0.463, L0.472, L0.481, L0.490, L0.499, L0.508, L0.1257"
    - "  Creating 1 files"
    - "**** Simulation run 165, type=compact(ManySmallFiles). 6 Input Files, 179b total:"
    - "L0                                                                                                                 "
    - "L0.1258[153078,306153] 115ns 141b|----------------------------------------L0.1258-----------------------------------------|"
    - "L0.1267[153078,306153] 135ns 20b|----------------------------------------L0.1267-----------------------------------------|"
    - "L0.1268[153078,306153] 155ns 18b|----------------------------------------L0.1268-----------------------------------------|"
    - "L0.1259[153078,306153] 175ns 0b|----------------------------------------L0.1259-----------------------------------------|"
    - "L0.1260[153078,306153] 195ns 0b|----------------------------------------L0.1260-----------------------------------------|"
    - "L0.1261[153078,306153] 199ns 0b|----------------------------------------L0.1261-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 179b total:"
    - "L0, all files 179b                                                                                                 "
    - "L0.?[153078,306153] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1258, L0.1259, L0.1260, L0.1261, L0.1267, L0.1268"
    - "  Creating 1 files"
    - "**** Simulation run 166, type=compact(ManySmallFiles). 6 Input Files, 179b total:"
    - "L0                                                                                                                 "
    - "L0.1262[306154,459229] 115ns 141b|----------------------------------------L0.1262-----------------------------------------|"
    - "L0.1263[306154,459229] 135ns 20b|----------------------------------------L0.1263-----------------------------------------|"
    - "L0.1264[306154,459229] 155ns 18b|----------------------------------------L0.1264-----------------------------------------|"
    - "L0.1265[306154,459229] 175ns 0b|----------------------------------------L0.1265-----------------------------------------|"
    - "L0.1266[306154,459229] 195ns 0b|----------------------------------------L0.1266-----------------------------------------|"
    - "L0.1269[306154,459229] 199ns 0b|----------------------------------------L0.1269-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 179b total:"
    - "L0, all files 179b                                                                                                 "
    - "L0.?[306154,459229] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1262, L0.1263, L0.1264, L0.1265, L0.1266, L0.1269"
    - "  Creating 1 files"
    - "**** Simulation run 167, type=compact(ManySmallFiles). 6 Input Files, 179b total:"
    - "L0                                                                                                                 "
    - "L0.1270[459230,612305] 115ns 141b|----------------------------------------L0.1270-----------------------------------------|"
    - "L0.1271[459230,612305] 135ns 20b|----------------------------------------L0.1271-----------------------------------------|"
    - "L0.1272[459230,612305] 155ns 18b|----------------------------------------L0.1272-----------------------------------------|"
    - "L0.1273[459230,612305] 175ns 0b|----------------------------------------L0.1273-----------------------------------------|"
    - "L0.1274[459230,612305] 195ns 0b|----------------------------------------L0.1274-----------------------------------------|"
    - "L0.1275[459230,612305] 199ns 0b|----------------------------------------L0.1275-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 179b total:"
    - "L0, all files 179b                                                                                                 "
    - "L0.?[459230,612305] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1270, L0.1271, L0.1272, L0.1273, L0.1274, L0.1275"
    - "  Creating 1 files"
    - "**** Simulation run 168, type=compact(ManySmallFiles). 6 Input Files, 179b total:"
    - "L0                                                                                                                 "
    - "L0.1276[612306,765381] 115ns 141b|----------------------------------------L0.1276-----------------------------------------|"
    - "L0.1277[612306,765381] 135ns 20b|----------------------------------------L0.1277-----------------------------------------|"
    - "L0.1278[612306,765381] 155ns 18b|----------------------------------------L0.1278-----------------------------------------|"
    - "L0.1279[612306,765381] 175ns 0b|----------------------------------------L0.1279-----------------------------------------|"
    - "L0.1280[612306,765381] 195ns 0b|----------------------------------------L0.1280-----------------------------------------|"
    - "L0.1281[612306,765381] 199ns 0b|----------------------------------------L0.1281-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 179b total:"
    - "L0, all files 179b                                                                                                 "
    - "L0.?[612306,765381] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1276, L0.1277, L0.1278, L0.1279, L0.1280, L0.1281"
    - "  Creating 1 files"
    - "**** Simulation run 169, type=compact(ManySmallFiles). 6 Input Files, 179b total:"
    - "L0                                                                                                                 "
    - "L0.1282[765382,918457] 115ns 141b|----------------------------------------L0.1282-----------------------------------------|"
    - "L0.1283[765382,918457] 135ns 20b|----------------------------------------L0.1283-----------------------------------------|"
    - "L0.1284[765382,918457] 155ns 18b|----------------------------------------L0.1284-----------------------------------------|"
    - "L0.1293[765382,918457] 175ns 0b|----------------------------------------L0.1293-----------------------------------------|"
    - "L0.1285[765382,918457] 195ns 0b|----------------------------------------L0.1285-----------------------------------------|"
    - "L0.1286[765382,918457] 199ns 0b|----------------------------------------L0.1286-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 179b total:"
    - "L0, all files 179b                                                                                                 "
    - "L0.?[765382,918457] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1282, L0.1283, L0.1284, L0.1285, L0.1286, L0.1293"
    - "  Creating 1 files"
    - "**** Simulation run 170, type=compact(ManySmallFiles). 6 Input Files, 128b total:"
    - "L0                                                                                                                 "
    - "L0.1287[918458,1071533] 115ns 90b|----------------------------------------L0.1287-----------------------------------------|"
    - "L0.1288[918458,1071533] 135ns 20b|----------------------------------------L0.1288-----------------------------------------|"
    - "L0.1289[918458,1071533] 155ns 18b|----------------------------------------L0.1289-----------------------------------------|"
    - "L0.1290[918458,1071533] 175ns 0b|----------------------------------------L0.1290-----------------------------------------|"
    - "L0.1291[918458,1071533] 195ns 0b|----------------------------------------L0.1291-----------------------------------------|"
    - "L0.1292[918458,1071533] 199ns 0b|----------------------------------------L0.1292-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 128b total:"
    - "L0, all files 128b                                                                                                 "
    - "L0.?[918458,1071533] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.1287, L0.1288, L0.1289, L0.1290, L0.1291, L0.1292"
    - "  Creating 1 files"
    - "**** Simulation run 171, type=compact(ManySmallFiles). 5 Input Files, 76b total:"
    - "L0                                                                                                                 "
    - "L0.1294[1071534,1224609] 127ns 50b|----------------------------------------L0.1294-----------------------------------------|"
    - "L0.1295[1071534,1224609] 147ns 20b|----------------------------------------L0.1295-----------------------------------------|"
    - "L0.1296[1071534,1224609] 167ns 6b|----------------------------------------L0.1296-----------------------------------------|"
    - "L0.1297[1071534,1224609] 187ns 0b|----------------------------------------L0.1297-----------------------------------------|"
    - "L0.1298[1071534,1224609] 199ns 0b|----------------------------------------L0.1298-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 76b total:"
    - "L0, all files 76b                                                                                                  "
    - "L0.?[1071534,1224609] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 5 files: L0.1294, L0.1295, L0.1296, L0.1297, L0.1298"
    - "  Creating 1 files"
    - "**** Simulation run 172, type=compact(ManySmallFiles). 4 Input Files, 46b total:"
    - "L0                                                                                                                 "
    - "L0.1299[1224610,1377685] 142ns 35b|----------------------------------------L0.1299-----------------------------------------|"
    - "L0.1300[1224610,1377685] 162ns 11b|----------------------------------------L0.1300-----------------------------------------|"
    - "L0.1301[1224610,1377685] 182ns 0b|----------------------------------------L0.1301-----------------------------------------|"
    - "L0.1302[1224610,1377685] 199ns 0b|----------------------------------------L0.1302-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 46b total:"
    - "L0, all files 46b                                                                                                  "
    - "L0.?[1224610,1377685] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1299, L0.1300, L0.1301, L0.1302"
    - "  Creating 1 files"
    - "**** Simulation run 173, type=compact(ManySmallFiles). 4 Input Files, 17b total:"
    - "L0                                                                                                                 "
    - "L0.1303[1377686,1530761] 157ns 17b|----------------------------------------L0.1303-----------------------------------------|"
    - "L0.1304[1377686,1530761] 177ns 0b|----------------------------------------L0.1304-----------------------------------------|"
    - "L0.1305[1377686,1530761] 197ns 0b|----------------------------------------L0.1305-----------------------------------------|"
    - "L0.1306[1377686,1530761] 199ns 0b|----------------------------------------L0.1306-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 17b total:"
    - "L0, all files 17b                                                                                                  "
    - "L0.?[1377686,1530761] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.1303, L0.1304, L0.1305, L0.1306"
    - "  Creating 1 files"
    - "**** Simulation run 174, type=compact(ManySmallFiles). 3 Input Files, 150b total:"
    - "L0                                                                                                                 "
    - "L0.1307[1530762,1683837] 173ns 150b|----------------------------------------L0.1307-----------------------------------------|"
    - "L0.1308[1530762,1683837] 193ns 0b|----------------------------------------L0.1308-----------------------------------------|"
    - "L0.1309[1530762,1683837] 199ns 0b|----------------------------------------L0.1309-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 150b total:"
    - "L0, all files 150b                                                                                                 "
    - "L0.?[1530762,1683837] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.1307, L0.1308, L0.1309"
    - "  Creating 1 files"
    - "**** Simulation run 175, type=compact(ManySmallFiles). 2 Input Files, 150b total:"
    - "L0                                                                                                                 "
    - "L0.1311[1683838,1836913] 188ns 150b|----------------------------------------L0.1311-----------------------------------------|"
    - "L0.1312[1683838,1836913] 199ns 0b|----------------------------------------L0.1312-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 150b total:"
    - "L0, all files 150b                                                                                                 "
    - "L0.?[1683838,1836913] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.1311, L0.1312"
    - "  Creating 1 files"
    - "**** Simulation run 176, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 160b total:"
    - "L0, all files 160b                                                                                                 "
    - "L0.1313[20,153077] 134ns |----------------------------------------L0.1313-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 160b total:"
    - "L0                                                                                                                 "
    - "L0.?[20,11776] 134ns 12b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 134ns 12b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 134ns 12b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 134ns 12b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 134ns 12b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 134ns 12b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 134ns 12b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 134ns 12b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 134ns 12b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 134ns 12b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 134ns 12b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 134ns 12b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 134ns 16b                                                                                   |L0.?| "
    - "**** Simulation run 177, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.517[135,153077] 135ns |-----------------------------------------L0.517-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[135,11776] 135ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 135ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 135ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 135ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 135ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 135ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 135ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 135ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 135ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 135ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 135ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 135ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 135ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 178, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.526[136,153077] 136ns |-----------------------------------------L0.526-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[136,11776] 136ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 136ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 136ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 136ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 136ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 136ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 136ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 136ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 136ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 136ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 136ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 136ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 136ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 179, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.535[137,153077] 137ns |-----------------------------------------L0.535-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[137,11776] 137ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 137ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 137ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 137ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 137ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 137ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 137ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 137ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 137ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 137ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 137ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 137ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 137ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 180, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.544[138,153077] 138ns |-----------------------------------------L0.544-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[138,11776] 138ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 138ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 138ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 138ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 138ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 138ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 138ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 138ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 138ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 138ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 138ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 138ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 138ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 181, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.554[139,153077] 139ns |-----------------------------------------L0.554-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[139,11776] 139ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 139ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 139ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 139ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 139ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 139ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 139ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 139ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 139ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 139ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 139ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 139ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 139ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 182, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.564[140,153077] 140ns |-----------------------------------------L0.564-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[140,11776] 140ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 140ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 140ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 140ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 140ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 140ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 140ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 140ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 140ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 140ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 140ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 140ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 140ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 183, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.574[141,153077] 141ns |-----------------------------------------L0.574-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[141,11776] 141ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 141ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 141ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 141ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 141ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 141ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 141ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 141ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 141ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 141ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 141ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 141ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 141ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 184, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.584[142,153077] 142ns |-----------------------------------------L0.584-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[142,11776] 142ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 142ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 142ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 142ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 142ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 142ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 142ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 142ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 142ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 142ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 142ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 142ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 142ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 185, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.594[143,153077] 143ns |-----------------------------------------L0.594-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[143,11776] 143ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 143ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 143ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 143ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 143ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 143ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 143ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 143ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 143ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 143ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 143ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 143ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 143ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 186, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.604[144,153077] 144ns |-----------------------------------------L0.604-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[144,11776] 144ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 144ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 144ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 144ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 144ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 144ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 144ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 144ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 144ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 144ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 144ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 144ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 144ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 187, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.614[145,153077] 145ns |-----------------------------------------L0.614-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[145,11776] 145ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 145ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 145ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 145ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 145ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 145ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 145ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 145ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 145ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 145ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 145ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 145ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 145ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 188, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.624[146,153077] 146ns |-----------------------------------------L0.624-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[146,11776] 146ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 146ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 146ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 146ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 146ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 146ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 146ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 146ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 146ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 146ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 146ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 146ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 146ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 189, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.634[147,153077] 147ns |-----------------------------------------L0.634-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[147,11776] 147ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 147ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 147ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 147ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 147ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 147ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 147ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 147ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 147ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 147ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 147ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 147ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 147ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 190, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.644[148,153077] 148ns |-----------------------------------------L0.644-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[148,11776] 148ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 148ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 148ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 148ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 148ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 148ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 148ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 148ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 148ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 148ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 148ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 148ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 148ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 191, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.654[149,153077] 149ns |-----------------------------------------L0.654-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[149,11776] 149ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 149ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 149ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 149ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 149ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 149ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 149ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 149ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 149ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 149ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 149ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 149ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 149ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 192, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.664[150,153077] 150ns |-----------------------------------------L0.664-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[150,11776] 150ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 150ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 150ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 150ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 150ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 150ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 150ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 150ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 150ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 150ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 150ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 150ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 150ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 193, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.674[151,153077] 151ns |-----------------------------------------L0.674-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[151,11776] 151ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 151ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 151ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 151ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 151ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 151ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 151ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 151ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 151ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 151ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 151ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 151ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 151ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 194, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 1b total:"
    - "L0, all files 1b                                                                                                   "
    - "L0.684[152,153077] 152ns |-----------------------------------------L0.684-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 1b total:"
    - "L0                                                                                                                 "
    - "L0.?[152,11776] 152ns 0b |L0.?|                                                                                    "
    - "L0.?[11777,23551] 152ns 0b      |L0.?|                                                                              "
    - "L0.?[23552,35326] 152ns 0b             |L0.?|                                                                       "
    - "L0.?[35327,47101] 152ns 0b                    |L0.?|                                                                "
    - "L0.?[47102,58876] 152ns 0b                           |L0.?|                                                         "
    - "L0.?[58877,70651] 152ns 0b                                  |L0.?|                                                  "
    - "L0.?[70652,82426] 152ns 0b                                         |L0.?|                                           "
    - "L0.?[82427,94201] 152ns 0b                                                |L0.?|                                    "
    - "L0.?[94202,105976] 152ns 0b                                                       |L0.?|                             "
    - "L0.?[105977,117751] 152ns 0b                                                              |L0.?|                      "
    - "L0.?[117752,129526] 152ns 0b                                                                     |L0.?|               "
    - "L0.?[129527,141301] 152ns 0b                                                                            |L0.?|        "
    - "L0.?[141302,153077] 152ns 1b                                                                                   |L0.?| "
    - "**** Simulation run 195, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.694[153,153077] 153ns |-----------------------------------------L0.694-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[153,11776] 153ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 153ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 153ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 153ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 153ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 153ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 153ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 153ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 153ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 153ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 153ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 153ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 153ns                                                                                   |L0.?| "
    - "**** Simulation run 196, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.704[154,153077] 154ns |-----------------------------------------L0.704-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[154,11776] 154ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 154ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 154ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 154ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 154ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 154ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 154ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 154ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 154ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 154ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 154ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 154ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 154ns                                                                                   |L0.?| "
    - "**** Simulation run 197, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.715[155,153077] 155ns |-----------------------------------------L0.715-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[155,11776] 155ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 155ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 155ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 155ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 155ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 155ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 155ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 155ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 155ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 155ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 155ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 155ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 155ns                                                                                   |L0.?| "
    - "**** Simulation run 198, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.726[156,153077] 156ns |-----------------------------------------L0.726-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[156,11776] 156ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 156ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 156ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 156ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 156ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 156ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 156ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 156ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 156ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 156ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 156ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 156ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 156ns                                                                                   |L0.?| "
    - "**** Simulation run 199, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.737[157,153077] 157ns |-----------------------------------------L0.737-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[157,11776] 157ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 157ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 157ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 157ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 157ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 157ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 157ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 157ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 157ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 157ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 157ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 157ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 157ns                                                                                   |L0.?| "
    - "**** Simulation run 200, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.748[158,153077] 158ns |-----------------------------------------L0.748-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[158,11776] 158ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 158ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 158ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 158ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 158ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 158ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 158ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 158ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 158ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 158ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 158ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 158ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 158ns                                                                                   |L0.?| "
    - "**** Simulation run 201, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.759[159,153077] 159ns |-----------------------------------------L0.759-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[159,11776] 159ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 159ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 159ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 159ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 159ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 159ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 159ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 159ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 159ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 159ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 159ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 159ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 159ns                                                                                   |L0.?| "
    - "**** Simulation run 202, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.770[160,153077] 160ns |-----------------------------------------L0.770-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[160,11776] 160ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 160ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 160ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 160ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 160ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 160ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 160ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 160ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 160ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 160ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 160ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 160ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 160ns                                                                                   |L0.?| "
    - "**** Simulation run 203, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.781[161,153077] 161ns |-----------------------------------------L0.781-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[161,11776] 161ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 161ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 161ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 161ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 161ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 161ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 161ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 161ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 161ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 161ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 161ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 161ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 161ns                                                                                   |L0.?| "
    - "**** Simulation run 204, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.792[162,153077] 162ns |-----------------------------------------L0.792-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[162,11776] 162ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 162ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 162ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 162ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 162ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 162ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 162ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 162ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 162ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 162ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 162ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 162ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 162ns                                                                                   |L0.?| "
    - "**** Simulation run 205, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.803[163,153077] 163ns |-----------------------------------------L0.803-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[163,11776] 163ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 163ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 163ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 163ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 163ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 163ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 163ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 163ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 163ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 163ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 163ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 163ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 163ns                                                                                   |L0.?| "
    - "**** Simulation run 206, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.814[164,153077] 164ns |-----------------------------------------L0.814-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[164,11776] 164ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 164ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 164ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 164ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 164ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 164ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 164ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 164ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 164ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 164ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 164ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 164ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 164ns                                                                                   |L0.?| "
    - "**** Simulation run 207, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.825[165,153077] 165ns |-----------------------------------------L0.825-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[165,11776] 165ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 165ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 165ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 165ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 165ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 165ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 165ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 165ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 165ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 165ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 165ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 165ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 165ns                                                                                   |L0.?| "
    - "**** Simulation run 208, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.836[166,153077] 166ns |-----------------------------------------L0.836-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[166,11776] 166ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 166ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 166ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 166ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 166ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 166ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 166ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 166ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 166ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 166ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 166ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 166ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 166ns                                                                                   |L0.?| "
    - "**** Simulation run 209, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.847[167,153077] 167ns |-----------------------------------------L0.847-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[167,11776] 167ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 167ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 167ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 167ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 167ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 167ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 167ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 167ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 167ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 167ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 167ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 167ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 167ns                                                                                   |L0.?| "
    - "**** Simulation run 210, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.858[168,153077] 168ns |-----------------------------------------L0.858-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[168,11776] 168ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 168ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 168ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 168ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 168ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 168ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 168ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 168ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 168ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 168ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 168ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 168ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 168ns                                                                                   |L0.?| "
    - "**** Simulation run 211, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.869[169,153077] 169ns |-----------------------------------------L0.869-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[169,11776] 169ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 169ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 169ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 169ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 169ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 169ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 169ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 169ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 169ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 169ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 169ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 169ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 169ns                                                                                   |L0.?| "
    - "**** Simulation run 212, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.881[170,153077] 170ns |-----------------------------------------L0.881-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[170,11776] 170ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 170ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 170ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 170ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 170ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 170ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 170ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 170ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 170ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 170ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 170ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 170ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 170ns                                                                                   |L0.?| "
    - "**** Simulation run 213, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.893[171,153077] 171ns |-----------------------------------------L0.893-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[171,11776] 171ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 171ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 171ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 171ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 171ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 171ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 171ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 171ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 171ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 171ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 171ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 171ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 171ns                                                                                   |L0.?| "
    - "**** Simulation run 214, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.905[172,153077] 172ns |-----------------------------------------L0.905-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[172,11776] 172ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 172ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 172ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 172ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 172ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 172ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 172ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 172ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 172ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 172ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 172ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 172ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 172ns                                                                                   |L0.?| "
    - "**** Simulation run 215, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.917[173,153077] 173ns |-----------------------------------------L0.917-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[173,11776] 173ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 173ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 173ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 173ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 173ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 173ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 173ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 173ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 173ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 173ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 173ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 173ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 173ns                                                                                   |L0.?| "
    - "Committing partition 1:"
    - "  Soft Deleting 40 files: L0.517, L0.526, L0.535, L0.544, L0.554, L0.564, L0.574, L0.584, L0.594, L0.604, L0.614, L0.624, L0.634, L0.644, L0.654, L0.664, L0.674, L0.684, L0.694, L0.704, L0.715, L0.726, L0.737, L0.748, L0.759, L0.770, L0.781, L0.792, L0.803, L0.814, L0.825, L0.836, L0.847, L0.858, L0.869, L0.881, L0.893, L0.905, L0.917, L0.1313"
    - "  Creating 520 files"
    - "**** Simulation run 216, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.929[174,153077] 174ns |-----------------------------------------L0.929-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[174,11776] 174ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 174ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 174ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 174ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 174ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 174ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 174ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 174ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 174ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 174ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 174ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 174ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 174ns                                                                                   |L0.?| "
    - "**** Simulation run 217, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.941[175,153077] 175ns |-----------------------------------------L0.941-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[175,11776] 175ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 175ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 175ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 175ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 175ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 175ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 175ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 175ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 175ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 175ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 175ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 175ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 175ns                                                                                   |L0.?| "
    - "**** Simulation run 218, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.953[176,153077] 176ns |-----------------------------------------L0.953-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[176,11776] 176ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 176ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 176ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 176ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 176ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 176ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 176ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 176ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 176ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 176ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 176ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 176ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 176ns                                                                                   |L0.?| "
    - "**** Simulation run 219, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.965[177,153077] 177ns |-----------------------------------------L0.965-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[177,11776] 177ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 177ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 177ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 177ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 177ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 177ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 177ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 177ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 177ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 177ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 177ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 177ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 177ns                                                                                   |L0.?| "
    - "**** Simulation run 220, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.977[178,153077] 178ns |-----------------------------------------L0.977-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[178,11776] 178ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 178ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 178ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 178ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 178ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 178ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 178ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 178ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 178ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 178ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 178ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 178ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 178ns                                                                                   |L0.?| "
    - "**** Simulation run 221, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.989[179,153077] 179ns |-----------------------------------------L0.989-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[179,11776] 179ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 179ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 179ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 179ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 179ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 179ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 179ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 179ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 179ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 179ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 179ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 179ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 179ns                                                                                   |L0.?| "
    - "**** Simulation run 222, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1001[180,153077] 180ns|----------------------------------------L0.1001-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[180,11776] 180ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 180ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 180ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 180ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 180ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 180ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 180ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 180ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 180ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 180ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 180ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 180ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 180ns                                                                                   |L0.?| "
    - "**** Simulation run 223, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1013[181,153077] 181ns|----------------------------------------L0.1013-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[181,11776] 181ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 181ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 181ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 181ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 181ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 181ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 181ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 181ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 181ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 181ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 181ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 181ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 181ns                                                                                   |L0.?| "
    - "**** Simulation run 224, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1025[182,153077] 182ns|----------------------------------------L0.1025-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[182,11776] 182ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 182ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 182ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 182ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 182ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 182ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 182ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 182ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 182ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 182ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 182ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 182ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 182ns                                                                                   |L0.?| "
    - "**** Simulation run 225, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1037[183,153077] 183ns|----------------------------------------L0.1037-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[183,11776] 183ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 183ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 183ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 183ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 183ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 183ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 183ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 183ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 183ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 183ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 183ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 183ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 183ns                                                                                   |L0.?| "
    - "**** Simulation run 226, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1049[184,153077] 184ns|----------------------------------------L0.1049-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[184,11776] 184ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 184ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 184ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 184ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 184ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 184ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 184ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 184ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 184ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 184ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 184ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 184ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 184ns                                                                                   |L0.?| "
    - "**** Simulation run 227, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1088[187,153077] 187ns|----------------------------------------L0.1088-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[187,11776] 187ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 187ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 187ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 187ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 187ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 187ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 187ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 187ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 187ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 187ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 187ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 187ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 187ns                                                                                   |L0.?| "
    - "**** Simulation run 228, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1101[188,153077] 188ns|----------------------------------------L0.1101-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[188,11776] 188ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 188ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 188ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 188ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 188ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 188ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 188ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 188ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 188ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 188ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 188ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 188ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 188ns                                                                                   |L0.?| "
    - "**** Simulation run 229, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1114[189,153077] 189ns|----------------------------------------L0.1114-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[189,11776] 189ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 189ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 189ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 189ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 189ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 189ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 189ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 189ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 189ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 189ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 189ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 189ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 189ns                                                                                   |L0.?| "
    - "**** Simulation run 230, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1127[190,153077] 190ns|----------------------------------------L0.1127-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[190,11776] 190ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 190ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 190ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 190ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 190ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 190ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 190ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 190ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 190ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 190ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 190ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 190ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 190ns                                                                                   |L0.?| "
    - "**** Simulation run 231, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1140[191,153077] 191ns|----------------------------------------L0.1140-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[191,11776] 191ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 191ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 191ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 191ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 191ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 191ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 191ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 191ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 191ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 191ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 191ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 191ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 191ns                                                                                   |L0.?| "
    - "**** Simulation run 232, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1153[192,153077] 192ns|----------------------------------------L0.1153-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[192,11776] 192ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 192ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 192ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 192ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 192ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 192ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 192ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 192ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 192ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 192ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 192ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 192ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 192ns                                                                                   |L0.?| "
    - "**** Simulation run 233, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1166[193,153077] 193ns|----------------------------------------L0.1166-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[193,11776] 193ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 193ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 193ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 193ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 193ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 193ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 193ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 193ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 193ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 193ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 193ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 193ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 193ns                                                                                   |L0.?| "
    - "**** Simulation run 234, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1179[194,153077] 194ns|----------------------------------------L0.1179-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[194,11776] 194ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 194ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 194ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 194ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 194ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 194ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 194ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 194ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 194ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 194ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 194ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 194ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 194ns                                                                                   |L0.?| "
    - "**** Simulation run 235, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1062[185,153077] 185ns|----------------------------------------L0.1062-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[185,11776] 185ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 185ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 185ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 185ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 185ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 185ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 185ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 185ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 185ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 185ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 185ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 185ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 185ns                                                                                   |L0.?| "
    - "**** Simulation run 236, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1075[186,153077] 186ns|----------------------------------------L0.1075-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[186,11776] 186ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 186ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 186ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 186ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 186ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 186ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 186ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 186ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 186ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 186ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 186ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 186ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 186ns                                                                                   |L0.?| "
    - "**** Simulation run 237, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1192[195,153077] 195ns|----------------------------------------L0.1192-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[195,11776] 195ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 195ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 195ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 195ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 195ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 195ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 195ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 195ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 195ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 195ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 195ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 195ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 195ns                                                                                   |L0.?| "
    - "**** Simulation run 238, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1205[196,153077] 196ns|----------------------------------------L0.1205-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[196,11776] 196ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 196ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 196ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 196ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 196ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 196ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 196ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 196ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 196ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 196ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 196ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 196ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 196ns                                                                                   |L0.?| "
    - "**** Simulation run 239, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1218[197,153077] 197ns|----------------------------------------L0.1218-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[197,11776] 197ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 197ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 197ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 197ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 197ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 197ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 197ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 197ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 197ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 197ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 197ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 197ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 197ns                                                                                   |L0.?| "
    - "**** Simulation run 240, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1231[198,153077] 198ns|----------------------------------------L0.1231-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[198,11776] 198ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 198ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 198ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 198ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 198ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 198ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 198ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 198ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 198ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 198ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 198ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 198ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 198ns                                                                                   |L0.?| "
    - "**** Simulation run 241, type=split(HighL0OverlapTotalBacklog)(split_times=[11776, 23551, 35326, 47101, 58876, 70651, 82426, 94201, 105976, 117751, 129526, 141301]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1244[199,153077] 199ns|----------------------------------------L0.1244-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[199,11776] 199ns    |L0.?|                                                                                    "
    - "L0.?[11777,23551] 199ns        |L0.?|                                                                              "
    - "L0.?[23552,35326] 199ns               |L0.?|                                                                       "
    - "L0.?[35327,47101] 199ns                      |L0.?|                                                                "
    - "L0.?[47102,58876] 199ns                             |L0.?|                                                         "
    - "L0.?[58877,70651] 199ns                                    |L0.?|                                                  "
    - "L0.?[70652,82426] 199ns                                           |L0.?|                                           "
    - "L0.?[82427,94201] 199ns                                                  |L0.?|                                    "
    - "L0.?[94202,105976] 199ns                                                        |L0.?|                             "
    - "L0.?[105977,117751] 199ns                                                              |L0.?|                      "
    - "L0.?[117752,129526] 199ns                                                                     |L0.?|               "
    - "L0.?[129527,141301] 199ns                                                                            |L0.?|        "
    - "L0.?[141302,153077] 199ns                                                                                   |L0.?| "
    - "Committing partition 1:"
    - "  Soft Deleting 26 files: L0.929, L0.941, L0.953, L0.965, L0.977, L0.989, L0.1001, L0.1013, L0.1025, L0.1037, L0.1049, L0.1062, L0.1075, L0.1088, L0.1101, L0.1114, L0.1127, L0.1140, L0.1153, L0.1166, L0.1179, L0.1192, L0.1205, L0.1218, L0.1231, L0.1244"
    - "  Creating 338 files"
    - "**** Simulation run 242, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1325[20,11776] 134ns 12b|----------------------------------------L0.1325----------------------------------------| "
    - "L0.1338[135,11776] 135ns 0b|----------------------------------------L0.1338----------------------------------------| "
    - "L0.1351[136,11776] 136ns 0b|----------------------------------------L0.1351----------------------------------------| "
    - "L0.1364[137,11776] 137ns 0b|----------------------------------------L0.1364----------------------------------------| "
    - "L0.1377[138,11776] 138ns 0b|----------------------------------------L0.1377----------------------------------------| "
    - "L0.1390[139,11776] 139ns 0b|----------------------------------------L0.1390----------------------------------------| "
    - "L0.1403[140,11776] 140ns 0b|----------------------------------------L0.1403----------------------------------------| "
    - "L0.1416[141,11776] 141ns 0b|----------------------------------------L0.1416----------------------------------------| "
    - "L0.1429[142,11776] 142ns 0b|----------------------------------------L0.1429----------------------------------------| "
    - "L0.1442[143,11776] 143ns 0b|----------------------------------------L0.1442----------------------------------------| "
    - "L0.1455[144,11776] 144ns 0b|----------------------------------------L0.1455----------------------------------------| "
    - "L0.1468[145,11776] 145ns 0b|----------------------------------------L0.1468----------------------------------------| "
    - "L0.1481[146,11776] 146ns 0b|----------------------------------------L0.1481----------------------------------------| "
    - "L0.1494[147,11776] 147ns 0b|----------------------------------------L0.1494----------------------------------------| "
    - "L0.1507[148,11776] 148ns 0b|----------------------------------------L0.1507----------------------------------------| "
    - "L0.1520[149,11776] 149ns 0b|----------------------------------------L0.1520----------------------------------------| "
    - "L0.1533[150,11776] 150ns 0b|----------------------------------------L0.1533----------------------------------------| "
    - "L0.1546[151,11776] 151ns 0b |---------------------------------------L0.1546----------------------------------------| "
    - "L0.1559[152,11776] 152ns 0b |---------------------------------------L0.1559----------------------------------------| "
    - "L0.1572[153,11776] 153ns 0b |---------------------------------------L0.1572----------------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[20,11776] 153ns     |-----------------------------------------L0.?------------------------------------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1325, L0.1338, L0.1351, L0.1364, L0.1377, L0.1390, L0.1403, L0.1416, L0.1429, L0.1442, L0.1455, L0.1468, L0.1481, L0.1494, L0.1507, L0.1520, L0.1533, L0.1546, L0.1559, L0.1572"
    - "  Creating 1 files"
    - "**** Simulation run 243, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1326[11777,23551] 134ns 12b|----------------------------------------L0.1326-----------------------------------------|"
    - "L0.1339[11777,23551] 135ns 0b|----------------------------------------L0.1339-----------------------------------------|"
    - "L0.1352[11777,23551] 136ns 0b|----------------------------------------L0.1352-----------------------------------------|"
    - "L0.1365[11777,23551] 137ns 0b|----------------------------------------L0.1365-----------------------------------------|"
    - "L0.1378[11777,23551] 138ns 0b|----------------------------------------L0.1378-----------------------------------------|"
    - "L0.1391[11777,23551] 139ns 0b|----------------------------------------L0.1391-----------------------------------------|"
    - "L0.1404[11777,23551] 140ns 0b|----------------------------------------L0.1404-----------------------------------------|"
    - "L0.1417[11777,23551] 141ns 0b|----------------------------------------L0.1417-----------------------------------------|"
    - "L0.1430[11777,23551] 142ns 0b|----------------------------------------L0.1430-----------------------------------------|"
    - "L0.1443[11777,23551] 143ns 0b|----------------------------------------L0.1443-----------------------------------------|"
    - "L0.1456[11777,23551] 144ns 0b|----------------------------------------L0.1456-----------------------------------------|"
    - "L0.1469[11777,23551] 145ns 0b|----------------------------------------L0.1469-----------------------------------------|"
    - "L0.1482[11777,23551] 146ns 0b|----------------------------------------L0.1482-----------------------------------------|"
    - "L0.1495[11777,23551] 147ns 0b|----------------------------------------L0.1495-----------------------------------------|"
    - "L0.1508[11777,23551] 148ns 0b|----------------------------------------L0.1508-----------------------------------------|"
    - "L0.1521[11777,23551] 149ns 0b|----------------------------------------L0.1521-----------------------------------------|"
    - "L0.1534[11777,23551] 150ns 0b|----------------------------------------L0.1534-----------------------------------------|"
    - "L0.1547[11777,23551] 151ns 0b|----------------------------------------L0.1547-----------------------------------------|"
    - "L0.1560[11777,23551] 152ns 0b|----------------------------------------L0.1560-----------------------------------------|"
    - "L0.1573[11777,23551] 153ns 0b|----------------------------------------L0.1573-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[11777,23551] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1326, L0.1339, L0.1352, L0.1365, L0.1378, L0.1391, L0.1404, L0.1417, L0.1430, L0.1443, L0.1456, L0.1469, L0.1482, L0.1495, L0.1508, L0.1521, L0.1534, L0.1547, L0.1560, L0.1573"
    - "  Creating 1 files"
    - "**** Simulation run 244, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1586[11777,23551] 154ns|----------------------------------------L0.1586-----------------------------------------|"
    - "L0.1599[11777,23551] 155ns|----------------------------------------L0.1599-----------------------------------------|"
    - "L0.1612[11777,23551] 156ns|----------------------------------------L0.1612-----------------------------------------|"
    - "L0.1625[11777,23551] 157ns|----------------------------------------L0.1625-----------------------------------------|"
    - "L0.1638[11777,23551] 158ns|----------------------------------------L0.1638-----------------------------------------|"
    - "L0.1651[11777,23551] 159ns|----------------------------------------L0.1651-----------------------------------------|"
    - "L0.1664[11777,23551] 160ns|----------------------------------------L0.1664-----------------------------------------|"
    - "L0.1677[11777,23551] 161ns|----------------------------------------L0.1677-----------------------------------------|"
    - "L0.1690[11777,23551] 162ns|----------------------------------------L0.1690-----------------------------------------|"
    - "L0.1703[11777,23551] 163ns|----------------------------------------L0.1703-----------------------------------------|"
    - "L0.1716[11777,23551] 164ns|----------------------------------------L0.1716-----------------------------------------|"
    - "L0.1729[11777,23551] 165ns|----------------------------------------L0.1729-----------------------------------------|"
    - "L0.1742[11777,23551] 166ns|----------------------------------------L0.1742-----------------------------------------|"
    - "L0.1755[11777,23551] 167ns|----------------------------------------L0.1755-----------------------------------------|"
    - "L0.1768[11777,23551] 168ns|----------------------------------------L0.1768-----------------------------------------|"
    - "L0.1781[11777,23551] 169ns|----------------------------------------L0.1781-----------------------------------------|"
    - "L0.1794[11777,23551] 170ns|----------------------------------------L0.1794-----------------------------------------|"
    - "L0.1807[11777,23551] 171ns|----------------------------------------L0.1807-----------------------------------------|"
    - "L0.1820[11777,23551] 172ns|----------------------------------------L0.1820-----------------------------------------|"
    - "L0.1833[11777,23551] 173ns|----------------------------------------L0.1833-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[11777,23551] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1586, L0.1599, L0.1612, L0.1625, L0.1638, L0.1651, L0.1664, L0.1677, L0.1690, L0.1703, L0.1716, L0.1729, L0.1742, L0.1755, L0.1768, L0.1781, L0.1794, L0.1807, L0.1820, L0.1833"
    - "  Creating 1 files"
    - "**** Simulation run 245, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1846[11777,23551] 174ns|----------------------------------------L0.1846-----------------------------------------|"
    - "L0.1859[11777,23551] 175ns|----------------------------------------L0.1859-----------------------------------------|"
    - "L0.1872[11777,23551] 176ns|----------------------------------------L0.1872-----------------------------------------|"
    - "L0.1885[11777,23551] 177ns|----------------------------------------L0.1885-----------------------------------------|"
    - "L0.1898[11777,23551] 178ns|----------------------------------------L0.1898-----------------------------------------|"
    - "L0.1911[11777,23551] 179ns|----------------------------------------L0.1911-----------------------------------------|"
    - "L0.1924[11777,23551] 180ns|----------------------------------------L0.1924-----------------------------------------|"
    - "L0.1937[11777,23551] 181ns|----------------------------------------L0.1937-----------------------------------------|"
    - "L0.1950[11777,23551] 182ns|----------------------------------------L0.1950-----------------------------------------|"
    - "L0.1963[11777,23551] 183ns|----------------------------------------L0.1963-----------------------------------------|"
    - "L0.1976[11777,23551] 184ns|----------------------------------------L0.1976-----------------------------------------|"
    - "L0.2093[11777,23551] 185ns|----------------------------------------L0.2093-----------------------------------------|"
    - "L0.2106[11777,23551] 186ns|----------------------------------------L0.2106-----------------------------------------|"
    - "L0.1989[11777,23551] 187ns|----------------------------------------L0.1989-----------------------------------------|"
    - "L0.2002[11777,23551] 188ns|----------------------------------------L0.2002-----------------------------------------|"
    - "L0.2015[11777,23551] 189ns|----------------------------------------L0.2015-----------------------------------------|"
    - "L0.2028[11777,23551] 190ns|----------------------------------------L0.2028-----------------------------------------|"
    - "L0.2041[11777,23551] 191ns|----------------------------------------L0.2041-----------------------------------------|"
    - "L0.2054[11777,23551] 192ns|----------------------------------------L0.2054-----------------------------------------|"
    - "L0.2067[11777,23551] 193ns|----------------------------------------L0.2067-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[11777,23551] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1846, L0.1859, L0.1872, L0.1885, L0.1898, L0.1911, L0.1924, L0.1937, L0.1950, L0.1963, L0.1976, L0.1989, L0.2002, L0.2015, L0.2028, L0.2041, L0.2054, L0.2067, L0.2093, L0.2106"
    - "  Creating 1 files"
    - "**** Simulation run 246, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2080[11777,23551] 194ns|----------------------------------------L0.2080-----------------------------------------|"
    - "L0.2119[11777,23551] 195ns|----------------------------------------L0.2119-----------------------------------------|"
    - "L0.2132[11777,23551] 196ns|----------------------------------------L0.2132-----------------------------------------|"
    - "L0.2145[11777,23551] 197ns|----------------------------------------L0.2145-----------------------------------------|"
    - "L0.2158[11777,23551] 198ns|----------------------------------------L0.2158-----------------------------------------|"
    - "L0.2171[11777,23551] 199ns|----------------------------------------L0.2171-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[11777,23551] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2080, L0.2119, L0.2132, L0.2145, L0.2158, L0.2171"
    - "  Creating 1 files"
    - "**** Simulation run 247, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1327[23552,35326] 134ns 12b|----------------------------------------L0.1327-----------------------------------------|"
    - "L0.1340[23552,35326] 135ns 0b|----------------------------------------L0.1340-----------------------------------------|"
    - "L0.1353[23552,35326] 136ns 0b|----------------------------------------L0.1353-----------------------------------------|"
    - "L0.1366[23552,35326] 137ns 0b|----------------------------------------L0.1366-----------------------------------------|"
    - "L0.1379[23552,35326] 138ns 0b|----------------------------------------L0.1379-----------------------------------------|"
    - "L0.1392[23552,35326] 139ns 0b|----------------------------------------L0.1392-----------------------------------------|"
    - "L0.1405[23552,35326] 140ns 0b|----------------------------------------L0.1405-----------------------------------------|"
    - "L0.1418[23552,35326] 141ns 0b|----------------------------------------L0.1418-----------------------------------------|"
    - "L0.1431[23552,35326] 142ns 0b|----------------------------------------L0.1431-----------------------------------------|"
    - "L0.1444[23552,35326] 143ns 0b|----------------------------------------L0.1444-----------------------------------------|"
    - "L0.1457[23552,35326] 144ns 0b|----------------------------------------L0.1457-----------------------------------------|"
    - "L0.1470[23552,35326] 145ns 0b|----------------------------------------L0.1470-----------------------------------------|"
    - "L0.1483[23552,35326] 146ns 0b|----------------------------------------L0.1483-----------------------------------------|"
    - "L0.1496[23552,35326] 147ns 0b|----------------------------------------L0.1496-----------------------------------------|"
    - "L0.1509[23552,35326] 148ns 0b|----------------------------------------L0.1509-----------------------------------------|"
    - "L0.1522[23552,35326] 149ns 0b|----------------------------------------L0.1522-----------------------------------------|"
    - "L0.1535[23552,35326] 150ns 0b|----------------------------------------L0.1535-----------------------------------------|"
    - "L0.1548[23552,35326] 151ns 0b|----------------------------------------L0.1548-----------------------------------------|"
    - "L0.1561[23552,35326] 152ns 0b|----------------------------------------L0.1561-----------------------------------------|"
    - "L0.1574[23552,35326] 153ns 0b|----------------------------------------L0.1574-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[23552,35326] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1327, L0.1340, L0.1353, L0.1366, L0.1379, L0.1392, L0.1405, L0.1418, L0.1431, L0.1444, L0.1457, L0.1470, L0.1483, L0.1496, L0.1509, L0.1522, L0.1535, L0.1548, L0.1561, L0.1574"
    - "  Creating 1 files"
    - "**** Simulation run 248, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1587[23552,35326] 154ns|----------------------------------------L0.1587-----------------------------------------|"
    - "L0.1600[23552,35326] 155ns|----------------------------------------L0.1600-----------------------------------------|"
    - "L0.1613[23552,35326] 156ns|----------------------------------------L0.1613-----------------------------------------|"
    - "L0.1626[23552,35326] 157ns|----------------------------------------L0.1626-----------------------------------------|"
    - "L0.1639[23552,35326] 158ns|----------------------------------------L0.1639-----------------------------------------|"
    - "L0.1652[23552,35326] 159ns|----------------------------------------L0.1652-----------------------------------------|"
    - "L0.1665[23552,35326] 160ns|----------------------------------------L0.1665-----------------------------------------|"
    - "L0.1678[23552,35326] 161ns|----------------------------------------L0.1678-----------------------------------------|"
    - "L0.1691[23552,35326] 162ns|----------------------------------------L0.1691-----------------------------------------|"
    - "L0.1704[23552,35326] 163ns|----------------------------------------L0.1704-----------------------------------------|"
    - "L0.1717[23552,35326] 164ns|----------------------------------------L0.1717-----------------------------------------|"
    - "L0.1730[23552,35326] 165ns|----------------------------------------L0.1730-----------------------------------------|"
    - "L0.1743[23552,35326] 166ns|----------------------------------------L0.1743-----------------------------------------|"
    - "L0.1756[23552,35326] 167ns|----------------------------------------L0.1756-----------------------------------------|"
    - "L0.1769[23552,35326] 168ns|----------------------------------------L0.1769-----------------------------------------|"
    - "L0.1782[23552,35326] 169ns|----------------------------------------L0.1782-----------------------------------------|"
    - "L0.1795[23552,35326] 170ns|----------------------------------------L0.1795-----------------------------------------|"
    - "L0.1808[23552,35326] 171ns|----------------------------------------L0.1808-----------------------------------------|"
    - "L0.1821[23552,35326] 172ns|----------------------------------------L0.1821-----------------------------------------|"
    - "L0.1834[23552,35326] 173ns|----------------------------------------L0.1834-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[23552,35326] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1587, L0.1600, L0.1613, L0.1626, L0.1639, L0.1652, L0.1665, L0.1678, L0.1691, L0.1704, L0.1717, L0.1730, L0.1743, L0.1756, L0.1769, L0.1782, L0.1795, L0.1808, L0.1821, L0.1834"
    - "  Creating 1 files"
    - "**** Simulation run 249, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1847[23552,35326] 174ns|----------------------------------------L0.1847-----------------------------------------|"
    - "L0.1860[23552,35326] 175ns|----------------------------------------L0.1860-----------------------------------------|"
    - "L0.1873[23552,35326] 176ns|----------------------------------------L0.1873-----------------------------------------|"
    - "L0.1886[23552,35326] 177ns|----------------------------------------L0.1886-----------------------------------------|"
    - "L0.1899[23552,35326] 178ns|----------------------------------------L0.1899-----------------------------------------|"
    - "L0.1912[23552,35326] 179ns|----------------------------------------L0.1912-----------------------------------------|"
    - "L0.1925[23552,35326] 180ns|----------------------------------------L0.1925-----------------------------------------|"
    - "L0.1938[23552,35326] 181ns|----------------------------------------L0.1938-----------------------------------------|"
    - "L0.1951[23552,35326] 182ns|----------------------------------------L0.1951-----------------------------------------|"
    - "L0.1964[23552,35326] 183ns|----------------------------------------L0.1964-----------------------------------------|"
    - "L0.1977[23552,35326] 184ns|----------------------------------------L0.1977-----------------------------------------|"
    - "L0.2094[23552,35326] 185ns|----------------------------------------L0.2094-----------------------------------------|"
    - "L0.2107[23552,35326] 186ns|----------------------------------------L0.2107-----------------------------------------|"
    - "L0.1990[23552,35326] 187ns|----------------------------------------L0.1990-----------------------------------------|"
    - "L0.2003[23552,35326] 188ns|----------------------------------------L0.2003-----------------------------------------|"
    - "L0.2016[23552,35326] 189ns|----------------------------------------L0.2016-----------------------------------------|"
    - "L0.2029[23552,35326] 190ns|----------------------------------------L0.2029-----------------------------------------|"
    - "L0.2042[23552,35326] 191ns|----------------------------------------L0.2042-----------------------------------------|"
    - "L0.2055[23552,35326] 192ns|----------------------------------------L0.2055-----------------------------------------|"
    - "L0.2068[23552,35326] 193ns|----------------------------------------L0.2068-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[23552,35326] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1847, L0.1860, L0.1873, L0.1886, L0.1899, L0.1912, L0.1925, L0.1938, L0.1951, L0.1964, L0.1977, L0.1990, L0.2003, L0.2016, L0.2029, L0.2042, L0.2055, L0.2068, L0.2094, L0.2107"
    - "  Creating 1 files"
    - "**** Simulation run 250, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2081[23552,35326] 194ns|----------------------------------------L0.2081-----------------------------------------|"
    - "L0.2120[23552,35326] 195ns|----------------------------------------L0.2120-----------------------------------------|"
    - "L0.2133[23552,35326] 196ns|----------------------------------------L0.2133-----------------------------------------|"
    - "L0.2146[23552,35326] 197ns|----------------------------------------L0.2146-----------------------------------------|"
    - "L0.2159[23552,35326] 198ns|----------------------------------------L0.2159-----------------------------------------|"
    - "L0.2172[23552,35326] 199ns|----------------------------------------L0.2172-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[23552,35326] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2081, L0.2120, L0.2133, L0.2146, L0.2159, L0.2172"
    - "  Creating 1 files"
    - "**** Simulation run 251, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1328[35327,47101] 134ns 12b|----------------------------------------L0.1328-----------------------------------------|"
    - "L0.1341[35327,47101] 135ns 0b|----------------------------------------L0.1341-----------------------------------------|"
    - "L0.1354[35327,47101] 136ns 0b|----------------------------------------L0.1354-----------------------------------------|"
    - "L0.1367[35327,47101] 137ns 0b|----------------------------------------L0.1367-----------------------------------------|"
    - "L0.1380[35327,47101] 138ns 0b|----------------------------------------L0.1380-----------------------------------------|"
    - "L0.1393[35327,47101] 139ns 0b|----------------------------------------L0.1393-----------------------------------------|"
    - "L0.1406[35327,47101] 140ns 0b|----------------------------------------L0.1406-----------------------------------------|"
    - "L0.1419[35327,47101] 141ns 0b|----------------------------------------L0.1419-----------------------------------------|"
    - "L0.1432[35327,47101] 142ns 0b|----------------------------------------L0.1432-----------------------------------------|"
    - "L0.1445[35327,47101] 143ns 0b|----------------------------------------L0.1445-----------------------------------------|"
    - "L0.1458[35327,47101] 144ns 0b|----------------------------------------L0.1458-----------------------------------------|"
    - "L0.1471[35327,47101] 145ns 0b|----------------------------------------L0.1471-----------------------------------------|"
    - "L0.1484[35327,47101] 146ns 0b|----------------------------------------L0.1484-----------------------------------------|"
    - "L0.1497[35327,47101] 147ns 0b|----------------------------------------L0.1497-----------------------------------------|"
    - "L0.1510[35327,47101] 148ns 0b|----------------------------------------L0.1510-----------------------------------------|"
    - "L0.1523[35327,47101] 149ns 0b|----------------------------------------L0.1523-----------------------------------------|"
    - "L0.1536[35327,47101] 150ns 0b|----------------------------------------L0.1536-----------------------------------------|"
    - "L0.1549[35327,47101] 151ns 0b|----------------------------------------L0.1549-----------------------------------------|"
    - "L0.1562[35327,47101] 152ns 0b|----------------------------------------L0.1562-----------------------------------------|"
    - "L0.1575[35327,47101] 153ns 0b|----------------------------------------L0.1575-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[35327,47101] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1328, L0.1341, L0.1354, L0.1367, L0.1380, L0.1393, L0.1406, L0.1419, L0.1432, L0.1445, L0.1458, L0.1471, L0.1484, L0.1497, L0.1510, L0.1523, L0.1536, L0.1549, L0.1562, L0.1575"
    - "  Creating 1 files"
    - "**** Simulation run 252, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1588[35327,47101] 154ns|----------------------------------------L0.1588-----------------------------------------|"
    - "L0.1601[35327,47101] 155ns|----------------------------------------L0.1601-----------------------------------------|"
    - "L0.1614[35327,47101] 156ns|----------------------------------------L0.1614-----------------------------------------|"
    - "L0.1627[35327,47101] 157ns|----------------------------------------L0.1627-----------------------------------------|"
    - "L0.1640[35327,47101] 158ns|----------------------------------------L0.1640-----------------------------------------|"
    - "L0.1653[35327,47101] 159ns|----------------------------------------L0.1653-----------------------------------------|"
    - "L0.1666[35327,47101] 160ns|----------------------------------------L0.1666-----------------------------------------|"
    - "L0.1679[35327,47101] 161ns|----------------------------------------L0.1679-----------------------------------------|"
    - "L0.1692[35327,47101] 162ns|----------------------------------------L0.1692-----------------------------------------|"
    - "L0.1705[35327,47101] 163ns|----------------------------------------L0.1705-----------------------------------------|"
    - "L0.1718[35327,47101] 164ns|----------------------------------------L0.1718-----------------------------------------|"
    - "L0.1731[35327,47101] 165ns|----------------------------------------L0.1731-----------------------------------------|"
    - "L0.1744[35327,47101] 166ns|----------------------------------------L0.1744-----------------------------------------|"
    - "L0.1757[35327,47101] 167ns|----------------------------------------L0.1757-----------------------------------------|"
    - "L0.1770[35327,47101] 168ns|----------------------------------------L0.1770-----------------------------------------|"
    - "L0.1783[35327,47101] 169ns|----------------------------------------L0.1783-----------------------------------------|"
    - "L0.1796[35327,47101] 170ns|----------------------------------------L0.1796-----------------------------------------|"
    - "L0.1809[35327,47101] 171ns|----------------------------------------L0.1809-----------------------------------------|"
    - "L0.1822[35327,47101] 172ns|----------------------------------------L0.1822-----------------------------------------|"
    - "L0.1835[35327,47101] 173ns|----------------------------------------L0.1835-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[35327,47101] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1588, L0.1601, L0.1614, L0.1627, L0.1640, L0.1653, L0.1666, L0.1679, L0.1692, L0.1705, L0.1718, L0.1731, L0.1744, L0.1757, L0.1770, L0.1783, L0.1796, L0.1809, L0.1822, L0.1835"
    - "  Creating 1 files"
    - "**** Simulation run 253, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1848[35327,47101] 174ns|----------------------------------------L0.1848-----------------------------------------|"
    - "L0.1861[35327,47101] 175ns|----------------------------------------L0.1861-----------------------------------------|"
    - "L0.1874[35327,47101] 176ns|----------------------------------------L0.1874-----------------------------------------|"
    - "L0.1887[35327,47101] 177ns|----------------------------------------L0.1887-----------------------------------------|"
    - "L0.1900[35327,47101] 178ns|----------------------------------------L0.1900-----------------------------------------|"
    - "L0.1913[35327,47101] 179ns|----------------------------------------L0.1913-----------------------------------------|"
    - "L0.1926[35327,47101] 180ns|----------------------------------------L0.1926-----------------------------------------|"
    - "L0.1939[35327,47101] 181ns|----------------------------------------L0.1939-----------------------------------------|"
    - "L0.1952[35327,47101] 182ns|----------------------------------------L0.1952-----------------------------------------|"
    - "L0.1965[35327,47101] 183ns|----------------------------------------L0.1965-----------------------------------------|"
    - "L0.1978[35327,47101] 184ns|----------------------------------------L0.1978-----------------------------------------|"
    - "L0.2095[35327,47101] 185ns|----------------------------------------L0.2095-----------------------------------------|"
    - "L0.2108[35327,47101] 186ns|----------------------------------------L0.2108-----------------------------------------|"
    - "L0.1991[35327,47101] 187ns|----------------------------------------L0.1991-----------------------------------------|"
    - "L0.2004[35327,47101] 188ns|----------------------------------------L0.2004-----------------------------------------|"
    - "L0.2017[35327,47101] 189ns|----------------------------------------L0.2017-----------------------------------------|"
    - "L0.2030[35327,47101] 190ns|----------------------------------------L0.2030-----------------------------------------|"
    - "L0.2043[35327,47101] 191ns|----------------------------------------L0.2043-----------------------------------------|"
    - "L0.2056[35327,47101] 192ns|----------------------------------------L0.2056-----------------------------------------|"
    - "L0.2069[35327,47101] 193ns|----------------------------------------L0.2069-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[35327,47101] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1848, L0.1861, L0.1874, L0.1887, L0.1900, L0.1913, L0.1926, L0.1939, L0.1952, L0.1965, L0.1978, L0.1991, L0.2004, L0.2017, L0.2030, L0.2043, L0.2056, L0.2069, L0.2095, L0.2108"
    - "  Creating 1 files"
    - "**** Simulation run 254, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2082[35327,47101] 194ns|----------------------------------------L0.2082-----------------------------------------|"
    - "L0.2121[35327,47101] 195ns|----------------------------------------L0.2121-----------------------------------------|"
    - "L0.2134[35327,47101] 196ns|----------------------------------------L0.2134-----------------------------------------|"
    - "L0.2147[35327,47101] 197ns|----------------------------------------L0.2147-----------------------------------------|"
    - "L0.2160[35327,47101] 198ns|----------------------------------------L0.2160-----------------------------------------|"
    - "L0.2173[35327,47101] 199ns|----------------------------------------L0.2173-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[35327,47101] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2082, L0.2121, L0.2134, L0.2147, L0.2160, L0.2173"
    - "  Creating 1 files"
    - "**** Simulation run 255, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1329[47102,58876] 134ns 12b|----------------------------------------L0.1329-----------------------------------------|"
    - "L0.1342[47102,58876] 135ns 0b|----------------------------------------L0.1342-----------------------------------------|"
    - "L0.1355[47102,58876] 136ns 0b|----------------------------------------L0.1355-----------------------------------------|"
    - "L0.1368[47102,58876] 137ns 0b|----------------------------------------L0.1368-----------------------------------------|"
    - "L0.1381[47102,58876] 138ns 0b|----------------------------------------L0.1381-----------------------------------------|"
    - "L0.1394[47102,58876] 139ns 0b|----------------------------------------L0.1394-----------------------------------------|"
    - "L0.1407[47102,58876] 140ns 0b|----------------------------------------L0.1407-----------------------------------------|"
    - "L0.1420[47102,58876] 141ns 0b|----------------------------------------L0.1420-----------------------------------------|"
    - "L0.1433[47102,58876] 142ns 0b|----------------------------------------L0.1433-----------------------------------------|"
    - "L0.1446[47102,58876] 143ns 0b|----------------------------------------L0.1446-----------------------------------------|"
    - "L0.1459[47102,58876] 144ns 0b|----------------------------------------L0.1459-----------------------------------------|"
    - "L0.1472[47102,58876] 145ns 0b|----------------------------------------L0.1472-----------------------------------------|"
    - "L0.1485[47102,58876] 146ns 0b|----------------------------------------L0.1485-----------------------------------------|"
    - "L0.1498[47102,58876] 147ns 0b|----------------------------------------L0.1498-----------------------------------------|"
    - "L0.1511[47102,58876] 148ns 0b|----------------------------------------L0.1511-----------------------------------------|"
    - "L0.1524[47102,58876] 149ns 0b|----------------------------------------L0.1524-----------------------------------------|"
    - "L0.1537[47102,58876] 150ns 0b|----------------------------------------L0.1537-----------------------------------------|"
    - "L0.1550[47102,58876] 151ns 0b|----------------------------------------L0.1550-----------------------------------------|"
    - "L0.1563[47102,58876] 152ns 0b|----------------------------------------L0.1563-----------------------------------------|"
    - "L0.1576[47102,58876] 153ns 0b|----------------------------------------L0.1576-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[47102,58876] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1329, L0.1342, L0.1355, L0.1368, L0.1381, L0.1394, L0.1407, L0.1420, L0.1433, L0.1446, L0.1459, L0.1472, L0.1485, L0.1498, L0.1511, L0.1524, L0.1537, L0.1550, L0.1563, L0.1576"
    - "  Creating 1 files"
    - "**** Simulation run 256, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1589[47102,58876] 154ns|----------------------------------------L0.1589-----------------------------------------|"
    - "L0.1602[47102,58876] 155ns|----------------------------------------L0.1602-----------------------------------------|"
    - "L0.1615[47102,58876] 156ns|----------------------------------------L0.1615-----------------------------------------|"
    - "L0.1628[47102,58876] 157ns|----------------------------------------L0.1628-----------------------------------------|"
    - "L0.1641[47102,58876] 158ns|----------------------------------------L0.1641-----------------------------------------|"
    - "L0.1654[47102,58876] 159ns|----------------------------------------L0.1654-----------------------------------------|"
    - "L0.1667[47102,58876] 160ns|----------------------------------------L0.1667-----------------------------------------|"
    - "L0.1680[47102,58876] 161ns|----------------------------------------L0.1680-----------------------------------------|"
    - "L0.1693[47102,58876] 162ns|----------------------------------------L0.1693-----------------------------------------|"
    - "L0.1706[47102,58876] 163ns|----------------------------------------L0.1706-----------------------------------------|"
    - "L0.1719[47102,58876] 164ns|----------------------------------------L0.1719-----------------------------------------|"
    - "L0.1732[47102,58876] 165ns|----------------------------------------L0.1732-----------------------------------------|"
    - "L0.1745[47102,58876] 166ns|----------------------------------------L0.1745-----------------------------------------|"
    - "L0.1758[47102,58876] 167ns|----------------------------------------L0.1758-----------------------------------------|"
    - "L0.1771[47102,58876] 168ns|----------------------------------------L0.1771-----------------------------------------|"
    - "L0.1784[47102,58876] 169ns|----------------------------------------L0.1784-----------------------------------------|"
    - "L0.1797[47102,58876] 170ns|----------------------------------------L0.1797-----------------------------------------|"
    - "L0.1810[47102,58876] 171ns|----------------------------------------L0.1810-----------------------------------------|"
    - "L0.1823[47102,58876] 172ns|----------------------------------------L0.1823-----------------------------------------|"
    - "L0.1836[47102,58876] 173ns|----------------------------------------L0.1836-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[47102,58876] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1589, L0.1602, L0.1615, L0.1628, L0.1641, L0.1654, L0.1667, L0.1680, L0.1693, L0.1706, L0.1719, L0.1732, L0.1745, L0.1758, L0.1771, L0.1784, L0.1797, L0.1810, L0.1823, L0.1836"
    - "  Creating 1 files"
    - "**** Simulation run 257, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1849[47102,58876] 174ns|----------------------------------------L0.1849-----------------------------------------|"
    - "L0.1862[47102,58876] 175ns|----------------------------------------L0.1862-----------------------------------------|"
    - "L0.1875[47102,58876] 176ns|----------------------------------------L0.1875-----------------------------------------|"
    - "L0.1888[47102,58876] 177ns|----------------------------------------L0.1888-----------------------------------------|"
    - "L0.1901[47102,58876] 178ns|----------------------------------------L0.1901-----------------------------------------|"
    - "L0.1914[47102,58876] 179ns|----------------------------------------L0.1914-----------------------------------------|"
    - "L0.1927[47102,58876] 180ns|----------------------------------------L0.1927-----------------------------------------|"
    - "L0.1940[47102,58876] 181ns|----------------------------------------L0.1940-----------------------------------------|"
    - "L0.1953[47102,58876] 182ns|----------------------------------------L0.1953-----------------------------------------|"
    - "L0.1966[47102,58876] 183ns|----------------------------------------L0.1966-----------------------------------------|"
    - "L0.1979[47102,58876] 184ns|----------------------------------------L0.1979-----------------------------------------|"
    - "L0.2096[47102,58876] 185ns|----------------------------------------L0.2096-----------------------------------------|"
    - "L0.2109[47102,58876] 186ns|----------------------------------------L0.2109-----------------------------------------|"
    - "L0.1992[47102,58876] 187ns|----------------------------------------L0.1992-----------------------------------------|"
    - "L0.2005[47102,58876] 188ns|----------------------------------------L0.2005-----------------------------------------|"
    - "L0.2018[47102,58876] 189ns|----------------------------------------L0.2018-----------------------------------------|"
    - "L0.2031[47102,58876] 190ns|----------------------------------------L0.2031-----------------------------------------|"
    - "L0.2044[47102,58876] 191ns|----------------------------------------L0.2044-----------------------------------------|"
    - "L0.2057[47102,58876] 192ns|----------------------------------------L0.2057-----------------------------------------|"
    - "L0.2070[47102,58876] 193ns|----------------------------------------L0.2070-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[47102,58876] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1849, L0.1862, L0.1875, L0.1888, L0.1901, L0.1914, L0.1927, L0.1940, L0.1953, L0.1966, L0.1979, L0.1992, L0.2005, L0.2018, L0.2031, L0.2044, L0.2057, L0.2070, L0.2096, L0.2109"
    - "  Creating 1 files"
    - "**** Simulation run 258, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2083[47102,58876] 194ns|----------------------------------------L0.2083-----------------------------------------|"
    - "L0.2122[47102,58876] 195ns|----------------------------------------L0.2122-----------------------------------------|"
    - "L0.2135[47102,58876] 196ns|----------------------------------------L0.2135-----------------------------------------|"
    - "L0.2148[47102,58876] 197ns|----------------------------------------L0.2148-----------------------------------------|"
    - "L0.2161[47102,58876] 198ns|----------------------------------------L0.2161-----------------------------------------|"
    - "L0.2174[47102,58876] 199ns|----------------------------------------L0.2174-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[47102,58876] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2083, L0.2122, L0.2135, L0.2148, L0.2161, L0.2174"
    - "  Creating 1 files"
    - "**** Simulation run 259, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1330[58877,70651] 134ns 12b|----------------------------------------L0.1330-----------------------------------------|"
    - "L0.1343[58877,70651] 135ns 0b|----------------------------------------L0.1343-----------------------------------------|"
    - "L0.1356[58877,70651] 136ns 0b|----------------------------------------L0.1356-----------------------------------------|"
    - "L0.1369[58877,70651] 137ns 0b|----------------------------------------L0.1369-----------------------------------------|"
    - "L0.1382[58877,70651] 138ns 0b|----------------------------------------L0.1382-----------------------------------------|"
    - "L0.1395[58877,70651] 139ns 0b|----------------------------------------L0.1395-----------------------------------------|"
    - "L0.1408[58877,70651] 140ns 0b|----------------------------------------L0.1408-----------------------------------------|"
    - "L0.1421[58877,70651] 141ns 0b|----------------------------------------L0.1421-----------------------------------------|"
    - "L0.1434[58877,70651] 142ns 0b|----------------------------------------L0.1434-----------------------------------------|"
    - "L0.1447[58877,70651] 143ns 0b|----------------------------------------L0.1447-----------------------------------------|"
    - "L0.1460[58877,70651] 144ns 0b|----------------------------------------L0.1460-----------------------------------------|"
    - "L0.1473[58877,70651] 145ns 0b|----------------------------------------L0.1473-----------------------------------------|"
    - "L0.1486[58877,70651] 146ns 0b|----------------------------------------L0.1486-----------------------------------------|"
    - "L0.1499[58877,70651] 147ns 0b|----------------------------------------L0.1499-----------------------------------------|"
    - "L0.1512[58877,70651] 148ns 0b|----------------------------------------L0.1512-----------------------------------------|"
    - "L0.1525[58877,70651] 149ns 0b|----------------------------------------L0.1525-----------------------------------------|"
    - "L0.1538[58877,70651] 150ns 0b|----------------------------------------L0.1538-----------------------------------------|"
    - "L0.1551[58877,70651] 151ns 0b|----------------------------------------L0.1551-----------------------------------------|"
    - "L0.1564[58877,70651] 152ns 0b|----------------------------------------L0.1564-----------------------------------------|"
    - "L0.1577[58877,70651] 153ns 0b|----------------------------------------L0.1577-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[58877,70651] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1330, L0.1343, L0.1356, L0.1369, L0.1382, L0.1395, L0.1408, L0.1421, L0.1434, L0.1447, L0.1460, L0.1473, L0.1486, L0.1499, L0.1512, L0.1525, L0.1538, L0.1551, L0.1564, L0.1577"
    - "  Creating 1 files"
    - "**** Simulation run 260, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1590[58877,70651] 154ns|----------------------------------------L0.1590-----------------------------------------|"
    - "L0.1603[58877,70651] 155ns|----------------------------------------L0.1603-----------------------------------------|"
    - "L0.1616[58877,70651] 156ns|----------------------------------------L0.1616-----------------------------------------|"
    - "L0.1629[58877,70651] 157ns|----------------------------------------L0.1629-----------------------------------------|"
    - "L0.1642[58877,70651] 158ns|----------------------------------------L0.1642-----------------------------------------|"
    - "L0.1655[58877,70651] 159ns|----------------------------------------L0.1655-----------------------------------------|"
    - "L0.1668[58877,70651] 160ns|----------------------------------------L0.1668-----------------------------------------|"
    - "L0.1681[58877,70651] 161ns|----------------------------------------L0.1681-----------------------------------------|"
    - "L0.1694[58877,70651] 162ns|----------------------------------------L0.1694-----------------------------------------|"
    - "L0.1707[58877,70651] 163ns|----------------------------------------L0.1707-----------------------------------------|"
    - "L0.1720[58877,70651] 164ns|----------------------------------------L0.1720-----------------------------------------|"
    - "L0.1733[58877,70651] 165ns|----------------------------------------L0.1733-----------------------------------------|"
    - "L0.1746[58877,70651] 166ns|----------------------------------------L0.1746-----------------------------------------|"
    - "L0.1759[58877,70651] 167ns|----------------------------------------L0.1759-----------------------------------------|"
    - "L0.1772[58877,70651] 168ns|----------------------------------------L0.1772-----------------------------------------|"
    - "L0.1785[58877,70651] 169ns|----------------------------------------L0.1785-----------------------------------------|"
    - "L0.1798[58877,70651] 170ns|----------------------------------------L0.1798-----------------------------------------|"
    - "L0.1811[58877,70651] 171ns|----------------------------------------L0.1811-----------------------------------------|"
    - "L0.1824[58877,70651] 172ns|----------------------------------------L0.1824-----------------------------------------|"
    - "L0.1837[58877,70651] 173ns|----------------------------------------L0.1837-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[58877,70651] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1590, L0.1603, L0.1616, L0.1629, L0.1642, L0.1655, L0.1668, L0.1681, L0.1694, L0.1707, L0.1720, L0.1733, L0.1746, L0.1759, L0.1772, L0.1785, L0.1798, L0.1811, L0.1824, L0.1837"
    - "  Creating 1 files"
    - "**** Simulation run 261, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1850[58877,70651] 174ns|----------------------------------------L0.1850-----------------------------------------|"
    - "L0.1863[58877,70651] 175ns|----------------------------------------L0.1863-----------------------------------------|"
    - "L0.1876[58877,70651] 176ns|----------------------------------------L0.1876-----------------------------------------|"
    - "L0.1889[58877,70651] 177ns|----------------------------------------L0.1889-----------------------------------------|"
    - "L0.1902[58877,70651] 178ns|----------------------------------------L0.1902-----------------------------------------|"
    - "L0.1915[58877,70651] 179ns|----------------------------------------L0.1915-----------------------------------------|"
    - "L0.1928[58877,70651] 180ns|----------------------------------------L0.1928-----------------------------------------|"
    - "L0.1941[58877,70651] 181ns|----------------------------------------L0.1941-----------------------------------------|"
    - "L0.1954[58877,70651] 182ns|----------------------------------------L0.1954-----------------------------------------|"
    - "L0.1967[58877,70651] 183ns|----------------------------------------L0.1967-----------------------------------------|"
    - "L0.1980[58877,70651] 184ns|----------------------------------------L0.1980-----------------------------------------|"
    - "L0.2097[58877,70651] 185ns|----------------------------------------L0.2097-----------------------------------------|"
    - "L0.2110[58877,70651] 186ns|----------------------------------------L0.2110-----------------------------------------|"
    - "L0.1993[58877,70651] 187ns|----------------------------------------L0.1993-----------------------------------------|"
    - "L0.2006[58877,70651] 188ns|----------------------------------------L0.2006-----------------------------------------|"
    - "L0.2019[58877,70651] 189ns|----------------------------------------L0.2019-----------------------------------------|"
    - "L0.2032[58877,70651] 190ns|----------------------------------------L0.2032-----------------------------------------|"
    - "L0.2045[58877,70651] 191ns|----------------------------------------L0.2045-----------------------------------------|"
    - "L0.2058[58877,70651] 192ns|----------------------------------------L0.2058-----------------------------------------|"
    - "L0.2071[58877,70651] 193ns|----------------------------------------L0.2071-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[58877,70651] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1850, L0.1863, L0.1876, L0.1889, L0.1902, L0.1915, L0.1928, L0.1941, L0.1954, L0.1967, L0.1980, L0.1993, L0.2006, L0.2019, L0.2032, L0.2045, L0.2058, L0.2071, L0.2097, L0.2110"
    - "  Creating 1 files"
    - "**** Simulation run 262, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2084[58877,70651] 194ns|----------------------------------------L0.2084-----------------------------------------|"
    - "L0.2123[58877,70651] 195ns|----------------------------------------L0.2123-----------------------------------------|"
    - "L0.2136[58877,70651] 196ns|----------------------------------------L0.2136-----------------------------------------|"
    - "L0.2149[58877,70651] 197ns|----------------------------------------L0.2149-----------------------------------------|"
    - "L0.2162[58877,70651] 198ns|----------------------------------------L0.2162-----------------------------------------|"
    - "L0.2175[58877,70651] 199ns|----------------------------------------L0.2175-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[58877,70651] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2084, L0.2123, L0.2136, L0.2149, L0.2162, L0.2175"
    - "  Creating 1 files"
    - "**** Simulation run 263, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1331[70652,82426] 134ns 12b|----------------------------------------L0.1331-----------------------------------------|"
    - "L0.1344[70652,82426] 135ns 0b|----------------------------------------L0.1344-----------------------------------------|"
    - "L0.1357[70652,82426] 136ns 0b|----------------------------------------L0.1357-----------------------------------------|"
    - "L0.1370[70652,82426] 137ns 0b|----------------------------------------L0.1370-----------------------------------------|"
    - "L0.1383[70652,82426] 138ns 0b|----------------------------------------L0.1383-----------------------------------------|"
    - "L0.1396[70652,82426] 139ns 0b|----------------------------------------L0.1396-----------------------------------------|"
    - "L0.1409[70652,82426] 140ns 0b|----------------------------------------L0.1409-----------------------------------------|"
    - "L0.1422[70652,82426] 141ns 0b|----------------------------------------L0.1422-----------------------------------------|"
    - "L0.1435[70652,82426] 142ns 0b|----------------------------------------L0.1435-----------------------------------------|"
    - "L0.1448[70652,82426] 143ns 0b|----------------------------------------L0.1448-----------------------------------------|"
    - "L0.1461[70652,82426] 144ns 0b|----------------------------------------L0.1461-----------------------------------------|"
    - "L0.1474[70652,82426] 145ns 0b|----------------------------------------L0.1474-----------------------------------------|"
    - "L0.1487[70652,82426] 146ns 0b|----------------------------------------L0.1487-----------------------------------------|"
    - "L0.1500[70652,82426] 147ns 0b|----------------------------------------L0.1500-----------------------------------------|"
    - "L0.1513[70652,82426] 148ns 0b|----------------------------------------L0.1513-----------------------------------------|"
    - "L0.1526[70652,82426] 149ns 0b|----------------------------------------L0.1526-----------------------------------------|"
    - "L0.1539[70652,82426] 150ns 0b|----------------------------------------L0.1539-----------------------------------------|"
    - "L0.1552[70652,82426] 151ns 0b|----------------------------------------L0.1552-----------------------------------------|"
    - "L0.1565[70652,82426] 152ns 0b|----------------------------------------L0.1565-----------------------------------------|"
    - "L0.1578[70652,82426] 153ns 0b|----------------------------------------L0.1578-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[70652,82426] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1331, L0.1344, L0.1357, L0.1370, L0.1383, L0.1396, L0.1409, L0.1422, L0.1435, L0.1448, L0.1461, L0.1474, L0.1487, L0.1500, L0.1513, L0.1526, L0.1539, L0.1552, L0.1565, L0.1578"
    - "  Creating 1 files"
    - "**** Simulation run 264, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1851[70652,82426] 174ns|----------------------------------------L0.1851-----------------------------------------|"
    - "L0.1864[70652,82426] 175ns|----------------------------------------L0.1864-----------------------------------------|"
    - "L0.1877[70652,82426] 176ns|----------------------------------------L0.1877-----------------------------------------|"
    - "L0.1890[70652,82426] 177ns|----------------------------------------L0.1890-----------------------------------------|"
    - "L0.1903[70652,82426] 178ns|----------------------------------------L0.1903-----------------------------------------|"
    - "L0.1916[70652,82426] 179ns|----------------------------------------L0.1916-----------------------------------------|"
    - "L0.1929[70652,82426] 180ns|----------------------------------------L0.1929-----------------------------------------|"
    - "L0.1942[70652,82426] 181ns|----------------------------------------L0.1942-----------------------------------------|"
    - "L0.1955[70652,82426] 182ns|----------------------------------------L0.1955-----------------------------------------|"
    - "L0.1968[70652,82426] 183ns|----------------------------------------L0.1968-----------------------------------------|"
    - "L0.1981[70652,82426] 184ns|----------------------------------------L0.1981-----------------------------------------|"
    - "L0.2098[70652,82426] 185ns|----------------------------------------L0.2098-----------------------------------------|"
    - "L0.2111[70652,82426] 186ns|----------------------------------------L0.2111-----------------------------------------|"
    - "L0.1994[70652,82426] 187ns|----------------------------------------L0.1994-----------------------------------------|"
    - "L0.2007[70652,82426] 188ns|----------------------------------------L0.2007-----------------------------------------|"
    - "L0.2020[70652,82426] 189ns|----------------------------------------L0.2020-----------------------------------------|"
    - "L0.2033[70652,82426] 190ns|----------------------------------------L0.2033-----------------------------------------|"
    - "L0.2046[70652,82426] 191ns|----------------------------------------L0.2046-----------------------------------------|"
    - "L0.2059[70652,82426] 192ns|----------------------------------------L0.2059-----------------------------------------|"
    - "L0.2072[70652,82426] 193ns|----------------------------------------L0.2072-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[70652,82426] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1851, L0.1864, L0.1877, L0.1890, L0.1903, L0.1916, L0.1929, L0.1942, L0.1955, L0.1968, L0.1981, L0.1994, L0.2007, L0.2020, L0.2033, L0.2046, L0.2059, L0.2072, L0.2098, L0.2111"
    - "  Creating 1 files"
    - "**** Simulation run 265, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2085[70652,82426] 194ns|----------------------------------------L0.2085-----------------------------------------|"
    - "L0.2124[70652,82426] 195ns|----------------------------------------L0.2124-----------------------------------------|"
    - "L0.2137[70652,82426] 196ns|----------------------------------------L0.2137-----------------------------------------|"
    - "L0.2150[70652,82426] 197ns|----------------------------------------L0.2150-----------------------------------------|"
    - "L0.2163[70652,82426] 198ns|----------------------------------------L0.2163-----------------------------------------|"
    - "L0.2176[70652,82426] 199ns|----------------------------------------L0.2176-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[70652,82426] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2085, L0.2124, L0.2137, L0.2150, L0.2163, L0.2176"
    - "  Creating 1 files"
    - "**** Simulation run 266, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1332[82427,94201] 134ns 12b|----------------------------------------L0.1332-----------------------------------------|"
    - "L0.1345[82427,94201] 135ns 0b|----------------------------------------L0.1345-----------------------------------------|"
    - "L0.1358[82427,94201] 136ns 0b|----------------------------------------L0.1358-----------------------------------------|"
    - "L0.1371[82427,94201] 137ns 0b|----------------------------------------L0.1371-----------------------------------------|"
    - "L0.1384[82427,94201] 138ns 0b|----------------------------------------L0.1384-----------------------------------------|"
    - "L0.1397[82427,94201] 139ns 0b|----------------------------------------L0.1397-----------------------------------------|"
    - "L0.1410[82427,94201] 140ns 0b|----------------------------------------L0.1410-----------------------------------------|"
    - "L0.1423[82427,94201] 141ns 0b|----------------------------------------L0.1423-----------------------------------------|"
    - "L0.1436[82427,94201] 142ns 0b|----------------------------------------L0.1436-----------------------------------------|"
    - "L0.1449[82427,94201] 143ns 0b|----------------------------------------L0.1449-----------------------------------------|"
    - "L0.1462[82427,94201] 144ns 0b|----------------------------------------L0.1462-----------------------------------------|"
    - "L0.1475[82427,94201] 145ns 0b|----------------------------------------L0.1475-----------------------------------------|"
    - "L0.1488[82427,94201] 146ns 0b|----------------------------------------L0.1488-----------------------------------------|"
    - "L0.1501[82427,94201] 147ns 0b|----------------------------------------L0.1501-----------------------------------------|"
    - "L0.1514[82427,94201] 148ns 0b|----------------------------------------L0.1514-----------------------------------------|"
    - "L0.1527[82427,94201] 149ns 0b|----------------------------------------L0.1527-----------------------------------------|"
    - "L0.1540[82427,94201] 150ns 0b|----------------------------------------L0.1540-----------------------------------------|"
    - "L0.1553[82427,94201] 151ns 0b|----------------------------------------L0.1553-----------------------------------------|"
    - "L0.1566[82427,94201] 152ns 0b|----------------------------------------L0.1566-----------------------------------------|"
    - "L0.1579[82427,94201] 153ns 0b|----------------------------------------L0.1579-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[82427,94201] 153ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1332, L0.1345, L0.1358, L0.1371, L0.1384, L0.1397, L0.1410, L0.1423, L0.1436, L0.1449, L0.1462, L0.1475, L0.1488, L0.1501, L0.1514, L0.1527, L0.1540, L0.1553, L0.1566, L0.1579"
    - "  Creating 1 files"
    - "**** Simulation run 267, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1592[82427,94201] 154ns|----------------------------------------L0.1592-----------------------------------------|"
    - "L0.1605[82427,94201] 155ns|----------------------------------------L0.1605-----------------------------------------|"
    - "L0.1618[82427,94201] 156ns|----------------------------------------L0.1618-----------------------------------------|"
    - "L0.1631[82427,94201] 157ns|----------------------------------------L0.1631-----------------------------------------|"
    - "L0.1644[82427,94201] 158ns|----------------------------------------L0.1644-----------------------------------------|"
    - "L0.1657[82427,94201] 159ns|----------------------------------------L0.1657-----------------------------------------|"
    - "L0.1670[82427,94201] 160ns|----------------------------------------L0.1670-----------------------------------------|"
    - "L0.1683[82427,94201] 161ns|----------------------------------------L0.1683-----------------------------------------|"
    - "L0.1696[82427,94201] 162ns|----------------------------------------L0.1696-----------------------------------------|"
    - "L0.1709[82427,94201] 163ns|----------------------------------------L0.1709-----------------------------------------|"
    - "L0.1722[82427,94201] 164ns|----------------------------------------L0.1722-----------------------------------------|"
    - "L0.1735[82427,94201] 165ns|----------------------------------------L0.1735-----------------------------------------|"
    - "L0.1748[82427,94201] 166ns|----------------------------------------L0.1748-----------------------------------------|"
    - "L0.1761[82427,94201] 167ns|----------------------------------------L0.1761-----------------------------------------|"
    - "L0.1774[82427,94201] 168ns|----------------------------------------L0.1774-----------------------------------------|"
    - "L0.1787[82427,94201] 169ns|----------------------------------------L0.1787-----------------------------------------|"
    - "L0.1800[82427,94201] 170ns|----------------------------------------L0.1800-----------------------------------------|"
    - "L0.1813[82427,94201] 171ns|----------------------------------------L0.1813-----------------------------------------|"
    - "L0.1826[82427,94201] 172ns|----------------------------------------L0.1826-----------------------------------------|"
    - "L0.1839[82427,94201] 173ns|----------------------------------------L0.1839-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[82427,94201] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1592, L0.1605, L0.1618, L0.1631, L0.1644, L0.1657, L0.1670, L0.1683, L0.1696, L0.1709, L0.1722, L0.1735, L0.1748, L0.1761, L0.1774, L0.1787, L0.1800, L0.1813, L0.1826, L0.1839"
    - "  Creating 1 files"
    - "**** Simulation run 268, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1852[82427,94201] 174ns|----------------------------------------L0.1852-----------------------------------------|"
    - "L0.1865[82427,94201] 175ns|----------------------------------------L0.1865-----------------------------------------|"
    - "L0.1878[82427,94201] 176ns|----------------------------------------L0.1878-----------------------------------------|"
    - "L0.1891[82427,94201] 177ns|----------------------------------------L0.1891-----------------------------------------|"
    - "L0.1904[82427,94201] 178ns|----------------------------------------L0.1904-----------------------------------------|"
    - "L0.1917[82427,94201] 179ns|----------------------------------------L0.1917-----------------------------------------|"
    - "L0.1930[82427,94201] 180ns|----------------------------------------L0.1930-----------------------------------------|"
    - "L0.1943[82427,94201] 181ns|----------------------------------------L0.1943-----------------------------------------|"
    - "L0.1956[82427,94201] 182ns|----------------------------------------L0.1956-----------------------------------------|"
    - "L0.1969[82427,94201] 183ns|----------------------------------------L0.1969-----------------------------------------|"
    - "L0.1982[82427,94201] 184ns|----------------------------------------L0.1982-----------------------------------------|"
    - "L0.2099[82427,94201] 185ns|----------------------------------------L0.2099-----------------------------------------|"
    - "L0.2112[82427,94201] 186ns|----------------------------------------L0.2112-----------------------------------------|"
    - "L0.1995[82427,94201] 187ns|----------------------------------------L0.1995-----------------------------------------|"
    - "L0.2008[82427,94201] 188ns|----------------------------------------L0.2008-----------------------------------------|"
    - "L0.2021[82427,94201] 189ns|----------------------------------------L0.2021-----------------------------------------|"
    - "L0.2034[82427,94201] 190ns|----------------------------------------L0.2034-----------------------------------------|"
    - "L0.2047[82427,94201] 191ns|----------------------------------------L0.2047-----------------------------------------|"
    - "L0.2060[82427,94201] 192ns|----------------------------------------L0.2060-----------------------------------------|"
    - "L0.2073[82427,94201] 193ns|----------------------------------------L0.2073-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[82427,94201] 193ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1852, L0.1865, L0.1878, L0.1891, L0.1904, L0.1917, L0.1930, L0.1943, L0.1956, L0.1969, L0.1982, L0.1995, L0.2008, L0.2021, L0.2034, L0.2047, L0.2060, L0.2073, L0.2099, L0.2112"
    - "  Creating 1 files"
    - "**** Simulation run 269, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2086[82427,94201] 194ns|----------------------------------------L0.2086-----------------------------------------|"
    - "L0.2125[82427,94201] 195ns|----------------------------------------L0.2125-----------------------------------------|"
    - "L0.2138[82427,94201] 196ns|----------------------------------------L0.2138-----------------------------------------|"
    - "L0.2151[82427,94201] 197ns|----------------------------------------L0.2151-----------------------------------------|"
    - "L0.2164[82427,94201] 198ns|----------------------------------------L0.2164-----------------------------------------|"
    - "L0.2177[82427,94201] 199ns|----------------------------------------L0.2177-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[82427,94201] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2086, L0.2125, L0.2138, L0.2151, L0.2164, L0.2177"
    - "  Creating 1 files"
    - "**** Simulation run 270, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1333[94202,105976] 134ns 12b|----------------------------------------L0.1333-----------------------------------------|"
    - "L0.1346[94202,105976] 135ns 0b|----------------------------------------L0.1346-----------------------------------------|"
    - "L0.1359[94202,105976] 136ns 0b|----------------------------------------L0.1359-----------------------------------------|"
    - "L0.1372[94202,105976] 137ns 0b|----------------------------------------L0.1372-----------------------------------------|"
    - "L0.1385[94202,105976] 138ns 0b|----------------------------------------L0.1385-----------------------------------------|"
    - "L0.1398[94202,105976] 139ns 0b|----------------------------------------L0.1398-----------------------------------------|"
    - "L0.1411[94202,105976] 140ns 0b|----------------------------------------L0.1411-----------------------------------------|"
    - "L0.1424[94202,105976] 141ns 0b|----------------------------------------L0.1424-----------------------------------------|"
    - "L0.1437[94202,105976] 142ns 0b|----------------------------------------L0.1437-----------------------------------------|"
    - "L0.1450[94202,105976] 143ns 0b|----------------------------------------L0.1450-----------------------------------------|"
    - "L0.1463[94202,105976] 144ns 0b|----------------------------------------L0.1463-----------------------------------------|"
    - "L0.1476[94202,105976] 145ns 0b|----------------------------------------L0.1476-----------------------------------------|"
    - "L0.1489[94202,105976] 146ns 0b|----------------------------------------L0.1489-----------------------------------------|"
    - "L0.1502[94202,105976] 147ns 0b|----------------------------------------L0.1502-----------------------------------------|"
    - "L0.1515[94202,105976] 148ns 0b|----------------------------------------L0.1515-----------------------------------------|"
    - "L0.1528[94202,105976] 149ns 0b|----------------------------------------L0.1528-----------------------------------------|"
    - "L0.1541[94202,105976] 150ns 0b|----------------------------------------L0.1541-----------------------------------------|"
    - "L0.1554[94202,105976] 151ns 0b|----------------------------------------L0.1554-----------------------------------------|"
    - "L0.1567[94202,105976] 152ns 0b|----------------------------------------L0.1567-----------------------------------------|"
    - "L0.1580[94202,105976] 153ns 0b|----------------------------------------L0.1580-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[94202,105976] 153ns |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1333, L0.1346, L0.1359, L0.1372, L0.1385, L0.1398, L0.1411, L0.1424, L0.1437, L0.1450, L0.1463, L0.1476, L0.1489, L0.1502, L0.1515, L0.1528, L0.1541, L0.1554, L0.1567, L0.1580"
    - "  Creating 1 files"
    - "**** Simulation run 271, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1593[94202,105976] 154ns|----------------------------------------L0.1593-----------------------------------------|"
    - "L0.1606[94202,105976] 155ns|----------------------------------------L0.1606-----------------------------------------|"
    - "L0.1619[94202,105976] 156ns|----------------------------------------L0.1619-----------------------------------------|"
    - "L0.1632[94202,105976] 157ns|----------------------------------------L0.1632-----------------------------------------|"
    - "L0.1645[94202,105976] 158ns|----------------------------------------L0.1645-----------------------------------------|"
    - "L0.1658[94202,105976] 159ns|----------------------------------------L0.1658-----------------------------------------|"
    - "L0.1671[94202,105976] 160ns|----------------------------------------L0.1671-----------------------------------------|"
    - "L0.1684[94202,105976] 161ns|----------------------------------------L0.1684-----------------------------------------|"
    - "L0.1697[94202,105976] 162ns|----------------------------------------L0.1697-----------------------------------------|"
    - "L0.1710[94202,105976] 163ns|----------------------------------------L0.1710-----------------------------------------|"
    - "L0.1723[94202,105976] 164ns|----------------------------------------L0.1723-----------------------------------------|"
    - "L0.1736[94202,105976] 165ns|----------------------------------------L0.1736-----------------------------------------|"
    - "L0.1749[94202,105976] 166ns|----------------------------------------L0.1749-----------------------------------------|"
    - "L0.1762[94202,105976] 167ns|----------------------------------------L0.1762-----------------------------------------|"
    - "L0.1775[94202,105976] 168ns|----------------------------------------L0.1775-----------------------------------------|"
    - "L0.1788[94202,105976] 169ns|----------------------------------------L0.1788-----------------------------------------|"
    - "L0.1801[94202,105976] 170ns|----------------------------------------L0.1801-----------------------------------------|"
    - "L0.1814[94202,105976] 171ns|----------------------------------------L0.1814-----------------------------------------|"
    - "L0.1827[94202,105976] 172ns|----------------------------------------L0.1827-----------------------------------------|"
    - "L0.1840[94202,105976] 173ns|----------------------------------------L0.1840-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[94202,105976] 173ns |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1593, L0.1606, L0.1619, L0.1632, L0.1645, L0.1658, L0.1671, L0.1684, L0.1697, L0.1710, L0.1723, L0.1736, L0.1749, L0.1762, L0.1775, L0.1788, L0.1801, L0.1814, L0.1827, L0.1840"
    - "  Creating 1 files"
    - "**** Simulation run 272, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1591[70652,82426] 154ns|----------------------------------------L0.1591-----------------------------------------|"
    - "L0.1604[70652,82426] 155ns|----------------------------------------L0.1604-----------------------------------------|"
    - "L0.1617[70652,82426] 156ns|----------------------------------------L0.1617-----------------------------------------|"
    - "L0.1630[70652,82426] 157ns|----------------------------------------L0.1630-----------------------------------------|"
    - "L0.1643[70652,82426] 158ns|----------------------------------------L0.1643-----------------------------------------|"
    - "L0.1656[70652,82426] 159ns|----------------------------------------L0.1656-----------------------------------------|"
    - "L0.1669[70652,82426] 160ns|----------------------------------------L0.1669-----------------------------------------|"
    - "L0.1682[70652,82426] 161ns|----------------------------------------L0.1682-----------------------------------------|"
    - "L0.1695[70652,82426] 162ns|----------------------------------------L0.1695-----------------------------------------|"
    - "L0.1708[70652,82426] 163ns|----------------------------------------L0.1708-----------------------------------------|"
    - "L0.1721[70652,82426] 164ns|----------------------------------------L0.1721-----------------------------------------|"
    - "L0.1734[70652,82426] 165ns|----------------------------------------L0.1734-----------------------------------------|"
    - "L0.1747[70652,82426] 166ns|----------------------------------------L0.1747-----------------------------------------|"
    - "L0.1760[70652,82426] 167ns|----------------------------------------L0.1760-----------------------------------------|"
    - "L0.1773[70652,82426] 168ns|----------------------------------------L0.1773-----------------------------------------|"
    - "L0.1786[70652,82426] 169ns|----------------------------------------L0.1786-----------------------------------------|"
    - "L0.1799[70652,82426] 170ns|----------------------------------------L0.1799-----------------------------------------|"
    - "L0.1812[70652,82426] 171ns|----------------------------------------L0.1812-----------------------------------------|"
    - "L0.1825[70652,82426] 172ns|----------------------------------------L0.1825-----------------------------------------|"
    - "L0.1838[70652,82426] 173ns|----------------------------------------L0.1838-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[70652,82426] 173ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1591, L0.1604, L0.1617, L0.1630, L0.1643, L0.1656, L0.1669, L0.1682, L0.1695, L0.1708, L0.1721, L0.1734, L0.1747, L0.1760, L0.1773, L0.1786, L0.1799, L0.1812, L0.1825, L0.1838"
    - "  Creating 1 files"
    - "**** Simulation run 273, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1853[94202,105976] 174ns|----------------------------------------L0.1853-----------------------------------------|"
    - "L0.1866[94202,105976] 175ns|----------------------------------------L0.1866-----------------------------------------|"
    - "L0.1879[94202,105976] 176ns|----------------------------------------L0.1879-----------------------------------------|"
    - "L0.1892[94202,105976] 177ns|----------------------------------------L0.1892-----------------------------------------|"
    - "L0.1905[94202,105976] 178ns|----------------------------------------L0.1905-----------------------------------------|"
    - "L0.1918[94202,105976] 179ns|----------------------------------------L0.1918-----------------------------------------|"
    - "L0.1931[94202,105976] 180ns|----------------------------------------L0.1931-----------------------------------------|"
    - "L0.1944[94202,105976] 181ns|----------------------------------------L0.1944-----------------------------------------|"
    - "L0.1957[94202,105976] 182ns|----------------------------------------L0.1957-----------------------------------------|"
    - "L0.1970[94202,105976] 183ns|----------------------------------------L0.1970-----------------------------------------|"
    - "L0.1983[94202,105976] 184ns|----------------------------------------L0.1983-----------------------------------------|"
    - "L0.2100[94202,105976] 185ns|----------------------------------------L0.2100-----------------------------------------|"
    - "L0.2113[94202,105976] 186ns|----------------------------------------L0.2113-----------------------------------------|"
    - "L0.1996[94202,105976] 187ns|----------------------------------------L0.1996-----------------------------------------|"
    - "L0.2009[94202,105976] 188ns|----------------------------------------L0.2009-----------------------------------------|"
    - "L0.2022[94202,105976] 189ns|----------------------------------------L0.2022-----------------------------------------|"
    - "L0.2035[94202,105976] 190ns|----------------------------------------L0.2035-----------------------------------------|"
    - "L0.2048[94202,105976] 191ns|----------------------------------------L0.2048-----------------------------------------|"
    - "L0.2061[94202,105976] 192ns|----------------------------------------L0.2061-----------------------------------------|"
    - "L0.2074[94202,105976] 193ns|----------------------------------------L0.2074-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[94202,105976] 193ns |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1853, L0.1866, L0.1879, L0.1892, L0.1905, L0.1918, L0.1931, L0.1944, L0.1957, L0.1970, L0.1983, L0.1996, L0.2009, L0.2022, L0.2035, L0.2048, L0.2061, L0.2074, L0.2100, L0.2113"
    - "  Creating 1 files"
    - "**** Simulation run 274, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2087[94202,105976] 194ns|----------------------------------------L0.2087-----------------------------------------|"
    - "L0.2126[94202,105976] 195ns|----------------------------------------L0.2126-----------------------------------------|"
    - "L0.2139[94202,105976] 196ns|----------------------------------------L0.2139-----------------------------------------|"
    - "L0.2152[94202,105976] 197ns|----------------------------------------L0.2152-----------------------------------------|"
    - "L0.2165[94202,105976] 198ns|----------------------------------------L0.2165-----------------------------------------|"
    - "L0.2178[94202,105976] 199ns|----------------------------------------L0.2178-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[94202,105976] 199ns |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2087, L0.2126, L0.2139, L0.2152, L0.2165, L0.2178"
    - "  Creating 1 files"
    - "**** Simulation run 275, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1334[105977,117751] 134ns 12b|----------------------------------------L0.1334-----------------------------------------|"
    - "L0.1347[105977,117751] 135ns 0b|----------------------------------------L0.1347-----------------------------------------|"
    - "L0.1360[105977,117751] 136ns 0b|----------------------------------------L0.1360-----------------------------------------|"
    - "L0.1373[105977,117751] 137ns 0b|----------------------------------------L0.1373-----------------------------------------|"
    - "L0.1386[105977,117751] 138ns 0b|----------------------------------------L0.1386-----------------------------------------|"
    - "L0.1399[105977,117751] 139ns 0b|----------------------------------------L0.1399-----------------------------------------|"
    - "L0.1412[105977,117751] 140ns 0b|----------------------------------------L0.1412-----------------------------------------|"
    - "L0.1425[105977,117751] 141ns 0b|----------------------------------------L0.1425-----------------------------------------|"
    - "L0.1438[105977,117751] 142ns 0b|----------------------------------------L0.1438-----------------------------------------|"
    - "L0.1451[105977,117751] 143ns 0b|----------------------------------------L0.1451-----------------------------------------|"
    - "L0.1464[105977,117751] 144ns 0b|----------------------------------------L0.1464-----------------------------------------|"
    - "L0.1477[105977,117751] 145ns 0b|----------------------------------------L0.1477-----------------------------------------|"
    - "L0.1490[105977,117751] 146ns 0b|----------------------------------------L0.1490-----------------------------------------|"
    - "L0.1503[105977,117751] 147ns 0b|----------------------------------------L0.1503-----------------------------------------|"
    - "L0.1516[105977,117751] 148ns 0b|----------------------------------------L0.1516-----------------------------------------|"
    - "L0.1529[105977,117751] 149ns 0b|----------------------------------------L0.1529-----------------------------------------|"
    - "L0.1542[105977,117751] 150ns 0b|----------------------------------------L0.1542-----------------------------------------|"
    - "L0.1555[105977,117751] 151ns 0b|----------------------------------------L0.1555-----------------------------------------|"
    - "L0.1568[105977,117751] 152ns 0b|----------------------------------------L0.1568-----------------------------------------|"
    - "L0.1581[105977,117751] 153ns 0b|----------------------------------------L0.1581-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[105977,117751] 153ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1334, L0.1347, L0.1360, L0.1373, L0.1386, L0.1399, L0.1412, L0.1425, L0.1438, L0.1451, L0.1464, L0.1477, L0.1490, L0.1503, L0.1516, L0.1529, L0.1542, L0.1555, L0.1568, L0.1581"
    - "  Creating 1 files"
    - "**** Simulation run 276, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1594[105977,117751] 154ns|----------------------------------------L0.1594-----------------------------------------|"
    - "L0.1607[105977,117751] 155ns|----------------------------------------L0.1607-----------------------------------------|"
    - "L0.1620[105977,117751] 156ns|----------------------------------------L0.1620-----------------------------------------|"
    - "L0.1633[105977,117751] 157ns|----------------------------------------L0.1633-----------------------------------------|"
    - "L0.1646[105977,117751] 158ns|----------------------------------------L0.1646-----------------------------------------|"
    - "L0.1659[105977,117751] 159ns|----------------------------------------L0.1659-----------------------------------------|"
    - "L0.1672[105977,117751] 160ns|----------------------------------------L0.1672-----------------------------------------|"
    - "L0.1685[105977,117751] 161ns|----------------------------------------L0.1685-----------------------------------------|"
    - "L0.1698[105977,117751] 162ns|----------------------------------------L0.1698-----------------------------------------|"
    - "L0.1711[105977,117751] 163ns|----------------------------------------L0.1711-----------------------------------------|"
    - "L0.1724[105977,117751] 164ns|----------------------------------------L0.1724-----------------------------------------|"
    - "L0.1737[105977,117751] 165ns|----------------------------------------L0.1737-----------------------------------------|"
    - "L0.1750[105977,117751] 166ns|----------------------------------------L0.1750-----------------------------------------|"
    - "L0.1763[105977,117751] 167ns|----------------------------------------L0.1763-----------------------------------------|"
    - "L0.1776[105977,117751] 168ns|----------------------------------------L0.1776-----------------------------------------|"
    - "L0.1789[105977,117751] 169ns|----------------------------------------L0.1789-----------------------------------------|"
    - "L0.1802[105977,117751] 170ns|----------------------------------------L0.1802-----------------------------------------|"
    - "L0.1815[105977,117751] 171ns|----------------------------------------L0.1815-----------------------------------------|"
    - "L0.1828[105977,117751] 172ns|----------------------------------------L0.1828-----------------------------------------|"
    - "L0.1841[105977,117751] 173ns|----------------------------------------L0.1841-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[105977,117751] 173ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1594, L0.1607, L0.1620, L0.1633, L0.1646, L0.1659, L0.1672, L0.1685, L0.1698, L0.1711, L0.1724, L0.1737, L0.1750, L0.1763, L0.1776, L0.1789, L0.1802, L0.1815, L0.1828, L0.1841"
    - "  Creating 1 files"
    - "**** Simulation run 277, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1854[105977,117751] 174ns|----------------------------------------L0.1854-----------------------------------------|"
    - "L0.1867[105977,117751] 175ns|----------------------------------------L0.1867-----------------------------------------|"
    - "L0.1880[105977,117751] 176ns|----------------------------------------L0.1880-----------------------------------------|"
    - "L0.1893[105977,117751] 177ns|----------------------------------------L0.1893-----------------------------------------|"
    - "L0.1906[105977,117751] 178ns|----------------------------------------L0.1906-----------------------------------------|"
    - "L0.1919[105977,117751] 179ns|----------------------------------------L0.1919-----------------------------------------|"
    - "L0.1932[105977,117751] 180ns|----------------------------------------L0.1932-----------------------------------------|"
    - "L0.1945[105977,117751] 181ns|----------------------------------------L0.1945-----------------------------------------|"
    - "L0.1958[105977,117751] 182ns|----------------------------------------L0.1958-----------------------------------------|"
    - "L0.1971[105977,117751] 183ns|----------------------------------------L0.1971-----------------------------------------|"
    - "L0.1984[105977,117751] 184ns|----------------------------------------L0.1984-----------------------------------------|"
    - "L0.2101[105977,117751] 185ns|----------------------------------------L0.2101-----------------------------------------|"
    - "L0.2114[105977,117751] 186ns|----------------------------------------L0.2114-----------------------------------------|"
    - "L0.1997[105977,117751] 187ns|----------------------------------------L0.1997-----------------------------------------|"
    - "L0.2010[105977,117751] 188ns|----------------------------------------L0.2010-----------------------------------------|"
    - "L0.2023[105977,117751] 189ns|----------------------------------------L0.2023-----------------------------------------|"
    - "L0.2036[105977,117751] 190ns|----------------------------------------L0.2036-----------------------------------------|"
    - "L0.2049[105977,117751] 191ns|----------------------------------------L0.2049-----------------------------------------|"
    - "L0.2062[105977,117751] 192ns|----------------------------------------L0.2062-----------------------------------------|"
    - "L0.2075[105977,117751] 193ns|----------------------------------------L0.2075-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[105977,117751] 193ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1854, L0.1867, L0.1880, L0.1893, L0.1906, L0.1919, L0.1932, L0.1945, L0.1958, L0.1971, L0.1984, L0.1997, L0.2010, L0.2023, L0.2036, L0.2049, L0.2062, L0.2075, L0.2101, L0.2114"
    - "  Creating 1 files"
    - "**** Simulation run 278, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2088[105977,117751] 194ns|----------------------------------------L0.2088-----------------------------------------|"
    - "L0.2127[105977,117751] 195ns|----------------------------------------L0.2127-----------------------------------------|"
    - "L0.2140[105977,117751] 196ns|----------------------------------------L0.2140-----------------------------------------|"
    - "L0.2153[105977,117751] 197ns|----------------------------------------L0.2153-----------------------------------------|"
    - "L0.2166[105977,117751] 198ns|----------------------------------------L0.2166-----------------------------------------|"
    - "L0.2179[105977,117751] 199ns|----------------------------------------L0.2179-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[105977,117751] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2088, L0.2127, L0.2140, L0.2153, L0.2166, L0.2179"
    - "  Creating 1 files"
    - "**** Simulation run 279, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1335[117752,129526] 134ns 12b|----------------------------------------L0.1335-----------------------------------------|"
    - "L0.1348[117752,129526] 135ns 0b|----------------------------------------L0.1348-----------------------------------------|"
    - "L0.1361[117752,129526] 136ns 0b|----------------------------------------L0.1361-----------------------------------------|"
    - "L0.1374[117752,129526] 137ns 0b|----------------------------------------L0.1374-----------------------------------------|"
    - "L0.1387[117752,129526] 138ns 0b|----------------------------------------L0.1387-----------------------------------------|"
    - "L0.1400[117752,129526] 139ns 0b|----------------------------------------L0.1400-----------------------------------------|"
    - "L0.1413[117752,129526] 140ns 0b|----------------------------------------L0.1413-----------------------------------------|"
    - "L0.1426[117752,129526] 141ns 0b|----------------------------------------L0.1426-----------------------------------------|"
    - "L0.1439[117752,129526] 142ns 0b|----------------------------------------L0.1439-----------------------------------------|"
    - "L0.1452[117752,129526] 143ns 0b|----------------------------------------L0.1452-----------------------------------------|"
    - "L0.1465[117752,129526] 144ns 0b|----------------------------------------L0.1465-----------------------------------------|"
    - "L0.1478[117752,129526] 145ns 0b|----------------------------------------L0.1478-----------------------------------------|"
    - "L0.1491[117752,129526] 146ns 0b|----------------------------------------L0.1491-----------------------------------------|"
    - "L0.1504[117752,129526] 147ns 0b|----------------------------------------L0.1504-----------------------------------------|"
    - "L0.1517[117752,129526] 148ns 0b|----------------------------------------L0.1517-----------------------------------------|"
    - "L0.1530[117752,129526] 149ns 0b|----------------------------------------L0.1530-----------------------------------------|"
    - "L0.1543[117752,129526] 150ns 0b|----------------------------------------L0.1543-----------------------------------------|"
    - "L0.1556[117752,129526] 151ns 0b|----------------------------------------L0.1556-----------------------------------------|"
    - "L0.1569[117752,129526] 152ns 0b|----------------------------------------L0.1569-----------------------------------------|"
    - "L0.1582[117752,129526] 153ns 0b|----------------------------------------L0.1582-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[117752,129526] 153ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1335, L0.1348, L0.1361, L0.1374, L0.1387, L0.1400, L0.1413, L0.1426, L0.1439, L0.1452, L0.1465, L0.1478, L0.1491, L0.1504, L0.1517, L0.1530, L0.1543, L0.1556, L0.1569, L0.1582"
    - "  Creating 1 files"
    - "**** Simulation run 280, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1595[117752,129526] 154ns|----------------------------------------L0.1595-----------------------------------------|"
    - "L0.1608[117752,129526] 155ns|----------------------------------------L0.1608-----------------------------------------|"
    - "L0.1621[117752,129526] 156ns|----------------------------------------L0.1621-----------------------------------------|"
    - "L0.1634[117752,129526] 157ns|----------------------------------------L0.1634-----------------------------------------|"
    - "L0.1647[117752,129526] 158ns|----------------------------------------L0.1647-----------------------------------------|"
    - "L0.1660[117752,129526] 159ns|----------------------------------------L0.1660-----------------------------------------|"
    - "L0.1673[117752,129526] 160ns|----------------------------------------L0.1673-----------------------------------------|"
    - "L0.1686[117752,129526] 161ns|----------------------------------------L0.1686-----------------------------------------|"
    - "L0.1699[117752,129526] 162ns|----------------------------------------L0.1699-----------------------------------------|"
    - "L0.1712[117752,129526] 163ns|----------------------------------------L0.1712-----------------------------------------|"
    - "L0.1725[117752,129526] 164ns|----------------------------------------L0.1725-----------------------------------------|"
    - "L0.1738[117752,129526] 165ns|----------------------------------------L0.1738-----------------------------------------|"
    - "L0.1751[117752,129526] 166ns|----------------------------------------L0.1751-----------------------------------------|"
    - "L0.1764[117752,129526] 167ns|----------------------------------------L0.1764-----------------------------------------|"
    - "L0.1777[117752,129526] 168ns|----------------------------------------L0.1777-----------------------------------------|"
    - "L0.1790[117752,129526] 169ns|----------------------------------------L0.1790-----------------------------------------|"
    - "L0.1803[117752,129526] 170ns|----------------------------------------L0.1803-----------------------------------------|"
    - "L0.1816[117752,129526] 171ns|----------------------------------------L0.1816-----------------------------------------|"
    - "L0.1829[117752,129526] 172ns|----------------------------------------L0.1829-----------------------------------------|"
    - "L0.1842[117752,129526] 173ns|----------------------------------------L0.1842-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[117752,129526] 173ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1595, L0.1608, L0.1621, L0.1634, L0.1647, L0.1660, L0.1673, L0.1686, L0.1699, L0.1712, L0.1725, L0.1738, L0.1751, L0.1764, L0.1777, L0.1790, L0.1803, L0.1816, L0.1829, L0.1842"
    - "  Creating 1 files"
    - "**** Simulation run 281, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1855[117752,129526] 174ns|----------------------------------------L0.1855-----------------------------------------|"
    - "L0.1868[117752,129526] 175ns|----------------------------------------L0.1868-----------------------------------------|"
    - "L0.1881[117752,129526] 176ns|----------------------------------------L0.1881-----------------------------------------|"
    - "L0.1894[117752,129526] 177ns|----------------------------------------L0.1894-----------------------------------------|"
    - "L0.1907[117752,129526] 178ns|----------------------------------------L0.1907-----------------------------------------|"
    - "L0.1920[117752,129526] 179ns|----------------------------------------L0.1920-----------------------------------------|"
    - "L0.1933[117752,129526] 180ns|----------------------------------------L0.1933-----------------------------------------|"
    - "L0.1946[117752,129526] 181ns|----------------------------------------L0.1946-----------------------------------------|"
    - "L0.1959[117752,129526] 182ns|----------------------------------------L0.1959-----------------------------------------|"
    - "L0.1972[117752,129526] 183ns|----------------------------------------L0.1972-----------------------------------------|"
    - "L0.1985[117752,129526] 184ns|----------------------------------------L0.1985-----------------------------------------|"
    - "L0.2102[117752,129526] 185ns|----------------------------------------L0.2102-----------------------------------------|"
    - "L0.2115[117752,129526] 186ns|----------------------------------------L0.2115-----------------------------------------|"
    - "L0.1998[117752,129526] 187ns|----------------------------------------L0.1998-----------------------------------------|"
    - "L0.2011[117752,129526] 188ns|----------------------------------------L0.2011-----------------------------------------|"
    - "L0.2024[117752,129526] 189ns|----------------------------------------L0.2024-----------------------------------------|"
    - "L0.2037[117752,129526] 190ns|----------------------------------------L0.2037-----------------------------------------|"
    - "L0.2050[117752,129526] 191ns|----------------------------------------L0.2050-----------------------------------------|"
    - "L0.2063[117752,129526] 192ns|----------------------------------------L0.2063-----------------------------------------|"
    - "L0.2076[117752,129526] 193ns|----------------------------------------L0.2076-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[117752,129526] 193ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1855, L0.1868, L0.1881, L0.1894, L0.1907, L0.1920, L0.1933, L0.1946, L0.1959, L0.1972, L0.1985, L0.1998, L0.2011, L0.2024, L0.2037, L0.2050, L0.2063, L0.2076, L0.2102, L0.2115"
    - "  Creating 1 files"
    - "**** Simulation run 282, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2089[117752,129526] 194ns|----------------------------------------L0.2089-----------------------------------------|"
    - "L0.2128[117752,129526] 195ns|----------------------------------------L0.2128-----------------------------------------|"
    - "L0.2141[117752,129526] 196ns|----------------------------------------L0.2141-----------------------------------------|"
    - "L0.2154[117752,129526] 197ns|----------------------------------------L0.2154-----------------------------------------|"
    - "L0.2167[117752,129526] 198ns|----------------------------------------L0.2167-----------------------------------------|"
    - "L0.2180[117752,129526] 199ns|----------------------------------------L0.2180-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[117752,129526] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2089, L0.2128, L0.2141, L0.2154, L0.2167, L0.2180"
    - "  Creating 1 files"
    - "**** Simulation run 283, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.1336[129527,141301] 134ns 12b|----------------------------------------L0.1336-----------------------------------------|"
    - "L0.1349[129527,141301] 135ns 0b|----------------------------------------L0.1349-----------------------------------------|"
    - "L0.1362[129527,141301] 136ns 0b|----------------------------------------L0.1362-----------------------------------------|"
    - "L0.1375[129527,141301] 137ns 0b|----------------------------------------L0.1375-----------------------------------------|"
    - "L0.1388[129527,141301] 138ns 0b|----------------------------------------L0.1388-----------------------------------------|"
    - "L0.1401[129527,141301] 139ns 0b|----------------------------------------L0.1401-----------------------------------------|"
    - "L0.1414[129527,141301] 140ns 0b|----------------------------------------L0.1414-----------------------------------------|"
    - "L0.1427[129527,141301] 141ns 0b|----------------------------------------L0.1427-----------------------------------------|"
    - "L0.1440[129527,141301] 142ns 0b|----------------------------------------L0.1440-----------------------------------------|"
    - "L0.1453[129527,141301] 143ns 0b|----------------------------------------L0.1453-----------------------------------------|"
    - "L0.1466[129527,141301] 144ns 0b|----------------------------------------L0.1466-----------------------------------------|"
    - "L0.1479[129527,141301] 145ns 0b|----------------------------------------L0.1479-----------------------------------------|"
    - "L0.1492[129527,141301] 146ns 0b|----------------------------------------L0.1492-----------------------------------------|"
    - "L0.1505[129527,141301] 147ns 0b|----------------------------------------L0.1505-----------------------------------------|"
    - "L0.1518[129527,141301] 148ns 0b|----------------------------------------L0.1518-----------------------------------------|"
    - "L0.1531[129527,141301] 149ns 0b|----------------------------------------L0.1531-----------------------------------------|"
    - "L0.1544[129527,141301] 150ns 0b|----------------------------------------L0.1544-----------------------------------------|"
    - "L0.1557[129527,141301] 151ns 0b|----------------------------------------L0.1557-----------------------------------------|"
    - "L0.1570[129527,141301] 152ns 0b|----------------------------------------L0.1570-----------------------------------------|"
    - "L0.1583[129527,141301] 153ns 0b|----------------------------------------L0.1583-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[129527,141301] 153ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1336, L0.1349, L0.1362, L0.1375, L0.1388, L0.1401, L0.1414, L0.1427, L0.1440, L0.1453, L0.1466, L0.1479, L0.1492, L0.1505, L0.1518, L0.1531, L0.1544, L0.1557, L0.1570, L0.1583"
    - "  Creating 1 files"
    - "**** Simulation run 284, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1596[129527,141301] 154ns|----------------------------------------L0.1596-----------------------------------------|"
    - "L0.1609[129527,141301] 155ns|----------------------------------------L0.1609-----------------------------------------|"
    - "L0.1622[129527,141301] 156ns|----------------------------------------L0.1622-----------------------------------------|"
    - "L0.1635[129527,141301] 157ns|----------------------------------------L0.1635-----------------------------------------|"
    - "L0.1648[129527,141301] 158ns|----------------------------------------L0.1648-----------------------------------------|"
    - "L0.1661[129527,141301] 159ns|----------------------------------------L0.1661-----------------------------------------|"
    - "L0.1674[129527,141301] 160ns|----------------------------------------L0.1674-----------------------------------------|"
    - "L0.1687[129527,141301] 161ns|----------------------------------------L0.1687-----------------------------------------|"
    - "L0.1700[129527,141301] 162ns|----------------------------------------L0.1700-----------------------------------------|"
    - "L0.1713[129527,141301] 163ns|----------------------------------------L0.1713-----------------------------------------|"
    - "L0.1726[129527,141301] 164ns|----------------------------------------L0.1726-----------------------------------------|"
    - "L0.1739[129527,141301] 165ns|----------------------------------------L0.1739-----------------------------------------|"
    - "L0.1752[129527,141301] 166ns|----------------------------------------L0.1752-----------------------------------------|"
    - "L0.1765[129527,141301] 167ns|----------------------------------------L0.1765-----------------------------------------|"
    - "L0.1778[129527,141301] 168ns|----------------------------------------L0.1778-----------------------------------------|"
    - "L0.1791[129527,141301] 169ns|----------------------------------------L0.1791-----------------------------------------|"
    - "L0.1804[129527,141301] 170ns|----------------------------------------L0.1804-----------------------------------------|"
    - "L0.1817[129527,141301] 171ns|----------------------------------------L0.1817-----------------------------------------|"
    - "L0.1830[129527,141301] 172ns|----------------------------------------L0.1830-----------------------------------------|"
    - "L0.1843[129527,141301] 173ns|----------------------------------------L0.1843-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[129527,141301] 173ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1596, L0.1609, L0.1622, L0.1635, L0.1648, L0.1661, L0.1674, L0.1687, L0.1700, L0.1713, L0.1726, L0.1739, L0.1752, L0.1765, L0.1778, L0.1791, L0.1804, L0.1817, L0.1830, L0.1843"
    - "  Creating 1 files"
    - "**** Simulation run 285, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1856[129527,141301] 174ns|----------------------------------------L0.1856-----------------------------------------|"
    - "L0.1869[129527,141301] 175ns|----------------------------------------L0.1869-----------------------------------------|"
    - "L0.1882[129527,141301] 176ns|----------------------------------------L0.1882-----------------------------------------|"
    - "L0.1895[129527,141301] 177ns|----------------------------------------L0.1895-----------------------------------------|"
    - "L0.1908[129527,141301] 178ns|----------------------------------------L0.1908-----------------------------------------|"
    - "L0.1921[129527,141301] 179ns|----------------------------------------L0.1921-----------------------------------------|"
    - "L0.1934[129527,141301] 180ns|----------------------------------------L0.1934-----------------------------------------|"
    - "L0.1947[129527,141301] 181ns|----------------------------------------L0.1947-----------------------------------------|"
    - "L0.1960[129527,141301] 182ns|----------------------------------------L0.1960-----------------------------------------|"
    - "L0.1973[129527,141301] 183ns|----------------------------------------L0.1973-----------------------------------------|"
    - "L0.1986[129527,141301] 184ns|----------------------------------------L0.1986-----------------------------------------|"
    - "L0.2103[129527,141301] 185ns|----------------------------------------L0.2103-----------------------------------------|"
    - "L0.2116[129527,141301] 186ns|----------------------------------------L0.2116-----------------------------------------|"
    - "L0.1999[129527,141301] 187ns|----------------------------------------L0.1999-----------------------------------------|"
    - "L0.2012[129527,141301] 188ns|----------------------------------------L0.2012-----------------------------------------|"
    - "L0.2025[129527,141301] 189ns|----------------------------------------L0.2025-----------------------------------------|"
    - "L0.2038[129527,141301] 190ns|----------------------------------------L0.2038-----------------------------------------|"
    - "L0.2051[129527,141301] 191ns|----------------------------------------L0.2051-----------------------------------------|"
    - "L0.2064[129527,141301] 192ns|----------------------------------------L0.2064-----------------------------------------|"
    - "L0.2077[129527,141301] 193ns|----------------------------------------L0.2077-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[129527,141301] 193ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1856, L0.1869, L0.1882, L0.1895, L0.1908, L0.1921, L0.1934, L0.1947, L0.1960, L0.1973, L0.1986, L0.1999, L0.2012, L0.2025, L0.2038, L0.2051, L0.2064, L0.2077, L0.2103, L0.2116"
    - "  Creating 1 files"
    - "**** Simulation run 286, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2090[129527,141301] 194ns|----------------------------------------L0.2090-----------------------------------------|"
    - "L0.2129[129527,141301] 195ns|----------------------------------------L0.2129-----------------------------------------|"
    - "L0.2142[129527,141301] 196ns|----------------------------------------L0.2142-----------------------------------------|"
    - "L0.2155[129527,141301] 197ns|----------------------------------------L0.2155-----------------------------------------|"
    - "L0.2168[129527,141301] 198ns|----------------------------------------L0.2168-----------------------------------------|"
    - "L0.2181[129527,141301] 199ns|----------------------------------------L0.2181-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[129527,141301] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2090, L0.2129, L0.2142, L0.2155, L0.2168, L0.2181"
    - "  Creating 1 files"
    - "**** Simulation run 287, type=compact(ManySmallFiles). 20 Input Files, 34b total:"
    - "L0                                                                                                                 "
    - "L0.1337[141302,153077] 134ns 16b|----------------------------------------L0.1337-----------------------------------------|"
    - "L0.1350[141302,153077] 135ns 1b|----------------------------------------L0.1350-----------------------------------------|"
    - "L0.1363[141302,153077] 136ns 1b|----------------------------------------L0.1363-----------------------------------------|"
    - "L0.1376[141302,153077] 137ns 1b|----------------------------------------L0.1376-----------------------------------------|"
    - "L0.1389[141302,153077] 138ns 1b|----------------------------------------L0.1389-----------------------------------------|"
    - "L0.1402[141302,153077] 139ns 1b|----------------------------------------L0.1402-----------------------------------------|"
    - "L0.1415[141302,153077] 140ns 1b|----------------------------------------L0.1415-----------------------------------------|"
    - "L0.1428[141302,153077] 141ns 1b|----------------------------------------L0.1428-----------------------------------------|"
    - "L0.1441[141302,153077] 142ns 1b|----------------------------------------L0.1441-----------------------------------------|"
    - "L0.1454[141302,153077] 143ns 1b|----------------------------------------L0.1454-----------------------------------------|"
    - "L0.1467[141302,153077] 144ns 1b|----------------------------------------L0.1467-----------------------------------------|"
    - "L0.1480[141302,153077] 145ns 1b|----------------------------------------L0.1480-----------------------------------------|"
    - "L0.1493[141302,153077] 146ns 1b|----------------------------------------L0.1493-----------------------------------------|"
    - "L0.1506[141302,153077] 147ns 1b|----------------------------------------L0.1506-----------------------------------------|"
    - "L0.1519[141302,153077] 148ns 1b|----------------------------------------L0.1519-----------------------------------------|"
    - "L0.1532[141302,153077] 149ns 1b|----------------------------------------L0.1532-----------------------------------------|"
    - "L0.1545[141302,153077] 150ns 1b|----------------------------------------L0.1545-----------------------------------------|"
    - "L0.1558[141302,153077] 151ns 1b|----------------------------------------L0.1558-----------------------------------------|"
    - "L0.1571[141302,153077] 152ns 1b|----------------------------------------L0.1571-----------------------------------------|"
    - "L0.1584[141302,153077] 153ns 0b|----------------------------------------L0.1584-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 34b total:"
    - "L0, all files 34b                                                                                                  "
    - "L0.?[141302,153077] 153ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1337, L0.1350, L0.1363, L0.1376, L0.1389, L0.1402, L0.1415, L0.1428, L0.1441, L0.1454, L0.1467, L0.1480, L0.1493, L0.1506, L0.1519, L0.1532, L0.1545, L0.1558, L0.1571, L0.1584"
    - "  Creating 1 files"
    - "**** Simulation run 288, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1597[141302,153077] 154ns|----------------------------------------L0.1597-----------------------------------------|"
    - "L0.1610[141302,153077] 155ns|----------------------------------------L0.1610-----------------------------------------|"
    - "L0.1623[141302,153077] 156ns|----------------------------------------L0.1623-----------------------------------------|"
    - "L0.1636[141302,153077] 157ns|----------------------------------------L0.1636-----------------------------------------|"
    - "L0.1649[141302,153077] 158ns|----------------------------------------L0.1649-----------------------------------------|"
    - "L0.1662[141302,153077] 159ns|----------------------------------------L0.1662-----------------------------------------|"
    - "L0.1675[141302,153077] 160ns|----------------------------------------L0.1675-----------------------------------------|"
    - "L0.1688[141302,153077] 161ns|----------------------------------------L0.1688-----------------------------------------|"
    - "L0.1701[141302,153077] 162ns|----------------------------------------L0.1701-----------------------------------------|"
    - "L0.1714[141302,153077] 163ns|----------------------------------------L0.1714-----------------------------------------|"
    - "L0.1727[141302,153077] 164ns|----------------------------------------L0.1727-----------------------------------------|"
    - "L0.1740[141302,153077] 165ns|----------------------------------------L0.1740-----------------------------------------|"
    - "L0.1753[141302,153077] 166ns|----------------------------------------L0.1753-----------------------------------------|"
    - "L0.1766[141302,153077] 167ns|----------------------------------------L0.1766-----------------------------------------|"
    - "L0.1779[141302,153077] 168ns|----------------------------------------L0.1779-----------------------------------------|"
    - "L0.1792[141302,153077] 169ns|----------------------------------------L0.1792-----------------------------------------|"
    - "L0.1805[141302,153077] 170ns|----------------------------------------L0.1805-----------------------------------------|"
    - "L0.1818[141302,153077] 171ns|----------------------------------------L0.1818-----------------------------------------|"
    - "L0.1831[141302,153077] 172ns|----------------------------------------L0.1831-----------------------------------------|"
    - "L0.1844[141302,153077] 173ns|----------------------------------------L0.1844-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[141302,153077] 173ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1597, L0.1610, L0.1623, L0.1636, L0.1649, L0.1662, L0.1675, L0.1688, L0.1701, L0.1714, L0.1727, L0.1740, L0.1753, L0.1766, L0.1779, L0.1792, L0.1805, L0.1818, L0.1831, L0.1844"
    - "  Creating 1 files"
    - "**** Simulation run 289, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1857[141302,153077] 174ns|----------------------------------------L0.1857-----------------------------------------|"
    - "L0.1870[141302,153077] 175ns|----------------------------------------L0.1870-----------------------------------------|"
    - "L0.1883[141302,153077] 176ns|----------------------------------------L0.1883-----------------------------------------|"
    - "L0.1896[141302,153077] 177ns|----------------------------------------L0.1896-----------------------------------------|"
    - "L0.1909[141302,153077] 178ns|----------------------------------------L0.1909-----------------------------------------|"
    - "L0.1922[141302,153077] 179ns|----------------------------------------L0.1922-----------------------------------------|"
    - "L0.1935[141302,153077] 180ns|----------------------------------------L0.1935-----------------------------------------|"
    - "L0.1948[141302,153077] 181ns|----------------------------------------L0.1948-----------------------------------------|"
    - "L0.1961[141302,153077] 182ns|----------------------------------------L0.1961-----------------------------------------|"
    - "L0.1974[141302,153077] 183ns|----------------------------------------L0.1974-----------------------------------------|"
    - "L0.1987[141302,153077] 184ns|----------------------------------------L0.1987-----------------------------------------|"
    - "L0.2104[141302,153077] 185ns|----------------------------------------L0.2104-----------------------------------------|"
    - "L0.2117[141302,153077] 186ns|----------------------------------------L0.2117-----------------------------------------|"
    - "L0.2000[141302,153077] 187ns|----------------------------------------L0.2000-----------------------------------------|"
    - "L0.2013[141302,153077] 188ns|----------------------------------------L0.2013-----------------------------------------|"
    - "L0.2026[141302,153077] 189ns|----------------------------------------L0.2026-----------------------------------------|"
    - "L0.2039[141302,153077] 190ns|----------------------------------------L0.2039-----------------------------------------|"
    - "L0.2052[141302,153077] 191ns|----------------------------------------L0.2052-----------------------------------------|"
    - "L0.2065[141302,153077] 192ns|----------------------------------------L0.2065-----------------------------------------|"
    - "L0.2078[141302,153077] 193ns|----------------------------------------L0.2078-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[141302,153077] 193ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.1857, L0.1870, L0.1883, L0.1896, L0.1909, L0.1922, L0.1935, L0.1948, L0.1961, L0.1974, L0.1987, L0.2000, L0.2013, L0.2026, L0.2039, L0.2052, L0.2065, L0.2078, L0.2104, L0.2117"
    - "  Creating 1 files"
    - "**** Simulation run 290, type=compact(ManySmallFiles). 6 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2091[141302,153077] 194ns|----------------------------------------L0.2091-----------------------------------------|"
    - "L0.2130[141302,153077] 195ns|----------------------------------------L0.2130-----------------------------------------|"
    - "L0.2143[141302,153077] 196ns|----------------------------------------L0.2143-----------------------------------------|"
    - "L0.2156[141302,153077] 197ns|----------------------------------------L0.2156-----------------------------------------|"
    - "L0.2169[141302,153077] 198ns|----------------------------------------L0.2169-----------------------------------------|"
    - "L0.2182[141302,153077] 199ns|----------------------------------------L0.2182-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[141302,153077] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L0.2091, L0.2130, L0.2143, L0.2156, L0.2169, L0.2182"
    - "  Creating 1 files"
    - "**** Simulation run 291, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.1[1,2000] 0ns         |------------------------------------------L0.1------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 0ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 0ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 0ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 292, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.2[1,2000] 1ns         |------------------------------------------L0.2------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 1ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 1ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 1ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 293, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.3[1,2000] 2ns         |------------------------------------------L0.3------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 2ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 2ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 2ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 294, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.4[1,2000] 3ns         |------------------------------------------L0.4------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 3ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 3ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 3ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 295, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.5[1,2000] 4ns         |------------------------------------------L0.5------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 4ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 4ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 4ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 296, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.6[1,2000] 5ns         |------------------------------------------L0.6------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 5ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 5ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 5ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 297, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.7[1,2000] 6ns         |------------------------------------------L0.7------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 6ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 6ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 6ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 298, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.8[1,2000] 7ns         |------------------------------------------L0.8------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 7ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 7ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 7ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 299, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.9[1,2000] 8ns         |------------------------------------------L0.9------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 8ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 8ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 8ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 300, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.10[1,2000] 9ns        |-----------------------------------------L0.10------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 9ns 45mb     |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 9ns 45mb                                          |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 9ns 9mb                                                                                   |-L0.?-| "
    - "**** Simulation run 301, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.11[1,2000] 10ns       |-----------------------------------------L0.11------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 10ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 10ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 10ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 302, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.12[1,2000] 11ns       |-----------------------------------------L0.12------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 11ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 11ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 11ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 303, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.13[1,2000] 12ns       |-----------------------------------------L0.13------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 12ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 12ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 12ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 304, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.14[1,2000] 13ns       |-----------------------------------------L0.14------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 13ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 13ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 13ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 305, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.15[1,2000] 14ns       |-----------------------------------------L0.15------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 14ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 14ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 14ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 306, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.16[1,2000] 15ns       |-----------------------------------------L0.16------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 15ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 15ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 15ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 307, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.17[1,2000] 16ns       |-----------------------------------------L0.17------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 16ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 16ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 16ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 308, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.18[1,2000] 17ns       |-----------------------------------------L0.18------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 17ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 17ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 17ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 309, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.19[1,2000] 18ns       |-----------------------------------------L0.19------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 18ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 18ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 18ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 310, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811]). 1 Input Files, 100mb total:"
    - "L0, all files 100mb                                                                                                "
    - "L0.20[1,2000] 19ns       |-----------------------------------------L0.20------------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 100mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,906] 19ns 45mb    |-----------------L0.?-----------------|                                                  "
    - "L0.?[907,1811] 19ns 45mb                                         |-----------------L0.?-----------------|          "
    - "L0.?[1812,2000] 19ns 9mb                                                                                  |-L0.?-| "
    - "**** Simulation run 311, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.2183[20,11776] 153ns  |----------------------------------------L0.2183----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0                                                                                                                 "
    - "L0.?[20,906] 153ns 0b    |L0.?|                                                                                    "
    - "L0.?[907,1811] 153ns 0b        |L0.?|                                                                              "
    - "L0.?[1812,2716] 153ns 0b              |L0.?|                                                                       "
    - "L0.?[2717,3621] 153ns 0b                     |L0.?|                                                                "
    - "L0.?[3622,4526] 153ns 0b                            |L0.?|                                                         "
    - "L0.?[4527,5431] 153ns 0b                                   |L0.?|                                                  "
    - "L0.?[5432,6336] 153ns 0b                                          |L0.?|                                           "
    - "L0.?[6337,7241] 153ns 0b                                                 |L0.?|                                    "
    - "L0.?[7242,8146] 153ns 0b                                                        |L0.?|                             "
    - "L0.?[8147,9051] 153ns 0b                                                               |L0.?|                      "
    - "L0.?[9052,9956] 153ns 0b                                                                      |L0.?|               "
    - "L0.?[9957,10861] 153ns 0b                                                                            |L0.?|        "
    - "L0.?[10862,11776] 153ns 12b                                                                                   |L0.?| "
    - "**** Simulation run 312, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1585[154,11776] 154ns |----------------------------------------L0.1585-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[154,906] 154ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 154ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 154ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 154ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 154ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 154ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 154ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 154ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 154ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 154ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 154ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 154ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 154ns                                                                                    |L0.?-| "
    - "**** Simulation run 313, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1598[155,11776] 155ns |----------------------------------------L0.1598----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[155,906] 155ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 155ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 155ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 155ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 155ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 155ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 155ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 155ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 155ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 155ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 155ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 155ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 155ns                                                                                    |L0.?-| "
    - "**** Simulation run 314, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1611[156,11776] 156ns |----------------------------------------L0.1611-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[156,906] 156ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 156ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 156ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 156ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 156ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 156ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 156ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 156ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 156ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 156ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 156ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 156ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 156ns                                                                                    |L0.?-| "
    - "**** Simulation run 315, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1624[157,11776] 157ns |----------------------------------------L0.1624-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[157,906] 157ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 157ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 157ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 157ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 157ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 157ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 157ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 157ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 157ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 157ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 157ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 157ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 157ns                                                                                    |L0.?-| "
    - "**** Simulation run 316, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1637[158,11776] 158ns |----------------------------------------L0.1637----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[158,906] 158ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 158ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 158ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 158ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 158ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 158ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 158ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 158ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 158ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 158ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 158ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 158ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 158ns                                                                                    |L0.?-| "
    - "**** Simulation run 317, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1650[159,11776] 159ns |----------------------------------------L0.1650-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[159,906] 159ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 159ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 159ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 159ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 159ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 159ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 159ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 159ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 159ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 159ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 159ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 159ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 159ns                                                                                    |L0.?-| "
    - "**** Simulation run 318, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1663[160,11776] 160ns |----------------------------------------L0.1663-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[160,906] 160ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 160ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 160ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 160ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 160ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 160ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 160ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 160ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 160ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 160ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 160ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 160ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 160ns                                                                                    |L0.?-| "
    - "**** Simulation run 319, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1676[161,11776] 161ns |----------------------------------------L0.1676-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[161,906] 161ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 161ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 161ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 161ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 161ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 161ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 161ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 161ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 161ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 161ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 161ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 161ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 161ns                                                                                    |L0.?-| "
    - "**** Simulation run 320, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1689[162,11776] 162ns |----------------------------------------L0.1689-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[162,906] 162ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 162ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 162ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 162ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 162ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 162ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 162ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 162ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 162ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 162ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 162ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 162ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 162ns                                                                                    |L0.?-| "
    - "**** Simulation run 321, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1702[163,11776] 163ns |----------------------------------------L0.1702-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[163,906] 163ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 163ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 163ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 163ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 163ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 163ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 163ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 163ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 163ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 163ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 163ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 163ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 163ns                                                                                    |L0.?-| "
    - "**** Simulation run 322, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1715[164,11776] 164ns |----------------------------------------L0.1715-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[164,906] 164ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 164ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 164ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 164ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 164ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 164ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 164ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 164ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 164ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 164ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 164ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 164ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 164ns                                                                                    |L0.?-| "
    - "**** Simulation run 323, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1728[165,11776] 165ns |----------------------------------------L0.1728-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[165,906] 165ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 165ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 165ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 165ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 165ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 165ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 165ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 165ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 165ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 165ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 165ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 165ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 165ns                                                                                    |L0.?-| "
    - "**** Simulation run 324, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1741[166,11776] 166ns |----------------------------------------L0.1741-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[166,906] 166ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 166ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 166ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 166ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 166ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 166ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 166ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 166ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 166ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 166ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 166ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 166ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 166ns                                                                                    |L0.?-| "
    - "**** Simulation run 325, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1754[167,11776] 167ns |----------------------------------------L0.1754-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[167,906] 167ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 167ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 167ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 167ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 167ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 167ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 167ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 167ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 167ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 167ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 167ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 167ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 167ns                                                                                    |L0.?-| "
    - "**** Simulation run 326, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1767[168,11776] 168ns |----------------------------------------L0.1767----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[168,906] 168ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 168ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 168ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 168ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 168ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 168ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 168ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 168ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 168ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 168ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 168ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 168ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 168ns                                                                                    |L0.?-| "
    - "**** Simulation run 327, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1780[169,11776] 169ns |----------------------------------------L0.1780-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[169,906] 169ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 169ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 169ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 169ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 169ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 169ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 169ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 169ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 169ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 169ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 169ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 169ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 169ns                                                                                    |L0.?-| "
    - "**** Simulation run 328, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1793[170,11776] 170ns |----------------------------------------L0.1793-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[170,906] 170ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 170ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 170ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 170ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 170ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 170ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 170ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 170ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 170ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 170ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 170ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 170ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 170ns                                                                                    |L0.?-| "
    - "**** Simulation run 329, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1806[171,11776] 171ns |----------------------------------------L0.1806----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[171,906] 171ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 171ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 171ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 171ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 171ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 171ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 171ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 171ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 171ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 171ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 171ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 171ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 171ns                                                                                    |L0.?-| "
    - "**** Simulation run 330, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1819[172,11776] 172ns |----------------------------------------L0.1819-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[172,906] 172ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 172ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 172ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 172ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 172ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 172ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 172ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 172ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 172ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 172ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 172ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 172ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 172ns                                                                                    |L0.?-| "
    - "Committing partition 1:"
    - "  Soft Deleting 40 files: L0.1, L0.2, L0.3, L0.4, L0.5, L0.6, L0.7, L0.8, L0.9, L0.10, L0.11, L0.12, L0.13, L0.14, L0.15, L0.16, L0.17, L0.18, L0.19, L0.20, L0.1585, L0.1598, L0.1611, L0.1624, L0.1637, L0.1650, L0.1663, L0.1676, L0.1689, L0.1702, L0.1715, L0.1728, L0.1741, L0.1754, L0.1767, L0.1780, L0.1793, L0.1806, L0.1819, L0.2183"
    - "  Creating 320 files"
    - "**** Simulation run 331, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1832[173,11776] 173ns |----------------------------------------L0.1832-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[173,906] 173ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 173ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 173ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 173ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 173ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 173ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 173ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 173ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 173ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 173ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 173ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 173ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 173ns                                                                                    |L0.?-| "
    - "**** Simulation run 332, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1845[174,11776] 174ns |----------------------------------------L0.1845-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[174,906] 174ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 174ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 174ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 174ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 174ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 174ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 174ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 174ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 174ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 174ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 174ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 174ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 174ns                                                                                    |L0.?-| "
    - "**** Simulation run 333, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1858[175,11776] 175ns |----------------------------------------L0.1858-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[175,906] 175ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 175ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 175ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 175ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 175ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 175ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 175ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 175ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 175ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 175ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 175ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 175ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 175ns                                                                                    |L0.?-| "
    - "**** Simulation run 334, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1871[176,11776] 176ns |----------------------------------------L0.1871-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[176,906] 176ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 176ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 176ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 176ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 176ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 176ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 176ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 176ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 176ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 176ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 176ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 176ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 176ns                                                                                    |L0.?-| "
    - "**** Simulation run 335, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1884[177,11776] 177ns |----------------------------------------L0.1884-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[177,906] 177ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 177ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 177ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 177ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 177ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 177ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 177ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 177ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 177ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 177ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 177ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 177ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 177ns                                                                                    |L0.?-| "
    - "**** Simulation run 336, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1897[178,11776] 178ns |----------------------------------------L0.1897-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[178,906] 178ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 178ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 178ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 178ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 178ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 178ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 178ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 178ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 178ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 178ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 178ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 178ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 178ns                                                                                    |L0.?-| "
    - "**** Simulation run 337, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1910[179,11776] 179ns |----------------------------------------L0.1910-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[179,906] 179ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 179ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 179ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 179ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 179ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 179ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 179ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 179ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 179ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 179ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 179ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 179ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 179ns                                                                                    |L0.?-| "
    - "**** Simulation run 338, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1923[180,11776] 180ns |----------------------------------------L0.1923-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[180,906] 180ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 180ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 180ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 180ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 180ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 180ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 180ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 180ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 180ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 180ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 180ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 180ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 180ns                                                                                    |L0.?-| "
    - "**** Simulation run 339, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1936[181,11776] 181ns |----------------------------------------L0.1936-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[181,906] 181ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 181ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 181ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 181ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 181ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 181ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 181ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 181ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 181ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 181ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 181ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 181ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 181ns                                                                                    |L0.?-| "
    - "**** Simulation run 340, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1949[182,11776] 182ns |----------------------------------------L0.1949-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[182,906] 182ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 182ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 182ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 182ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 182ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 182ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 182ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 182ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 182ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 182ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 182ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 182ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 182ns                                                                                    |L0.?-| "
    - "**** Simulation run 341, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1962[183,11776] 183ns |----------------------------------------L0.1962-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[183,906] 183ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 183ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 183ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 183ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 183ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 183ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 183ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 183ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 183ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 183ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 183ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 183ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 183ns                                                                                    |L0.?-| "
    - "**** Simulation run 342, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1975[184,11776] 184ns |----------------------------------------L0.1975----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[184,906] 184ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 184ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 184ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 184ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 184ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 184ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 184ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 184ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 184ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 184ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 184ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 184ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 184ns                                                                                    |L0.?-| "
    - "**** Simulation run 343, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2092[185,11776] 185ns |----------------------------------------L0.2092-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[185,906] 185ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 185ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 185ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 185ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 185ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 185ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 185ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 185ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 185ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 185ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 185ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 185ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 185ns                                                                                    |L0.?-| "
    - "**** Simulation run 344, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2105[186,11776] 186ns |----------------------------------------L0.2105-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[186,906] 186ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 186ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 186ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 186ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 186ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 186ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 186ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 186ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 186ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 186ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 186ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 186ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 186ns                                                                                    |L0.?-| "
    - "**** Simulation run 345, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.1988[187,11776] 187ns |----------------------------------------L0.1988----------------------------------------| "
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[187,906] 187ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 187ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 187ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 187ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 187ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 187ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 187ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 187ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 187ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 187ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 187ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 187ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 187ns                                                                                    |L0.?-| "
    - "**** Simulation run 346, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2001[188,11776] 188ns |----------------------------------------L0.2001-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[188,906] 188ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 188ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 188ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 188ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 188ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 188ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 188ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 188ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 188ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 188ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 188ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 188ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 188ns                                                                                    |L0.?-| "
    - "**** Simulation run 347, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2014[189,11776] 189ns |----------------------------------------L0.2014-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[189,906] 189ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 189ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 189ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 189ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 189ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 189ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 189ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 189ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 189ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 189ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 189ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 189ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 189ns                                                                                    |L0.?-| "
    - "**** Simulation run 348, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2027[190,11776] 190ns |----------------------------------------L0.2027-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[190,906] 190ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 190ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 190ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 190ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 190ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 190ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 190ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 190ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 190ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 190ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 190ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 190ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 190ns                                                                                    |L0.?-| "
    - "**** Simulation run 349, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2040[191,11776] 191ns |----------------------------------------L0.2040-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[191,906] 191ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 191ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 191ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 191ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 191ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 191ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 191ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 191ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 191ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 191ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 191ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 191ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 191ns                                                                                    |L0.?-| "
    - "**** Simulation run 350, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2053[192,11776] 192ns |----------------------------------------L0.2053-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[192,906] 192ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 192ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 192ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 192ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 192ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 192ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 192ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 192ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 192ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 192ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 192ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 192ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 192ns                                                                                    |L0.?-| "
    - "**** Simulation run 351, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2066[193,11776] 193ns |----------------------------------------L0.2066-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[193,906] 193ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 193ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 193ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 193ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 193ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 193ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 193ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 193ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 193ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 193ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 193ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 193ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 193ns                                                                                    |L0.?-| "
    - "**** Simulation run 352, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2079[194,11776] 194ns |----------------------------------------L0.2079-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[194,906] 194ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 194ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 194ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 194ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 194ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 194ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 194ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 194ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 194ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 194ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 194ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 194ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 194ns                                                                                    |L0.?-| "
    - "**** Simulation run 353, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2118[195,11776] 195ns |----------------------------------------L0.2118-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[195,906] 195ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 195ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 195ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 195ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 195ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 195ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 195ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 195ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 195ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 195ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 195ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 195ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 195ns                                                                                    |L0.?-| "
    - "**** Simulation run 354, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2131[196,11776] 196ns |----------------------------------------L0.2131-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[196,906] 196ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 196ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 196ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 196ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 196ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 196ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 196ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 196ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 196ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 196ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 196ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 196ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 196ns                                                                                    |L0.?-| "
    - "**** Simulation run 355, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2144[197,11776] 197ns |----------------------------------------L0.2144-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[197,906] 197ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 197ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 197ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 197ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 197ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 197ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 197ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 197ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 197ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 197ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 197ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 197ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 197ns                                                                                    |L0.?-| "
    - "**** Simulation run 356, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2157[198,11776] 198ns |----------------------------------------L0.2157-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[198,906] 198ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 198ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 198ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 198ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 198ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 198ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 198ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 198ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 198ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 198ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 198ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 198ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 198ns                                                                                    |L0.?-| "
    - "**** Simulation run 357, type=split(HighL0OverlapTotalBacklog)(split_times=[906, 1811, 2716, 3621, 4526, 5431, 6336, 7241, 8146, 9051, 9956, 10861]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2170[199,11776] 199ns |----------------------------------------L0.2170-----------------------------------------|"
    - "**** 13 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[199,906] 199ns      |L0.?|                                                                                    "
    - "L0.?[907,1811] 199ns          |L0.?-|                                                                              "
    - "L0.?[1812,2716] 199ns                |L0.?-|                                                                       "
    - "L0.?[2717,3621] 199ns                       |L0.?-|                                                                "
    - "L0.?[3622,4526] 199ns                              |L0.?-|                                                         "
    - "L0.?[4527,5431] 199ns                                     |L0.?-|                                                  "
    - "L0.?[5432,6336] 199ns                                            |L0.?-|                                           "
    - "L0.?[6337,7241] 199ns                                                   |L0.?-|                                    "
    - "L0.?[7242,8146] 199ns                                                          |L0.?-|                             "
    - "L0.?[8147,9051] 199ns                                                                 |L0.?-|                      "
    - "L0.?[9052,9956] 199ns                                                                        |L0.?-|               "
    - "L0.?[9957,10861] 199ns                                                                              |L0.?-|        "
    - "L0.?[10862,11776] 199ns                                                                                    |L0.?-| "
    - "Committing partition 1:"
    - "  Soft Deleting 27 files: L0.1832, L0.1845, L0.1858, L0.1871, L0.1884, L0.1897, L0.1910, L0.1923, L0.1936, L0.1949, L0.1962, L0.1975, L0.1988, L0.2001, L0.2014, L0.2027, L0.2040, L0.2053, L0.2066, L0.2079, L0.2092, L0.2105, L0.2118, L0.2131, L0.2144, L0.2157, L0.2170"
    - "  Creating 351 files"
    - "**** Simulation run 358, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2292[20,906] 153ns    |----------------------------------------L0.2292-----------------------------------------|"
    - "L0.2305[154,906] 154ns                |---------------------------------L0.2305----------------------------------| "
    - "L0.2318[155,906] 155ns                |---------------------------------L0.2318----------------------------------| "
    - "L0.2331[156,906] 156ns                |---------------------------------L0.2331----------------------------------| "
    - "L0.2344[157,906] 157ns                |---------------------------------L0.2344----------------------------------| "
    - "L0.2357[158,906] 158ns                 |---------------------------------L0.2357---------------------------------| "
    - "L0.2370[159,906] 159ns                 |---------------------------------L0.2370---------------------------------| "
    - "L0.2383[160,906] 160ns                 |---------------------------------L0.2383---------------------------------| "
    - "L0.2396[161,906] 161ns                 |---------------------------------L0.2396---------------------------------| "
    - "L0.2409[162,906] 162ns                 |---------------------------------L0.2409---------------------------------| "
    - "L0.2422[163,906] 163ns                 |---------------------------------L0.2422---------------------------------| "
    - "L0.2435[164,906] 164ns                 |---------------------------------L0.2435---------------------------------| "
    - "L0.2448[165,906] 165ns                 |---------------------------------L0.2448---------------------------------| "
    - "L0.2461[166,906] 166ns                 |---------------------------------L0.2461---------------------------------| "
    - "L0.2474[167,906] 167ns                 |---------------------------------L0.2474---------------------------------| "
    - "L0.2487[168,906] 168ns                  |--------------------------------L0.2487---------------------------------| "
    - "L0.2500[169,906] 169ns                  |--------------------------------L0.2500---------------------------------| "
    - "L0.2513[170,906] 170ns                  |--------------------------------L0.2513---------------------------------| "
    - "L0.2526[171,906] 171ns                  |--------------------------------L0.2526---------------------------------| "
    - "L0.2539[172,906] 172ns                  |--------------------------------L0.2539---------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[20,906] 172ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2292, L0.2305, L0.2318, L0.2331, L0.2344, L0.2357, L0.2370, L0.2383, L0.2396, L0.2409, L0.2422, L0.2435, L0.2448, L0.2461, L0.2474, L0.2487, L0.2500, L0.2513, L0.2526, L0.2539"
    - "  Creating 1 files"
    - "**** Simulation run 359, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2293[907,1811] 153ns  |----------------------------------------L0.2293-----------------------------------------|"
    - "L0.2306[907,1811] 154ns  |----------------------------------------L0.2306-----------------------------------------|"
    - "L0.2319[907,1811] 155ns  |----------------------------------------L0.2319-----------------------------------------|"
    - "L0.2332[907,1811] 156ns  |----------------------------------------L0.2332-----------------------------------------|"
    - "L0.2345[907,1811] 157ns  |----------------------------------------L0.2345-----------------------------------------|"
    - "L0.2358[907,1811] 158ns  |----------------------------------------L0.2358-----------------------------------------|"
    - "L0.2371[907,1811] 159ns  |----------------------------------------L0.2371-----------------------------------------|"
    - "L0.2384[907,1811] 160ns  |----------------------------------------L0.2384-----------------------------------------|"
    - "L0.2397[907,1811] 161ns  |----------------------------------------L0.2397-----------------------------------------|"
    - "L0.2410[907,1811] 162ns  |----------------------------------------L0.2410-----------------------------------------|"
    - "L0.2423[907,1811] 163ns  |----------------------------------------L0.2423-----------------------------------------|"
    - "L0.2436[907,1811] 164ns  |----------------------------------------L0.2436-----------------------------------------|"
    - "L0.2449[907,1811] 165ns  |----------------------------------------L0.2449-----------------------------------------|"
    - "L0.2462[907,1811] 166ns  |----------------------------------------L0.2462-----------------------------------------|"
    - "L0.2475[907,1811] 167ns  |----------------------------------------L0.2475-----------------------------------------|"
    - "L0.2488[907,1811] 168ns  |----------------------------------------L0.2488-----------------------------------------|"
    - "L0.2501[907,1811] 169ns  |----------------------------------------L0.2501-----------------------------------------|"
    - "L0.2514[907,1811] 170ns  |----------------------------------------L0.2514-----------------------------------------|"
    - "L0.2527[907,1811] 171ns  |----------------------------------------L0.2527-----------------------------------------|"
    - "L0.2540[907,1811] 172ns  |----------------------------------------L0.2540-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1811] 172ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2293, L0.2306, L0.2319, L0.2332, L0.2345, L0.2358, L0.2371, L0.2384, L0.2397, L0.2410, L0.2423, L0.2436, L0.2449, L0.2462, L0.2475, L0.2488, L0.2501, L0.2514, L0.2527, L0.2540"
    - "  Creating 1 files"
    - "**** Simulation run 360, type=compact(ManySmallFiles). 20 Input Files, 189mb total:"
    - "L0, all files 9mb                                                                                                  "
    - "L0.2234[1812,2000] 0ns   |----------------------------------------L0.2234-----------------------------------------|"
    - "L0.2237[1812,2000] 1ns   |----------------------------------------L0.2237-----------------------------------------|"
    - "L0.2240[1812,2000] 2ns   |----------------------------------------L0.2240-----------------------------------------|"
    - "L0.2243[1812,2000] 3ns   |----------------------------------------L0.2243-----------------------------------------|"
    - "L0.2246[1812,2000] 4ns   |----------------------------------------L0.2246-----------------------------------------|"
    - "L0.2249[1812,2000] 5ns   |----------------------------------------L0.2249-----------------------------------------|"
    - "L0.2252[1812,2000] 6ns   |----------------------------------------L0.2252-----------------------------------------|"
    - "L0.2255[1812,2000] 7ns   |----------------------------------------L0.2255-----------------------------------------|"
    - "L0.2258[1812,2000] 8ns   |----------------------------------------L0.2258-----------------------------------------|"
    - "L0.2261[1812,2000] 9ns   |----------------------------------------L0.2261-----------------------------------------|"
    - "L0.2264[1812,2000] 10ns  |----------------------------------------L0.2264-----------------------------------------|"
    - "L0.2267[1812,2000] 11ns  |----------------------------------------L0.2267-----------------------------------------|"
    - "L0.2270[1812,2000] 12ns  |----------------------------------------L0.2270-----------------------------------------|"
    - "L0.2273[1812,2000] 13ns  |----------------------------------------L0.2273-----------------------------------------|"
    - "L0.2276[1812,2000] 14ns  |----------------------------------------L0.2276-----------------------------------------|"
    - "L0.2279[1812,2000] 15ns  |----------------------------------------L0.2279-----------------------------------------|"
    - "L0.2282[1812,2000] 16ns  |----------------------------------------L0.2282-----------------------------------------|"
    - "L0.2285[1812,2000] 17ns  |----------------------------------------L0.2285-----------------------------------------|"
    - "L0.2288[1812,2000] 18ns  |----------------------------------------L0.2288-----------------------------------------|"
    - "L0.2291[1812,2000] 19ns  |----------------------------------------L0.2291-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 189mb total:"
    - "L0, all files 189mb                                                                                                "
    - "L0.?[1812,2000] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2234, L0.2237, L0.2240, L0.2243, L0.2246, L0.2249, L0.2252, L0.2255, L0.2258, L0.2261, L0.2264, L0.2267, L0.2270, L0.2273, L0.2276, L0.2279, L0.2282, L0.2285, L0.2288, L0.2291"
    - "  Creating 1 files"
    - "**** Simulation run 361, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2294[1812,2716] 153ns |----------------------------------------L0.2294-----------------------------------------|"
    - "L0.2307[1812,2716] 154ns |----------------------------------------L0.2307-----------------------------------------|"
    - "L0.2320[1812,2716] 155ns |----------------------------------------L0.2320-----------------------------------------|"
    - "L0.2333[1812,2716] 156ns |----------------------------------------L0.2333-----------------------------------------|"
    - "L0.2346[1812,2716] 157ns |----------------------------------------L0.2346-----------------------------------------|"
    - "L0.2359[1812,2716] 158ns |----------------------------------------L0.2359-----------------------------------------|"
    - "L0.2372[1812,2716] 159ns |----------------------------------------L0.2372-----------------------------------------|"
    - "L0.2385[1812,2716] 160ns |----------------------------------------L0.2385-----------------------------------------|"
    - "L0.2398[1812,2716] 161ns |----------------------------------------L0.2398-----------------------------------------|"
    - "L0.2411[1812,2716] 162ns |----------------------------------------L0.2411-----------------------------------------|"
    - "L0.2424[1812,2716] 163ns |----------------------------------------L0.2424-----------------------------------------|"
    - "L0.2437[1812,2716] 164ns |----------------------------------------L0.2437-----------------------------------------|"
    - "L0.2450[1812,2716] 165ns |----------------------------------------L0.2450-----------------------------------------|"
    - "L0.2463[1812,2716] 166ns |----------------------------------------L0.2463-----------------------------------------|"
    - "L0.2476[1812,2716] 167ns |----------------------------------------L0.2476-----------------------------------------|"
    - "L0.2489[1812,2716] 168ns |----------------------------------------L0.2489-----------------------------------------|"
    - "L0.2502[1812,2716] 169ns |----------------------------------------L0.2502-----------------------------------------|"
    - "L0.2515[1812,2716] 170ns |----------------------------------------L0.2515-----------------------------------------|"
    - "L0.2528[1812,2716] 171ns |----------------------------------------L0.2528-----------------------------------------|"
    - "L0.2541[1812,2716] 172ns |----------------------------------------L0.2541-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1812,2716] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2294, L0.2307, L0.2320, L0.2333, L0.2346, L0.2359, L0.2372, L0.2385, L0.2398, L0.2411, L0.2424, L0.2437, L0.2450, L0.2463, L0.2476, L0.2489, L0.2502, L0.2515, L0.2528, L0.2541"
    - "  Creating 1 files"
    - "**** Simulation run 362, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2554[1812,2716] 173ns |----------------------------------------L0.2554-----------------------------------------|"
    - "L0.2567[1812,2716] 174ns |----------------------------------------L0.2567-----------------------------------------|"
    - "L0.2580[1812,2716] 175ns |----------------------------------------L0.2580-----------------------------------------|"
    - "L0.2593[1812,2716] 176ns |----------------------------------------L0.2593-----------------------------------------|"
    - "L0.2606[1812,2716] 177ns |----------------------------------------L0.2606-----------------------------------------|"
    - "L0.2619[1812,2716] 178ns |----------------------------------------L0.2619-----------------------------------------|"
    - "L0.2632[1812,2716] 179ns |----------------------------------------L0.2632-----------------------------------------|"
    - "L0.2645[1812,2716] 180ns |----------------------------------------L0.2645-----------------------------------------|"
    - "L0.2658[1812,2716] 181ns |----------------------------------------L0.2658-----------------------------------------|"
    - "L0.2671[1812,2716] 182ns |----------------------------------------L0.2671-----------------------------------------|"
    - "L0.2684[1812,2716] 183ns |----------------------------------------L0.2684-----------------------------------------|"
    - "L0.2697[1812,2716] 184ns |----------------------------------------L0.2697-----------------------------------------|"
    - "L0.2710[1812,2716] 185ns |----------------------------------------L0.2710-----------------------------------------|"
    - "L0.2723[1812,2716] 186ns |----------------------------------------L0.2723-----------------------------------------|"
    - "L0.2736[1812,2716] 187ns |----------------------------------------L0.2736-----------------------------------------|"
    - "L0.2749[1812,2716] 188ns |----------------------------------------L0.2749-----------------------------------------|"
    - "L0.2762[1812,2716] 189ns |----------------------------------------L0.2762-----------------------------------------|"
    - "L0.2775[1812,2716] 190ns |----------------------------------------L0.2775-----------------------------------------|"
    - "L0.2788[1812,2716] 191ns |----------------------------------------L0.2788-----------------------------------------|"
    - "L0.2801[1812,2716] 192ns |----------------------------------------L0.2801-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1812,2716] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2554, L0.2567, L0.2580, L0.2593, L0.2606, L0.2619, L0.2632, L0.2645, L0.2658, L0.2671, L0.2684, L0.2697, L0.2710, L0.2723, L0.2736, L0.2749, L0.2762, L0.2775, L0.2788, L0.2801"
    - "  Creating 1 files"
    - "**** Simulation run 363, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2814[1812,2716] 193ns |----------------------------------------L0.2814-----------------------------------------|"
    - "L0.2827[1812,2716] 194ns |----------------------------------------L0.2827-----------------------------------------|"
    - "L0.2840[1812,2716] 195ns |----------------------------------------L0.2840-----------------------------------------|"
    - "L0.2853[1812,2716] 196ns |----------------------------------------L0.2853-----------------------------------------|"
    - "L0.2866[1812,2716] 197ns |----------------------------------------L0.2866-----------------------------------------|"
    - "L0.2879[1812,2716] 198ns |----------------------------------------L0.2879-----------------------------------------|"
    - "L0.2892[1812,2716] 199ns |----------------------------------------L0.2892-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1812,2716] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2814, L0.2827, L0.2840, L0.2853, L0.2866, L0.2879, L0.2892"
    - "  Creating 1 files"
    - "**** Simulation run 364, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2295[2717,3621] 153ns |----------------------------------------L0.2295-----------------------------------------|"
    - "L0.2308[2717,3621] 154ns |----------------------------------------L0.2308-----------------------------------------|"
    - "L0.2321[2717,3621] 155ns |----------------------------------------L0.2321-----------------------------------------|"
    - "L0.2334[2717,3621] 156ns |----------------------------------------L0.2334-----------------------------------------|"
    - "L0.2347[2717,3621] 157ns |----------------------------------------L0.2347-----------------------------------------|"
    - "L0.2360[2717,3621] 158ns |----------------------------------------L0.2360-----------------------------------------|"
    - "L0.2373[2717,3621] 159ns |----------------------------------------L0.2373-----------------------------------------|"
    - "L0.2386[2717,3621] 160ns |----------------------------------------L0.2386-----------------------------------------|"
    - "L0.2399[2717,3621] 161ns |----------------------------------------L0.2399-----------------------------------------|"
    - "L0.2412[2717,3621] 162ns |----------------------------------------L0.2412-----------------------------------------|"
    - "L0.2425[2717,3621] 163ns |----------------------------------------L0.2425-----------------------------------------|"
    - "L0.2438[2717,3621] 164ns |----------------------------------------L0.2438-----------------------------------------|"
    - "L0.2451[2717,3621] 165ns |----------------------------------------L0.2451-----------------------------------------|"
    - "L0.2464[2717,3621] 166ns |----------------------------------------L0.2464-----------------------------------------|"
    - "L0.2477[2717,3621] 167ns |----------------------------------------L0.2477-----------------------------------------|"
    - "L0.2490[2717,3621] 168ns |----------------------------------------L0.2490-----------------------------------------|"
    - "L0.2503[2717,3621] 169ns |----------------------------------------L0.2503-----------------------------------------|"
    - "L0.2516[2717,3621] 170ns |----------------------------------------L0.2516-----------------------------------------|"
    - "L0.2529[2717,3621] 171ns |----------------------------------------L0.2529-----------------------------------------|"
    - "L0.2542[2717,3621] 172ns |----------------------------------------L0.2542-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[2717,3621] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2295, L0.2308, L0.2321, L0.2334, L0.2347, L0.2360, L0.2373, L0.2386, L0.2399, L0.2412, L0.2425, L0.2438, L0.2451, L0.2464, L0.2477, L0.2490, L0.2503, L0.2516, L0.2529, L0.2542"
    - "  Creating 1 files"
    - "**** Simulation run 365, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2555[2717,3621] 173ns |----------------------------------------L0.2555-----------------------------------------|"
    - "L0.2568[2717,3621] 174ns |----------------------------------------L0.2568-----------------------------------------|"
    - "L0.2581[2717,3621] 175ns |----------------------------------------L0.2581-----------------------------------------|"
    - "L0.2594[2717,3621] 176ns |----------------------------------------L0.2594-----------------------------------------|"
    - "L0.2607[2717,3621] 177ns |----------------------------------------L0.2607-----------------------------------------|"
    - "L0.2620[2717,3621] 178ns |----------------------------------------L0.2620-----------------------------------------|"
    - "L0.2633[2717,3621] 179ns |----------------------------------------L0.2633-----------------------------------------|"
    - "L0.2646[2717,3621] 180ns |----------------------------------------L0.2646-----------------------------------------|"
    - "L0.2659[2717,3621] 181ns |----------------------------------------L0.2659-----------------------------------------|"
    - "L0.2672[2717,3621] 182ns |----------------------------------------L0.2672-----------------------------------------|"
    - "L0.2685[2717,3621] 183ns |----------------------------------------L0.2685-----------------------------------------|"
    - "L0.2698[2717,3621] 184ns |----------------------------------------L0.2698-----------------------------------------|"
    - "L0.2711[2717,3621] 185ns |----------------------------------------L0.2711-----------------------------------------|"
    - "L0.2724[2717,3621] 186ns |----------------------------------------L0.2724-----------------------------------------|"
    - "L0.2737[2717,3621] 187ns |----------------------------------------L0.2737-----------------------------------------|"
    - "L0.2750[2717,3621] 188ns |----------------------------------------L0.2750-----------------------------------------|"
    - "L0.2763[2717,3621] 189ns |----------------------------------------L0.2763-----------------------------------------|"
    - "L0.2776[2717,3621] 190ns |----------------------------------------L0.2776-----------------------------------------|"
    - "L0.2789[2717,3621] 191ns |----------------------------------------L0.2789-----------------------------------------|"
    - "L0.2802[2717,3621] 192ns |----------------------------------------L0.2802-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[2717,3621] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2555, L0.2568, L0.2581, L0.2594, L0.2607, L0.2620, L0.2633, L0.2646, L0.2659, L0.2672, L0.2685, L0.2698, L0.2711, L0.2724, L0.2737, L0.2750, L0.2763, L0.2776, L0.2789, L0.2802"
    - "  Creating 1 files"
    - "**** Simulation run 366, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2815[2717,3621] 193ns |----------------------------------------L0.2815-----------------------------------------|"
    - "L0.2828[2717,3621] 194ns |----------------------------------------L0.2828-----------------------------------------|"
    - "L0.2841[2717,3621] 195ns |----------------------------------------L0.2841-----------------------------------------|"
    - "L0.2854[2717,3621] 196ns |----------------------------------------L0.2854-----------------------------------------|"
    - "L0.2867[2717,3621] 197ns |----------------------------------------L0.2867-----------------------------------------|"
    - "L0.2880[2717,3621] 198ns |----------------------------------------L0.2880-----------------------------------------|"
    - "L0.2893[2717,3621] 199ns |----------------------------------------L0.2893-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[2717,3621] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2815, L0.2828, L0.2841, L0.2854, L0.2867, L0.2880, L0.2893"
    - "  Creating 1 files"
    - "**** Simulation run 367, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2816[3622,4526] 193ns |----------------------------------------L0.2816-----------------------------------------|"
    - "L0.2829[3622,4526] 194ns |----------------------------------------L0.2829-----------------------------------------|"
    - "L0.2842[3622,4526] 195ns |----------------------------------------L0.2842-----------------------------------------|"
    - "L0.2855[3622,4526] 196ns |----------------------------------------L0.2855-----------------------------------------|"
    - "L0.2868[3622,4526] 197ns |----------------------------------------L0.2868-----------------------------------------|"
    - "L0.2881[3622,4526] 198ns |----------------------------------------L0.2881-----------------------------------------|"
    - "L0.2894[3622,4526] 199ns |----------------------------------------L0.2894-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[3622,4526] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2816, L0.2829, L0.2842, L0.2855, L0.2868, L0.2881, L0.2894"
    - "  Creating 1 files"
    - "**** Simulation run 368, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2297[4527,5431] 153ns |----------------------------------------L0.2297-----------------------------------------|"
    - "L0.2310[4527,5431] 154ns |----------------------------------------L0.2310-----------------------------------------|"
    - "L0.2323[4527,5431] 155ns |----------------------------------------L0.2323-----------------------------------------|"
    - "L0.2336[4527,5431] 156ns |----------------------------------------L0.2336-----------------------------------------|"
    - "L0.2349[4527,5431] 157ns |----------------------------------------L0.2349-----------------------------------------|"
    - "L0.2362[4527,5431] 158ns |----------------------------------------L0.2362-----------------------------------------|"
    - "L0.2375[4527,5431] 159ns |----------------------------------------L0.2375-----------------------------------------|"
    - "L0.2388[4527,5431] 160ns |----------------------------------------L0.2388-----------------------------------------|"
    - "L0.2401[4527,5431] 161ns |----------------------------------------L0.2401-----------------------------------------|"
    - "L0.2414[4527,5431] 162ns |----------------------------------------L0.2414-----------------------------------------|"
    - "L0.2427[4527,5431] 163ns |----------------------------------------L0.2427-----------------------------------------|"
    - "L0.2440[4527,5431] 164ns |----------------------------------------L0.2440-----------------------------------------|"
    - "L0.2453[4527,5431] 165ns |----------------------------------------L0.2453-----------------------------------------|"
    - "L0.2466[4527,5431] 166ns |----------------------------------------L0.2466-----------------------------------------|"
    - "L0.2479[4527,5431] 167ns |----------------------------------------L0.2479-----------------------------------------|"
    - "L0.2492[4527,5431] 168ns |----------------------------------------L0.2492-----------------------------------------|"
    - "L0.2505[4527,5431] 169ns |----------------------------------------L0.2505-----------------------------------------|"
    - "L0.2518[4527,5431] 170ns |----------------------------------------L0.2518-----------------------------------------|"
    - "L0.2531[4527,5431] 171ns |----------------------------------------L0.2531-----------------------------------------|"
    - "L0.2544[4527,5431] 172ns |----------------------------------------L0.2544-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[4527,5431] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2297, L0.2310, L0.2323, L0.2336, L0.2349, L0.2362, L0.2375, L0.2388, L0.2401, L0.2414, L0.2427, L0.2440, L0.2453, L0.2466, L0.2479, L0.2492, L0.2505, L0.2518, L0.2531, L0.2544"
    - "  Creating 1 files"
    - "**** Simulation run 369, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2557[4527,5431] 173ns |----------------------------------------L0.2557-----------------------------------------|"
    - "L0.2570[4527,5431] 174ns |----------------------------------------L0.2570-----------------------------------------|"
    - "L0.2583[4527,5431] 175ns |----------------------------------------L0.2583-----------------------------------------|"
    - "L0.2596[4527,5431] 176ns |----------------------------------------L0.2596-----------------------------------------|"
    - "L0.2609[4527,5431] 177ns |----------------------------------------L0.2609-----------------------------------------|"
    - "L0.2622[4527,5431] 178ns |----------------------------------------L0.2622-----------------------------------------|"
    - "L0.2635[4527,5431] 179ns |----------------------------------------L0.2635-----------------------------------------|"
    - "L0.2648[4527,5431] 180ns |----------------------------------------L0.2648-----------------------------------------|"
    - "L0.2661[4527,5431] 181ns |----------------------------------------L0.2661-----------------------------------------|"
    - "L0.2674[4527,5431] 182ns |----------------------------------------L0.2674-----------------------------------------|"
    - "L0.2687[4527,5431] 183ns |----------------------------------------L0.2687-----------------------------------------|"
    - "L0.2700[4527,5431] 184ns |----------------------------------------L0.2700-----------------------------------------|"
    - "L0.2713[4527,5431] 185ns |----------------------------------------L0.2713-----------------------------------------|"
    - "L0.2726[4527,5431] 186ns |----------------------------------------L0.2726-----------------------------------------|"
    - "L0.2739[4527,5431] 187ns |----------------------------------------L0.2739-----------------------------------------|"
    - "L0.2752[4527,5431] 188ns |----------------------------------------L0.2752-----------------------------------------|"
    - "L0.2765[4527,5431] 189ns |----------------------------------------L0.2765-----------------------------------------|"
    - "L0.2778[4527,5431] 190ns |----------------------------------------L0.2778-----------------------------------------|"
    - "L0.2791[4527,5431] 191ns |----------------------------------------L0.2791-----------------------------------------|"
    - "L0.2804[4527,5431] 192ns |----------------------------------------L0.2804-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[4527,5431] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2557, L0.2570, L0.2583, L0.2596, L0.2609, L0.2622, L0.2635, L0.2648, L0.2661, L0.2674, L0.2687, L0.2700, L0.2713, L0.2726, L0.2739, L0.2752, L0.2765, L0.2778, L0.2791, L0.2804"
    - "  Creating 1 files"
    - "**** Simulation run 370, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2817[4527,5431] 193ns |----------------------------------------L0.2817-----------------------------------------|"
    - "L0.2830[4527,5431] 194ns |----------------------------------------L0.2830-----------------------------------------|"
    - "L0.2843[4527,5431] 195ns |----------------------------------------L0.2843-----------------------------------------|"
    - "L0.2856[4527,5431] 196ns |----------------------------------------L0.2856-----------------------------------------|"
    - "L0.2869[4527,5431] 197ns |----------------------------------------L0.2869-----------------------------------------|"
    - "L0.2882[4527,5431] 198ns |----------------------------------------L0.2882-----------------------------------------|"
    - "L0.2895[4527,5431] 199ns |----------------------------------------L0.2895-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[4527,5431] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2817, L0.2830, L0.2843, L0.2856, L0.2869, L0.2882, L0.2895"
    - "  Creating 1 files"
    - "**** Simulation run 371, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2298[5432,6336] 153ns |----------------------------------------L0.2298-----------------------------------------|"
    - "L0.2311[5432,6336] 154ns |----------------------------------------L0.2311-----------------------------------------|"
    - "L0.2324[5432,6336] 155ns |----------------------------------------L0.2324-----------------------------------------|"
    - "L0.2337[5432,6336] 156ns |----------------------------------------L0.2337-----------------------------------------|"
    - "L0.2350[5432,6336] 157ns |----------------------------------------L0.2350-----------------------------------------|"
    - "L0.2363[5432,6336] 158ns |----------------------------------------L0.2363-----------------------------------------|"
    - "L0.2376[5432,6336] 159ns |----------------------------------------L0.2376-----------------------------------------|"
    - "L0.2389[5432,6336] 160ns |----------------------------------------L0.2389-----------------------------------------|"
    - "L0.2402[5432,6336] 161ns |----------------------------------------L0.2402-----------------------------------------|"
    - "L0.2415[5432,6336] 162ns |----------------------------------------L0.2415-----------------------------------------|"
    - "L0.2428[5432,6336] 163ns |----------------------------------------L0.2428-----------------------------------------|"
    - "L0.2441[5432,6336] 164ns |----------------------------------------L0.2441-----------------------------------------|"
    - "L0.2454[5432,6336] 165ns |----------------------------------------L0.2454-----------------------------------------|"
    - "L0.2467[5432,6336] 166ns |----------------------------------------L0.2467-----------------------------------------|"
    - "L0.2480[5432,6336] 167ns |----------------------------------------L0.2480-----------------------------------------|"
    - "L0.2493[5432,6336] 168ns |----------------------------------------L0.2493-----------------------------------------|"
    - "L0.2506[5432,6336] 169ns |----------------------------------------L0.2506-----------------------------------------|"
    - "L0.2519[5432,6336] 170ns |----------------------------------------L0.2519-----------------------------------------|"
    - "L0.2532[5432,6336] 171ns |----------------------------------------L0.2532-----------------------------------------|"
    - "L0.2545[5432,6336] 172ns |----------------------------------------L0.2545-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[5432,6336] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2298, L0.2311, L0.2324, L0.2337, L0.2350, L0.2363, L0.2376, L0.2389, L0.2402, L0.2415, L0.2428, L0.2441, L0.2454, L0.2467, L0.2480, L0.2493, L0.2506, L0.2519, L0.2532, L0.2545"
    - "  Creating 1 files"
    - "**** Simulation run 372, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2558[5432,6336] 173ns |----------------------------------------L0.2558-----------------------------------------|"
    - "L0.2571[5432,6336] 174ns |----------------------------------------L0.2571-----------------------------------------|"
    - "L0.2584[5432,6336] 175ns |----------------------------------------L0.2584-----------------------------------------|"
    - "L0.2597[5432,6336] 176ns |----------------------------------------L0.2597-----------------------------------------|"
    - "L0.2610[5432,6336] 177ns |----------------------------------------L0.2610-----------------------------------------|"
    - "L0.2623[5432,6336] 178ns |----------------------------------------L0.2623-----------------------------------------|"
    - "L0.2636[5432,6336] 179ns |----------------------------------------L0.2636-----------------------------------------|"
    - "L0.2649[5432,6336] 180ns |----------------------------------------L0.2649-----------------------------------------|"
    - "L0.2662[5432,6336] 181ns |----------------------------------------L0.2662-----------------------------------------|"
    - "L0.2675[5432,6336] 182ns |----------------------------------------L0.2675-----------------------------------------|"
    - "L0.2688[5432,6336] 183ns |----------------------------------------L0.2688-----------------------------------------|"
    - "L0.2701[5432,6336] 184ns |----------------------------------------L0.2701-----------------------------------------|"
    - "L0.2714[5432,6336] 185ns |----------------------------------------L0.2714-----------------------------------------|"
    - "L0.2727[5432,6336] 186ns |----------------------------------------L0.2727-----------------------------------------|"
    - "L0.2740[5432,6336] 187ns |----------------------------------------L0.2740-----------------------------------------|"
    - "L0.2753[5432,6336] 188ns |----------------------------------------L0.2753-----------------------------------------|"
    - "L0.2766[5432,6336] 189ns |----------------------------------------L0.2766-----------------------------------------|"
    - "L0.2779[5432,6336] 190ns |----------------------------------------L0.2779-----------------------------------------|"
    - "L0.2792[5432,6336] 191ns |----------------------------------------L0.2792-----------------------------------------|"
    - "L0.2805[5432,6336] 192ns |----------------------------------------L0.2805-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[5432,6336] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2558, L0.2571, L0.2584, L0.2597, L0.2610, L0.2623, L0.2636, L0.2649, L0.2662, L0.2675, L0.2688, L0.2701, L0.2714, L0.2727, L0.2740, L0.2753, L0.2766, L0.2779, L0.2792, L0.2805"
    - "  Creating 1 files"
    - "**** Simulation run 373, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2818[5432,6336] 193ns |----------------------------------------L0.2818-----------------------------------------|"
    - "L0.2831[5432,6336] 194ns |----------------------------------------L0.2831-----------------------------------------|"
    - "L0.2844[5432,6336] 195ns |----------------------------------------L0.2844-----------------------------------------|"
    - "L0.2857[5432,6336] 196ns |----------------------------------------L0.2857-----------------------------------------|"
    - "L0.2870[5432,6336] 197ns |----------------------------------------L0.2870-----------------------------------------|"
    - "L0.2883[5432,6336] 198ns |----------------------------------------L0.2883-----------------------------------------|"
    - "L0.2896[5432,6336] 199ns |----------------------------------------L0.2896-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[5432,6336] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2818, L0.2831, L0.2844, L0.2857, L0.2870, L0.2883, L0.2896"
    - "  Creating 1 files"
    - "**** Simulation run 374, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2299[6337,7241] 153ns |----------------------------------------L0.2299-----------------------------------------|"
    - "L0.2312[6337,7241] 154ns |----------------------------------------L0.2312-----------------------------------------|"
    - "L0.2325[6337,7241] 155ns |----------------------------------------L0.2325-----------------------------------------|"
    - "L0.2338[6337,7241] 156ns |----------------------------------------L0.2338-----------------------------------------|"
    - "L0.2351[6337,7241] 157ns |----------------------------------------L0.2351-----------------------------------------|"
    - "L0.2364[6337,7241] 158ns |----------------------------------------L0.2364-----------------------------------------|"
    - "L0.2377[6337,7241] 159ns |----------------------------------------L0.2377-----------------------------------------|"
    - "L0.2390[6337,7241] 160ns |----------------------------------------L0.2390-----------------------------------------|"
    - "L0.2403[6337,7241] 161ns |----------------------------------------L0.2403-----------------------------------------|"
    - "L0.2416[6337,7241] 162ns |----------------------------------------L0.2416-----------------------------------------|"
    - "L0.2429[6337,7241] 163ns |----------------------------------------L0.2429-----------------------------------------|"
    - "L0.2442[6337,7241] 164ns |----------------------------------------L0.2442-----------------------------------------|"
    - "L0.2455[6337,7241] 165ns |----------------------------------------L0.2455-----------------------------------------|"
    - "L0.2468[6337,7241] 166ns |----------------------------------------L0.2468-----------------------------------------|"
    - "L0.2481[6337,7241] 167ns |----------------------------------------L0.2481-----------------------------------------|"
    - "L0.2494[6337,7241] 168ns |----------------------------------------L0.2494-----------------------------------------|"
    - "L0.2507[6337,7241] 169ns |----------------------------------------L0.2507-----------------------------------------|"
    - "L0.2520[6337,7241] 170ns |----------------------------------------L0.2520-----------------------------------------|"
    - "L0.2533[6337,7241] 171ns |----------------------------------------L0.2533-----------------------------------------|"
    - "L0.2546[6337,7241] 172ns |----------------------------------------L0.2546-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[6337,7241] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2299, L0.2312, L0.2325, L0.2338, L0.2351, L0.2364, L0.2377, L0.2390, L0.2403, L0.2416, L0.2429, L0.2442, L0.2455, L0.2468, L0.2481, L0.2494, L0.2507, L0.2520, L0.2533, L0.2546"
    - "  Creating 1 files"
    - "**** Simulation run 375, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2296[3622,4526] 153ns |----------------------------------------L0.2296-----------------------------------------|"
    - "L0.2309[3622,4526] 154ns |----------------------------------------L0.2309-----------------------------------------|"
    - "L0.2322[3622,4526] 155ns |----------------------------------------L0.2322-----------------------------------------|"
    - "L0.2335[3622,4526] 156ns |----------------------------------------L0.2335-----------------------------------------|"
    - "L0.2348[3622,4526] 157ns |----------------------------------------L0.2348-----------------------------------------|"
    - "L0.2361[3622,4526] 158ns |----------------------------------------L0.2361-----------------------------------------|"
    - "L0.2374[3622,4526] 159ns |----------------------------------------L0.2374-----------------------------------------|"
    - "L0.2387[3622,4526] 160ns |----------------------------------------L0.2387-----------------------------------------|"
    - "L0.2400[3622,4526] 161ns |----------------------------------------L0.2400-----------------------------------------|"
    - "L0.2413[3622,4526] 162ns |----------------------------------------L0.2413-----------------------------------------|"
    - "L0.2426[3622,4526] 163ns |----------------------------------------L0.2426-----------------------------------------|"
    - "L0.2439[3622,4526] 164ns |----------------------------------------L0.2439-----------------------------------------|"
    - "L0.2452[3622,4526] 165ns |----------------------------------------L0.2452-----------------------------------------|"
    - "L0.2465[3622,4526] 166ns |----------------------------------------L0.2465-----------------------------------------|"
    - "L0.2478[3622,4526] 167ns |----------------------------------------L0.2478-----------------------------------------|"
    - "L0.2491[3622,4526] 168ns |----------------------------------------L0.2491-----------------------------------------|"
    - "L0.2504[3622,4526] 169ns |----------------------------------------L0.2504-----------------------------------------|"
    - "L0.2517[3622,4526] 170ns |----------------------------------------L0.2517-----------------------------------------|"
    - "L0.2530[3622,4526] 171ns |----------------------------------------L0.2530-----------------------------------------|"
    - "L0.2543[3622,4526] 172ns |----------------------------------------L0.2543-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[3622,4526] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2296, L0.2309, L0.2322, L0.2335, L0.2348, L0.2361, L0.2374, L0.2387, L0.2400, L0.2413, L0.2426, L0.2439, L0.2452, L0.2465, L0.2478, L0.2491, L0.2504, L0.2517, L0.2530, L0.2543"
    - "  Creating 1 files"
    - "**** Simulation run 376, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2556[3622,4526] 173ns |----------------------------------------L0.2556-----------------------------------------|"
    - "L0.2569[3622,4526] 174ns |----------------------------------------L0.2569-----------------------------------------|"
    - "L0.2582[3622,4526] 175ns |----------------------------------------L0.2582-----------------------------------------|"
    - "L0.2595[3622,4526] 176ns |----------------------------------------L0.2595-----------------------------------------|"
    - "L0.2608[3622,4526] 177ns |----------------------------------------L0.2608-----------------------------------------|"
    - "L0.2621[3622,4526] 178ns |----------------------------------------L0.2621-----------------------------------------|"
    - "L0.2634[3622,4526] 179ns |----------------------------------------L0.2634-----------------------------------------|"
    - "L0.2647[3622,4526] 180ns |----------------------------------------L0.2647-----------------------------------------|"
    - "L0.2660[3622,4526] 181ns |----------------------------------------L0.2660-----------------------------------------|"
    - "L0.2673[3622,4526] 182ns |----------------------------------------L0.2673-----------------------------------------|"
    - "L0.2686[3622,4526] 183ns |----------------------------------------L0.2686-----------------------------------------|"
    - "L0.2699[3622,4526] 184ns |----------------------------------------L0.2699-----------------------------------------|"
    - "L0.2712[3622,4526] 185ns |----------------------------------------L0.2712-----------------------------------------|"
    - "L0.2725[3622,4526] 186ns |----------------------------------------L0.2725-----------------------------------------|"
    - "L0.2738[3622,4526] 187ns |----------------------------------------L0.2738-----------------------------------------|"
    - "L0.2751[3622,4526] 188ns |----------------------------------------L0.2751-----------------------------------------|"
    - "L0.2764[3622,4526] 189ns |----------------------------------------L0.2764-----------------------------------------|"
    - "L0.2777[3622,4526] 190ns |----------------------------------------L0.2777-----------------------------------------|"
    - "L0.2790[3622,4526] 191ns |----------------------------------------L0.2790-----------------------------------------|"
    - "L0.2803[3622,4526] 192ns |----------------------------------------L0.2803-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[3622,4526] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2556, L0.2569, L0.2582, L0.2595, L0.2608, L0.2621, L0.2634, L0.2647, L0.2660, L0.2673, L0.2686, L0.2699, L0.2712, L0.2725, L0.2738, L0.2751, L0.2764, L0.2777, L0.2790, L0.2803"
    - "  Creating 1 files"
    - "**** Simulation run 377, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2559[6337,7241] 173ns |----------------------------------------L0.2559-----------------------------------------|"
    - "L0.2572[6337,7241] 174ns |----------------------------------------L0.2572-----------------------------------------|"
    - "L0.2585[6337,7241] 175ns |----------------------------------------L0.2585-----------------------------------------|"
    - "L0.2598[6337,7241] 176ns |----------------------------------------L0.2598-----------------------------------------|"
    - "L0.2611[6337,7241] 177ns |----------------------------------------L0.2611-----------------------------------------|"
    - "L0.2624[6337,7241] 178ns |----------------------------------------L0.2624-----------------------------------------|"
    - "L0.2637[6337,7241] 179ns |----------------------------------------L0.2637-----------------------------------------|"
    - "L0.2650[6337,7241] 180ns |----------------------------------------L0.2650-----------------------------------------|"
    - "L0.2663[6337,7241] 181ns |----------------------------------------L0.2663-----------------------------------------|"
    - "L0.2676[6337,7241] 182ns |----------------------------------------L0.2676-----------------------------------------|"
    - "L0.2689[6337,7241] 183ns |----------------------------------------L0.2689-----------------------------------------|"
    - "L0.2702[6337,7241] 184ns |----------------------------------------L0.2702-----------------------------------------|"
    - "L0.2715[6337,7241] 185ns |----------------------------------------L0.2715-----------------------------------------|"
    - "L0.2728[6337,7241] 186ns |----------------------------------------L0.2728-----------------------------------------|"
    - "L0.2741[6337,7241] 187ns |----------------------------------------L0.2741-----------------------------------------|"
    - "L0.2754[6337,7241] 188ns |----------------------------------------L0.2754-----------------------------------------|"
    - "L0.2767[6337,7241] 189ns |----------------------------------------L0.2767-----------------------------------------|"
    - "L0.2780[6337,7241] 190ns |----------------------------------------L0.2780-----------------------------------------|"
    - "L0.2793[6337,7241] 191ns |----------------------------------------L0.2793-----------------------------------------|"
    - "L0.2806[6337,7241] 192ns |----------------------------------------L0.2806-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[6337,7241] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2559, L0.2572, L0.2585, L0.2598, L0.2611, L0.2624, L0.2637, L0.2650, L0.2663, L0.2676, L0.2689, L0.2702, L0.2715, L0.2728, L0.2741, L0.2754, L0.2767, L0.2780, L0.2793, L0.2806"
    - "  Creating 1 files"
    - "**** Simulation run 378, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2819[6337,7241] 193ns |----------------------------------------L0.2819-----------------------------------------|"
    - "L0.2832[6337,7241] 194ns |----------------------------------------L0.2832-----------------------------------------|"
    - "L0.2845[6337,7241] 195ns |----------------------------------------L0.2845-----------------------------------------|"
    - "L0.2858[6337,7241] 196ns |----------------------------------------L0.2858-----------------------------------------|"
    - "L0.2871[6337,7241] 197ns |----------------------------------------L0.2871-----------------------------------------|"
    - "L0.2884[6337,7241] 198ns |----------------------------------------L0.2884-----------------------------------------|"
    - "L0.2897[6337,7241] 199ns |----------------------------------------L0.2897-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[6337,7241] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2819, L0.2832, L0.2845, L0.2858, L0.2871, L0.2884, L0.2897"
    - "  Creating 1 files"
    - "**** Simulation run 379, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2300[7242,8146] 153ns |----------------------------------------L0.2300-----------------------------------------|"
    - "L0.2313[7242,8146] 154ns |----------------------------------------L0.2313-----------------------------------------|"
    - "L0.2326[7242,8146] 155ns |----------------------------------------L0.2326-----------------------------------------|"
    - "L0.2339[7242,8146] 156ns |----------------------------------------L0.2339-----------------------------------------|"
    - "L0.2352[7242,8146] 157ns |----------------------------------------L0.2352-----------------------------------------|"
    - "L0.2365[7242,8146] 158ns |----------------------------------------L0.2365-----------------------------------------|"
    - "L0.2378[7242,8146] 159ns |----------------------------------------L0.2378-----------------------------------------|"
    - "L0.2391[7242,8146] 160ns |----------------------------------------L0.2391-----------------------------------------|"
    - "L0.2404[7242,8146] 161ns |----------------------------------------L0.2404-----------------------------------------|"
    - "L0.2417[7242,8146] 162ns |----------------------------------------L0.2417-----------------------------------------|"
    - "L0.2430[7242,8146] 163ns |----------------------------------------L0.2430-----------------------------------------|"
    - "L0.2443[7242,8146] 164ns |----------------------------------------L0.2443-----------------------------------------|"
    - "L0.2456[7242,8146] 165ns |----------------------------------------L0.2456-----------------------------------------|"
    - "L0.2469[7242,8146] 166ns |----------------------------------------L0.2469-----------------------------------------|"
    - "L0.2482[7242,8146] 167ns |----------------------------------------L0.2482-----------------------------------------|"
    - "L0.2495[7242,8146] 168ns |----------------------------------------L0.2495-----------------------------------------|"
    - "L0.2508[7242,8146] 169ns |----------------------------------------L0.2508-----------------------------------------|"
    - "L0.2521[7242,8146] 170ns |----------------------------------------L0.2521-----------------------------------------|"
    - "L0.2534[7242,8146] 171ns |----------------------------------------L0.2534-----------------------------------------|"
    - "L0.2547[7242,8146] 172ns |----------------------------------------L0.2547-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[7242,8146] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2300, L0.2313, L0.2326, L0.2339, L0.2352, L0.2365, L0.2378, L0.2391, L0.2404, L0.2417, L0.2430, L0.2443, L0.2456, L0.2469, L0.2482, L0.2495, L0.2508, L0.2521, L0.2534, L0.2547"
    - "  Creating 1 files"
    - "**** Simulation run 380, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2560[7242,8146] 173ns |----------------------------------------L0.2560-----------------------------------------|"
    - "L0.2573[7242,8146] 174ns |----------------------------------------L0.2573-----------------------------------------|"
    - "L0.2586[7242,8146] 175ns |----------------------------------------L0.2586-----------------------------------------|"
    - "L0.2599[7242,8146] 176ns |----------------------------------------L0.2599-----------------------------------------|"
    - "L0.2612[7242,8146] 177ns |----------------------------------------L0.2612-----------------------------------------|"
    - "L0.2625[7242,8146] 178ns |----------------------------------------L0.2625-----------------------------------------|"
    - "L0.2638[7242,8146] 179ns |----------------------------------------L0.2638-----------------------------------------|"
    - "L0.2651[7242,8146] 180ns |----------------------------------------L0.2651-----------------------------------------|"
    - "L0.2664[7242,8146] 181ns |----------------------------------------L0.2664-----------------------------------------|"
    - "L0.2677[7242,8146] 182ns |----------------------------------------L0.2677-----------------------------------------|"
    - "L0.2690[7242,8146] 183ns |----------------------------------------L0.2690-----------------------------------------|"
    - "L0.2703[7242,8146] 184ns |----------------------------------------L0.2703-----------------------------------------|"
    - "L0.2716[7242,8146] 185ns |----------------------------------------L0.2716-----------------------------------------|"
    - "L0.2729[7242,8146] 186ns |----------------------------------------L0.2729-----------------------------------------|"
    - "L0.2742[7242,8146] 187ns |----------------------------------------L0.2742-----------------------------------------|"
    - "L0.2755[7242,8146] 188ns |----------------------------------------L0.2755-----------------------------------------|"
    - "L0.2768[7242,8146] 189ns |----------------------------------------L0.2768-----------------------------------------|"
    - "L0.2781[7242,8146] 190ns |----------------------------------------L0.2781-----------------------------------------|"
    - "L0.2794[7242,8146] 191ns |----------------------------------------L0.2794-----------------------------------------|"
    - "L0.2807[7242,8146] 192ns |----------------------------------------L0.2807-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[7242,8146] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2560, L0.2573, L0.2586, L0.2599, L0.2612, L0.2625, L0.2638, L0.2651, L0.2664, L0.2677, L0.2690, L0.2703, L0.2716, L0.2729, L0.2742, L0.2755, L0.2768, L0.2781, L0.2794, L0.2807"
    - "  Creating 1 files"
    - "**** Simulation run 381, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2820[7242,8146] 193ns |----------------------------------------L0.2820-----------------------------------------|"
    - "L0.2833[7242,8146] 194ns |----------------------------------------L0.2833-----------------------------------------|"
    - "L0.2846[7242,8146] 195ns |----------------------------------------L0.2846-----------------------------------------|"
    - "L0.2859[7242,8146] 196ns |----------------------------------------L0.2859-----------------------------------------|"
    - "L0.2872[7242,8146] 197ns |----------------------------------------L0.2872-----------------------------------------|"
    - "L0.2885[7242,8146] 198ns |----------------------------------------L0.2885-----------------------------------------|"
    - "L0.2898[7242,8146] 199ns |----------------------------------------L0.2898-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[7242,8146] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2820, L0.2833, L0.2846, L0.2859, L0.2872, L0.2885, L0.2898"
    - "  Creating 1 files"
    - "**** Simulation run 382, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2301[8147,9051] 153ns |----------------------------------------L0.2301-----------------------------------------|"
    - "L0.2314[8147,9051] 154ns |----------------------------------------L0.2314-----------------------------------------|"
    - "L0.2327[8147,9051] 155ns |----------------------------------------L0.2327-----------------------------------------|"
    - "L0.2340[8147,9051] 156ns |----------------------------------------L0.2340-----------------------------------------|"
    - "L0.2353[8147,9051] 157ns |----------------------------------------L0.2353-----------------------------------------|"
    - "L0.2366[8147,9051] 158ns |----------------------------------------L0.2366-----------------------------------------|"
    - "L0.2379[8147,9051] 159ns |----------------------------------------L0.2379-----------------------------------------|"
    - "L0.2392[8147,9051] 160ns |----------------------------------------L0.2392-----------------------------------------|"
    - "L0.2405[8147,9051] 161ns |----------------------------------------L0.2405-----------------------------------------|"
    - "L0.2418[8147,9051] 162ns |----------------------------------------L0.2418-----------------------------------------|"
    - "L0.2431[8147,9051] 163ns |----------------------------------------L0.2431-----------------------------------------|"
    - "L0.2444[8147,9051] 164ns |----------------------------------------L0.2444-----------------------------------------|"
    - "L0.2457[8147,9051] 165ns |----------------------------------------L0.2457-----------------------------------------|"
    - "L0.2470[8147,9051] 166ns |----------------------------------------L0.2470-----------------------------------------|"
    - "L0.2483[8147,9051] 167ns |----------------------------------------L0.2483-----------------------------------------|"
    - "L0.2496[8147,9051] 168ns |----------------------------------------L0.2496-----------------------------------------|"
    - "L0.2509[8147,9051] 169ns |----------------------------------------L0.2509-----------------------------------------|"
    - "L0.2522[8147,9051] 170ns |----------------------------------------L0.2522-----------------------------------------|"
    - "L0.2535[8147,9051] 171ns |----------------------------------------L0.2535-----------------------------------------|"
    - "L0.2548[8147,9051] 172ns |----------------------------------------L0.2548-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[8147,9051] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2301, L0.2314, L0.2327, L0.2340, L0.2353, L0.2366, L0.2379, L0.2392, L0.2405, L0.2418, L0.2431, L0.2444, L0.2457, L0.2470, L0.2483, L0.2496, L0.2509, L0.2522, L0.2535, L0.2548"
    - "  Creating 1 files"
    - "**** Simulation run 383, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2561[8147,9051] 173ns |----------------------------------------L0.2561-----------------------------------------|"
    - "L0.2574[8147,9051] 174ns |----------------------------------------L0.2574-----------------------------------------|"
    - "L0.2587[8147,9051] 175ns |----------------------------------------L0.2587-----------------------------------------|"
    - "L0.2600[8147,9051] 176ns |----------------------------------------L0.2600-----------------------------------------|"
    - "L0.2613[8147,9051] 177ns |----------------------------------------L0.2613-----------------------------------------|"
    - "L0.2626[8147,9051] 178ns |----------------------------------------L0.2626-----------------------------------------|"
    - "L0.2639[8147,9051] 179ns |----------------------------------------L0.2639-----------------------------------------|"
    - "L0.2652[8147,9051] 180ns |----------------------------------------L0.2652-----------------------------------------|"
    - "L0.2665[8147,9051] 181ns |----------------------------------------L0.2665-----------------------------------------|"
    - "L0.2678[8147,9051] 182ns |----------------------------------------L0.2678-----------------------------------------|"
    - "L0.2691[8147,9051] 183ns |----------------------------------------L0.2691-----------------------------------------|"
    - "L0.2704[8147,9051] 184ns |----------------------------------------L0.2704-----------------------------------------|"
    - "L0.2717[8147,9051] 185ns |----------------------------------------L0.2717-----------------------------------------|"
    - "L0.2730[8147,9051] 186ns |----------------------------------------L0.2730-----------------------------------------|"
    - "L0.2743[8147,9051] 187ns |----------------------------------------L0.2743-----------------------------------------|"
    - "L0.2756[8147,9051] 188ns |----------------------------------------L0.2756-----------------------------------------|"
    - "L0.2769[8147,9051] 189ns |----------------------------------------L0.2769-----------------------------------------|"
    - "L0.2782[8147,9051] 190ns |----------------------------------------L0.2782-----------------------------------------|"
    - "L0.2795[8147,9051] 191ns |----------------------------------------L0.2795-----------------------------------------|"
    - "L0.2808[8147,9051] 192ns |----------------------------------------L0.2808-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[8147,9051] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2561, L0.2574, L0.2587, L0.2600, L0.2613, L0.2626, L0.2639, L0.2652, L0.2665, L0.2678, L0.2691, L0.2704, L0.2717, L0.2730, L0.2743, L0.2756, L0.2769, L0.2782, L0.2795, L0.2808"
    - "  Creating 1 files"
    - "**** Simulation run 384, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2821[8147,9051] 193ns |----------------------------------------L0.2821-----------------------------------------|"
    - "L0.2834[8147,9051] 194ns |----------------------------------------L0.2834-----------------------------------------|"
    - "L0.2847[8147,9051] 195ns |----------------------------------------L0.2847-----------------------------------------|"
    - "L0.2860[8147,9051] 196ns |----------------------------------------L0.2860-----------------------------------------|"
    - "L0.2873[8147,9051] 197ns |----------------------------------------L0.2873-----------------------------------------|"
    - "L0.2886[8147,9051] 198ns |----------------------------------------L0.2886-----------------------------------------|"
    - "L0.2899[8147,9051] 199ns |----------------------------------------L0.2899-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[8147,9051] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2821, L0.2834, L0.2847, L0.2860, L0.2873, L0.2886, L0.2899"
    - "  Creating 1 files"
    - "**** Simulation run 385, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2302[9052,9956] 153ns |----------------------------------------L0.2302-----------------------------------------|"
    - "L0.2315[9052,9956] 154ns |----------------------------------------L0.2315-----------------------------------------|"
    - "L0.2328[9052,9956] 155ns |----------------------------------------L0.2328-----------------------------------------|"
    - "L0.2341[9052,9956] 156ns |----------------------------------------L0.2341-----------------------------------------|"
    - "L0.2354[9052,9956] 157ns |----------------------------------------L0.2354-----------------------------------------|"
    - "L0.2367[9052,9956] 158ns |----------------------------------------L0.2367-----------------------------------------|"
    - "L0.2380[9052,9956] 159ns |----------------------------------------L0.2380-----------------------------------------|"
    - "L0.2393[9052,9956] 160ns |----------------------------------------L0.2393-----------------------------------------|"
    - "L0.2406[9052,9956] 161ns |----------------------------------------L0.2406-----------------------------------------|"
    - "L0.2419[9052,9956] 162ns |----------------------------------------L0.2419-----------------------------------------|"
    - "L0.2432[9052,9956] 163ns |----------------------------------------L0.2432-----------------------------------------|"
    - "L0.2445[9052,9956] 164ns |----------------------------------------L0.2445-----------------------------------------|"
    - "L0.2458[9052,9956] 165ns |----------------------------------------L0.2458-----------------------------------------|"
    - "L0.2471[9052,9956] 166ns |----------------------------------------L0.2471-----------------------------------------|"
    - "L0.2484[9052,9956] 167ns |----------------------------------------L0.2484-----------------------------------------|"
    - "L0.2497[9052,9956] 168ns |----------------------------------------L0.2497-----------------------------------------|"
    - "L0.2510[9052,9956] 169ns |----------------------------------------L0.2510-----------------------------------------|"
    - "L0.2523[9052,9956] 170ns |----------------------------------------L0.2523-----------------------------------------|"
    - "L0.2536[9052,9956] 171ns |----------------------------------------L0.2536-----------------------------------------|"
    - "L0.2549[9052,9956] 172ns |----------------------------------------L0.2549-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9052,9956] 172ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2302, L0.2315, L0.2328, L0.2341, L0.2354, L0.2367, L0.2380, L0.2393, L0.2406, L0.2419, L0.2432, L0.2445, L0.2458, L0.2471, L0.2484, L0.2497, L0.2510, L0.2523, L0.2536, L0.2549"
    - "  Creating 1 files"
    - "**** Simulation run 386, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2562[9052,9956] 173ns |----------------------------------------L0.2562-----------------------------------------|"
    - "L0.2575[9052,9956] 174ns |----------------------------------------L0.2575-----------------------------------------|"
    - "L0.2588[9052,9956] 175ns |----------------------------------------L0.2588-----------------------------------------|"
    - "L0.2601[9052,9956] 176ns |----------------------------------------L0.2601-----------------------------------------|"
    - "L0.2614[9052,9956] 177ns |----------------------------------------L0.2614-----------------------------------------|"
    - "L0.2627[9052,9956] 178ns |----------------------------------------L0.2627-----------------------------------------|"
    - "L0.2640[9052,9956] 179ns |----------------------------------------L0.2640-----------------------------------------|"
    - "L0.2653[9052,9956] 180ns |----------------------------------------L0.2653-----------------------------------------|"
    - "L0.2666[9052,9956] 181ns |----------------------------------------L0.2666-----------------------------------------|"
    - "L0.2679[9052,9956] 182ns |----------------------------------------L0.2679-----------------------------------------|"
    - "L0.2692[9052,9956] 183ns |----------------------------------------L0.2692-----------------------------------------|"
    - "L0.2705[9052,9956] 184ns |----------------------------------------L0.2705-----------------------------------------|"
    - "L0.2718[9052,9956] 185ns |----------------------------------------L0.2718-----------------------------------------|"
    - "L0.2731[9052,9956] 186ns |----------------------------------------L0.2731-----------------------------------------|"
    - "L0.2744[9052,9956] 187ns |----------------------------------------L0.2744-----------------------------------------|"
    - "L0.2757[9052,9956] 188ns |----------------------------------------L0.2757-----------------------------------------|"
    - "L0.2770[9052,9956] 189ns |----------------------------------------L0.2770-----------------------------------------|"
    - "L0.2783[9052,9956] 190ns |----------------------------------------L0.2783-----------------------------------------|"
    - "L0.2796[9052,9956] 191ns |----------------------------------------L0.2796-----------------------------------------|"
    - "L0.2809[9052,9956] 192ns |----------------------------------------L0.2809-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9052,9956] 192ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2562, L0.2575, L0.2588, L0.2601, L0.2614, L0.2627, L0.2640, L0.2653, L0.2666, L0.2679, L0.2692, L0.2705, L0.2718, L0.2731, L0.2744, L0.2757, L0.2770, L0.2783, L0.2796, L0.2809"
    - "  Creating 1 files"
    - "**** Simulation run 387, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2822[9052,9956] 193ns |----------------------------------------L0.2822-----------------------------------------|"
    - "L0.2835[9052,9956] 194ns |----------------------------------------L0.2835-----------------------------------------|"
    - "L0.2848[9052,9956] 195ns |----------------------------------------L0.2848-----------------------------------------|"
    - "L0.2861[9052,9956] 196ns |----------------------------------------L0.2861-----------------------------------------|"
    - "L0.2874[9052,9956] 197ns |----------------------------------------L0.2874-----------------------------------------|"
    - "L0.2887[9052,9956] 198ns |----------------------------------------L0.2887-----------------------------------------|"
    - "L0.2900[9052,9956] 199ns |----------------------------------------L0.2900-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9052,9956] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2822, L0.2835, L0.2848, L0.2861, L0.2874, L0.2887, L0.2900"
    - "  Creating 1 files"
    - "**** Simulation run 388, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2303[9957,10861] 153ns|----------------------------------------L0.2303-----------------------------------------|"
    - "L0.2316[9957,10861] 154ns|----------------------------------------L0.2316-----------------------------------------|"
    - "L0.2329[9957,10861] 155ns|----------------------------------------L0.2329-----------------------------------------|"
    - "L0.2342[9957,10861] 156ns|----------------------------------------L0.2342-----------------------------------------|"
    - "L0.2355[9957,10861] 157ns|----------------------------------------L0.2355-----------------------------------------|"
    - "L0.2368[9957,10861] 158ns|----------------------------------------L0.2368-----------------------------------------|"
    - "L0.2381[9957,10861] 159ns|----------------------------------------L0.2381-----------------------------------------|"
    - "L0.2394[9957,10861] 160ns|----------------------------------------L0.2394-----------------------------------------|"
    - "L0.2407[9957,10861] 161ns|----------------------------------------L0.2407-----------------------------------------|"
    - "L0.2420[9957,10861] 162ns|----------------------------------------L0.2420-----------------------------------------|"
    - "L0.2433[9957,10861] 163ns|----------------------------------------L0.2433-----------------------------------------|"
    - "L0.2446[9957,10861] 164ns|----------------------------------------L0.2446-----------------------------------------|"
    - "L0.2459[9957,10861] 165ns|----------------------------------------L0.2459-----------------------------------------|"
    - "L0.2472[9957,10861] 166ns|----------------------------------------L0.2472-----------------------------------------|"
    - "L0.2485[9957,10861] 167ns|----------------------------------------L0.2485-----------------------------------------|"
    - "L0.2498[9957,10861] 168ns|----------------------------------------L0.2498-----------------------------------------|"
    - "L0.2511[9957,10861] 169ns|----------------------------------------L0.2511-----------------------------------------|"
    - "L0.2524[9957,10861] 170ns|----------------------------------------L0.2524-----------------------------------------|"
    - "L0.2537[9957,10861] 171ns|----------------------------------------L0.2537-----------------------------------------|"
    - "L0.2550[9957,10861] 172ns|----------------------------------------L0.2550-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9957,10861] 172ns   |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2303, L0.2316, L0.2329, L0.2342, L0.2355, L0.2368, L0.2381, L0.2394, L0.2407, L0.2420, L0.2433, L0.2446, L0.2459, L0.2472, L0.2485, L0.2498, L0.2511, L0.2524, L0.2537, L0.2550"
    - "  Creating 1 files"
    - "**** Simulation run 389, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2563[9957,10861] 173ns|----------------------------------------L0.2563-----------------------------------------|"
    - "L0.2576[9957,10861] 174ns|----------------------------------------L0.2576-----------------------------------------|"
    - "L0.2589[9957,10861] 175ns|----------------------------------------L0.2589-----------------------------------------|"
    - "L0.2602[9957,10861] 176ns|----------------------------------------L0.2602-----------------------------------------|"
    - "L0.2615[9957,10861] 177ns|----------------------------------------L0.2615-----------------------------------------|"
    - "L0.2628[9957,10861] 178ns|----------------------------------------L0.2628-----------------------------------------|"
    - "L0.2641[9957,10861] 179ns|----------------------------------------L0.2641-----------------------------------------|"
    - "L0.2654[9957,10861] 180ns|----------------------------------------L0.2654-----------------------------------------|"
    - "L0.2667[9957,10861] 181ns|----------------------------------------L0.2667-----------------------------------------|"
    - "L0.2680[9957,10861] 182ns|----------------------------------------L0.2680-----------------------------------------|"
    - "L0.2693[9957,10861] 183ns|----------------------------------------L0.2693-----------------------------------------|"
    - "L0.2706[9957,10861] 184ns|----------------------------------------L0.2706-----------------------------------------|"
    - "L0.2719[9957,10861] 185ns|----------------------------------------L0.2719-----------------------------------------|"
    - "L0.2732[9957,10861] 186ns|----------------------------------------L0.2732-----------------------------------------|"
    - "L0.2745[9957,10861] 187ns|----------------------------------------L0.2745-----------------------------------------|"
    - "L0.2758[9957,10861] 188ns|----------------------------------------L0.2758-----------------------------------------|"
    - "L0.2771[9957,10861] 189ns|----------------------------------------L0.2771-----------------------------------------|"
    - "L0.2784[9957,10861] 190ns|----------------------------------------L0.2784-----------------------------------------|"
    - "L0.2797[9957,10861] 191ns|----------------------------------------L0.2797-----------------------------------------|"
    - "L0.2810[9957,10861] 192ns|----------------------------------------L0.2810-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9957,10861] 192ns   |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2563, L0.2576, L0.2589, L0.2602, L0.2615, L0.2628, L0.2641, L0.2654, L0.2667, L0.2680, L0.2693, L0.2706, L0.2719, L0.2732, L0.2745, L0.2758, L0.2771, L0.2784, L0.2797, L0.2810"
    - "  Creating 1 files"
    - "**** Simulation run 390, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2823[9957,10861] 193ns|----------------------------------------L0.2823-----------------------------------------|"
    - "L0.2836[9957,10861] 194ns|----------------------------------------L0.2836-----------------------------------------|"
    - "L0.2849[9957,10861] 195ns|----------------------------------------L0.2849-----------------------------------------|"
    - "L0.2862[9957,10861] 196ns|----------------------------------------L0.2862-----------------------------------------|"
    - "L0.2875[9957,10861] 197ns|----------------------------------------L0.2875-----------------------------------------|"
    - "L0.2888[9957,10861] 198ns|----------------------------------------L0.2888-----------------------------------------|"
    - "L0.2901[9957,10861] 199ns|----------------------------------------L0.2901-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9957,10861] 199ns   |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2823, L0.2836, L0.2849, L0.2862, L0.2875, L0.2888, L0.2901"
    - "  Creating 1 files"
    - "**** Simulation run 391, type=compact(ManySmallFiles). 20 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2304[10862,11776] 153ns 12b|----------------------------------------L0.2304-----------------------------------------|"
    - "L0.2317[10862,11776] 154ns 0b|----------------------------------------L0.2317-----------------------------------------|"
    - "L0.2330[10862,11776] 155ns 0b|----------------------------------------L0.2330-----------------------------------------|"
    - "L0.2343[10862,11776] 156ns 0b|----------------------------------------L0.2343-----------------------------------------|"
    - "L0.2356[10862,11776] 157ns 0b|----------------------------------------L0.2356-----------------------------------------|"
    - "L0.2369[10862,11776] 158ns 0b|----------------------------------------L0.2369-----------------------------------------|"
    - "L0.2382[10862,11776] 159ns 0b|----------------------------------------L0.2382-----------------------------------------|"
    - "L0.2395[10862,11776] 160ns 0b|----------------------------------------L0.2395-----------------------------------------|"
    - "L0.2408[10862,11776] 161ns 0b|----------------------------------------L0.2408-----------------------------------------|"
    - "L0.2421[10862,11776] 162ns 0b|----------------------------------------L0.2421-----------------------------------------|"
    - "L0.2434[10862,11776] 163ns 0b|----------------------------------------L0.2434-----------------------------------------|"
    - "L0.2447[10862,11776] 164ns 0b|----------------------------------------L0.2447-----------------------------------------|"
    - "L0.2460[10862,11776] 165ns 0b|----------------------------------------L0.2460-----------------------------------------|"
    - "L0.2473[10862,11776] 166ns 0b|----------------------------------------L0.2473-----------------------------------------|"
    - "L0.2486[10862,11776] 167ns 0b|----------------------------------------L0.2486-----------------------------------------|"
    - "L0.2499[10862,11776] 168ns 0b|----------------------------------------L0.2499-----------------------------------------|"
    - "L0.2512[10862,11776] 169ns 0b|----------------------------------------L0.2512-----------------------------------------|"
    - "L0.2525[10862,11776] 170ns 0b|----------------------------------------L0.2525-----------------------------------------|"
    - "L0.2538[10862,11776] 171ns 0b|----------------------------------------L0.2538-----------------------------------------|"
    - "L0.2551[10862,11776] 172ns 0b|----------------------------------------L0.2551-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[10862,11776] 172ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2304, L0.2317, L0.2330, L0.2343, L0.2356, L0.2369, L0.2382, L0.2395, L0.2408, L0.2421, L0.2434, L0.2447, L0.2460, L0.2473, L0.2486, L0.2499, L0.2512, L0.2525, L0.2538, L0.2551"
    - "  Creating 1 files"
    - "**** Simulation run 392, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2564[10862,11776] 173ns|----------------------------------------L0.2564-----------------------------------------|"
    - "L0.2577[10862,11776] 174ns|----------------------------------------L0.2577-----------------------------------------|"
    - "L0.2590[10862,11776] 175ns|----------------------------------------L0.2590-----------------------------------------|"
    - "L0.2603[10862,11776] 176ns|----------------------------------------L0.2603-----------------------------------------|"
    - "L0.2616[10862,11776] 177ns|----------------------------------------L0.2616-----------------------------------------|"
    - "L0.2629[10862,11776] 178ns|----------------------------------------L0.2629-----------------------------------------|"
    - "L0.2642[10862,11776] 179ns|----------------------------------------L0.2642-----------------------------------------|"
    - "L0.2655[10862,11776] 180ns|----------------------------------------L0.2655-----------------------------------------|"
    - "L0.2668[10862,11776] 181ns|----------------------------------------L0.2668-----------------------------------------|"
    - "L0.2681[10862,11776] 182ns|----------------------------------------L0.2681-----------------------------------------|"
    - "L0.2694[10862,11776] 183ns|----------------------------------------L0.2694-----------------------------------------|"
    - "L0.2707[10862,11776] 184ns|----------------------------------------L0.2707-----------------------------------------|"
    - "L0.2720[10862,11776] 185ns|----------------------------------------L0.2720-----------------------------------------|"
    - "L0.2733[10862,11776] 186ns|----------------------------------------L0.2733-----------------------------------------|"
    - "L0.2746[10862,11776] 187ns|----------------------------------------L0.2746-----------------------------------------|"
    - "L0.2759[10862,11776] 188ns|----------------------------------------L0.2759-----------------------------------------|"
    - "L0.2772[10862,11776] 189ns|----------------------------------------L0.2772-----------------------------------------|"
    - "L0.2785[10862,11776] 190ns|----------------------------------------L0.2785-----------------------------------------|"
    - "L0.2798[10862,11776] 191ns|----------------------------------------L0.2798-----------------------------------------|"
    - "L0.2811[10862,11776] 192ns|----------------------------------------L0.2811-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[10862,11776] 192ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2564, L0.2577, L0.2590, L0.2603, L0.2616, L0.2629, L0.2642, L0.2655, L0.2668, L0.2681, L0.2694, L0.2707, L0.2720, L0.2733, L0.2746, L0.2759, L0.2772, L0.2785, L0.2798, L0.2811"
    - "  Creating 1 files"
    - "**** Simulation run 393, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2184[11777,23551] 153ns 12b|----------------------------------------L0.2184-----------------------------------------|"
    - "L0.2185[11777,23551] 173ns 0b|----------------------------------------L0.2185-----------------------------------------|"
    - "L0.2186[11777,23551] 193ns 0b|----------------------------------------L0.2186-----------------------------------------|"
    - "L0.2187[11777,23551] 199ns 0b|----------------------------------------L0.2187-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[11777,23551] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2184, L0.2185, L0.2186, L0.2187"
    - "  Creating 1 files"
    - "**** Simulation run 394, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2188[23552,35326] 153ns 12b|----------------------------------------L0.2188-----------------------------------------|"
    - "L0.2189[23552,35326] 173ns 0b|----------------------------------------L0.2189-----------------------------------------|"
    - "L0.2190[23552,35326] 193ns 0b|----------------------------------------L0.2190-----------------------------------------|"
    - "L0.2191[23552,35326] 199ns 0b|----------------------------------------L0.2191-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[23552,35326] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2188, L0.2189, L0.2190, L0.2191"
    - "  Creating 1 files"
    - "**** Simulation run 395, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2192[35327,47101] 153ns 12b|----------------------------------------L0.2192-----------------------------------------|"
    - "L0.2193[35327,47101] 173ns 0b|----------------------------------------L0.2193-----------------------------------------|"
    - "L0.2194[35327,47101] 193ns 0b|----------------------------------------L0.2194-----------------------------------------|"
    - "L0.2195[35327,47101] 199ns 0b|----------------------------------------L0.2195-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[35327,47101] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2192, L0.2193, L0.2194, L0.2195"
    - "  Creating 1 files"
    - "**** Simulation run 396, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2196[47102,58876] 153ns 12b|----------------------------------------L0.2196-----------------------------------------|"
    - "L0.2197[47102,58876] 173ns 0b|----------------------------------------L0.2197-----------------------------------------|"
    - "L0.2198[47102,58876] 193ns 0b|----------------------------------------L0.2198-----------------------------------------|"
    - "L0.2199[47102,58876] 199ns 0b|----------------------------------------L0.2199-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[47102,58876] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2196, L0.2197, L0.2198, L0.2199"
    - "  Creating 1 files"
    - "**** Simulation run 397, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2200[58877,70651] 153ns 12b|----------------------------------------L0.2200-----------------------------------------|"
    - "L0.2201[58877,70651] 173ns 0b|----------------------------------------L0.2201-----------------------------------------|"
    - "L0.2202[58877,70651] 193ns 0b|----------------------------------------L0.2202-----------------------------------------|"
    - "L0.2203[58877,70651] 199ns 0b|----------------------------------------L0.2203-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[58877,70651] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2200, L0.2201, L0.2202, L0.2203"
    - "  Creating 1 files"
    - "**** Simulation run 398, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2204[70652,82426] 153ns 12b|----------------------------------------L0.2204-----------------------------------------|"
    - "L0.2213[70652,82426] 173ns 0b|----------------------------------------L0.2213-----------------------------------------|"
    - "L0.2205[70652,82426] 193ns 0b|----------------------------------------L0.2205-----------------------------------------|"
    - "L0.2206[70652,82426] 199ns 0b|----------------------------------------L0.2206-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[70652,82426] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2204, L0.2205, L0.2206, L0.2213"
    - "  Creating 1 files"
    - "**** Simulation run 399, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2207[82427,94201] 153ns 12b|----------------------------------------L0.2207-----------------------------------------|"
    - "L0.2208[82427,94201] 173ns 0b|----------------------------------------L0.2208-----------------------------------------|"
    - "L0.2209[82427,94201] 193ns 0b|----------------------------------------L0.2209-----------------------------------------|"
    - "L0.2210[82427,94201] 199ns 0b|----------------------------------------L0.2210-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[82427,94201] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2207, L0.2208, L0.2209, L0.2210"
    - "  Creating 1 files"
    - "**** Simulation run 400, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2211[94202,105976] 153ns 12b|----------------------------------------L0.2211-----------------------------------------|"
    - "L0.2212[94202,105976] 173ns 0b|----------------------------------------L0.2212-----------------------------------------|"
    - "L0.2214[94202,105976] 193ns 0b|----------------------------------------L0.2214-----------------------------------------|"
    - "L0.2215[94202,105976] 199ns 0b|----------------------------------------L0.2215-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[94202,105976] 199ns |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2211, L0.2212, L0.2214, L0.2215"
    - "  Creating 1 files"
    - "**** Simulation run 401, type=compact(ManySmallFiles). 7 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2824[10862,11776] 193ns|----------------------------------------L0.2824-----------------------------------------|"
    - "L0.2837[10862,11776] 194ns|----------------------------------------L0.2837-----------------------------------------|"
    - "L0.2850[10862,11776] 195ns|----------------------------------------L0.2850-----------------------------------------|"
    - "L0.2863[10862,11776] 196ns|----------------------------------------L0.2863-----------------------------------------|"
    - "L0.2876[10862,11776] 197ns|----------------------------------------L0.2876-----------------------------------------|"
    - "L0.2889[10862,11776] 198ns|----------------------------------------L0.2889-----------------------------------------|"
    - "L0.2902[10862,11776] 199ns|----------------------------------------L0.2902-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[10862,11776] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.2824, L0.2837, L0.2850, L0.2863, L0.2876, L0.2889, L0.2902"
    - "  Creating 1 files"
    - "**** Simulation run 402, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2216[105977,117751] 153ns 12b|----------------------------------------L0.2216-----------------------------------------|"
    - "L0.2217[105977,117751] 173ns 0b|----------------------------------------L0.2217-----------------------------------------|"
    - "L0.2218[105977,117751] 193ns 0b|----------------------------------------L0.2218-----------------------------------------|"
    - "L0.2219[105977,117751] 199ns 0b|----------------------------------------L0.2219-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[105977,117751] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2216, L0.2217, L0.2218, L0.2219"
    - "  Creating 1 files"
    - "**** Simulation run 403, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2220[117752,129526] 153ns 12b|----------------------------------------L0.2220-----------------------------------------|"
    - "L0.2221[117752,129526] 173ns 0b|----------------------------------------L0.2221-----------------------------------------|"
    - "L0.2222[117752,129526] 193ns 0b|----------------------------------------L0.2222-----------------------------------------|"
    - "L0.2223[117752,129526] 199ns 0b|----------------------------------------L0.2223-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[117752,129526] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2220, L0.2221, L0.2222, L0.2223"
    - "  Creating 1 files"
    - "**** Simulation run 404, type=compact(ManySmallFiles). 4 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2224[129527,141301] 153ns 12b|----------------------------------------L0.2224-----------------------------------------|"
    - "L0.2225[129527,141301] 173ns 0b|----------------------------------------L0.2225-----------------------------------------|"
    - "L0.2226[129527,141301] 193ns 0b|----------------------------------------L0.2226-----------------------------------------|"
    - "L0.2227[129527,141301] 199ns 0b|----------------------------------------L0.2227-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[129527,141301] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2224, L0.2225, L0.2226, L0.2227"
    - "  Creating 1 files"
    - "**** Simulation run 405, type=compact(ManySmallFiles). 4 Input Files, 34b total:"
    - "L0                                                                                                                 "
    - "L0.2228[141302,153077] 153ns 34b|----------------------------------------L0.2228-----------------------------------------|"
    - "L0.2229[141302,153077] 173ns 0b|----------------------------------------L0.2229-----------------------------------------|"
    - "L0.2230[141302,153077] 193ns 0b|----------------------------------------L0.2230-----------------------------------------|"
    - "L0.2231[141302,153077] 199ns 0b|----------------------------------------L0.2231-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 34b total:"
    - "L0, all files 34b                                                                                                  "
    - "L0.?[141302,153077] 199ns|------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2228, L0.2229, L0.2230, L0.2231"
    - "  Creating 1 files"
    - "**** Simulation run 406, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2903[20,906] 172ns    |----------------------------------------L0.2903-----------------------------------------|"
    - "L0.2552[173,906] 173ns                  |--------------------------------L0.2552---------------------------------| "
    - "L0.2565[174,906] 174ns                  |--------------------------------L0.2565---------------------------------| "
    - "L0.2578[175,906] 175ns                  |--------------------------------L0.2578---------------------------------| "
    - "L0.2591[176,906] 176ns                  |--------------------------------L0.2591---------------------------------| "
    - "L0.2604[177,906] 177ns                  |--------------------------------L0.2604---------------------------------| "
    - "L0.2617[178,906] 178ns                   |--------------------------------L0.2617--------------------------------| "
    - "L0.2630[179,906] 179ns                   |--------------------------------L0.2630--------------------------------| "
    - "L0.2643[180,906] 180ns                   |--------------------------------L0.2643--------------------------------| "
    - "L0.2656[181,906] 181ns                   |--------------------------------L0.2656--------------------------------| "
    - "L0.2669[182,906] 182ns                   |--------------------------------L0.2669--------------------------------| "
    - "L0.2682[183,906] 183ns                   |--------------------------------L0.2682--------------------------------| "
    - "L0.2695[184,906] 184ns                   |--------------------------------L0.2695--------------------------------| "
    - "L0.2708[185,906] 185ns                   |--------------------------------L0.2708--------------------------------| "
    - "L0.2721[186,906] 186ns                   |--------------------------------L0.2721--------------------------------| "
    - "L0.2734[187,906] 187ns                   |--------------------------------L0.2734--------------------------------| "
    - "L0.2747[188,906] 188ns                    |-------------------------------L0.2747--------------------------------| "
    - "L0.2760[189,906] 189ns                    |-------------------------------L0.2760--------------------------------| "
    - "L0.2773[190,906] 190ns                    |-------------------------------L0.2773--------------------------------| "
    - "L0.2786[191,906] 191ns                    |-------------------------------L0.2786--------------------------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[20,906] 191ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2552, L0.2565, L0.2578, L0.2591, L0.2604, L0.2617, L0.2630, L0.2643, L0.2656, L0.2669, L0.2682, L0.2695, L0.2708, L0.2721, L0.2734, L0.2747, L0.2760, L0.2773, L0.2786, L0.2903"
    - "  Creating 1 files"
    - "**** Simulation run 407, type=compact(ManySmallFiles). 20 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2904[907,1811] 172ns  |----------------------------------------L0.2904-----------------------------------------|"
    - "L0.2553[907,1811] 173ns  |----------------------------------------L0.2553-----------------------------------------|"
    - "L0.2566[907,1811] 174ns  |----------------------------------------L0.2566-----------------------------------------|"
    - "L0.2579[907,1811] 175ns  |----------------------------------------L0.2579-----------------------------------------|"
    - "L0.2592[907,1811] 176ns  |----------------------------------------L0.2592-----------------------------------------|"
    - "L0.2605[907,1811] 177ns  |----------------------------------------L0.2605-----------------------------------------|"
    - "L0.2618[907,1811] 178ns  |----------------------------------------L0.2618-----------------------------------------|"
    - "L0.2631[907,1811] 179ns  |----------------------------------------L0.2631-----------------------------------------|"
    - "L0.2644[907,1811] 180ns  |----------------------------------------L0.2644-----------------------------------------|"
    - "L0.2657[907,1811] 181ns  |----------------------------------------L0.2657-----------------------------------------|"
    - "L0.2670[907,1811] 182ns  |----------------------------------------L0.2670-----------------------------------------|"
    - "L0.2683[907,1811] 183ns  |----------------------------------------L0.2683-----------------------------------------|"
    - "L0.2696[907,1811] 184ns  |----------------------------------------L0.2696-----------------------------------------|"
    - "L0.2709[907,1811] 185ns  |----------------------------------------L0.2709-----------------------------------------|"
    - "L0.2722[907,1811] 186ns  |----------------------------------------L0.2722-----------------------------------------|"
    - "L0.2735[907,1811] 187ns  |----------------------------------------L0.2735-----------------------------------------|"
    - "L0.2748[907,1811] 188ns  |----------------------------------------L0.2748-----------------------------------------|"
    - "L0.2761[907,1811] 189ns  |----------------------------------------L0.2761-----------------------------------------|"
    - "L0.2774[907,1811] 190ns  |----------------------------------------L0.2774-----------------------------------------|"
    - "L0.2787[907,1811] 191ns  |----------------------------------------L0.2787-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1811] 191ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2553, L0.2566, L0.2579, L0.2592, L0.2605, L0.2618, L0.2631, L0.2644, L0.2657, L0.2670, L0.2683, L0.2696, L0.2709, L0.2722, L0.2735, L0.2748, L0.2761, L0.2774, L0.2787, L0.2904"
    - "  Creating 1 files"
    - "**** Simulation run 408, type=compact(ManySmallFiles). 4 Input Files, 189mb total:"
    - "L0                                                                                                                 "
    - "L0.2905[1812,2000] 19ns 189mb|----L0.2905-----|                                                                        "
    - "L0.2906[1812,2716] 172ns 0b|----------------------------------------L0.2906-----------------------------------------|"
    - "L0.2907[1812,2716] 192ns 0b|----------------------------------------L0.2907-----------------------------------------|"
    - "L0.2908[1812,2716] 199ns 0b|----------------------------------------L0.2908-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 189mb total:"
    - "L0, all files 189mb                                                                                                "
    - "L0.?[1812,2716] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L0.2905, L0.2906, L0.2907, L0.2908"
    - "  Creating 1 files"
    - "**** Simulation run 409, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2909[2717,3621] 172ns |----------------------------------------L0.2909-----------------------------------------|"
    - "L0.2910[2717,3621] 192ns |----------------------------------------L0.2910-----------------------------------------|"
    - "L0.2911[2717,3621] 199ns |----------------------------------------L0.2911-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[2717,3621] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2909, L0.2910, L0.2911"
    - "  Creating 1 files"
    - "**** Simulation run 410, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2920[3622,4526] 172ns |----------------------------------------L0.2920-----------------------------------------|"
    - "L0.2921[3622,4526] 192ns |----------------------------------------L0.2921-----------------------------------------|"
    - "L0.2912[3622,4526] 199ns |----------------------------------------L0.2912-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[3622,4526] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2912, L0.2920, L0.2921"
    - "  Creating 1 files"
    - "**** Simulation run 411, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2913[4527,5431] 172ns |----------------------------------------L0.2913-----------------------------------------|"
    - "L0.2914[4527,5431] 192ns |----------------------------------------L0.2914-----------------------------------------|"
    - "L0.2915[4527,5431] 199ns |----------------------------------------L0.2915-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[4527,5431] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2913, L0.2914, L0.2915"
    - "  Creating 1 files"
    - "**** Simulation run 412, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2916[5432,6336] 172ns |----------------------------------------L0.2916-----------------------------------------|"
    - "L0.2917[5432,6336] 192ns |----------------------------------------L0.2917-----------------------------------------|"
    - "L0.2918[5432,6336] 199ns |----------------------------------------L0.2918-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[5432,6336] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2916, L0.2917, L0.2918"
    - "  Creating 1 files"
    - "**** Simulation run 413, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2919[6337,7241] 172ns |----------------------------------------L0.2919-----------------------------------------|"
    - "L0.2922[6337,7241] 192ns |----------------------------------------L0.2922-----------------------------------------|"
    - "L0.2923[6337,7241] 199ns |----------------------------------------L0.2923-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[6337,7241] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2919, L0.2922, L0.2923"
    - "  Creating 1 files"
    - "**** Simulation run 414, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2924[7242,8146] 172ns |----------------------------------------L0.2924-----------------------------------------|"
    - "L0.2925[7242,8146] 192ns |----------------------------------------L0.2925-----------------------------------------|"
    - "L0.2926[7242,8146] 199ns |----------------------------------------L0.2926-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[7242,8146] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2924, L0.2925, L0.2926"
    - "  Creating 1 files"
    - "**** Simulation run 415, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2927[8147,9051] 172ns |----------------------------------------L0.2927-----------------------------------------|"
    - "L0.2928[8147,9051] 192ns |----------------------------------------L0.2928-----------------------------------------|"
    - "L0.2929[8147,9051] 199ns |----------------------------------------L0.2929-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[8147,9051] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2927, L0.2928, L0.2929"
    - "  Creating 1 files"
    - "**** Simulation run 416, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2930[9052,9956] 172ns |----------------------------------------L0.2930-----------------------------------------|"
    - "L0.2931[9052,9956] 192ns |----------------------------------------L0.2931-----------------------------------------|"
    - "L0.2932[9052,9956] 199ns |----------------------------------------L0.2932-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9052,9956] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2930, L0.2931, L0.2932"
    - "  Creating 1 files"
    - "**** Simulation run 417, type=compact(ManySmallFiles). 3 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2933[9957,10861] 172ns|----------------------------------------L0.2933-----------------------------------------|"
    - "L0.2934[9957,10861] 192ns|----------------------------------------L0.2934-----------------------------------------|"
    - "L0.2935[9957,10861] 199ns|----------------------------------------L0.2935-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[9957,10861] 199ns   |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2933, L0.2934, L0.2935"
    - "  Creating 1 files"
    - "**** Simulation run 418, type=compact(ManySmallFiles). 3 Input Files, 12b total:"
    - "L0                                                                                                                 "
    - "L0.2936[10862,11776] 172ns 12b|----------------------------------------L0.2936-----------------------------------------|"
    - "L0.2945[10862,11776] 192ns 0b|----------------------------------------L0.2945-----------------------------------------|"
    - "L0.2946[10862,11776] 199ns 0b|----------------------------------------L0.2946-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 12b total:"
    - "L0, all files 12b                                                                                                  "
    - "L0.?[10862,11776] 199ns  |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L0.2936, L0.2945, L0.2946"
    - "  Creating 1 files"
    - "**** Simulation run 419, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2232[1,906] 0ns       |----------------------------------------L0.2232-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 0ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 0ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 0ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 0ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 0ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 0ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 0ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 420, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2235[1,906] 1ns       |----------------------------------------L0.2235-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 1ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 1ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 1ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 1ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 1ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 1ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 1ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 421, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2238[1,906] 2ns       |----------------------------------------L0.2238-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 2ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 2ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 2ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 2ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 2ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 2ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 2ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 422, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2241[1,906] 3ns       |----------------------------------------L0.2241-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 3ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 3ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 3ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 3ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 3ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 3ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 3ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 423, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2244[1,906] 4ns       |----------------------------------------L0.2244-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 4ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 4ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 4ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 4ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 4ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 4ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 4ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 424, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2247[1,906] 5ns       |----------------------------------------L0.2247-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 5ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 5ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 5ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 5ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 5ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 5ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 5ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 425, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2250[1,906] 6ns       |----------------------------------------L0.2250-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 6ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 6ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 6ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 6ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 6ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 6ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 6ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 426, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2253[1,906] 7ns       |----------------------------------------L0.2253-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 7ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 7ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 7ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 7ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 7ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 7ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 7ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 427, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2256[1,906] 8ns       |----------------------------------------L0.2256-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 8ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 8ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 8ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 8ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 8ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 8ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 8ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 428, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2259[1,906] 9ns       |----------------------------------------L0.2259-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 9ns 6mb      |---L0.?---|                                                                              "
    - "L0.?[131,259] 9ns 6mb                |---L0.?---|                                                                  "
    - "L0.?[260,388] 9ns 6mb                             |---L0.?---|                                                     "
    - "L0.?[389,517] 9ns 6mb                                          |---L0.?---|                                        "
    - "L0.?[518,646] 9ns 6mb                                                       |---L0.?---|                           "
    - "L0.?[647,775] 9ns 6mb                                                                    |---L0.?---|              "
    - "L0.?[776,906] 9ns 7mb                                                                                 |---L0.?---| "
    - "**** Simulation run 429, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2262[1,906] 10ns      |----------------------------------------L0.2262-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 10ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 10ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 10ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 10ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 10ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 10ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 10ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 430, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2265[1,906] 11ns      |----------------------------------------L0.2265-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 11ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 11ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 11ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 11ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 11ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 11ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 11ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 431, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2268[1,906] 12ns      |----------------------------------------L0.2268-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 12ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 12ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 12ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 12ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 12ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 12ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 12ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 432, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2271[1,906] 13ns      |----------------------------------------L0.2271-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 13ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 13ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 13ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 13ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 13ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 13ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 13ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 433, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2274[1,906] 14ns      |----------------------------------------L0.2274-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 14ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 14ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 14ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 14ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 14ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 14ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 14ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 434, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2277[1,906] 15ns      |----------------------------------------L0.2277-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 15ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 15ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 15ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 15ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 15ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 15ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 15ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 435, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2280[1,906] 16ns      |----------------------------------------L0.2280-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 16ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 16ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 16ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 16ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 16ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 16ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 16ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 436, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2283[1,906] 17ns      |----------------------------------------L0.2283-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 17ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 17ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 17ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 17ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 17ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 17ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 17ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 437, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2286[1,906] 18ns      |----------------------------------------L0.2286-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 18ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 18ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 18ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 18ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 18ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 18ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 18ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 438, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2289[1,906] 19ns      |----------------------------------------L0.2289-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[1,130] 19ns 6mb     |---L0.?---|                                                                              "
    - "L0.?[131,259] 19ns 6mb               |---L0.?---|                                                                  "
    - "L0.?[260,388] 19ns 6mb                            |---L0.?---|                                                     "
    - "L0.?[389,517] 19ns 6mb                                         |---L0.?---|                                        "
    - "L0.?[518,646] 19ns 6mb                                                      |---L0.?---|                           "
    - "L0.?[647,775] 19ns 6mb                                                                   |---L0.?---|              "
    - "L0.?[776,906] 19ns 7mb                                                                                |---L0.?---| "
    - "**** Simulation run 439, type=split(HighL0OverlapTotalBacklog)(split_times=[130, 259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2951[20,906] 191ns    |----------------------------------------L0.2951-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[20,130] 191ns       |--L0.?---|                                                                               "
    - "L0.?[131,259] 191ns                 |---L0.?----|                                                                  "
    - "L0.?[260,388] 191ns                              |---L0.?----|                                                     "
    - "L0.?[389,517] 191ns                                           |---L0.?----|                                        "
    - "L0.?[518,646] 191ns                                                        |---L0.?----|                           "
    - "L0.?[647,775] 191ns                                                                     |---L0.?----|              "
    - "L0.?[776,906] 191ns                                                                                  |---L0.?----| "
    - "**** Simulation run 440, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2799[192,906] 192ns   |----------------------------------------L0.2799-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[192,259] 192ns      |-L0.?-|                                                                                  "
    - "L0.?[260,388] 192ns              |-----L0.?-----|                                                                  "
    - "L0.?[389,517] 192ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 192ns                                               |-----L0.?-----|                                 "
    - "L0.?[647,775] 192ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 192ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 441, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2812[193,906] 193ns   |----------------------------------------L0.2812-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[193,259] 193ns      |-L0.?-|                                                                                  "
    - "L0.?[260,388] 193ns              |-----L0.?-----|                                                                  "
    - "L0.?[389,517] 193ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 193ns                                               |-----L0.?-----|                                 "
    - "L0.?[647,775] 193ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 193ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 442, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2825[194,906] 194ns   |----------------------------------------L0.2825-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[194,259] 194ns      |-L0.?-|                                                                                  "
    - "L0.?[260,388] 194ns              |-----L0.?-----|                                                                  "
    - "L0.?[389,517] 194ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 194ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 194ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 194ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 443, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2838[195,906] 195ns   |----------------------------------------L0.2838-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[195,259] 195ns      |-L0.?-|                                                                                  "
    - "L0.?[260,388] 195ns              |-----L0.?-----|                                                                  "
    - "L0.?[389,517] 195ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 195ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 195ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 195ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 444, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2851[196,906] 196ns   |----------------------------------------L0.2851-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[196,259] 196ns      |L0.?-|                                                                                   "
    - "L0.?[260,388] 196ns              |-----L0.?-----|                                                                  "
    - "L0.?[389,517] 196ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 196ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 196ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 196ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 445, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2864[197,906] 197ns   |----------------------------------------L0.2864-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[197,259] 197ns      |L0.?-|                                                                                   "
    - "L0.?[260,388] 197ns             |-----L0.?-----|                                                                   "
    - "L0.?[389,517] 197ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 197ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 197ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 197ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 446, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2877[198,906] 198ns   |----------------------------------------L0.2877-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[198,259] 198ns      |L0.?-|                                                                                   "
    - "L0.?[260,388] 198ns             |-----L0.?-----|                                                                   "
    - "L0.?[389,517] 198ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 198ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 198ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 198ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 447, type=split(HighL0OverlapTotalBacklog)(split_times=[259, 388, 517, 646, 775]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2890[199,906] 199ns   |----------------------------------------L0.2890-----------------------------------------|"
    - "**** 6 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[199,259] 199ns      |L0.?-|                                                                                   "
    - "L0.?[260,388] 199ns             |-----L0.?-----|                                                                   "
    - "L0.?[389,517] 199ns                              |-----L0.?-----|                                                  "
    - "L0.?[518,646] 199ns                                              |-----L0.?-----|                                  "
    - "L0.?[647,775] 199ns                                                               |-----L0.?-----|                 "
    - "L0.?[776,906] 199ns                                                                               |-----L0.?-----| "
    - "**** Simulation run 448, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2233[907,1811] 0ns    |----------------------------------------L0.2233-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 0ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 0ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 0ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 0ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 0ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 0ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 0ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 449, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2236[907,1811] 1ns    |----------------------------------------L0.2236-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 1ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 1ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 1ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 1ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 1ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 1ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 1ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 450, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2239[907,1811] 2ns    |----------------------------------------L0.2239-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 2ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 2ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 2ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 2ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 2ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 2ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 2ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 451, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2242[907,1811] 3ns    |----------------------------------------L0.2242-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 3ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 3ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 3ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 3ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 3ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 3ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 3ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 452, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2245[907,1811] 4ns    |----------------------------------------L0.2245-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 4ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 4ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 4ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 4ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 4ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 4ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 4ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 453, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2248[907,1811] 5ns    |----------------------------------------L0.2248-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 5ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 5ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 5ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 5ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 5ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 5ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 5ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 454, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2251[907,1811] 6ns    |----------------------------------------L0.2251-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 6ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 6ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 6ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 6ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 6ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 6ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 6ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 455, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2254[907,1811] 7ns    |----------------------------------------L0.2254-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 7ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 7ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 7ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 7ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 7ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 7ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 7ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 456, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2257[907,1811] 8ns    |----------------------------------------L0.2257-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 8ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 8ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 8ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 8ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 8ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 8ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 8ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 457, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2260[907,1811] 9ns    |----------------------------------------L0.2260-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 9ns 7mb   |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 9ns 6mb              |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 9ns 6mb                           |---L0.?---|                                                     "
    - "L0.?[1295,1423] 9ns 6mb                                        |---L0.?---|                                        "
    - "L0.?[1424,1552] 9ns 6mb                                                     |---L0.?---|                           "
    - "L0.?[1553,1681] 9ns 6mb                                                                  |---L0.?---|              "
    - "L0.?[1682,1811] 9ns 7mb                                                                               |---L0.?---| "
    - "**** Simulation run 458, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2263[907,1811] 10ns   |----------------------------------------L0.2263-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 10ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 10ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 10ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 10ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 10ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 10ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 10ns 7mb                                                                              |---L0.?---| "
    - "Committing partition 1:"
    - "  Soft Deleting 40 files: L0.2232, L0.2233, L0.2235, L0.2236, L0.2238, L0.2239, L0.2241, L0.2242, L0.2244, L0.2245, L0.2247, L0.2248, L0.2250, L0.2251, L0.2253, L0.2254, L0.2256, L0.2257, L0.2259, L0.2260, L0.2262, L0.2263, L0.2265, L0.2268, L0.2271, L0.2274, L0.2277, L0.2280, L0.2283, L0.2286, L0.2289, L0.2799, L0.2812, L0.2825, L0.2838, L0.2851, L0.2864, L0.2877, L0.2890, L0.2951"
    - "  Creating 272 files"
    - "**** Simulation run 459, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2266[907,1811] 11ns   |----------------------------------------L0.2266-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 11ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 11ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 11ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 11ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 11ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 11ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 11ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 460, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2269[907,1811] 12ns   |----------------------------------------L0.2269-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 12ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 12ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 12ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 12ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 12ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 12ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 12ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 461, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2272[907,1811] 13ns   |----------------------------------------L0.2272-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 13ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 13ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 13ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 13ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 13ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 13ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 13ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 462, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2275[907,1811] 14ns   |----------------------------------------L0.2275-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 14ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 14ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 14ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 14ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 14ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 14ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 14ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 463, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2278[907,1811] 15ns   |----------------------------------------L0.2278-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 15ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 15ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 15ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 15ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 15ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 15ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 15ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 464, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2281[907,1811] 16ns   |----------------------------------------L0.2281-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 16ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 16ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 16ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 16ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 16ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 16ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 16ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 465, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2284[907,1811] 17ns   |----------------------------------------L0.2284-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 17ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 17ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 17ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 17ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 17ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 17ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 17ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 466, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2287[907,1811] 18ns   |----------------------------------------L0.2287-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 18ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 18ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 18ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 18ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 18ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 18ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 18ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 467, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 45mb total:"
    - "L0, all files 45mb                                                                                                 "
    - "L0.2290[907,1811] 19ns   |----------------------------------------L0.2290-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 45mb total:"
    - "L0                                                                                                                 "
    - "L0.?[907,1036] 19ns 7mb  |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 19ns 6mb             |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 19ns 6mb                          |---L0.?---|                                                     "
    - "L0.?[1295,1423] 19ns 6mb                                       |---L0.?---|                                        "
    - "L0.?[1424,1552] 19ns 6mb                                                    |---L0.?---|                           "
    - "L0.?[1553,1681] 19ns 6mb                                                                 |---L0.?---|              "
    - "L0.?[1682,1811] 19ns 7mb                                                                              |---L0.?---| "
    - "**** Simulation run 468, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2952[907,1811] 191ns  |----------------------------------------L0.2952-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 191ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 191ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 191ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 191ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 191ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 191ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 191ns                                                                                 |---L0.?---| "
    - "**** Simulation run 469, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2800[907,1811] 192ns  |----------------------------------------L0.2800-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 192ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 192ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 192ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 192ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 192ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 192ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 192ns                                                                                 |---L0.?---| "
    - "**** Simulation run 470, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2813[907,1811] 193ns  |----------------------------------------L0.2813-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 193ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 193ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 193ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 193ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 193ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 193ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 193ns                                                                                 |---L0.?---| "
    - "**** Simulation run 471, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2826[907,1811] 194ns  |----------------------------------------L0.2826-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 194ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 194ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 194ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 194ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 194ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 194ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 194ns                                                                                 |---L0.?---| "
    - "**** Simulation run 472, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2839[907,1811] 195ns  |----------------------------------------L0.2839-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 195ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 195ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 195ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 195ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 195ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 195ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 195ns                                                                                 |---L0.?---| "
    - "**** Simulation run 473, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2852[907,1811] 196ns  |----------------------------------------L0.2852-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 196ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 196ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 196ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 196ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 196ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 196ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 196ns                                                                                 |---L0.?---| "
    - "**** Simulation run 474, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2865[907,1811] 197ns  |----------------------------------------L0.2865-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 197ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 197ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 197ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 197ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 197ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 197ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 197ns                                                                                 |---L0.?---| "
    - "**** Simulation run 475, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2878[907,1811] 198ns  |----------------------------------------L0.2878-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 198ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 198ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 198ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 198ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 198ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 198ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 198ns                                                                                 |---L0.?---| "
    - "**** Simulation run 476, type=split(HighL0OverlapTotalBacklog)(split_times=[1036, 1165, 1294, 1423, 1552, 1681]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.2891[907,1811] 199ns  |----------------------------------------L0.2891-----------------------------------------|"
    - "**** 7 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 199ns     |---L0.?---|                                                                              "
    - "L0.?[1037,1165] 199ns                |---L0.?---|                                                                  "
    - "L0.?[1166,1294] 199ns                             |---L0.?---|                                                     "
    - "L0.?[1295,1423] 199ns                                          |---L0.?---|                                        "
    - "L0.?[1424,1552] 199ns                                                       |---L0.?---|                           "
    - "L0.?[1553,1681] 199ns                                                                    |---L0.?---|              "
    - "L0.?[1682,1811] 199ns                                                                                 |---L0.?---| "
    - "Committing partition 1:"
    - "  Soft Deleting 18 files: L0.2266, L0.2269, L0.2272, L0.2275, L0.2278, L0.2281, L0.2284, L0.2287, L0.2290, L0.2800, L0.2813, L0.2826, L0.2839, L0.2852, L0.2865, L0.2878, L0.2891, L0.2952"
    - "  Creating 126 files"
    - "**** Simulation run 477, type=compact(ManySmallFiles). 20 Input Files, 130mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2964[1,130] 0ns       |----------------------------------------L0.2964-----------------------------------------|"
    - "L0.2971[1,130] 1ns       |----------------------------------------L0.2971-----------------------------------------|"
    - "L0.2978[1,130] 2ns       |----------------------------------------L0.2978-----------------------------------------|"
    - "L0.2985[1,130] 3ns       |----------------------------------------L0.2985-----------------------------------------|"
    - "L0.2992[1,130] 4ns       |----------------------------------------L0.2992-----------------------------------------|"
    - "L0.2999[1,130] 5ns       |----------------------------------------L0.2999-----------------------------------------|"
    - "L0.3006[1,130] 6ns       |----------------------------------------L0.3006-----------------------------------------|"
    - "L0.3013[1,130] 7ns       |----------------------------------------L0.3013-----------------------------------------|"
    - "L0.3020[1,130] 8ns       |----------------------------------------L0.3020-----------------------------------------|"
    - "L0.3027[1,130] 9ns       |----------------------------------------L0.3027-----------------------------------------|"
    - "L0.3034[1,130] 10ns      |----------------------------------------L0.3034-----------------------------------------|"
    - "L0.3041[1,130] 11ns      |----------------------------------------L0.3041-----------------------------------------|"
    - "L0.3048[1,130] 12ns      |----------------------------------------L0.3048-----------------------------------------|"
    - "L0.3055[1,130] 13ns      |----------------------------------------L0.3055-----------------------------------------|"
    - "L0.3062[1,130] 14ns      |----------------------------------------L0.3062-----------------------------------------|"
    - "L0.3069[1,130] 15ns      |----------------------------------------L0.3069-----------------------------------------|"
    - "L0.3076[1,130] 16ns      |----------------------------------------L0.3076-----------------------------------------|"
    - "L0.3083[1,130] 17ns      |----------------------------------------L0.3083-----------------------------------------|"
    - "L0.3090[1,130] 18ns      |----------------------------------------L0.3090-----------------------------------------|"
    - "L0.3097[1,130] 19ns      |----------------------------------------L0.3097-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 130mb total:"
    - "L0, all files 130mb                                                                                                "
    - "L0.?[1,130] 19ns         |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2964, L0.2971, L0.2978, L0.2985, L0.2992, L0.2999, L0.3006, L0.3013, L0.3020, L0.3027, L0.3034, L0.3041, L0.3048, L0.3055, L0.3062, L0.3069, L0.3076, L0.3083, L0.3090, L0.3097"
    - "  Creating 1 files"
    - "**** Simulation run 478, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2965[131,259] 0ns     |----------------------------------------L0.2965-----------------------------------------|"
    - "L0.2972[131,259] 1ns     |----------------------------------------L0.2972-----------------------------------------|"
    - "L0.2979[131,259] 2ns     |----------------------------------------L0.2979-----------------------------------------|"
    - "L0.2986[131,259] 3ns     |----------------------------------------L0.2986-----------------------------------------|"
    - "L0.2993[131,259] 4ns     |----------------------------------------L0.2993-----------------------------------------|"
    - "L0.3000[131,259] 5ns     |----------------------------------------L0.3000-----------------------------------------|"
    - "L0.3007[131,259] 6ns     |----------------------------------------L0.3007-----------------------------------------|"
    - "L0.3014[131,259] 7ns     |----------------------------------------L0.3014-----------------------------------------|"
    - "L0.3021[131,259] 8ns     |----------------------------------------L0.3021-----------------------------------------|"
    - "L0.3028[131,259] 9ns     |----------------------------------------L0.3028-----------------------------------------|"
    - "L0.3035[131,259] 10ns    |----------------------------------------L0.3035-----------------------------------------|"
    - "L0.3042[131,259] 11ns    |----------------------------------------L0.3042-----------------------------------------|"
    - "L0.3049[131,259] 12ns    |----------------------------------------L0.3049-----------------------------------------|"
    - "L0.3056[131,259] 13ns    |----------------------------------------L0.3056-----------------------------------------|"
    - "L0.3063[131,259] 14ns    |----------------------------------------L0.3063-----------------------------------------|"
    - "L0.3070[131,259] 15ns    |----------------------------------------L0.3070-----------------------------------------|"
    - "L0.3077[131,259] 16ns    |----------------------------------------L0.3077-----------------------------------------|"
    - "L0.3084[131,259] 17ns    |----------------------------------------L0.3084-----------------------------------------|"
    - "L0.3091[131,259] 18ns    |----------------------------------------L0.3091-----------------------------------------|"
    - "L0.3098[131,259] 19ns    |----------------------------------------L0.3098-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[131,259] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2965, L0.2972, L0.2979, L0.2986, L0.2993, L0.3000, L0.3007, L0.3014, L0.3021, L0.3028, L0.3035, L0.3042, L0.3049, L0.3056, L0.3063, L0.3070, L0.3077, L0.3084, L0.3091, L0.3098"
    - "  Creating 1 files"
    - "**** Simulation run 479, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3105[131,259] 191ns   |----------------------------------------L0.3105-----------------------------------------|"
    - "L0.3111[192,259] 192ns                                             |-------------------L0.3111-------------------| "
    - "L0.3117[193,259] 193ns                                              |------------------L0.3117-------------------| "
    - "L0.3123[194,259] 194ns                                               |------------------L0.3123------------------| "
    - "L0.3129[195,259] 195ns                                                |------------------L0.3129------------------|"
    - "L0.3135[196,259] 196ns                                                |-----------------L0.3135------------------| "
    - "L0.3141[197,259] 197ns                                                 |-----------------L0.3141-----------------| "
    - "L0.3147[198,259] 198ns                                                  |----------------L0.3147-----------------| "
    - "L0.3153[199,259] 199ns                                                  |----------------L0.3153-----------------| "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[131,259] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3105, L0.3111, L0.3117, L0.3123, L0.3129, L0.3135, L0.3141, L0.3147, L0.3153"
    - "  Creating 1 files"
    - "**** Simulation run 480, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2966[260,388] 0ns     |----------------------------------------L0.2966-----------------------------------------|"
    - "L0.2973[260,388] 1ns     |----------------------------------------L0.2973-----------------------------------------|"
    - "L0.2980[260,388] 2ns     |----------------------------------------L0.2980-----------------------------------------|"
    - "L0.2987[260,388] 3ns     |----------------------------------------L0.2987-----------------------------------------|"
    - "L0.2994[260,388] 4ns     |----------------------------------------L0.2994-----------------------------------------|"
    - "L0.3001[260,388] 5ns     |----------------------------------------L0.3001-----------------------------------------|"
    - "L0.3008[260,388] 6ns     |----------------------------------------L0.3008-----------------------------------------|"
    - "L0.3015[260,388] 7ns     |----------------------------------------L0.3015-----------------------------------------|"
    - "L0.3022[260,388] 8ns     |----------------------------------------L0.3022-----------------------------------------|"
    - "L0.3029[260,388] 9ns     |----------------------------------------L0.3029-----------------------------------------|"
    - "L0.3036[260,388] 10ns    |----------------------------------------L0.3036-----------------------------------------|"
    - "L0.3043[260,388] 11ns    |----------------------------------------L0.3043-----------------------------------------|"
    - "L0.3050[260,388] 12ns    |----------------------------------------L0.3050-----------------------------------------|"
    - "L0.3057[260,388] 13ns    |----------------------------------------L0.3057-----------------------------------------|"
    - "L0.3064[260,388] 14ns    |----------------------------------------L0.3064-----------------------------------------|"
    - "L0.3071[260,388] 15ns    |----------------------------------------L0.3071-----------------------------------------|"
    - "L0.3078[260,388] 16ns    |----------------------------------------L0.3078-----------------------------------------|"
    - "L0.3085[260,388] 17ns    |----------------------------------------L0.3085-----------------------------------------|"
    - "L0.3092[260,388] 18ns    |----------------------------------------L0.3092-----------------------------------------|"
    - "L0.3099[260,388] 19ns    |----------------------------------------L0.3099-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[260,388] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2966, L0.2973, L0.2980, L0.2987, L0.2994, L0.3001, L0.3008, L0.3015, L0.3022, L0.3029, L0.3036, L0.3043, L0.3050, L0.3057, L0.3064, L0.3071, L0.3078, L0.3085, L0.3092, L0.3099"
    - "  Creating 1 files"
    - "**** Simulation run 481, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3106[260,388] 191ns   |----------------------------------------L0.3106-----------------------------------------|"
    - "L0.3112[260,388] 192ns   |----------------------------------------L0.3112-----------------------------------------|"
    - "L0.3118[260,388] 193ns   |----------------------------------------L0.3118-----------------------------------------|"
    - "L0.3124[260,388] 194ns   |----------------------------------------L0.3124-----------------------------------------|"
    - "L0.3130[260,388] 195ns   |----------------------------------------L0.3130-----------------------------------------|"
    - "L0.3136[260,388] 196ns   |----------------------------------------L0.3136-----------------------------------------|"
    - "L0.3142[260,388] 197ns   |----------------------------------------L0.3142-----------------------------------------|"
    - "L0.3148[260,388] 198ns   |----------------------------------------L0.3148-----------------------------------------|"
    - "L0.3154[260,388] 199ns   |----------------------------------------L0.3154-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[260,388] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3106, L0.3112, L0.3118, L0.3124, L0.3130, L0.3136, L0.3142, L0.3148, L0.3154"
    - "  Creating 1 files"
    - "**** Simulation run 482, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2967[389,517] 0ns     |----------------------------------------L0.2967-----------------------------------------|"
    - "L0.2974[389,517] 1ns     |----------------------------------------L0.2974-----------------------------------------|"
    - "L0.2981[389,517] 2ns     |----------------------------------------L0.2981-----------------------------------------|"
    - "L0.2988[389,517] 3ns     |----------------------------------------L0.2988-----------------------------------------|"
    - "L0.2995[389,517] 4ns     |----------------------------------------L0.2995-----------------------------------------|"
    - "L0.3002[389,517] 5ns     |----------------------------------------L0.3002-----------------------------------------|"
    - "L0.3009[389,517] 6ns     |----------------------------------------L0.3009-----------------------------------------|"
    - "L0.3016[389,517] 7ns     |----------------------------------------L0.3016-----------------------------------------|"
    - "L0.3023[389,517] 8ns     |----------------------------------------L0.3023-----------------------------------------|"
    - "L0.3030[389,517] 9ns     |----------------------------------------L0.3030-----------------------------------------|"
    - "L0.3037[389,517] 10ns    |----------------------------------------L0.3037-----------------------------------------|"
    - "L0.3044[389,517] 11ns    |----------------------------------------L0.3044-----------------------------------------|"
    - "L0.3051[389,517] 12ns    |----------------------------------------L0.3051-----------------------------------------|"
    - "L0.3058[389,517] 13ns    |----------------------------------------L0.3058-----------------------------------------|"
    - "L0.3065[389,517] 14ns    |----------------------------------------L0.3065-----------------------------------------|"
    - "L0.3072[389,517] 15ns    |----------------------------------------L0.3072-----------------------------------------|"
    - "L0.3079[389,517] 16ns    |----------------------------------------L0.3079-----------------------------------------|"
    - "L0.3086[389,517] 17ns    |----------------------------------------L0.3086-----------------------------------------|"
    - "L0.3093[389,517] 18ns    |----------------------------------------L0.3093-----------------------------------------|"
    - "L0.3100[389,517] 19ns    |----------------------------------------L0.3100-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[389,517] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2967, L0.2974, L0.2981, L0.2988, L0.2995, L0.3002, L0.3009, L0.3016, L0.3023, L0.3030, L0.3037, L0.3044, L0.3051, L0.3058, L0.3065, L0.3072, L0.3079, L0.3086, L0.3093, L0.3100"
    - "  Creating 1 files"
    - "**** Simulation run 483, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3107[389,517] 191ns   |----------------------------------------L0.3107-----------------------------------------|"
    - "L0.3113[389,517] 192ns   |----------------------------------------L0.3113-----------------------------------------|"
    - "L0.3119[389,517] 193ns   |----------------------------------------L0.3119-----------------------------------------|"
    - "L0.3125[389,517] 194ns   |----------------------------------------L0.3125-----------------------------------------|"
    - "L0.3131[389,517] 195ns   |----------------------------------------L0.3131-----------------------------------------|"
    - "L0.3137[389,517] 196ns   |----------------------------------------L0.3137-----------------------------------------|"
    - "L0.3143[389,517] 197ns   |----------------------------------------L0.3143-----------------------------------------|"
    - "L0.3149[389,517] 198ns   |----------------------------------------L0.3149-----------------------------------------|"
    - "L0.3155[389,517] 199ns   |----------------------------------------L0.3155-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[389,517] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3107, L0.3113, L0.3119, L0.3125, L0.3131, L0.3137, L0.3143, L0.3149, L0.3155"
    - "  Creating 1 files"
    - "**** Simulation run 484, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2968[518,646] 0ns     |----------------------------------------L0.2968-----------------------------------------|"
    - "L0.2975[518,646] 1ns     |----------------------------------------L0.2975-----------------------------------------|"
    - "L0.2982[518,646] 2ns     |----------------------------------------L0.2982-----------------------------------------|"
    - "L0.2989[518,646] 3ns     |----------------------------------------L0.2989-----------------------------------------|"
    - "L0.2996[518,646] 4ns     |----------------------------------------L0.2996-----------------------------------------|"
    - "L0.3003[518,646] 5ns     |----------------------------------------L0.3003-----------------------------------------|"
    - "L0.3010[518,646] 6ns     |----------------------------------------L0.3010-----------------------------------------|"
    - "L0.3017[518,646] 7ns     |----------------------------------------L0.3017-----------------------------------------|"
    - "L0.3024[518,646] 8ns     |----------------------------------------L0.3024-----------------------------------------|"
    - "L0.3031[518,646] 9ns     |----------------------------------------L0.3031-----------------------------------------|"
    - "L0.3038[518,646] 10ns    |----------------------------------------L0.3038-----------------------------------------|"
    - "L0.3045[518,646] 11ns    |----------------------------------------L0.3045-----------------------------------------|"
    - "L0.3052[518,646] 12ns    |----------------------------------------L0.3052-----------------------------------------|"
    - "L0.3059[518,646] 13ns    |----------------------------------------L0.3059-----------------------------------------|"
    - "L0.3066[518,646] 14ns    |----------------------------------------L0.3066-----------------------------------------|"
    - "L0.3073[518,646] 15ns    |----------------------------------------L0.3073-----------------------------------------|"
    - "L0.3080[518,646] 16ns    |----------------------------------------L0.3080-----------------------------------------|"
    - "L0.3087[518,646] 17ns    |----------------------------------------L0.3087-----------------------------------------|"
    - "L0.3094[518,646] 18ns    |----------------------------------------L0.3094-----------------------------------------|"
    - "L0.3101[518,646] 19ns    |----------------------------------------L0.3101-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[518,646] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2968, L0.2975, L0.2982, L0.2989, L0.2996, L0.3003, L0.3010, L0.3017, L0.3024, L0.3031, L0.3038, L0.3045, L0.3052, L0.3059, L0.3066, L0.3073, L0.3080, L0.3087, L0.3094, L0.3101"
    - "  Creating 1 files"
    - "**** Simulation run 485, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3108[518,646] 191ns   |----------------------------------------L0.3108-----------------------------------------|"
    - "L0.3114[518,646] 192ns   |----------------------------------------L0.3114-----------------------------------------|"
    - "L0.3120[518,646] 193ns   |----------------------------------------L0.3120-----------------------------------------|"
    - "L0.3126[518,646] 194ns   |----------------------------------------L0.3126-----------------------------------------|"
    - "L0.3132[518,646] 195ns   |----------------------------------------L0.3132-----------------------------------------|"
    - "L0.3138[518,646] 196ns   |----------------------------------------L0.3138-----------------------------------------|"
    - "L0.3144[518,646] 197ns   |----------------------------------------L0.3144-----------------------------------------|"
    - "L0.3150[518,646] 198ns   |----------------------------------------L0.3150-----------------------------------------|"
    - "L0.3156[518,646] 199ns   |----------------------------------------L0.3156-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[518,646] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3108, L0.3114, L0.3120, L0.3126, L0.3132, L0.3138, L0.3144, L0.3150, L0.3156"
    - "  Creating 1 files"
    - "**** Simulation run 486, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.2969[647,775] 0ns     |----------------------------------------L0.2969-----------------------------------------|"
    - "L0.2976[647,775] 1ns     |----------------------------------------L0.2976-----------------------------------------|"
    - "L0.2983[647,775] 2ns     |----------------------------------------L0.2983-----------------------------------------|"
    - "L0.2990[647,775] 3ns     |----------------------------------------L0.2990-----------------------------------------|"
    - "L0.2997[647,775] 4ns     |----------------------------------------L0.2997-----------------------------------------|"
    - "L0.3004[647,775] 5ns     |----------------------------------------L0.3004-----------------------------------------|"
    - "L0.3011[647,775] 6ns     |----------------------------------------L0.3011-----------------------------------------|"
    - "L0.3018[647,775] 7ns     |----------------------------------------L0.3018-----------------------------------------|"
    - "L0.3025[647,775] 8ns     |----------------------------------------L0.3025-----------------------------------------|"
    - "L0.3032[647,775] 9ns     |----------------------------------------L0.3032-----------------------------------------|"
    - "L0.3039[647,775] 10ns    |----------------------------------------L0.3039-----------------------------------------|"
    - "L0.3046[647,775] 11ns    |----------------------------------------L0.3046-----------------------------------------|"
    - "L0.3053[647,775] 12ns    |----------------------------------------L0.3053-----------------------------------------|"
    - "L0.3060[647,775] 13ns    |----------------------------------------L0.3060-----------------------------------------|"
    - "L0.3067[647,775] 14ns    |----------------------------------------L0.3067-----------------------------------------|"
    - "L0.3074[647,775] 15ns    |----------------------------------------L0.3074-----------------------------------------|"
    - "L0.3081[647,775] 16ns    |----------------------------------------L0.3081-----------------------------------------|"
    - "L0.3088[647,775] 17ns    |----------------------------------------L0.3088-----------------------------------------|"
    - "L0.3095[647,775] 18ns    |----------------------------------------L0.3095-----------------------------------------|"
    - "L0.3102[647,775] 19ns    |----------------------------------------L0.3102-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[647,775] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2969, L0.2976, L0.2983, L0.2990, L0.2997, L0.3004, L0.3011, L0.3018, L0.3025, L0.3032, L0.3039, L0.3046, L0.3053, L0.3060, L0.3067, L0.3074, L0.3081, L0.3088, L0.3095, L0.3102"
    - "  Creating 1 files"
    - "**** Simulation run 487, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3109[647,775] 191ns   |----------------------------------------L0.3109-----------------------------------------|"
    - "L0.3115[647,775] 192ns   |----------------------------------------L0.3115-----------------------------------------|"
    - "L0.3121[647,775] 193ns   |----------------------------------------L0.3121-----------------------------------------|"
    - "L0.3127[647,775] 194ns   |----------------------------------------L0.3127-----------------------------------------|"
    - "L0.3133[647,775] 195ns   |----------------------------------------L0.3133-----------------------------------------|"
    - "L0.3139[647,775] 196ns   |----------------------------------------L0.3139-----------------------------------------|"
    - "L0.3145[647,775] 197ns   |----------------------------------------L0.3145-----------------------------------------|"
    - "L0.3151[647,775] 198ns   |----------------------------------------L0.3151-----------------------------------------|"
    - "L0.3157[647,775] 199ns   |----------------------------------------L0.3157-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[647,775] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3109, L0.3115, L0.3121, L0.3127, L0.3133, L0.3139, L0.3145, L0.3151, L0.3157"
    - "  Creating 1 files"
    - "**** Simulation run 488, type=compact(ManySmallFiles). 20 Input Files, 131mb total:"
    - "L0, all files 7mb                                                                                                  "
    - "L0.2970[776,906] 0ns     |----------------------------------------L0.2970-----------------------------------------|"
    - "L0.2977[776,906] 1ns     |----------------------------------------L0.2977-----------------------------------------|"
    - "L0.2984[776,906] 2ns     |----------------------------------------L0.2984-----------------------------------------|"
    - "L0.2991[776,906] 3ns     |----------------------------------------L0.2991-----------------------------------------|"
    - "L0.2998[776,906] 4ns     |----------------------------------------L0.2998-----------------------------------------|"
    - "L0.3005[776,906] 5ns     |----------------------------------------L0.3005-----------------------------------------|"
    - "L0.3012[776,906] 6ns     |----------------------------------------L0.3012-----------------------------------------|"
    - "L0.3019[776,906] 7ns     |----------------------------------------L0.3019-----------------------------------------|"
    - "L0.3026[776,906] 8ns     |----------------------------------------L0.3026-----------------------------------------|"
    - "L0.3033[776,906] 9ns     |----------------------------------------L0.3033-----------------------------------------|"
    - "L0.3040[776,906] 10ns    |----------------------------------------L0.3040-----------------------------------------|"
    - "L0.3047[776,906] 11ns    |----------------------------------------L0.3047-----------------------------------------|"
    - "L0.3054[776,906] 12ns    |----------------------------------------L0.3054-----------------------------------------|"
    - "L0.3061[776,906] 13ns    |----------------------------------------L0.3061-----------------------------------------|"
    - "L0.3068[776,906] 14ns    |----------------------------------------L0.3068-----------------------------------------|"
    - "L0.3075[776,906] 15ns    |----------------------------------------L0.3075-----------------------------------------|"
    - "L0.3082[776,906] 16ns    |----------------------------------------L0.3082-----------------------------------------|"
    - "L0.3089[776,906] 17ns    |----------------------------------------L0.3089-----------------------------------------|"
    - "L0.3096[776,906] 18ns    |----------------------------------------L0.3096-----------------------------------------|"
    - "L0.3103[776,906] 19ns    |----------------------------------------L0.3103-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 131mb total:"
    - "L0, all files 131mb                                                                                                "
    - "L0.?[776,906] 19ns       |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2970, L0.2977, L0.2984, L0.2991, L0.2998, L0.3005, L0.3012, L0.3019, L0.3026, L0.3033, L0.3040, L0.3047, L0.3054, L0.3061, L0.3068, L0.3075, L0.3082, L0.3089, L0.3096, L0.3103"
    - "  Creating 1 files"
    - "**** Simulation run 489, type=compact(ManySmallFiles). 20 Input Files, 130mb total:"
    - "L0, all files 7mb                                                                                                  "
    - "L0.3159[907,1036] 0ns    |----------------------------------------L0.3159-----------------------------------------|"
    - "L0.3166[907,1036] 1ns    |----------------------------------------L0.3166-----------------------------------------|"
    - "L0.3173[907,1036] 2ns    |----------------------------------------L0.3173-----------------------------------------|"
    - "L0.3180[907,1036] 3ns    |----------------------------------------L0.3180-----------------------------------------|"
    - "L0.3187[907,1036] 4ns    |----------------------------------------L0.3187-----------------------------------------|"
    - "L0.3194[907,1036] 5ns    |----------------------------------------L0.3194-----------------------------------------|"
    - "L0.3201[907,1036] 6ns    |----------------------------------------L0.3201-----------------------------------------|"
    - "L0.3208[907,1036] 7ns    |----------------------------------------L0.3208-----------------------------------------|"
    - "L0.3215[907,1036] 8ns    |----------------------------------------L0.3215-----------------------------------------|"
    - "L0.3222[907,1036] 9ns    |----------------------------------------L0.3222-----------------------------------------|"
    - "L0.3229[907,1036] 10ns   |----------------------------------------L0.3229-----------------------------------------|"
    - "L0.3236[907,1036] 11ns   |----------------------------------------L0.3236-----------------------------------------|"
    - "L0.3243[907,1036] 12ns   |----------------------------------------L0.3243-----------------------------------------|"
    - "L0.3250[907,1036] 13ns   |----------------------------------------L0.3250-----------------------------------------|"
    - "L0.3257[907,1036] 14ns   |----------------------------------------L0.3257-----------------------------------------|"
    - "L0.3264[907,1036] 15ns   |----------------------------------------L0.3264-----------------------------------------|"
    - "L0.3271[907,1036] 16ns   |----------------------------------------L0.3271-----------------------------------------|"
    - "L0.3278[907,1036] 17ns   |----------------------------------------L0.3278-----------------------------------------|"
    - "L0.3285[907,1036] 18ns   |----------------------------------------L0.3285-----------------------------------------|"
    - "L0.3292[907,1036] 19ns   |----------------------------------------L0.3292-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 130mb total:"
    - "L0, all files 130mb                                                                                                "
    - "L0.?[907,1036] 19ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3159, L0.3166, L0.3173, L0.3180, L0.3187, L0.3194, L0.3201, L0.3208, L0.3215, L0.3222, L0.3229, L0.3236, L0.3243, L0.3250, L0.3257, L0.3264, L0.3271, L0.3278, L0.3285, L0.3292"
    - "  Creating 1 files"
    - "**** Simulation run 490, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3299[907,1036] 191ns  |----------------------------------------L0.3299-----------------------------------------|"
    - "L0.3306[907,1036] 192ns  |----------------------------------------L0.3306-----------------------------------------|"
    - "L0.3313[907,1036] 193ns  |----------------------------------------L0.3313-----------------------------------------|"
    - "L0.3320[907,1036] 194ns  |----------------------------------------L0.3320-----------------------------------------|"
    - "L0.3327[907,1036] 195ns  |----------------------------------------L0.3327-----------------------------------------|"
    - "L0.3334[907,1036] 196ns  |----------------------------------------L0.3334-----------------------------------------|"
    - "L0.3341[907,1036] 197ns  |----------------------------------------L0.3341-----------------------------------------|"
    - "L0.3348[907,1036] 198ns  |----------------------------------------L0.3348-----------------------------------------|"
    - "L0.3355[907,1036] 199ns  |----------------------------------------L0.3355-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,1036] 199ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3299, L0.3306, L0.3313, L0.3320, L0.3327, L0.3334, L0.3341, L0.3348, L0.3355"
    - "  Creating 1 files"
    - "**** Simulation run 491, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.3160[1037,1165] 0ns   |----------------------------------------L0.3160-----------------------------------------|"
    - "L0.3167[1037,1165] 1ns   |----------------------------------------L0.3167-----------------------------------------|"
    - "L0.3174[1037,1165] 2ns   |----------------------------------------L0.3174-----------------------------------------|"
    - "L0.3181[1037,1165] 3ns   |----------------------------------------L0.3181-----------------------------------------|"
    - "L0.3188[1037,1165] 4ns   |----------------------------------------L0.3188-----------------------------------------|"
    - "L0.3195[1037,1165] 5ns   |----------------------------------------L0.3195-----------------------------------------|"
    - "L0.3202[1037,1165] 6ns   |----------------------------------------L0.3202-----------------------------------------|"
    - "L0.3209[1037,1165] 7ns   |----------------------------------------L0.3209-----------------------------------------|"
    - "L0.3216[1037,1165] 8ns   |----------------------------------------L0.3216-----------------------------------------|"
    - "L0.3223[1037,1165] 9ns   |----------------------------------------L0.3223-----------------------------------------|"
    - "L0.3230[1037,1165] 10ns  |----------------------------------------L0.3230-----------------------------------------|"
    - "L0.3237[1037,1165] 11ns  |----------------------------------------L0.3237-----------------------------------------|"
    - "L0.3244[1037,1165] 12ns  |----------------------------------------L0.3244-----------------------------------------|"
    - "L0.3251[1037,1165] 13ns  |----------------------------------------L0.3251-----------------------------------------|"
    - "L0.3258[1037,1165] 14ns  |----------------------------------------L0.3258-----------------------------------------|"
    - "L0.3265[1037,1165] 15ns  |----------------------------------------L0.3265-----------------------------------------|"
    - "L0.3272[1037,1165] 16ns  |----------------------------------------L0.3272-----------------------------------------|"
    - "L0.3279[1037,1165] 17ns  |----------------------------------------L0.3279-----------------------------------------|"
    - "L0.3286[1037,1165] 18ns  |----------------------------------------L0.3286-----------------------------------------|"
    - "L0.3293[1037,1165] 19ns  |----------------------------------------L0.3293-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[1037,1165] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3160, L0.3167, L0.3174, L0.3181, L0.3188, L0.3195, L0.3202, L0.3209, L0.3216, L0.3223, L0.3230, L0.3237, L0.3244, L0.3251, L0.3258, L0.3265, L0.3272, L0.3279, L0.3286, L0.3293"
    - "  Creating 1 files"
    - "**** Simulation run 492, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3300[1037,1165] 191ns |----------------------------------------L0.3300-----------------------------------------|"
    - "L0.3307[1037,1165] 192ns |----------------------------------------L0.3307-----------------------------------------|"
    - "L0.3314[1037,1165] 193ns |----------------------------------------L0.3314-----------------------------------------|"
    - "L0.3321[1037,1165] 194ns |----------------------------------------L0.3321-----------------------------------------|"
    - "L0.3328[1037,1165] 195ns |----------------------------------------L0.3328-----------------------------------------|"
    - "L0.3335[1037,1165] 196ns |----------------------------------------L0.3335-----------------------------------------|"
    - "L0.3342[1037,1165] 197ns |----------------------------------------L0.3342-----------------------------------------|"
    - "L0.3349[1037,1165] 198ns |----------------------------------------L0.3349-----------------------------------------|"
    - "L0.3356[1037,1165] 199ns |----------------------------------------L0.3356-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1037,1165] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3300, L0.3307, L0.3314, L0.3321, L0.3328, L0.3335, L0.3342, L0.3349, L0.3356"
    - "  Creating 1 files"
    - "**** Simulation run 493, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.3161[1166,1294] 0ns   |----------------------------------------L0.3161-----------------------------------------|"
    - "L0.3168[1166,1294] 1ns   |----------------------------------------L0.3168-----------------------------------------|"
    - "L0.3175[1166,1294] 2ns   |----------------------------------------L0.3175-----------------------------------------|"
    - "L0.3182[1166,1294] 3ns   |----------------------------------------L0.3182-----------------------------------------|"
    - "L0.3189[1166,1294] 4ns   |----------------------------------------L0.3189-----------------------------------------|"
    - "L0.3196[1166,1294] 5ns   |----------------------------------------L0.3196-----------------------------------------|"
    - "L0.3203[1166,1294] 6ns   |----------------------------------------L0.3203-----------------------------------------|"
    - "L0.3210[1166,1294] 7ns   |----------------------------------------L0.3210-----------------------------------------|"
    - "L0.3217[1166,1294] 8ns   |----------------------------------------L0.3217-----------------------------------------|"
    - "L0.3224[1166,1294] 9ns   |----------------------------------------L0.3224-----------------------------------------|"
    - "L0.3231[1166,1294] 10ns  |----------------------------------------L0.3231-----------------------------------------|"
    - "L0.3238[1166,1294] 11ns  |----------------------------------------L0.3238-----------------------------------------|"
    - "L0.3245[1166,1294] 12ns  |----------------------------------------L0.3245-----------------------------------------|"
    - "L0.3252[1166,1294] 13ns  |----------------------------------------L0.3252-----------------------------------------|"
    - "L0.3259[1166,1294] 14ns  |----------------------------------------L0.3259-----------------------------------------|"
    - "L0.3266[1166,1294] 15ns  |----------------------------------------L0.3266-----------------------------------------|"
    - "L0.3273[1166,1294] 16ns  |----------------------------------------L0.3273-----------------------------------------|"
    - "L0.3280[1166,1294] 17ns  |----------------------------------------L0.3280-----------------------------------------|"
    - "L0.3287[1166,1294] 18ns  |----------------------------------------L0.3287-----------------------------------------|"
    - "L0.3294[1166,1294] 19ns  |----------------------------------------L0.3294-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[1166,1294] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3161, L0.3168, L0.3175, L0.3182, L0.3189, L0.3196, L0.3203, L0.3210, L0.3217, L0.3224, L0.3231, L0.3238, L0.3245, L0.3252, L0.3259, L0.3266, L0.3273, L0.3280, L0.3287, L0.3294"
    - "  Creating 1 files"
    - "**** Simulation run 494, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3301[1166,1294] 191ns |----------------------------------------L0.3301-----------------------------------------|"
    - "L0.3308[1166,1294] 192ns |----------------------------------------L0.3308-----------------------------------------|"
    - "L0.3315[1166,1294] 193ns |----------------------------------------L0.3315-----------------------------------------|"
    - "L0.3322[1166,1294] 194ns |----------------------------------------L0.3322-----------------------------------------|"
    - "L0.3329[1166,1294] 195ns |----------------------------------------L0.3329-----------------------------------------|"
    - "L0.3336[1166,1294] 196ns |----------------------------------------L0.3336-----------------------------------------|"
    - "L0.3343[1166,1294] 197ns |----------------------------------------L0.3343-----------------------------------------|"
    - "L0.3350[1166,1294] 198ns |----------------------------------------L0.3350-----------------------------------------|"
    - "L0.3357[1166,1294] 199ns |----------------------------------------L0.3357-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1166,1294] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3301, L0.3308, L0.3315, L0.3322, L0.3329, L0.3336, L0.3343, L0.3350, L0.3357"
    - "  Creating 1 files"
    - "**** Simulation run 495, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.3162[1295,1423] 0ns   |----------------------------------------L0.3162-----------------------------------------|"
    - "L0.3169[1295,1423] 1ns   |----------------------------------------L0.3169-----------------------------------------|"
    - "L0.3176[1295,1423] 2ns   |----------------------------------------L0.3176-----------------------------------------|"
    - "L0.3183[1295,1423] 3ns   |----------------------------------------L0.3183-----------------------------------------|"
    - "L0.3190[1295,1423] 4ns   |----------------------------------------L0.3190-----------------------------------------|"
    - "L0.3197[1295,1423] 5ns   |----------------------------------------L0.3197-----------------------------------------|"
    - "L0.3204[1295,1423] 6ns   |----------------------------------------L0.3204-----------------------------------------|"
    - "L0.3211[1295,1423] 7ns   |----------------------------------------L0.3211-----------------------------------------|"
    - "L0.3218[1295,1423] 8ns   |----------------------------------------L0.3218-----------------------------------------|"
    - "L0.3225[1295,1423] 9ns   |----------------------------------------L0.3225-----------------------------------------|"
    - "L0.3232[1295,1423] 10ns  |----------------------------------------L0.3232-----------------------------------------|"
    - "L0.3239[1295,1423] 11ns  |----------------------------------------L0.3239-----------------------------------------|"
    - "L0.3246[1295,1423] 12ns  |----------------------------------------L0.3246-----------------------------------------|"
    - "L0.3253[1295,1423] 13ns  |----------------------------------------L0.3253-----------------------------------------|"
    - "L0.3260[1295,1423] 14ns  |----------------------------------------L0.3260-----------------------------------------|"
    - "L0.3267[1295,1423] 15ns  |----------------------------------------L0.3267-----------------------------------------|"
    - "L0.3274[1295,1423] 16ns  |----------------------------------------L0.3274-----------------------------------------|"
    - "L0.3281[1295,1423] 17ns  |----------------------------------------L0.3281-----------------------------------------|"
    - "L0.3288[1295,1423] 18ns  |----------------------------------------L0.3288-----------------------------------------|"
    - "L0.3295[1295,1423] 19ns  |----------------------------------------L0.3295-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[1295,1423] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3162, L0.3169, L0.3176, L0.3183, L0.3190, L0.3197, L0.3204, L0.3211, L0.3218, L0.3225, L0.3232, L0.3239, L0.3246, L0.3253, L0.3260, L0.3267, L0.3274, L0.3281, L0.3288, L0.3295"
    - "  Creating 1 files"
    - "**** Simulation run 496, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3302[1295,1423] 191ns |----------------------------------------L0.3302-----------------------------------------|"
    - "L0.3309[1295,1423] 192ns |----------------------------------------L0.3309-----------------------------------------|"
    - "L0.3316[1295,1423] 193ns |----------------------------------------L0.3316-----------------------------------------|"
    - "L0.3323[1295,1423] 194ns |----------------------------------------L0.3323-----------------------------------------|"
    - "L0.3330[1295,1423] 195ns |----------------------------------------L0.3330-----------------------------------------|"
    - "L0.3337[1295,1423] 196ns |----------------------------------------L0.3337-----------------------------------------|"
    - "L0.3344[1295,1423] 197ns |----------------------------------------L0.3344-----------------------------------------|"
    - "L0.3351[1295,1423] 198ns |----------------------------------------L0.3351-----------------------------------------|"
    - "L0.3358[1295,1423] 199ns |----------------------------------------L0.3358-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1295,1423] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3302, L0.3309, L0.3316, L0.3323, L0.3330, L0.3337, L0.3344, L0.3351, L0.3358"
    - "  Creating 1 files"
    - "**** Simulation run 497, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3110[776,906] 191ns   |----------------------------------------L0.3110-----------------------------------------|"
    - "L0.3116[776,906] 192ns   |----------------------------------------L0.3116-----------------------------------------|"
    - "L0.3122[776,906] 193ns   |----------------------------------------L0.3122-----------------------------------------|"
    - "L0.3128[776,906] 194ns   |----------------------------------------L0.3128-----------------------------------------|"
    - "L0.3134[776,906] 195ns   |----------------------------------------L0.3134-----------------------------------------|"
    - "L0.3140[776,906] 196ns   |----------------------------------------L0.3140-----------------------------------------|"
    - "L0.3146[776,906] 197ns   |----------------------------------------L0.3146-----------------------------------------|"
    - "L0.3152[776,906] 198ns   |----------------------------------------L0.3152-----------------------------------------|"
    - "L0.3158[776,906] 199ns   |----------------------------------------L0.3158-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[776,906] 199ns      |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3110, L0.3116, L0.3122, L0.3128, L0.3134, L0.3140, L0.3146, L0.3152, L0.3158"
    - "  Creating 1 files"
    - "**** Simulation run 498, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.3163[1424,1552] 0ns   |----------------------------------------L0.3163-----------------------------------------|"
    - "L0.3170[1424,1552] 1ns   |----------------------------------------L0.3170-----------------------------------------|"
    - "L0.3177[1424,1552] 2ns   |----------------------------------------L0.3177-----------------------------------------|"
    - "L0.3184[1424,1552] 3ns   |----------------------------------------L0.3184-----------------------------------------|"
    - "L0.3191[1424,1552] 4ns   |----------------------------------------L0.3191-----------------------------------------|"
    - "L0.3198[1424,1552] 5ns   |----------------------------------------L0.3198-----------------------------------------|"
    - "L0.3205[1424,1552] 6ns   |----------------------------------------L0.3205-----------------------------------------|"
    - "L0.3212[1424,1552] 7ns   |----------------------------------------L0.3212-----------------------------------------|"
    - "L0.3219[1424,1552] 8ns   |----------------------------------------L0.3219-----------------------------------------|"
    - "L0.3226[1424,1552] 9ns   |----------------------------------------L0.3226-----------------------------------------|"
    - "L0.3233[1424,1552] 10ns  |----------------------------------------L0.3233-----------------------------------------|"
    - "L0.3240[1424,1552] 11ns  |----------------------------------------L0.3240-----------------------------------------|"
    - "L0.3247[1424,1552] 12ns  |----------------------------------------L0.3247-----------------------------------------|"
    - "L0.3254[1424,1552] 13ns  |----------------------------------------L0.3254-----------------------------------------|"
    - "L0.3261[1424,1552] 14ns  |----------------------------------------L0.3261-----------------------------------------|"
    - "L0.3268[1424,1552] 15ns  |----------------------------------------L0.3268-----------------------------------------|"
    - "L0.3275[1424,1552] 16ns  |----------------------------------------L0.3275-----------------------------------------|"
    - "L0.3282[1424,1552] 17ns  |----------------------------------------L0.3282-----------------------------------------|"
    - "L0.3289[1424,1552] 18ns  |----------------------------------------L0.3289-----------------------------------------|"
    - "L0.3296[1424,1552] 19ns  |----------------------------------------L0.3296-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[1424,1552] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3163, L0.3170, L0.3177, L0.3184, L0.3191, L0.3198, L0.3205, L0.3212, L0.3219, L0.3226, L0.3233, L0.3240, L0.3247, L0.3254, L0.3261, L0.3268, L0.3275, L0.3282, L0.3289, L0.3296"
    - "  Creating 1 files"
    - "**** Simulation run 499, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3303[1424,1552] 191ns |----------------------------------------L0.3303-----------------------------------------|"
    - "L0.3310[1424,1552] 192ns |----------------------------------------L0.3310-----------------------------------------|"
    - "L0.3317[1424,1552] 193ns |----------------------------------------L0.3317-----------------------------------------|"
    - "L0.3324[1424,1552] 194ns |----------------------------------------L0.3324-----------------------------------------|"
    - "L0.3331[1424,1552] 195ns |----------------------------------------L0.3331-----------------------------------------|"
    - "L0.3338[1424,1552] 196ns |----------------------------------------L0.3338-----------------------------------------|"
    - "L0.3345[1424,1552] 197ns |----------------------------------------L0.3345-----------------------------------------|"
    - "L0.3352[1424,1552] 198ns |----------------------------------------L0.3352-----------------------------------------|"
    - "L0.3359[1424,1552] 199ns |----------------------------------------L0.3359-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1424,1552] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3303, L0.3310, L0.3317, L0.3324, L0.3331, L0.3338, L0.3345, L0.3352, L0.3359"
    - "  Creating 1 files"
    - "**** Simulation run 500, type=compact(ManySmallFiles). 20 Input Files, 129mb total:"
    - "L0, all files 6mb                                                                                                  "
    - "L0.3164[1553,1681] 0ns   |----------------------------------------L0.3164-----------------------------------------|"
    - "L0.3171[1553,1681] 1ns   |----------------------------------------L0.3171-----------------------------------------|"
    - "L0.3178[1553,1681] 2ns   |----------------------------------------L0.3178-----------------------------------------|"
    - "L0.3185[1553,1681] 3ns   |----------------------------------------L0.3185-----------------------------------------|"
    - "L0.3192[1553,1681] 4ns   |----------------------------------------L0.3192-----------------------------------------|"
    - "L0.3199[1553,1681] 5ns   |----------------------------------------L0.3199-----------------------------------------|"
    - "L0.3206[1553,1681] 6ns   |----------------------------------------L0.3206-----------------------------------------|"
    - "L0.3213[1553,1681] 7ns   |----------------------------------------L0.3213-----------------------------------------|"
    - "L0.3220[1553,1681] 8ns   |----------------------------------------L0.3220-----------------------------------------|"
    - "L0.3227[1553,1681] 9ns   |----------------------------------------L0.3227-----------------------------------------|"
    - "L0.3234[1553,1681] 10ns  |----------------------------------------L0.3234-----------------------------------------|"
    - "L0.3241[1553,1681] 11ns  |----------------------------------------L0.3241-----------------------------------------|"
    - "L0.3248[1553,1681] 12ns  |----------------------------------------L0.3248-----------------------------------------|"
    - "L0.3255[1553,1681] 13ns  |----------------------------------------L0.3255-----------------------------------------|"
    - "L0.3262[1553,1681] 14ns  |----------------------------------------L0.3262-----------------------------------------|"
    - "L0.3269[1553,1681] 15ns  |----------------------------------------L0.3269-----------------------------------------|"
    - "L0.3276[1553,1681] 16ns  |----------------------------------------L0.3276-----------------------------------------|"
    - "L0.3283[1553,1681] 17ns  |----------------------------------------L0.3283-----------------------------------------|"
    - "L0.3290[1553,1681] 18ns  |----------------------------------------L0.3290-----------------------------------------|"
    - "L0.3297[1553,1681] 19ns  |----------------------------------------L0.3297-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 129mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.?[1553,1681] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3164, L0.3171, L0.3178, L0.3185, L0.3192, L0.3199, L0.3206, L0.3213, L0.3220, L0.3227, L0.3234, L0.3241, L0.3248, L0.3255, L0.3262, L0.3269, L0.3276, L0.3283, L0.3290, L0.3297"
    - "  Creating 1 files"
    - "**** Simulation run 501, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3304[1553,1681] 191ns |----------------------------------------L0.3304-----------------------------------------|"
    - "L0.3311[1553,1681] 192ns |----------------------------------------L0.3311-----------------------------------------|"
    - "L0.3318[1553,1681] 193ns |----------------------------------------L0.3318-----------------------------------------|"
    - "L0.3325[1553,1681] 194ns |----------------------------------------L0.3325-----------------------------------------|"
    - "L0.3332[1553,1681] 195ns |----------------------------------------L0.3332-----------------------------------------|"
    - "L0.3339[1553,1681] 196ns |----------------------------------------L0.3339-----------------------------------------|"
    - "L0.3346[1553,1681] 197ns |----------------------------------------L0.3346-----------------------------------------|"
    - "L0.3353[1553,1681] 198ns |----------------------------------------L0.3353-----------------------------------------|"
    - "L0.3360[1553,1681] 199ns |----------------------------------------L0.3360-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1553,1681] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3304, L0.3311, L0.3318, L0.3325, L0.3332, L0.3339, L0.3346, L0.3353, L0.3360"
    - "  Creating 1 files"
    - "**** Simulation run 502, type=compact(ManySmallFiles). 20 Input Files, 130mb total:"
    - "L0, all files 7mb                                                                                                  "
    - "L0.3165[1682,1811] 0ns   |----------------------------------------L0.3165-----------------------------------------|"
    - "L0.3172[1682,1811] 1ns   |----------------------------------------L0.3172-----------------------------------------|"
    - "L0.3179[1682,1811] 2ns   |----------------------------------------L0.3179-----------------------------------------|"
    - "L0.3186[1682,1811] 3ns   |----------------------------------------L0.3186-----------------------------------------|"
    - "L0.3193[1682,1811] 4ns   |----------------------------------------L0.3193-----------------------------------------|"
    - "L0.3200[1682,1811] 5ns   |----------------------------------------L0.3200-----------------------------------------|"
    - "L0.3207[1682,1811] 6ns   |----------------------------------------L0.3207-----------------------------------------|"
    - "L0.3214[1682,1811] 7ns   |----------------------------------------L0.3214-----------------------------------------|"
    - "L0.3221[1682,1811] 8ns   |----------------------------------------L0.3221-----------------------------------------|"
    - "L0.3228[1682,1811] 9ns   |----------------------------------------L0.3228-----------------------------------------|"
    - "L0.3235[1682,1811] 10ns  |----------------------------------------L0.3235-----------------------------------------|"
    - "L0.3242[1682,1811] 11ns  |----------------------------------------L0.3242-----------------------------------------|"
    - "L0.3249[1682,1811] 12ns  |----------------------------------------L0.3249-----------------------------------------|"
    - "L0.3256[1682,1811] 13ns  |----------------------------------------L0.3256-----------------------------------------|"
    - "L0.3263[1682,1811] 14ns  |----------------------------------------L0.3263-----------------------------------------|"
    - "L0.3270[1682,1811] 15ns  |----------------------------------------L0.3270-----------------------------------------|"
    - "L0.3277[1682,1811] 16ns  |----------------------------------------L0.3277-----------------------------------------|"
    - "L0.3284[1682,1811] 17ns  |----------------------------------------L0.3284-----------------------------------------|"
    - "L0.3291[1682,1811] 18ns  |----------------------------------------L0.3291-----------------------------------------|"
    - "L0.3298[1682,1811] 19ns  |----------------------------------------L0.3298-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 130mb total:"
    - "L0, all files 130mb                                                                                                "
    - "L0.?[1682,1811] 19ns     |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3165, L0.3172, L0.3179, L0.3186, L0.3193, L0.3200, L0.3207, L0.3214, L0.3221, L0.3228, L0.3235, L0.3242, L0.3249, L0.3256, L0.3263, L0.3270, L0.3277, L0.3284, L0.3291, L0.3298"
    - "  Creating 1 files"
    - "**** Simulation run 503, type=compact(ManySmallFiles). 9 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3305[1682,1811] 191ns |----------------------------------------L0.3305-----------------------------------------|"
    - "L0.3312[1682,1811] 192ns |----------------------------------------L0.3312-----------------------------------------|"
    - "L0.3319[1682,1811] 193ns |----------------------------------------L0.3319-----------------------------------------|"
    - "L0.3326[1682,1811] 194ns |----------------------------------------L0.3326-----------------------------------------|"
    - "L0.3333[1682,1811] 195ns |----------------------------------------L0.3333-----------------------------------------|"
    - "L0.3340[1682,1811] 196ns |----------------------------------------L0.3340-----------------------------------------|"
    - "L0.3347[1682,1811] 197ns |----------------------------------------L0.3347-----------------------------------------|"
    - "L0.3354[1682,1811] 198ns |----------------------------------------L0.3354-----------------------------------------|"
    - "L0.3361[1682,1811] 199ns |----------------------------------------L0.3361-----------------------------------------|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1682,1811] 199ns    |------------------------------------------L0.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 9 files: L0.3305, L0.3312, L0.3319, L0.3326, L0.3333, L0.3340, L0.3347, L0.3354, L0.3361"
    - "  Creating 1 files"
    - "**** Simulation run 504, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[101, 201]). 2 Input Files, 259mb total:"
    - "L0                                                                                                                 "
    - "L0.3362[1,130] 19ns 130mb|------------------L0.3362------------------|                                             "
    - "L0.3363[131,259] 19ns 129mb                                             |-----------------L0.3363------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 259mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1,101] 19ns 101mb   |--------------L1.?--------------|                                                        "
    - "L1.?[102,201] 19ns 100mb                                    |--------------L1.?--------------|                     "
    - "L1.?[202,259] 19ns 58mb                                                                        |------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3362, L0.3363"
    - "  Creating 3 files"
    - "**** Simulation run 505, type=split(ReduceOverlap)(split_times=[101]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3104[20,130] 191ns    |----------------------------------------L0.3104----------------------------------------| "
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[20,101] 191ns       |------------------------------L0.?------------------------------|                        "
    - "L0.?[102,130] 191ns                                                                         |--------L0.?--------| "
    - "**** Simulation run 506, type=split(ReduceOverlap)(split_times=[201]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3364[131,259] 199ns   |----------------------------------------L0.3364-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[131,201] 199ns      |---------------------L0.?----------------------|                                         "
    - "L0.?[202,259] 199ns                                                       |-----------------L0.?-----------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3104, L0.3364"
    - "  Creating 4 files"
    - "**** Simulation run 507, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[360, 460]). 2 Input Files, 258mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.3365[260,388] 19ns    |-----------------L0.3365------------------|                                              "
    - "L0.3367[389,517] 19ns                                                 |-----------------L0.3367------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 258mb total:"
    - "L1                                                                                                                 "
    - "L1.?[260,360] 19ns 101mb |--------------L1.?---------------|                                                       "
    - "L1.?[361,460] 19ns 100mb                                    |--------------L1.?--------------|                     "
    - "L1.?[461,517] 19ns 57mb                                                                        |------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3365, L0.3367"
    - "  Creating 3 files"
    - "**** Simulation run 508, type=split(ReduceOverlap)(split_times=[360]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3366[260,388] 199ns   |----------------------------------------L0.3366-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[260,360] 199ns      |--------------------------------L0.?--------------------------------|                    "
    - "L0.?[361,388] 199ns                                                                             |------L0.?------| "
    - "**** Simulation run 509, type=split(ReduceOverlap)(split_times=[460]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3368[389,517] 199ns   |----------------------------------------L0.3368-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[389,460] 199ns      |---------------------L0.?----------------------|                                         "
    - "L0.?[461,517] 199ns                                                        |----------------L0.?-----------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3366, L0.3368"
    - "  Creating 4 files"
    - "**** Simulation run 510, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[618, 718]). 2 Input Files, 258mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.3369[518,646] 19ns    |-----------------L0.3369------------------|                                              "
    - "L0.3371[647,775] 19ns                                                 |-----------------L0.3371------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 258mb total:"
    - "L1                                                                                                                 "
    - "L1.?[518,618] 19ns 101mb |--------------L1.?---------------|                                                       "
    - "L1.?[619,718] 19ns 100mb                                    |--------------L1.?--------------|                     "
    - "L1.?[719,775] 19ns 57mb                                                                        |------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3369, L0.3371"
    - "  Creating 3 files"
    - "**** Simulation run 511, type=split(ReduceOverlap)(split_times=[618]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3370[518,646] 199ns   |----------------------------------------L0.3370-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[518,618] 199ns      |--------------------------------L0.?--------------------------------|                    "
    - "L0.?[619,646] 199ns                                                                             |------L0.?------| "
    - "**** Simulation run 512, type=split(ReduceOverlap)(split_times=[718]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3372[647,775] 199ns   |----------------------------------------L0.3372-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[647,718] 199ns      |---------------------L0.?----------------------|                                         "
    - "L0.?[719,775] 199ns                                                        |----------------L0.?-----------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3370, L0.3372"
    - "  Creating 4 files"
    - "**** Simulation run 513, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[876, 976]). 2 Input Files, 261mb total:"
    - "L0                                                                                                                 "
    - "L0.3381[776,906] 19ns 131mb|------------------L0.3381------------------|                                             "
    - "L0.3373[907,1036] 19ns 130mb                                             |-----------------L0.3373------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 261mb total:"
    - "L1                                                                                                                 "
    - "L1.?[776,876] 19ns 101mb |--------------L1.?--------------|                                                        "
    - "L1.?[877,976] 19ns 100mb                                   |--------------L1.?--------------|                      "
    - "L1.?[977,1036] 19ns 60mb                                                                      |-------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3373, L0.3381"
    - "  Creating 3 files"
    - "**** Simulation run 514, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[202, 302]). 3 Input Files, 259mb total:"
    - "L1                                                                                                                 "
    - "L1.3390[102,201] 19ns 100mb|------------L1.3390-------------|                                                        "
    - "L1.3391[202,259] 19ns 58mb                                  |-----L1.3391-----|                                     "
    - "L1.3396[260,360] 19ns 101mb                                                       |------------L1.3396-------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 259mb total:"
    - "L2                                                                                                                 "
    - "L2.?[102,202] 19ns 101mb |--------------L2.?--------------|                                                        "
    - "L2.?[203,302] 19ns 100mb                                    |--------------L2.?--------------|                     "
    - "L2.?[303,360] 19ns 58mb                                                                        |------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.3390, L1.3391, L1.3396"
    - "  Upgrading 1 files level to CompactionLevel::L2: L1.3389"
    - "  Creating 3 files"
    - "**** Simulation run 515, type=split(ReduceOverlap)(split_times=[876]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3382[776,906] 199ns   |----------------------------------------L0.3382-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[776,876] 199ns      |-------------------------------L0.?--------------------------------|                     "
    - "L0.?[877,906] 199ns                                                                           |-------L0.?-------| "
    - "**** Simulation run 516, type=split(ReduceOverlap)(split_times=[976]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3374[907,1036] 199ns  |----------------------------------------L0.3374-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,976] 199ns      |---------------------L0.?---------------------|                                          "
    - "L0.?[977,1036] 199ns                                                     |-----------------L0.?------------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3374, L0.3382"
    - "  Creating 4 files"
    - "**** Simulation run 517, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1137, 1237]). 2 Input Files, 258mb total:"
    - "L0, all files 129mb                                                                                                "
    - "L0.3375[1037,1165] 19ns  |-----------------L0.3375------------------|                                              "
    - "L0.3377[1166,1294] 19ns                                               |-----------------L0.3377------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 258mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1037,1137] 19ns 101mb|--------------L1.?---------------|                                                       "
    - "L1.?[1138,1237] 19ns 100mb                                   |--------------L1.?--------------|                     "
    - "L1.?[1238,1294] 19ns 57mb                                                                      |------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L0.3375, L0.3377"
    - "  Creating 3 files"
    - "**** Simulation run 518, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[461, 561]). 3 Input Files, 258mb total:"
    - "L1                                                                                                                 "
    - "L1.3397[361,460] 19ns 100mb|------------L1.3397-------------|                                                        "
    - "L1.3398[461,517] 19ns 57mb                                   |-----L1.3398-----|                                    "
    - "L1.3403[518,618] 19ns 101mb                                                      |-------------L1.3403-------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 258mb total:"
    - "L2                                                                                                                 "
    - "L2.?[361,461] 19ns 101mb |--------------L2.?---------------|                                                       "
    - "L2.?[462,561] 19ns 100mb                                    |--------------L2.?--------------|                     "
    - "L2.?[562,618] 19ns 57mb                                                                        |------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.3397, L1.3398, L1.3403"
    - "  Creating 3 files"
    - "**** Simulation run 519, type=split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))(split_times=[171, 322, 473, 624, 775, 926, 1077, 1228, 1379, 1530, 1681]). 20 Input Files, 1.17gb total:"
    - "L0                                                                                                                 "
    - "L0.3379[1295,1423] 19ns 129mb                                                                |L0.3379|                 "
    - "L0.3383[1424,1552] 19ns 129mb                                                                      |L0.3383|           "
    - "L0.3385[1553,1681] 19ns 129mb                                                                             |L0.3385|    "
    - "L0.3387[1682,1811] 19ns 130mb                                                                                   |L0.3387|"
    - "L0.3392[20,101] 191ns 0b |L0.3392|                                                                                 "
    - "L0.3393[102,130] 191ns 0b    |L0.3393|                                                                             "
    - "L0.3394[131,201] 199ns 0b     |L0.3394|                                                                            "
    - "L0.3395[202,259] 199ns 0b         |L0.3395|                                                                        "
    - "L0.3399[260,360] 199ns 0b            |L0.3399|                                                                     "
    - "L0.3400[361,388] 199ns 0b                 |L0.3400|                                                                "
    - "L0.3401[389,460] 199ns 0b                  |L0.3401|                                                               "
    - "L0.3402[461,517] 199ns 0b                      |L0.3402|                                                           "
    - "L1                                                                                                                 "
    - "L1.3404[619,718] 19ns 100mb                              |L1.3404|                                                   "
    - "L1.3405[719,775] 19ns 57mb                                   |L1.3405|                                              "
    - "L1.3410[776,876] 19ns 101mb                                     |L1.3410|                                            "
    - "L1.3411[877,976] 19ns 100mb                                           |L1.3411|                                      "
    - "L1.3412[977,1036] 19ns 60mb                                                |L1.3412|                                 "
    - "L1.3420[1037,1137] 19ns 101mb                                                   |L1.3420|                              "
    - "L1.3421[1138,1237] 19ns 100mb                                                        |L1.3421|                         "
    - "L1.3422[1238,1294] 19ns 57mb                                                             |L1.3422|                    "
    - "**** 12 Output Files (parquet_file_id not yet assigned), 1.17gb total:"
    - "L1                                                                                                                 "
    - "L1.?[20,171] 199ns 101mb |L1.?-|                                                                                   "
    - "L1.?[172,322] 199ns 101mb       |L1.?-|                                                                            "
    - "L1.?[323,473] 199ns 101mb               |L1.?-|                                                                    "
    - "L1.?[474,624] 199ns 101mb                      |L1.?-|                                                             "
    - "L1.?[625,775] 199ns 101mb                              |L1.?-|                                                     "
    - "L1.?[776,926] 199ns 101mb                                     |L1.?-|                                              "
    - "L1.?[927,1077] 199ns 101mb                                             |L1.?-|                                      "
    - "L1.?[1078,1228] 199ns 101mb                                                     |L1.?-|                              "
    - "L1.?[1229,1379] 199ns 101mb                                                            |L1.?-|                       "
    - "L1.?[1380,1530] 199ns 101mb                                                                    |L1.?-|               "
    - "L1.?[1531,1681] 199ns 101mb                                                                           |L1.?-|        "
    - "L1.?[1682,1811] 199ns 87mb                                                                                   |L1.?| "
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.3379, L0.3383, L0.3385, L0.3387, L0.3392, L0.3393, L0.3394, L0.3395, L0.3399, L0.3400, L0.3401, L0.3402, L1.3404, L1.3405, L1.3410, L1.3411, L1.3412, L1.3420, L1.3421, L1.3422"
    - "  Creating 12 files"
    - "**** Simulation run 520, type=split(ReduceOverlap)(split_times=[561, 618]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3429[474,624] 199ns   |----------------------------------------L1.3429-----------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[474,561] 199ns 59mb |-----------------------L1.?-----------------------|                                      "
    - "L1.?[562,618] 199ns 38mb                                                     |-------------L1.?--------------|     "
    - "L1.?[619,624] 199ns 4mb                                                                                         |L1.?|"
    - "**** Simulation run 521, type=split(ReduceOverlap)(split_times=[360, 461]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3428[323,473] 199ns   |----------------------------------------L1.3428-----------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[323,360] 199ns 25mb |--------L1.?--------|                                                                    "
    - "L1.?[361,461] 199ns 67mb                       |---------------------------L1.?---------------------------|        "
    - "L1.?[462,473] 199ns 8mb                                                                                     |L1.?| "
    - "**** Simulation run 522, type=split(ReduceOverlap)(split_times=[202, 302]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3427[172,322] 199ns   |----------------------------------------L1.3427-----------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[172,202] 199ns 21mb |------L1.?------|                                                                        "
    - "L1.?[203,302] 199ns 67mb                   |--------------------------L1.?---------------------------|             "
    - "L1.?[303,322] 199ns 13mb                                                                               |--L1.?---| "
    - "**** Simulation run 523, type=split(ReduceOverlap)(split_times=[101]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3426[20,171] 199ns    |----------------------------------------L1.3426-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[20,101] 199ns 55mb  |---------------------L1.?---------------------|                                          "
    - "L1.?[102,171] 199ns 47mb                                                 |-----------------L1.?------------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 4 files: L1.3426, L1.3427, L1.3428, L1.3429"
    - "  Creating 11 files"
    - "**** Simulation run 524, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[66]). 2 Input Files, 156mb total:"
    - "L1                                                                                                                 "
    - "L1.3447[20,101] 199ns 55mb                 |-------------------------------L1.3447--------------------------------| "
    - "L2                                                                                                                 "
    - "L2.3389[1,101] 19ns 101mb|----------------------------------------L2.3389-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 156mb total:"
    - "L2                                                                                                                 "
    - "L2.?[1,66] 199ns 102mb   |--------------------------L2.?--------------------------|                                "
    - "L2.?[67,101] 199ns 54mb                                                             |------------L2.?------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L2.3389, L1.3447"
    - "  Creating 2 files"
    - "**** Simulation run 525, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[162]). 3 Input Files, 168mb total:"
    - "L1                                                                                                                 "
    - "L1.3448[102,171] 199ns 47mb|--------------------------L1.3448---------------------------|                            "
    - "L1.3444[172,202] 199ns 21mb                                                               |---------L1.3444---------|"
    - "L2                                                                                                                 "
    - "L2.3413[102,202] 19ns 101mb|----------------------------------------L2.3413-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 168mb total:"
    - "L2                                                                                                                 "
    - "L2.?[102,162] 199ns 102mb|------------------------L2.?------------------------|                                    "
    - "L2.?[163,202] 199ns 67mb                                                       |--------------L2.?---------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L2.3413, L1.3444, L1.3448"
    - "  Creating 2 files"
    - "**** Simulation run 526, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[263, 323]). 5 Input Files, 263mb total:"
    - "L1                                                                                                                 "
    - "L1.3445[203,302] 199ns 67mb|-----------------------L1.3445------------------------|                                  "
    - "L1.3446[303,322] 199ns 13mb                                                         |L1.3446-|                       "
    - "L1.3441[323,360] 199ns 25mb                                                                    |------L1.3441------| "
    - "L2                                                                                                                 "
    - "L2.3414[203,302] 19ns 100mb|-----------------------L2.3414------------------------|                                  "
    - "L2.3415[303,360] 19ns 58mb                                                         |-----------L2.3415------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 263mb total:"
    - "L2                                                                                                                 "
    - "L2.?[203,263] 199ns 102mb|--------------L2.?--------------|                                                        "
    - "L2.?[264,323] 199ns 100mb                                  |-------------L2.?--------------|                       "
    - "L2.?[324,360] 199ns 62mb                                                                      |-------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 5 files: L2.3414, L2.3415, L1.3441, L1.3445, L1.3446"
    - "  Creating 3 files"
    - "**** Simulation run 527, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[421]). 2 Input Files, 168mb total:"
    - "L1                                                                                                                 "
    - "L1.3442[361,461] 199ns 67mb|----------------------------------------L1.3442-----------------------------------------|"
    - "L2                                                                                                                 "
    - "L2.3423[361,461] 19ns 101mb|----------------------------------------L2.3423-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 168mb total:"
    - "L2                                                                                                                 "
    - "L2.?[361,421] 199ns 102mb|------------------------L2.?------------------------|                                    "
    - "L2.?[422,461] 199ns 67mb                                                       |--------------L2.?---------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 2 files: L2.3423, L1.3442"
    - "  Creating 2 files"
    - "**** Simulation run 528, type=split(HighL0OverlapTotalBacklog)(split_times=[957]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3418[907,976] 199ns   |----------------------------------------L0.3418-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,957] 199ns      |-----------------------------L0.?------------------------------|                         "
    - "L0.?[958,976] 199ns                                                                        |--------L0.?---------| "
    - "**** Simulation run 529, type=split(HighL0OverlapTotalBacklog)(split_times=[957]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3432[927,1077] 199ns  |----------------------------------------L1.3432-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[927,957] 199ns 21mb |------L1.?------|                                                                        "
    - "L1.?[958,1077] 199ns 80mb                  |--------------------------------L1.?---------------------------------| "
    - "**** Simulation run 530, type=split(HighL0OverlapTotalBacklog)(split_times=[1138]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3376[1037,1165] 199ns |----------------------------------------L0.3376-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1037,1138] 199ns    |--------------------------------L0.?---------------------------------|                   "
    - "L0.?[1139,1165] 199ns                                                                           |------L0.?------| "
    - "**** Simulation run 531, type=split(HighL0OverlapTotalBacklog)(split_times=[1138]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3433[1078,1228] 199ns |----------------------------------------L1.3433-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1078,1138] 199ns 41mb|---------------L1.?---------------|                                                      "
    - "L1.?[1139,1228] 199ns 60mb                                    |-----------------------L1.?------------------------| "
    - "**** Simulation run 532, type=split(HighL0OverlapTotalBacklog)(split_times=[1319]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3434[1229,1379] 199ns |----------------------------------------L1.3434-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1229,1319] 199ns 61mb|------------------------L1.?------------------------|                                    "
    - "L1.?[1320,1379] 199ns 40mb                                                      |--------------L1.?---------------| "
    - "**** Simulation run 533, type=split(HighL0OverlapTotalBacklog)(split_times=[1319]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3380[1295,1423] 199ns |----------------------------------------L0.3380-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1295,1319] 199ns    |-----L0.?-----|                                                                          "
    - "L0.?[1320,1423] 199ns                     |---------------------------------L0.?---------------------------------| "
    - "**** Simulation run 534, type=split(HighL0OverlapTotalBacklog)(split_times=[1500]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3435[1380,1530] 199ns |----------------------------------------L1.3435-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1380,1500] 199ns 81mb|---------------------------------L1.?---------------------------------|                  "
    - "L1.?[1501,1530] 199ns 20mb                                                                        |-----L1.?------| "
    - "**** Simulation run 535, type=split(HighL0OverlapTotalBacklog)(split_times=[1500]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3384[1424,1552] 199ns |----------------------------------------L0.3384-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1424,1500] 199ns    |-----------------------L0.?------------------------|                                     "
    - "L0.?[1501,1552] 199ns                                                          |--------------L0.?---------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 8 files: L0.3376, L0.3380, L0.3384, L0.3418, L1.3432, L1.3433, L1.3434, L1.3435"
    - "  Creating 16 files"
    - "**** Simulation run 536, type=split(ReduceOverlap)(split_times=[561]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3406[518,618] 199ns   |----------------------------------------L0.3406-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[518,561] 199ns      |----------------L0.?----------------|                                                    "
    - "L0.?[562,618] 199ns                                             |----------------------L0.?----------------------| "
    - "**** Simulation run 537, type=split(ReduceOverlap)(split_times=[624]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3407[619,646] 199ns   |----------------------------------------L0.3407-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[619,624] 199ns      |-----L0.?-----|                                                                          "
    - "L0.?[625,646] 199ns                          |--------------------------------L0.?--------------------------------|"
    - "**** Simulation run 538, type=split(ReduceOverlap)(split_times=[926]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3458[907,957] 199ns   |----------------------------------------L0.3458-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[907,926] 199ns      |--------------L0.?--------------|                                                        "
    - "L0.?[927,957] 199ns                                          |------------------------L0.?------------------------|"
    - "**** Simulation run 539, type=split(ReduceOverlap)(split_times=[1077]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3462[1037,1138] 199ns |----------------------------------------L0.3462-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1037,1077] 199ns    |--------------L0.?---------------|                                                       "
    - "L0.?[1078,1138] 199ns                                        |-----------------------L0.?------------------------| "
    - "**** Simulation run 540, type=split(ReduceOverlap)(split_times=[1228]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3378[1166,1294] 199ns |----------------------------------------L0.3378-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1166,1228] 199ns    |------------------L0.?-------------------|                                               "
    - "L0.?[1229,1294] 199ns                                                |-------------------L0.?--------------------| "
    - "**** Simulation run 541, type=split(ReduceOverlap)(split_times=[1379]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3469[1320,1423] 199ns |----------------------------------------L0.3469-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1320,1379] 199ns    |----------------------L0.?-----------------------|                                       "
    - "L0.?[1380,1423] 199ns                                                        |---------------L0.?----------------| "
    - "**** Simulation run 542, type=split(ReduceOverlap)(split_times=[1530]). 1 Input Files, 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.3473[1501,1552] 199ns |----------------------------------------L0.3473-----------------------------------------|"
    - "**** 2 Output Files (parquet_file_id not yet assigned), 0b total:"
    - "L0, all files 0b                                                                                                   "
    - "L0.?[1501,1530] 199ns    |----------------------L0.?-----------------------|                                       "
    - "L0.?[1531,1552] 199ns                                                        |---------------L0.?----------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 7 files: L0.3378, L0.3406, L0.3407, L0.3458, L0.3462, L0.3469, L0.3473"
    - "  Creating 14 files"
    - "**** Simulation run 543, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[624, 774]). 10 Input Files, 201mb total:"
    - "L0                                                                                                                 "
    - "L0.3474[518,561] 199ns 0b             |-L0.3474--|                                                                 "
    - "L0.3475[562,618] 199ns 0b                          |---L0.3475----|                                                "
    - "L0.3476[619,624] 199ns 0b                                           |L0.3476|                                      "
    - "L0.3477[625,646] 199ns 0b                                             |L0.3477|                                    "
    - "L0.3408[647,718] 199ns 0b                                                   |------L0.3408------|                  "
    - "L0.3409[719,775] 199ns 0b                                                                         |---L0.3409----| "
    - "L1                                                                                                                 "
    - "L1.3438[474,561] 199ns 59mb|--------L1.3438---------|                                                                "
    - "L1.3439[562,618] 199ns 38mb                          |---L1.3439----|                                                "
    - "L1.3440[619,624] 199ns 4mb                                           |L1.3440|                                      "
    - "L1.3430[625,775] 199ns 101mb                                             |-----------------L1.3430------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 201mb total:"
    - "L1                                                                                                                 "
    - "L1.?[474,624] 199ns 101mb|-------------------L1.?-------------------|                                              "
    - "L1.?[625,774] 199ns 100mb                                             |-------------------L1.?-------------------| "
    - "L1.?[775,775] 199ns 682kb                                                                                          |L1.?|"
    - "Committing partition 1:"
    - "  Soft Deleting 10 files: L0.3408, L0.3409, L1.3430, L1.3438, L1.3439, L1.3440, L0.3474, L0.3475, L0.3476, L0.3477"
    - "  Creating 3 files"
    - "**** Simulation run 544, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[926, 1076]). 12 Input Files, 242mb total:"
    - "L0                                                                                                                 "
    - "L0.3416[776,876] 199ns 0b|-------L0.3416--------|                                                                  "
    - "L0.3417[877,906] 199ns 0b                         |L0.3417|                                                        "
    - "L0.3478[907,926] 199ns 0b                                |L0.3478|                                                 "
    - "L0.3479[927,957] 199ns 0b                                     |L0.3479|                                            "
    - "L0.3459[958,976] 199ns 0b                                             |L0.3459|                                    "
    - "L0.3419[977,1036] 199ns 0b                                                 |--L0.3419---|                           "
    - "L0.3480[1037,1077] 199ns 0b                                                                |L0.3480|                 "
    - "L0.3481[1078,1138] 199ns 0b                                                                           |--L0.3481---| "
    - "L1                                                                                                                 "
    - "L1.3431[776,926] 199ns 101mb|--------------L1.3431--------------|                                                     "
    - "L1.3460[927,957] 199ns 21mb                                     |L1.3460|                                            "
    - "L1.3461[958,1077] 199ns 80mb                                             |----------L1.3461----------|                "
    - "L1.3464[1078,1138] 199ns 41mb                                                                           |--L1.3464---| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 242mb total:"
    - "L1                                                                                                                 "
    - "L1.?[776,926] 199ns 101mb|---------------L1.?----------------|                                                     "
    - "L1.?[927,1076] 199ns 100mb                                     |---------------L1.?----------------|                "
    - "L1.?[1077,1138] 199ns 41mb                                                                          |----L1.?-----| "
    - "Committing partition 1:"
    - "  Soft Deleting 12 files: L0.3416, L0.3417, L0.3419, L1.3431, L0.3459, L1.3460, L1.3461, L1.3464, L0.3478, L0.3479, L0.3480, L0.3481"
    - "  Creating 3 files"
    - "**** Simulation run 545, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1289, 1439]). 13 Input Files, 261mb total:"
    - "L0                                                                                                                 "
    - "L0.3463[1139,1165] 199ns 0b|L0.3463|                                                                                 "
    - "L0.3482[1166,1228] 199ns 0b      |--L0.3482---|                                                                      "
    - "L0.3483[1229,1294] 199ns 0b                    |--L0.3483---|                                                        "
    - "L0.3468[1295,1319] 199ns 0b                                   |L0.3468|                                              "
    - "L0.3484[1320,1379] 199ns 0b                                         |--L0.3484--|                                    "
    - "L0.3485[1380,1423] 199ns 0b                                                       |L0.3485|                          "
    - "L0.3472[1424,1500] 199ns 0b                                                                 |----L0.3472----|        "
    - "L0.3486[1501,1530] 199ns 0b                                                                                   |L0.3486|"
    - "L1                                                                                                                 "
    - "L1.3465[1139,1228] 199ns 60mb|-----L1.3465------|                                                                      "
    - "L1.3466[1229,1319] 199ns 61mb                    |-----L1.3466------|                                                  "
    - "L1.3467[1320,1379] 199ns 40mb                                         |--L1.3467--|                                    "
    - "L1.3470[1380,1500] 199ns 81mb                                                       |---------L1.3470---------|        "
    - "L1.3471[1501,1530] 199ns 20mb                                                                                   |L1.3471|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 261mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1139,1289] 199ns 101mb|--------------L1.?--------------|                                                        "
    - "L1.?[1290,1439] 199ns 100mb                                  |--------------L1.?--------------|                      "
    - "L1.?[1440,1530] 199ns 61mb                                                                     |-------L1.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 13 files: L0.3463, L1.3465, L1.3466, L1.3467, L0.3468, L1.3470, L1.3471, L0.3472, L0.3482, L0.3483, L0.3484, L0.3485, L0.3486"
    - "  Creating 3 files"
    - "**** Simulation run 546, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1681]). 5 Input Files, 187mb total:"
    - "L0                                                                                                                 "
    - "L0.3487[1531,1552] 199ns 0b|L0.3487|                                                                                 "
    - "L0.3386[1553,1681] 199ns 0b       |----------------L0.3386----------------|                                          "
    - "L0.3388[1682,1811] 199ns 0b                                                |----------------L0.3388----------------| "
    - "L1                                                                                                                 "
    - "L1.3436[1531,1681] 199ns 101mb|-------------------L1.3436--------------------|                                          "
    - "L1.3437[1682,1811] 199ns 87mb                                                |----------------L1.3437----------------| "
    - "**** 2 Output Files (parquet_file_id not yet assigned), 187mb total:"
    - "L1                                                                                                                 "
    - "L1.?[1531,1681] 199ns 101mb|---------------------L1.?---------------------|                                          "
    - "L1.?[1682,1811] 199ns 87mb                                                |-----------------L1.?------------------| "
    - "Committing partition 1:"
    - "  Soft Deleting 5 files: L0.3386, L0.3388, L1.3436, L1.3437, L0.3487"
    - "  Creating 2 files"
    - "**** Simulation run 547, type=compact(FoundSubsetLessThanMaxCompactSize). 20 Input Files, 132b total:"
    - "L0                                                                                                                 "
    - "L0.2954[2717,3621] 199ns 0b|L0.2954|                                                                                 "
    - "L0.2955[3622,4526] 199ns 0b|L0.2955|                                                                                 "
    - "L0.2956[4527,5431] 199ns 0b |L0.2956|                                                                                "
    - "L0.2957[5432,6336] 199ns 0b |L0.2957|                                                                                "
    - "L0.2958[6337,7241] 199ns 0b  |L0.2958|                                                                               "
    - "L0.2959[7242,8146] 199ns 0b   |L0.2959|                                                                              "
    - "L0.2960[8147,9051] 199ns 0b   |L0.2960|                                                                              "
    - "L0.2961[9052,9956] 199ns 0b    |L0.2961|                                                                             "
    - "L0.2962[9957,10861] 199ns 0b     |L0.2962|                                                                            "
    - "L0.2963[10862,11776] 199ns 12b     |L0.2963|                                                                            "
    - "L0.2937[11777,23551] 199ns 12b      |L0.2937|                                                                           "
    - "L0.2938[23552,35326] 199ns 12b              |L0.2938|                                                                   "
    - "L0.2939[35327,47101] 199ns 12b                       |L0.2939|                                                          "
    - "L0.2940[47102,58876] 199ns 12b                               |L0.2940|                                                  "
    - "L0.2941[58877,70651] 199ns 12b                                       |L0.2941|                                          "
    - "L0.2942[70652,82426] 199ns 12b                                                |L0.2942|                                 "
    - "L0.2943[82427,94201] 199ns 12b                                                        |L0.2943|                         "
    - "L0.2944[94202,105976] 199ns 12b                                                                |L0.2944|                 "
    - "L0.2947[105977,117751] 199ns 12b                                                                         |L0.2947|        "
    - "L0.2948[117752,129526] 199ns 12b                                                                                 |L0.2948|"
    - "**** 1 Output Files (parquet_file_id not yet assigned), 132b total:"
    - "L1, all files 132b                                                                                                 "
    - "L1.?[2717,129526] 199ns  |------------------------------------------L1.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 20 files: L0.2937, L0.2938, L0.2939, L0.2940, L0.2941, L0.2942, L0.2943, L0.2944, L0.2947, L0.2948, L0.2954, L0.2955, L0.2956, L0.2957, L0.2958, L0.2959, L0.2960, L0.2961, L0.2962, L0.2963"
    - "  Upgrading 1 files level to CompactionLevel::L1: L0.2953"
    - "  Creating 1 files"
    - "**** Simulation run 548, type=compact(TotalSizeLessThanMaxCompactSize). 15 Input Files, 2kb total:"
    - "L0                                                                                                                 "
    - "L0.1310[1836914,1990000] 199ns 160b                                                                                   |L0.1310|"
    - "L0.1324[1683838,1836913] 199ns 150b                                                                            |L0.1324|     "
    - "L0.1323[1530762,1683837] 199ns 150b                                                                     |L0.1323|            "
    - "L0.1322[1377686,1530761] 199ns 17b                                                              |L0.1322|                   "
    - "L0.1321[1224610,1377685] 199ns 46b                                                       |L0.1321|                          "
    - "L0.1320[1071534,1224609] 199ns 76b                                                |L0.1320|                                 "
    - "L0.1319[918458,1071533] 199ns 128b                                         |L0.1319|                                        "
    - "L0.1318[765382,918457] 199ns 179b                                  |L0.1318|                                               "
    - "L0.1317[612306,765381] 199ns 179b                           |L0.1317|                                                      "
    - "L0.1316[459230,612305] 199ns 179b                    |L0.1316|                                                             "
    - "L0.1315[306154,459229] 199ns 179b             |L0.1315|                                                                    "
    - "L0.1314[153078,306153] 199ns 179b      |L0.1314|                                                                           "
    - "L0.2950[141302,153077] 199ns 34b      |L0.2950|                                                                           "
    - "L0.2949[129527,141301] 199ns 12b     |L0.2949|                                                                            "
    - "L1                                                                                                                 "
    - "L1.3499[2717,129526] 199ns 132b|L1.3499|                                                                                 "
    - "**** 1 Output Files (parquet_file_id not yet assigned), 2kb total:"
    - "L1, all files 2kb                                                                                                  "
    - "L1.?[2717,1990000] 199ns |------------------------------------------L1.?------------------------------------------|"
    - "Committing partition 1:"
    - "  Soft Deleting 15 files: L0.1310, L0.1314, L0.1315, L0.1316, L0.1317, L0.1318, L0.1319, L0.1320, L0.1321, L0.1322, L0.1323, L0.1324, L0.2949, L0.2950, L1.3499"
    - "  Creating 1 files"
    - "**** Simulation run 549, type=split(ReduceOverlap)(split_times=[561, 618]). 1 Input Files, 101mb total:"
    - "L1, all files 101mb                                                                                                "
    - "L1.3488[474,624] 199ns   |----------------------------------------L1.3488-----------------------------------------|"
    - "**** 3 Output Files (parquet_file_id not yet assigned), 101mb total:"
    - "L1                                                                                                                 "
    - "L1.?[474,561] 199ns 59mb |-----------------------L1.?-----------------------|                                      "
    - "L1.?[562,618] 199ns 38mb                                                     |-------------L1.?--------------|     "
    - "L1.?[619,624] 199ns 4mb                                                                                         |L1.?|"
    - "Committing partition 1:"
    - "  Soft Deleting 1 files: L1.3488"
    - "  Creating 3 files"
    - "**** Simulation run 550, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[524, 586]). 6 Input Files, 266mb total:"
    - "L1                                                                                                                 "
    - "L1.3443[462,473] 199ns 8mb|L1.3443|                                                                                 "
    - "L1.3501[474,561] 199ns 59mb      |-------------------L1.3501--------------------|                                    "
    - "L1.3502[562,618] 199ns 38mb                                                       |-----------L1.3502-----------|    "
    - "L1.3503[619,624] 199ns 4mb                                                                                       |L1.3503|"
    - "L2                                                                                                                 "
    - "L2.3424[462,561] 19ns 100mb|-----------------------L2.3424-----------------------|                                   "
    - "L2.3425[562,618] 19ns 57mb                                                       |-----------L2.3425-----------|    "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 266mb total:"
    - "L2                                                                                                                 "
    - "L2.?[462,524] 199ns 103mb|--------------L2.?--------------|                                                        "
    - "L2.?[525,586] 199ns 101mb                                   |-------------L2.?--------------|                      "
    - "L2.?[587,624] 199ns 62mb                                                                      |-------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 6 files: L2.3424, L2.3425, L1.3443, L1.3501, L1.3502, L1.3503"
    - "  Creating 3 files"
    - "**** Simulation run 551, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[775, 925]). 3 Input Files, 201mb total:"
    - "L1                                                                                                                 "
    - "L1.3489[625,774] 199ns 100mb|-----------------L1.3489------------------|                                              "
    - "L1.3490[775,775] 199ns 682kb                                            |L1.3490|                                     "
    - "L1.3491[776,926] 199ns 101mb                                             |-----------------L1.3491------------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 201mb total:"
    - "L2                                                                                                                 "
    - "L2.?[625,775] 199ns 101mb|-------------------L2.?-------------------|                                              "
    - "L2.?[776,925] 199ns 100mb                                             |-------------------L2.?-------------------| "
    - "L2.?[926,926] 199ns 682kb                                                                                          |L2.?|"
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.3489, L1.3490, L1.3491"
    - "  Creating 3 files"
    - "**** Simulation run 552, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1077, 1227]). 3 Input Files, 242mb total:"
    - "L1                                                                                                                 "
    - "L1.3492[927,1076] 199ns 100mb|--------------L1.3492--------------|                                                     "
    - "L1.3493[1077,1138] 199ns 41mb                                     |---L1.3493---|                                      "
    - "L1.3494[1139,1289] 199ns 101mb                                                    |--------------L1.3494--------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 242mb total:"
    - "L2                                                                                                                 "
    - "L2.?[927,1077] 199ns 101mb|---------------L2.?----------------|                                                     "
    - "L2.?[1078,1227] 199ns 100mb                                     |---------------L2.?----------------|                "
    - "L2.?[1228,1289] 199ns 41mb                                                                          |----L2.?-----| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.3492, L1.3493, L1.3494"
    - "  Creating 3 files"
    - "**** Simulation run 553, type=split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))(split_times=[1440, 1590]). 3 Input Files, 261mb total:"
    - "L1                                                                                                                 "
    - "L1.3495[1290,1439] 199ns 100mb|------------L1.3495-------------|                                                        "
    - "L1.3496[1440,1530] 199ns 61mb                                  |-----L1.3496------|                                    "
    - "L1.3497[1531,1681] 199ns 101mb                                                       |------------L1.3497-------------| "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 261mb total:"
    - "L2                                                                                                                 "
    - "L2.?[1290,1440] 199ns 101mb|--------------L2.?--------------|                                                        "
    - "L2.?[1441,1590] 199ns 100mb                                  |--------------L2.?--------------|                      "
    - "L2.?[1591,1681] 199ns 61mb                                                                     |-------L2.?-------| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.3495, L1.3496, L1.3497"
    - "  Creating 3 files"
    - "**** Simulation run 554, type=split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))(split_times=[723271, 1444860]). 3 Input Files, 276mb total:"
    - "L1                                                                                                                 "
    - "L1.3500[2717,1990000] 199ns 2kb|----------------------------------------L1.3500----------------------------------------| "
    - "L1.3498[1682,1811] 199ns 87mb|L1.3498|                                                                                 "
    - "L1.2953[1812,2716] 199ns 189mb|L1.2953|                                                                                 "
    - "**** 3 Output Files (parquet_file_id not yet assigned), 276mb total:"
    - "L2                                                                                                                 "
    - "L2.?[1682,723271] 199ns 100mb|-------------L2.?-------------|                                                          "
    - "L2.?[723272,1444860] 199ns 100mb                                |-------------L2.?-------------|                          "
    - "L2.?[1444861,1990000] 199ns 76mb                                                                 |---------L2.?---------| "
    - "Committing partition 1:"
    - "  Soft Deleting 3 files: L1.2953, L1.3498, L1.3500"
    - "  Creating 3 files"
    - "**** Final Output Files (12.5gb written)"
    - "L2                                                                                                                 "
    - "L2.3449[1,66] 199ns 102mb|L2.3449|                                                                                 "
    - "L2.3450[67,101] 199ns 54mb|L2.3450|                                                                                 "
    - "L2.3451[102,162] 199ns 102mb|L2.3451|                                                                                 "
    - "L2.3452[163,202] 199ns 67mb|L2.3452|                                                                                 "
    - "L2.3453[203,263] 199ns 102mb|L2.3453|                                                                                 "
    - "L2.3454[264,323] 199ns 100mb|L2.3454|                                                                                 "
    - "L2.3455[324,360] 199ns 62mb|L2.3455|                                                                                 "
    - "L2.3456[361,421] 199ns 102mb|L2.3456|                                                                                 "
    - "L2.3457[422,461] 199ns 67mb|L2.3457|                                                                                 "
    - "L2.3504[462,524] 199ns 103mb|L2.3504|                                                                                 "
    - "L2.3505[525,586] 199ns 101mb|L2.3505|                                                                                 "
    - "L2.3506[587,624] 199ns 62mb|L2.3506|                                                                                 "
    - "L2.3507[625,775] 199ns 101mb|L2.3507|                                                                                 "
    - "L2.3508[776,925] 199ns 100mb|L2.3508|                                                                                 "
    - "L2.3509[926,926] 199ns 682kb|L2.3509|                                                                                 "
    - "L2.3510[927,1077] 199ns 101mb|L2.3510|                                                                                 "
    - "L2.3511[1078,1227] 199ns 100mb|L2.3511|                                                                                 "
    - "L2.3512[1228,1289] 199ns 41mb|L2.3512|                                                                                 "
    - "L2.3513[1290,1440] 199ns 101mb|L2.3513|                                                                                 "
    - "L2.3514[1441,1590] 199ns 100mb|L2.3514|                                                                                 "
    - "L2.3515[1591,1681] 199ns 61mb|L2.3515|                                                                                 "
    - "L2.3516[1682,723271] 199ns 100mb|-----------L2.3516------------|                                                          "
    - "L2.3517[723272,1444860] 199ns 100mb                                |-----------L2.3517------------|                          "
    - "L2.3518[1444861,1990000] 199ns 76mb                                                                 |-------L2.3518--------| "
    - "**** Breakdown of where bytes were written"
    - 1.43gb written by split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))
    - 132b written by compact(FoundSubsetLessThanMaxCompactSize)
    - 2.14gb written by compact(ManySmallFiles)
    - 2kb written by compact(TotalSizeLessThanMaxCompactSize)
    - 4.11gb written by split(HighL0OverlapTotalBacklog)
    - 4.32gb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 503mb written by split(ReduceOverlap)
    "###
    );
}

// This case is taken from a catalog where the partition was stuck doing single file L0->L0 compactions with a ManySmallFiles classification.
// The key point is that there is 1 L0 file, and enough overlapping L1 files such that the sum of L0 and overlapping L1s are too many for
// a single compaction.  So it it tried to do L0->L0 compaction, but you can't get less than 1 L0 file...
#[tokio::test]
async fn single_file_compaction() {
    test_helpers::maybe_start_logging();

    let max_files = 20;
    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(max_files)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(1000))
        .with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681776057065884000)
                .with_max_time(1681848094846357000)
                .with_compaction_level(CompactionLevel::Final)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681848108803007952))
                .with_file_size_bytes(148352),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681848059723530000)
                .with_max_time(1681849022292840000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681849158083717413))
                .with_file_size_bytes(8532),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681849256770938000)
                .with_max_time(1681849612137939000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681849758018522867))
                .with_file_size_bytes(7180),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681849857540998000)
                .with_max_time(1681849933405747000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681850058063700468))
                .with_file_size_bytes(6354),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681850155949687000)
                .with_max_time(1681850525337964000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681850658095040165))
                .with_file_size_bytes(7224),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681850533564810000)
                .with_max_time(1681850800324334000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681850958072081740))
                .with_file_size_bytes(6442),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681850807902300000)
                .with_max_time(1681851109057342000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681851258099471556))
                .with_file_size_bytes(6467),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681851356697599000)
                .with_max_time(1681851731606438000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681851858069516381))
                .with_file_size_bytes(7202),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681851768198276000)
                .with_max_time(1681852656555310000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681852758025054620))
                .with_file_size_bytes(7901),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681852858788440000)
                .with_max_time(1681853202074816000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681853358030917913))
                .with_file_size_bytes(7175),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681853216031150000)
                .with_max_time(1681853533814380000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681853658084495307))
                .with_file_size_bytes(6461),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681853755089369000)
                .with_max_time(1681854114135030000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681854258102937522))
                .with_file_size_bytes(7172),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681854158528835000)
                .with_max_time(1681854411758250000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681854558107269518))
                .with_file_size_bytes(6445),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681854656198860000)
                .with_max_time(1681855901530453000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681856058068217803))
                .with_file_size_bytes(9388),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681855930016632000)
                .with_max_time(1681856215951881000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681856358077776391))
                .with_file_size_bytes(6411),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681856457094364000)
                .with_max_time(1681856572199715000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681856658099983774))
                .with_file_size_bytes(6471),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681856755669647000)
                .with_max_time(1681856797376786000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681856959540758502))
                .with_file_size_bytes(6347),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681857059467239000)
                .with_max_time(1681857411709822000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681857559463607724))
                .with_file_size_bytes(7179),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681857658708732000)
                .with_max_time(1681858001258834000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681858159653340111))
                .with_file_size_bytes(7171),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681858259089021000)
                .with_max_time(1681858311972651000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681858459694290981))
                .with_file_size_bytes(6417),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681858336136281000)
                .with_max_time(1681858611711634000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681858759770566450))
                .with_file_size_bytes(6432),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681858613076367000)
                .with_max_time(1681859207290151000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681859359651203045))
                .with_file_size_bytes(7211),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681859212497834000)
                .with_max_time(1681859549996540000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681859659796715205))
                .with_file_size_bytes(6408),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681859755984961000)
                .with_max_time(1681860397139689000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681860559596560745))
                .with_file_size_bytes(7919),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681860656403220000)
                .with_max_time(1681861312602593000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681861463769557785))
                .with_file_size_bytes(7920),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681861557592893000)
                .with_max_time(1681861592762435000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681861760075293126))
                .with_file_size_bytes(6432),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681861612304587000)
                .with_max_time(1681861928505695000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681862059957822724))
                .with_file_size_bytes(6456),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681862008720364000)
                .with_max_time(1681862268794595000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681862511938856063))
                .with_file_size_bytes(6453),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1681776002714783000)
                .with_max_time(1681862102913137000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1683039505904263771))
                .with_file_size_bytes(7225),
        )
        .await;

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.29[1681776002714783000,1681862102913137000] 1683039505.9s 7kb|-----------------------------------------L0.29-----------------------------------------| "
    - "L1                                                                                                                 "
    - "L1.2[1681848059723530000,1681849022292840000] 1681849158.08s 8kb                                                                           |L1.2|         "
    - "L1.3[1681849256770938000,1681849612137939000] 1681849758.02s 7kb                                                                            |L1.3|        "
    - "L1.4[1681849857540998000,1681849933405747000] 1681850058.06s 6kb                                                                             |L1.4|       "
    - "L1.5[1681850155949687000,1681850525337964000] 1681850658.1s 7kb                                                                             |L1.5|       "
    - "L1.6[1681850533564810000,1681850800324334000] 1681850958.07s 6kb                                                                             |L1.6|       "
    - "L1.7[1681850807902300000,1681851109057342000] 1681851258.1s 6kb                                                                              |L1.7|      "
    - "L1.8[1681851356697599000,1681851731606438000] 1681851858.07s 7kb                                                                              |L1.8|      "
    - "L1.9[1681851768198276000,1681852656555310000] 1681852758.03s 8kb                                                                               |L1.9|     "
    - "L1.10[1681852858788440000,1681853202074816000] 1681853358.03s 7kb                                                                                |L1.10|   "
    - "L1.11[1681853216031150000,1681853533814380000] 1681853658.08s 6kb                                                                                |L1.11|   "
    - "L1.12[1681853755089369000,1681854114135030000] 1681854258.1s 7kb                                                                                 |L1.12|  "
    - "L1.13[1681854158528835000,1681854411758250000] 1681854558.11s 6kb                                                                                 |L1.13|  "
    - "L1.14[1681854656198860000,1681855901530453000] 1681856058.07s 9kb                                                                                  |L1.14| "
    - "L1.15[1681855930016632000,1681856215951881000] 1681856358.08s 6kb                                                                                   |L1.15|"
    - "L1.16[1681856457094364000,1681856572199715000] 1681856658.1s 6kb                                                                                   |L1.16|"
    - "L1.17[1681856755669647000,1681856797376786000] 1681856959.54s 6kb                                                                                    |L1.17|"
    - "L1.18[1681857059467239000,1681857411709822000] 1681857559.46s 7kb                                                                                    |L1.18|"
    - "L1.19[1681857658708732000,1681858001258834000] 1681858159.65s 7kb                                                                                     |L1.19|"
    - "L1.20[1681858259089021000,1681858311972651000] 1681858459.69s 6kb                                                                                     |L1.20|"
    - "L1.21[1681858336136281000,1681858611711634000] 1681858759.77s 6kb                                                                                     |L1.21|"
    - "L1.22[1681858613076367000,1681859207290151000] 1681859359.65s 7kb                                                                                      |L1.22|"
    - "L1.23[1681859212497834000,1681859549996540000] 1681859659.8s 6kb                                                                                      |L1.23|"
    - "L1.24[1681859755984961000,1681860397139689000] 1681860559.6s 8kb                                                                                       |L1.24|"
    - "L1.25[1681860656403220000,1681861312602593000] 1681861463.77s 8kb                                                                                        |L1.25|"
    - "L1.26[1681861557592893000,1681861592762435000] 1681861760.08s 6kb                                                                                         |L1.26|"
    - "L1.27[1681861612304587000,1681861928505695000] 1681862059.96s 6kb                                                                                         |L1.27|"
    - "L1.28[1681862008720364000,1681862268794595000] 1681862511.94s 6kb                                                                                         |L1.28|"
    - "L2                                                                                                                 "
    - "L2.1[1681776057065884000,1681848094846357000] 1681848108.8s 145kb|----------------------------------L2.1-----------------------------------|               "
    - "**** Final Output Files (192kb written)"
    - "L1                                                                                                                 "
    - "L1.30[1681776002714783000,1681862268794595000] 1683039505.9s 192kb|-----------------------------------------L1.30------------------------------------------|"
    - "L2                                                                                                                 "
    - "L2.1[1681776057065884000,1681848094846357000] 1681848108.8s 145kb|----------------------------------L2.1-----------------------------------|               "
    - "**** Breakdown of where bytes were written"
    - 192kb written by compact(TotalSizeLessThanMaxCompactSize)
    "###
    );
}

// Another case from a real world catalog.  Originally this case resulted in (appropriately) splitting the L0s so they don't overlap so many L1s, then (inapproprately)
// compacting the L0s together again as a ManySmallFiles operation, then the cycle repeated.
#[tokio::test]
async fn split_then_undo_it() {
    test_helpers::maybe_start_logging();

    let max_files = 20;
    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(max_files)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(1000))
        .with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680045637389000000)
                .with_max_time(1680046202520000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1680564436898219406))
                .with_file_size_bytes(106355502),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680046202521000000)
                .with_max_time(1680046767652000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1680564436898219406))
                .with_file_size_bytes(104204199),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680046767653000000)
                .with_max_time(1680047223526000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1680564436898219406))
                .with_file_size_bytes(84022852),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680047223527000000)
                .with_max_time(1680047793776000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1680564436898219406))
                .with_file_size_bytes(105366839),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680047793777000000)
                .with_max_time(1680047999999000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1680564436898219406))
                .with_file_size_bytes(37340524),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1679962892196000000)
                .with_max_time(1679969727828000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(585995),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1679979814583000000)
                .with_max_time(1679989863127000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(124967),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1679994942502000000)
                .with_max_time(1679996159985000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(174089),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1679996160115000000)
                .with_max_time(1680013439626000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(1448943),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680013440066000000)
                .with_max_time(1680019937530000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(443531),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680019960376000000)
                .with_max_time(1680030670313000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(187534),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680030903802000000)
                .with_max_time(1680033957192000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(50882),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680035266427000000)
                .with_max_time(1680037607284000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(62993),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680037696661000000)
                .with_max_time(1680041087999000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(9732222),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680041088000000000)
                .with_max_time(1680044543999000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(116659999),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680044544000000000)
                .with_max_time(1680045637388000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681186614522129445))
                .with_file_size_bytes(177095940),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1679961600071000000)
                .with_max_time(1680030719900000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681420678891928705))
                .with_file_size_bytes(11208773),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1680030720000000000)
                .with_max_time(1680047999900000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1681420678891928705))
                .with_file_size_bytes(2806765),
        )
        .await;

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.17[1679961600071000000,1680030719900000000] 1681420678.89s 11mb|--------------------------------L0.17--------------------------------|                   "
    - "L0.18[1680030720000000000,1680047999900000000] 1681420678.89s 3mb                                                                       |-----L0.18-----|  "
    - "L1                                                                                                                 "
    - "L1.1[1680045637389000000,1680046202520000000] 1680564436.9s 101mb                                                                                       |L1.1|"
    - "L1.2[1680046202521000000,1680046767652000000] 1680564436.9s 99mb                                                                                        |L1.2|"
    - "L1.3[1680046767653000000,1680047223526000000] 1680564436.9s 80mb                                                                                        |L1.3|"
    - "L1.4[1680047223527000000,1680047793776000000] 1680564436.9s 100mb                                                                                         |L1.4|"
    - "L1.5[1680047793777000000,1680047999999000000] 1680564436.9s 36mb                                                                                         |L1.5|"
    - "L1.6[1679962892196000000,1679969727828000000] 1681186614.52s 572kb |L1.6-|                                                                                  "
    - "L1.7[1679979814583000000,1679989863127000000] 1681186614.52s 122kb                  |--L1.7--|                                                              "
    - "L1.8[1679994942502000000,1679996159985000000] 1681186614.52s 170kb                                  |L1.8|                                                  "
    - "L1.9[1679996160115000000,1680013439626000000] 1681186614.52s 1mb                                    |-----L1.9------|                                     "
    - "L1.10[1680013440066000000,1680019937530000000] 1681186614.52s 433kb                                                      |L1.10|                             "
    - "L1.11[1680019960376000000,1680030670313000000] 1681186614.52s 183kb                                                            |--L1.11--|                   "
    - "L1.12[1680030903802000000,1680033957192000000] 1681186614.52s 50kb                                                                        |L1.12|           "
    - "L1.13[1680035266427000000,1680037607284000000] 1681186614.52s 62kb                                                                            |L1.13|       "
    - "L1.14[1680037696661000000,1680041087999000000] 1681186614.52s 9mb                                                                               |L1.14|    "
    - "L1.15[1680041088000000000,1680044543999000000] 1681186614.52s 111mb                                                                                  |L1.15| "
    - "L1.16[1680044544000000000,1680045637388000000] 1681186614.52s 169mb                                                                                      |L1.16|"
    - "WARNING: file L1.16[1680044544000000000,1680045637388000000] 1681186614.52s 169mb exceeds soft limit 100mb by more than 50%"
    - "**** Final Output Files (1.51gb written)"
    - "L2                                                                                                                 "
    - "L2.46[1679961600071000000,1680022452125054234] 1681420678.89s 100mb|----------------------------L2.46----------------------------|                           "
    - "L2.56[1680022452125054235,1680032319822421508] 1681420678.89s 100mb                                                               |-L2.56--|                 "
    - "L2.57[1680032319822421509,1680042187519788781] 1681420678.89s 100mb                                                                         |-L2.57--|       "
    - "L2.58[1680042187519788782,1680045769912063525] 1681420678.89s 36mb                                                                                   |L2.58|"
    - "L2.59[1680045769912063526,1680046349505534795] 1681420678.89s 100mb                                                                                       |L2.59|"
    - "L2.60[1680046349505534796,1680046929099006064] 1681420678.89s 100mb                                                                                        |L2.60|"
    - "L2.61[1680046929099006065,1680047338160274709] 1681420678.89s 71mb                                                                                        |L2.61|"
    - "L2.62[1680047338160274710,1680047867631254942] 1681420678.89s 93mb                                                                                         |L2.62|"
    - "L2.63[1680047867631254943,1680047999999000000] 1681420678.89s 23mb                                                                                         |L2.63|"
    - "**** Breakdown of where bytes were written"
    - 1.17gb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 11mb written by split(ReduceOverlap)
    - 152mb written by split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))
    - 185mb written by split(HighL0OverlapTotalBacklog)
    "###
    );
}

// This test comes from a real world catalog scenario where the configured split percentage caused a loop.  ManySmallFiles decided to compact just 2 files, which
// happen to already be split at the target percentage.  So the compaction creates 2 output files that are the same as the input files, resulting in a loop.
#[tokio::test]
async fn split_precent_loop() {
    test_helpers::maybe_start_logging();

    let max_files = 20;
    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(max_files)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(1000))
        .with_percentage_max_file_size(5)
        .with_split_percentage(80)
        .with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987200001000000)
                .with_max_time(1675996179137000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676010160053162493))
                .with_file_size_bytes(103403616),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675996179142000000)
                .with_max_time(1676005158275000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676010160053162493))
                .with_file_size_bytes(102072124),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676005158277000000)
                .with_max_time(1676010156669000000)
                .with_compaction_level(CompactionLevel::FileNonOverlapped)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676010160053162493))
                .with_file_size_bytes(61186631),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989300563000000)
                .with_max_time(1676036409167000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036411377096481))
                .with_file_size_bytes(2219347),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987777260000000)
                .with_max_time(1676036474324000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036476572081862))
                .with_file_size_bytes(2159488),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987902254000000)
                .with_max_time(1676036529744000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036533523024586))
                .with_file_size_bytes(2267826),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987264233000000)
                .with_max_time(1676036708522000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036711284678620))
                .with_file_size_bytes(2262710),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987208765000000)
                .with_max_time(1676036773664000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036776492734973))
                .with_file_size_bytes(2283847),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987969189000000)
                .with_max_time(1676036830287000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036833578815748))
                .with_file_size_bytes(2173838),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987448630000000)
                .with_max_time(1676037009945000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037011333912856))
                .with_file_size_bytes(2215286),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675991332100000000)
                .with_max_time(1676037072975000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037076612171888))
                .with_file_size_bytes(2175613),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989374650000000)
                .with_max_time(1676037129342000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037133683428336))
                .with_file_size_bytes(2244289),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987252382000000)
                .with_max_time(1676037308408000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037311292474524))
                .with_file_size_bytes(2217991),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987574435000000)
                .with_max_time(1676037374115000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037376589707454))
                .with_file_size_bytes(2188472),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989488901000000)
                .with_max_time(1676037430277000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037433529280795))
                .with_file_size_bytes(2247953),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987956301000000)
                .with_max_time(1676037608139000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037611337404983))
                .with_file_size_bytes(2230257),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987840745000000)
                .with_max_time(1676037673346000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037676565165201))
                .with_file_size_bytes(2197670),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987620819000000)
                .with_max_time(1676037730350000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037733819595619))
                .with_file_size_bytes(2181963),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987267649000000)
                .with_max_time(1676037909084000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037911429564851))
                .with_file_size_bytes(2225185),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675988167750000000)
                .with_max_time(1676037975214000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676037976761976812))
                .with_file_size_bytes(2241751),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675995240778000000)
                .with_max_time(1676063934345000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676063936517933405))
                .with_file_size_bytes(2117926),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987292063000000)
                .with_max_time(1676064071432000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064075612113418))
                .with_file_size_bytes(2197086),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675991856673000000)
                .with_max_time(1676064136664000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064139132278475))
                .with_file_size_bytes(2179185),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675990277246000000)
                .with_max_time(1676064234591000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064236557583838))
                .with_file_size_bytes(2229863),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989676787000000)
                .with_max_time(1676064371697000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064375723383965))
                .with_file_size_bytes(2164138),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675992075734000000)
                .with_max_time(1676064437297000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064439064292184))
                .with_file_size_bytes(2139050),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675991814786000000)
                .with_max_time(1676064533585000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064536460879736))
                .with_file_size_bytes(2215298),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675994514058000000)
                .with_max_time(1676064670409000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064675911178179))
                .with_file_size_bytes(2081641),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989994664000000)
                .with_max_time(1676064736678000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064740172902173))
                .with_file_size_bytes(2270347),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675989093150000000)
                .with_max_time(1676064834639000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064836484625744))
                .with_file_size_bytes(2241366),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987304054000000)
                .with_max_time(1676064970327000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676064975861286528))
                .with_file_size_bytes(2127038),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987787688000000)
                .with_max_time(1676065036871000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065039959254669))
                .with_file_size_bytes(2234389),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675994030979000000)
                .with_max_time(1676065133988000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065136539751838))
                .with_file_size_bytes(2162239),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675988375191000000)
                .with_max_time(1676065272216000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065275804926272))
                .with_file_size_bytes(2225432),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987584851000000)
                .with_max_time(1676065337320000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065339850486840))
                .with_file_size_bytes(2199543),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987883656000000)
                .with_max_time(1676065434070000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065436477743987))
                .with_file_size_bytes(2189675),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987838080000000)
                .with_max_time(1676065568770000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676065575902973989))
                .with_file_size_bytes(2240286),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987203524000000)
                .with_max_time(1676003982168000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036233843843417))
                .with_file_size_bytes(249698),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676003983105000000)
                .with_max_time(1676020762353000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036233843843417))
                .with_file_size_bytes(118322672),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676020762355000000)
                .with_max_time(1676036230752000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676036233843843417))
                .with_file_size_bytes(167000529),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987206125000000)
                .with_max_time(1676013525882000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676063839068577846))
                .with_file_size_bytes(299209),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676013530331000000)
                .with_max_time(1676039845772000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676063839068577846))
                .with_file_size_bytes(29163092),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676039845773000000)
                .with_max_time(1676063836202000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676063839068577846))
                .with_file_size_bytes(253799912),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1675987825375000000)
                .with_max_time(1676050466145000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676066475259188285))
                .with_file_size_bytes(20949),
        )
        .await;

    setup
        .partition
        .create_parquet_file(
            parquet_builder()
                .with_min_time(1676050539639000000)
                .with_max_time(1676066212011000000)
                .with_compaction_level(CompactionLevel::Initial)
                .with_max_l0_created_at(Time::from_timestamp_nanos(1676066475259188285))
                .with_file_size_bytes(13133322),
        )
        .await;

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.4[1675989300563000000,1676036409167000000] 1676036411.38s 2mb  |-----------------------L0.4------------------------|                                   "
    - "L0.5[1675987777260000000,1676036474324000000] 1676036476.57s 2mb|------------------------L0.5-------------------------|                                   "
    - "L0.6[1675987902254000000,1676036529744000000] 1676036533.52s 2mb|------------------------L0.6-------------------------|                                   "
    - "L0.7[1675987264233000000,1676036708522000000] 1676036711.28s 2mb|-------------------------L0.7-------------------------|                                  "
    - "L0.8[1675987208765000000,1676036773664000000] 1676036776.49s 2mb|-------------------------L0.8-------------------------|                                  "
    - "L0.9[1675987969189000000,1676036830287000000] 1676036833.58s 2mb|------------------------L0.9-------------------------|                                   "
    - "L0.10[1675987448630000000,1676037009945000000] 1676037011.33s 2mb|------------------------L0.10-------------------------|                                  "
    - "L0.11[1675991332100000000,1676037072975000000] 1676037076.61s 2mb    |----------------------L0.11-----------------------|                                  "
    - "L0.12[1675989374650000000,1676037129342000000] 1676037133.68s 2mb  |-----------------------L0.12------------------------|                                  "
    - "L0.13[1675987252382000000,1676037308408000000] 1676037311.29s 2mb|-------------------------L0.13-------------------------|                                 "
    - "L0.14[1675987574435000000,1676037374115000000] 1676037376.59s 2mb|------------------------L0.14-------------------------|                                  "
    - "L0.15[1675989488901000000,1676037430277000000] 1676037433.53s 2mb  |-----------------------L0.15------------------------|                                  "
    - "L0.16[1675987956301000000,1676037608139000000] 1676037611.34s 2mb|------------------------L0.16-------------------------|                                  "
    - "L0.17[1675987840745000000,1676037673346000000] 1676037676.57s 2mb|------------------------L0.17-------------------------|                                  "
    - "L0.18[1675987620819000000,1676037730350000000] 1676037733.82s 2mb|-------------------------L0.18-------------------------|                                 "
    - "L0.19[1675987267649000000,1676037909084000000] 1676037911.43s 2mb|-------------------------L0.19-------------------------|                                 "
    - "L0.20[1675988167750000000,1676037975214000000] 1676037976.76s 2mb |------------------------L0.20-------------------------|                                 "
    - "L0.21[1675995240778000000,1676063934345000000] 1676063936.52s 2mb         |-----------------------------------L0.21------------------------------------|   "
    - "L0.22[1675987292063000000,1676064071432000000] 1676064075.61s 2mb|----------------------------------------L0.22----------------------------------------|   "
    - "L0.23[1675991856673000000,1676064136664000000] 1676064139.13s 2mb     |-------------------------------------L0.23--------------------------------------|   "
    - "L0.24[1675990277246000000,1676064234591000000] 1676064236.56s 2mb   |--------------------------------------L0.24---------------------------------------|   "
    - "L0.25[1675989676787000000,1676064371697000000] 1676064375.72s 2mb  |---------------------------------------L0.25---------------------------------------|   "
    - "L0.26[1675992075734000000,1676064437297000000] 1676064439.06s 2mb     |-------------------------------------L0.26--------------------------------------|   "
    - "L0.27[1675991814786000000,1676064533585000000] 1676064536.46s 2mb     |-------------------------------------L0.27--------------------------------------|   "
    - "L0.28[1675994514058000000,1676064670409000000] 1676064675.91s 2mb        |------------------------------------L0.28------------------------------------|   "
    - "L0.29[1675989994664000000,1676064736678000000] 1676064740.17s 2mb   |---------------------------------------L0.29---------------------------------------|  "
    - "L0.30[1675989093150000000,1676064834639000000] 1676064836.48s 2mb  |---------------------------------------L0.30----------------------------------------|  "
    - "L0.31[1675987304054000000,1676064970327000000] 1676064975.86s 2mb|----------------------------------------L0.31-----------------------------------------|  "
    - "L0.32[1675987787688000000,1676065036871000000] 1676065039.96s 2mb|----------------------------------------L0.32----------------------------------------|   "
    - "L0.33[1675994030979000000,1676065133988000000] 1676065136.54s 2mb       |------------------------------------L0.33-------------------------------------|   "
    - "L0.34[1675988375191000000,1676065272216000000] 1676065275.8s 2mb |----------------------------------------L0.34----------------------------------------|  "
    - "L0.35[1675987584851000000,1676065337320000000] 1676065339.85s 2mb|----------------------------------------L0.35-----------------------------------------|  "
    - "L0.36[1675987883656000000,1676065434070000000] 1676065436.48s 2mb|----------------------------------------L0.36-----------------------------------------|  "
    - "L0.37[1675987838080000000,1676065568770000000] 1676065575.9s 2mb|----------------------------------------L0.37-----------------------------------------|  "
    - "L0.38[1675987203524000000,1676003982168000000] 1676036233.84s 244kb|------L0.38------|                                                                       "
    - "L0.39[1676003983105000000,1676020762353000000] 1676036233.84s 113mb                   |------L0.39------|                                                    "
    - "L0.40[1676020762355000000,1676036230752000000] 1676036233.84s 159mb                                      |-----L0.40-----|                                   "
    - "L0.41[1675987206125000000,1676013525882000000] 1676063839.07s 292kb|-----------L0.41-----------|                                                             "
    - "L0.42[1676013530331000000,1676039845772000000] 1676063839.07s 28mb                             |-----------L0.42-----------|                                "
    - "L0.43[1676039845773000000,1676063836202000000] 1676063839.07s 242mb                                                           |----------L0.43----------|    "
    - "L0.44[1675987825375000000,1676050466145000000] 1676066475.26s 20kb|--------------------------------L0.44--------------------------------|                   "
    - "L0.45[1676050539639000000,1676066212011000000] 1676066475.26s 13mb                                                                        |-----L0.45-----| "
    - "L1                                                                                                                 "
    - "L1.1[1675987200001000000,1675996179137000000] 1676010160.05s 99mb|--L1.1--|                                                                                "
    - "L1.2[1675996179142000000,1676005158275000000] 1676010160.05s 97mb          |--L1.2--|                                                                      "
    - "L1.3[1676005158277000000,1676010156669000000] 1676010160.05s 58mb                    |L1.3|                                                                "
    - "WARNING: file L0.40[1676020762355000000,1676036230752000000] 1676036233.84s 159mb exceeds soft limit 100mb by more than 50%"
    - "WARNING: file L0.43[1676039845773000000,1676063836202000000] 1676063839.07s 242mb exceeds soft limit 100mb by more than 50%"
    - "**** Final Output Files (3.1gb written)"
    - "L1                                                                                                                 "
    - "L1.260[1676045833054395546,1676050409609000000] 1676066475.26s 41mb                                                                  |L1.260|                "
    - "L2                                                                                                                 "
    - "L2.228[1676050409609000001,1676066212011000000] 1676066475.26s 145mb                                                                        |----L2.228-----| "
    - "L2.251[1675987200001000000,1675995209209749739] 1676066475.26s 100mb|L2.251-|                                                                                 "
    - "L2.252[1675995209209749740,1676003218418499478] 1676066475.26s 100mb         |L2.252-|                                                                        "
    - "L2.254[1676003218418499479,1676010967023992579] 1676066475.26s 100mb                  |L2.254|                                                                "
    - "L2.255[1676010967023992580,1676018715629485679] 1676066475.26s 100mb                           |L2.255|                                                       "
    - "L2.261[1676018715629485680,1676027900853158703] 1676066475.26s 100mb                                   |-L2.261-|                                             "
    - "L2.262[1676027900853158704,1676037086076831726] 1676066475.26s 100mb                                              |-L2.262-|                                  "
    - "L2.263[1676037086076831727,1676045833054395545] 1676066475.26s 95mb                                                        |L2.263-|                         "
    - "**** Breakdown of where bytes were written"
    - 1.46gb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 141mb written by split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))
    - 176mb written by split(ReduceOverlap)
    - 639mb written by compact(ManySmallFiles)
    - 724mb written by split(HighL0OverlapTotalBacklog)
    "###
    );
}

// This is a simplified version of a test generated from actual catalog contents (which was thousands of lines).
// The key attributes are:
//  - there are enough bytes of L0 to trigger vertical splitting
//  - there are enough L0 files that the individual files are tiny
//  - there are lots of L1s that make it a pain to merge down from L0
//  - when the L0s get split, they're split into enough pieces that the algorigthm (pre-fix) would put the L0s back together in a single file.
// The result, prior to the fix motivating this test case, is that the L0s would be vertically split, then regrouped together in a single chain,
// so they get recompacted together, which again prompts the need for vertical splitting, resulting in an unproductive cycle.
#[tokio::test]
async fn very_big_overlapped_backlog() {
    test_helpers::maybe_start_logging();

    let max_files = 20;
    let setup = layout_setup_builder()
        .await
        .with_max_num_files_per_plan(max_files)
        .with_max_desired_file_size_bytes(MAX_DESIRED_FILE_SIZE)
        .with_partition_timeout(Duration::from_millis(100000))
        .with_suppress_run_output() // remove this to debug
        .with_writes_breakdown()
        .build()
        .await;

    let max_time: i64 = 200000;
    let l0_cnt: i64 = 200;
    let l0_interval = max_time / l0_cnt;
    let l0_size = MAX_DESIRED_FILE_SIZE * 4 / l0_cnt as u64;
    let l1_cnt = 100;
    let l1_interval = max_time / l1_cnt;

    // Create 100s of overlapping L0s
    for i in 0..l0_cnt {
        setup
            .partition
            .create_parquet_file(
                parquet_builder()
                    .with_min_time(i * l0_interval)
                    .with_max_time((i + 1) * l0_interval)
                    .with_compaction_level(CompactionLevel::Initial)
                    .with_max_l0_created_at(Time::from_timestamp_nanos(l1_cnt + i))
                    .with_file_size_bytes(l0_size),
            )
            .await;
    }

    // Create a lot of L1s, on the same time range as the L0s
    for i in 0..l1_cnt {
        setup
            .partition
            .create_parquet_file(
                parquet_builder()
                    .with_min_time(i * l1_interval)
                    .with_max_time((i + 1) * l1_interval - 1)
                    .with_compaction_level(CompactionLevel::FileNonOverlapped)
                    .with_max_l0_created_at(Time::from_timestamp_nanos(i))
                    .with_file_size_bytes(MAX_DESIRED_FILE_SIZE),
            )
            .await;
    }

    // Create a lot of L2s, on the same time range as the L0s and L1s
    for i in 0..l1_cnt {
        setup
            .partition
            .create_parquet_file(
                parquet_builder()
                    .with_min_time(i * l1_interval)
                    .with_max_time((i + 1) * l1_interval - 1)
                    .with_compaction_level(CompactionLevel::Final)
                    .with_max_l0_created_at(Time::from_timestamp_nanos(i))
                    .with_file_size_bytes(MAX_DESIRED_FILE_SIZE),
            )
            .await;
    }

    insta::assert_yaml_snapshot!(
        run_layout_scenario(&setup).await,
        @r###"
    ---
    - "**** Input Files "
    - "L0                                                                                                                 "
    - "L0.1[0,1000] 100ns 2mb   |L0.1|                                                                                    "
    - "L0.2[1000,2000] 101ns 2mb|L0.2|                                                                                    "
    - "L0.3[2000,3000] 102ns 2mb|L0.3|                                                                                    "
    - "L0.4[3000,4000] 103ns 2mb |L0.4|                                                                                   "
    - "L0.5[4000,5000] 104ns 2mb |L0.5|                                                                                   "
    - "L0.6[5000,6000] 105ns 2mb  |L0.6|                                                                                  "
    - "L0.7[6000,7000] 106ns 2mb  |L0.7|                                                                                  "
    - "L0.8[7000,8000] 107ns 2mb   |L0.8|                                                                                 "
    - "L0.9[8000,9000] 108ns 2mb   |L0.9|                                                                                 "
    - "L0.10[9000,10000] 109ns 2mb    |L0.10|                                                                               "
    - "L0.11[10000,11000] 110ns 2mb    |L0.11|                                                                               "
    - "L0.12[11000,12000] 111ns 2mb    |L0.12|                                                                               "
    - "L0.13[12000,13000] 112ns 2mb     |L0.13|                                                                              "
    - "L0.14[13000,14000] 113ns 2mb     |L0.14|                                                                              "
    - "L0.15[14000,15000] 114ns 2mb      |L0.15|                                                                             "
    - "L0.16[15000,16000] 115ns 2mb      |L0.16|                                                                             "
    - "L0.17[16000,17000] 116ns 2mb       |L0.17|                                                                            "
    - "L0.18[17000,18000] 117ns 2mb       |L0.18|                                                                            "
    - "L0.19[18000,19000] 118ns 2mb        |L0.19|                                                                           "
    - "L0.20[19000,20000] 119ns 2mb        |L0.20|                                                                           "
    - "L0.21[20000,21000] 120ns 2mb         |L0.21|                                                                          "
    - "L0.22[21000,22000] 121ns 2mb         |L0.22|                                                                          "
    - "L0.23[22000,23000] 122ns 2mb         |L0.23|                                                                          "
    - "L0.24[23000,24000] 123ns 2mb          |L0.24|                                                                         "
    - "L0.25[24000,25000] 124ns 2mb          |L0.25|                                                                         "
    - "L0.26[25000,26000] 125ns 2mb           |L0.26|                                                                        "
    - "L0.27[26000,27000] 126ns 2mb           |L0.27|                                                                        "
    - "L0.28[27000,28000] 127ns 2mb            |L0.28|                                                                       "
    - "L0.29[28000,29000] 128ns 2mb            |L0.29|                                                                       "
    - "L0.30[29000,30000] 129ns 2mb             |L0.30|                                                                      "
    - "L0.31[30000,31000] 130ns 2mb             |L0.31|                                                                      "
    - "L0.32[31000,32000] 131ns 2mb             |L0.32|                                                                      "
    - "L0.33[32000,33000] 132ns 2mb              |L0.33|                                                                     "
    - "L0.34[33000,34000] 133ns 2mb              |L0.34|                                                                     "
    - "L0.35[34000,35000] 134ns 2mb               |L0.35|                                                                    "
    - "L0.36[35000,36000] 135ns 2mb               |L0.36|                                                                    "
    - "L0.37[36000,37000] 136ns 2mb                |L0.37|                                                                   "
    - "L0.38[37000,38000] 137ns 2mb                |L0.38|                                                                   "
    - "L0.39[38000,39000] 138ns 2mb                 |L0.39|                                                                  "
    - "L0.40[39000,40000] 139ns 2mb                 |L0.40|                                                                  "
    - "L0.41[40000,41000] 140ns 2mb                  |L0.41|                                                                 "
    - "L0.42[41000,42000] 141ns 2mb                  |L0.42|                                                                 "
    - "L0.43[42000,43000] 142ns 2mb                  |L0.43|                                                                 "
    - "L0.44[43000,44000] 143ns 2mb                   |L0.44|                                                                "
    - "L0.45[44000,45000] 144ns 2mb                   |L0.45|                                                                "
    - "L0.46[45000,46000] 145ns 2mb                    |L0.46|                                                               "
    - "L0.47[46000,47000] 146ns 2mb                    |L0.47|                                                               "
    - "L0.48[47000,48000] 147ns 2mb                     |L0.48|                                                              "
    - "L0.49[48000,49000] 148ns 2mb                     |L0.49|                                                              "
    - "L0.50[49000,50000] 149ns 2mb                      |L0.50|                                                             "
    - "L0.51[50000,51000] 150ns 2mb                      |L0.51|                                                             "
    - "L0.52[51000,52000] 151ns 2mb                      |L0.52|                                                             "
    - "L0.53[52000,53000] 152ns 2mb                       |L0.53|                                                            "
    - "L0.54[53000,54000] 153ns 2mb                       |L0.54|                                                            "
    - "L0.55[54000,55000] 154ns 2mb                        |L0.55|                                                           "
    - "L0.56[55000,56000] 155ns 2mb                        |L0.56|                                                           "
    - "L0.57[56000,57000] 156ns 2mb                         |L0.57|                                                          "
    - "L0.58[57000,58000] 157ns 2mb                         |L0.58|                                                          "
    - "L0.59[58000,59000] 158ns 2mb                          |L0.59|                                                         "
    - "L0.60[59000,60000] 159ns 2mb                          |L0.60|                                                         "
    - "L0.61[60000,61000] 160ns 2mb                           |L0.61|                                                        "
    - "L0.62[61000,62000] 161ns 2mb                           |L0.62|                                                        "
    - "L0.63[62000,63000] 162ns 2mb                           |L0.63|                                                        "
    - "L0.64[63000,64000] 163ns 2mb                            |L0.64|                                                       "
    - "L0.65[64000,65000] 164ns 2mb                            |L0.65|                                                       "
    - "L0.66[65000,66000] 165ns 2mb                             |L0.66|                                                      "
    - "L0.67[66000,67000] 166ns 2mb                             |L0.67|                                                      "
    - "L0.68[67000,68000] 167ns 2mb                              |L0.68|                                                     "
    - "L0.69[68000,69000] 168ns 2mb                              |L0.69|                                                     "
    - "L0.70[69000,70000] 169ns 2mb                               |L0.70|                                                    "
    - "L0.71[70000,71000] 170ns 2mb                               |L0.71|                                                    "
    - "L0.72[71000,72000] 171ns 2mb                               |L0.72|                                                    "
    - "L0.73[72000,73000] 172ns 2mb                                |L0.73|                                                   "
    - "L0.74[73000,74000] 173ns 2mb                                |L0.74|                                                   "
    - "L0.75[74000,75000] 174ns 2mb                                 |L0.75|                                                  "
    - "L0.76[75000,76000] 175ns 2mb                                 |L0.76|                                                  "
    - "L0.77[76000,77000] 176ns 2mb                                  |L0.77|                                                 "
    - "L0.78[77000,78000] 177ns 2mb                                  |L0.78|                                                 "
    - "L0.79[78000,79000] 178ns 2mb                                   |L0.79|                                                "
    - "L0.80[79000,80000] 179ns 2mb                                   |L0.80|                                                "
    - "L0.81[80000,81000] 180ns 2mb                                    |L0.81|                                               "
    - "L0.82[81000,82000] 181ns 2mb                                    |L0.82|                                               "
    - "L0.83[82000,83000] 182ns 2mb                                    |L0.83|                                               "
    - "L0.84[83000,84000] 183ns 2mb                                     |L0.84|                                              "
    - "L0.85[84000,85000] 184ns 2mb                                     |L0.85|                                              "
    - "L0.86[85000,86000] 185ns 2mb                                      |L0.86|                                             "
    - "L0.87[86000,87000] 186ns 2mb                                      |L0.87|                                             "
    - "L0.88[87000,88000] 187ns 2mb                                       |L0.88|                                            "
    - "L0.89[88000,89000] 188ns 2mb                                       |L0.89|                                            "
    - "L0.90[89000,90000] 189ns 2mb                                        |L0.90|                                           "
    - "L0.91[90000,91000] 190ns 2mb                                        |L0.91|                                           "
    - "L0.92[91000,92000] 191ns 2mb                                        |L0.92|                                           "
    - "L0.93[92000,93000] 192ns 2mb                                         |L0.93|                                          "
    - "L0.94[93000,94000] 193ns 2mb                                         |L0.94|                                          "
    - "L0.95[94000,95000] 194ns 2mb                                          |L0.95|                                         "
    - "L0.96[95000,96000] 195ns 2mb                                          |L0.96|                                         "
    - "L0.97[96000,97000] 196ns 2mb                                           |L0.97|                                        "
    - "L0.98[97000,98000] 197ns 2mb                                           |L0.98|                                        "
    - "L0.99[98000,99000] 198ns 2mb                                            |L0.99|                                       "
    - "L0.100[99000,100000] 199ns 2mb                                            |L0.100|                                      "
    - "L0.101[100000,101000] 200ns 2mb                                             |L0.101|                                     "
    - "L0.102[101000,102000] 201ns 2mb                                             |L0.102|                                     "
    - "L0.103[102000,103000] 202ns 2mb                                             |L0.103|                                     "
    - "L0.104[103000,104000] 203ns 2mb                                              |L0.104|                                    "
    - "L0.105[104000,105000] 204ns 2mb                                              |L0.105|                                    "
    - "L0.106[105000,106000] 205ns 2mb                                               |L0.106|                                   "
    - "L0.107[106000,107000] 206ns 2mb                                               |L0.107|                                   "
    - "L0.108[107000,108000] 207ns 2mb                                                |L0.108|                                  "
    - "L0.109[108000,109000] 208ns 2mb                                                |L0.109|                                  "
    - "L0.110[109000,110000] 209ns 2mb                                                 |L0.110|                                 "
    - "L0.111[110000,111000] 210ns 2mb                                                 |L0.111|                                 "
    - "L0.112[111000,112000] 211ns 2mb                                                 |L0.112|                                 "
    - "L0.113[112000,113000] 212ns 2mb                                                  |L0.113|                                "
    - "L0.114[113000,114000] 213ns 2mb                                                  |L0.114|                                "
    - "L0.115[114000,115000] 214ns 2mb                                                   |L0.115|                               "
    - "L0.116[115000,116000] 215ns 2mb                                                   |L0.116|                               "
    - "L0.117[116000,117000] 216ns 2mb                                                    |L0.117|                              "
    - "L0.118[117000,118000] 217ns 2mb                                                    |L0.118|                              "
    - "L0.119[118000,119000] 218ns 2mb                                                     |L0.119|                             "
    - "L0.120[119000,120000] 219ns 2mb                                                     |L0.120|                             "
    - "L0.121[120000,121000] 220ns 2mb                                                      |L0.121|                            "
    - "L0.122[121000,122000] 221ns 2mb                                                      |L0.122|                            "
    - "L0.123[122000,123000] 222ns 2mb                                                      |L0.123|                            "
    - "L0.124[123000,124000] 223ns 2mb                                                       |L0.124|                           "
    - "L0.125[124000,125000] 224ns 2mb                                                       |L0.125|                           "
    - "L0.126[125000,126000] 225ns 2mb                                                        |L0.126|                          "
    - "L0.127[126000,127000] 226ns 2mb                                                        |L0.127|                          "
    - "L0.128[127000,128000] 227ns 2mb                                                         |L0.128|                         "
    - "L0.129[128000,129000] 228ns 2mb                                                         |L0.129|                         "
    - "L0.130[129000,130000] 229ns 2mb                                                          |L0.130|                        "
    - "L0.131[130000,131000] 230ns 2mb                                                          |L0.131|                        "
    - "L0.132[131000,132000] 231ns 2mb                                                          |L0.132|                        "
    - "L0.133[132000,133000] 232ns 2mb                                                           |L0.133|                       "
    - "L0.134[133000,134000] 233ns 2mb                                                           |L0.134|                       "
    - "L0.135[134000,135000] 234ns 2mb                                                            |L0.135|                      "
    - "L0.136[135000,136000] 235ns 2mb                                                            |L0.136|                      "
    - "L0.137[136000,137000] 236ns 2mb                                                             |L0.137|                     "
    - "L0.138[137000,138000] 237ns 2mb                                                             |L0.138|                     "
    - "L0.139[138000,139000] 238ns 2mb                                                              |L0.139|                    "
    - "L0.140[139000,140000] 239ns 2mb                                                              |L0.140|                    "
    - "L0.141[140000,141000] 240ns 2mb                                                               |L0.141|                   "
    - "L0.142[141000,142000] 241ns 2mb                                                               |L0.142|                   "
    - "L0.143[142000,143000] 242ns 2mb                                                               |L0.143|                   "
    - "L0.144[143000,144000] 243ns 2mb                                                                |L0.144|                  "
    - "L0.145[144000,145000] 244ns 2mb                                                                |L0.145|                  "
    - "L0.146[145000,146000] 245ns 2mb                                                                 |L0.146|                 "
    - "L0.147[146000,147000] 246ns 2mb                                                                 |L0.147|                 "
    - "L0.148[147000,148000] 247ns 2mb                                                                  |L0.148|                "
    - "L0.149[148000,149000] 248ns 2mb                                                                  |L0.149|                "
    - "L0.150[149000,150000] 249ns 2mb                                                                   |L0.150|               "
    - "L0.151[150000,151000] 250ns 2mb                                                                   |L0.151|               "
    - "L0.152[151000,152000] 251ns 2mb                                                                   |L0.152|               "
    - "L0.153[152000,153000] 252ns 2mb                                                                    |L0.153|              "
    - "L0.154[153000,154000] 253ns 2mb                                                                    |L0.154|              "
    - "L0.155[154000,155000] 254ns 2mb                                                                     |L0.155|             "
    - "L0.156[155000,156000] 255ns 2mb                                                                     |L0.156|             "
    - "L0.157[156000,157000] 256ns 2mb                                                                      |L0.157|            "
    - "L0.158[157000,158000] 257ns 2mb                                                                      |L0.158|            "
    - "L0.159[158000,159000] 258ns 2mb                                                                       |L0.159|           "
    - "L0.160[159000,160000] 259ns 2mb                                                                       |L0.160|           "
    - "L0.161[160000,161000] 260ns 2mb                                                                        |L0.161|          "
    - "L0.162[161000,162000] 261ns 2mb                                                                        |L0.162|          "
    - "L0.163[162000,163000] 262ns 2mb                                                                        |L0.163|          "
    - "L0.164[163000,164000] 263ns 2mb                                                                         |L0.164|         "
    - "L0.165[164000,165000] 264ns 2mb                                                                         |L0.165|         "
    - "L0.166[165000,166000] 265ns 2mb                                                                          |L0.166|        "
    - "L0.167[166000,167000] 266ns 2mb                                                                          |L0.167|        "
    - "L0.168[167000,168000] 267ns 2mb                                                                           |L0.168|       "
    - "L0.169[168000,169000] 268ns 2mb                                                                           |L0.169|       "
    - "L0.170[169000,170000] 269ns 2mb                                                                            |L0.170|      "
    - "L0.171[170000,171000] 270ns 2mb                                                                            |L0.171|      "
    - "L0.172[171000,172000] 271ns 2mb                                                                            |L0.172|      "
    - "L0.173[172000,173000] 272ns 2mb                                                                             |L0.173|     "
    - "L0.174[173000,174000] 273ns 2mb                                                                             |L0.174|     "
    - "L0.175[174000,175000] 274ns 2mb                                                                              |L0.175|    "
    - "L0.176[175000,176000] 275ns 2mb                                                                              |L0.176|    "
    - "L0.177[176000,177000] 276ns 2mb                                                                               |L0.177|   "
    - "L0.178[177000,178000] 277ns 2mb                                                                               |L0.178|   "
    - "L0.179[178000,179000] 278ns 2mb                                                                                |L0.179|  "
    - "L0.180[179000,180000] 279ns 2mb                                                                                |L0.180|  "
    - "L0.181[180000,181000] 280ns 2mb                                                                                 |L0.181| "
    - "L0.182[181000,182000] 281ns 2mb                                                                                 |L0.182| "
    - "L0.183[182000,183000] 282ns 2mb                                                                                 |L0.183| "
    - "L0.184[183000,184000] 283ns 2mb                                                                                  |L0.184|"
    - "L0.185[184000,185000] 284ns 2mb                                                                                  |L0.185|"
    - "L0.186[185000,186000] 285ns 2mb                                                                                   |L0.186|"
    - "L0.187[186000,187000] 286ns 2mb                                                                                   |L0.187|"
    - "L0.188[187000,188000] 287ns 2mb                                                                                    |L0.188|"
    - "L0.189[188000,189000] 288ns 2mb                                                                                    |L0.189|"
    - "L0.190[189000,190000] 289ns 2mb                                                                                     |L0.190|"
    - "L0.191[190000,191000] 290ns 2mb                                                                                     |L0.191|"
    - "L0.192[191000,192000] 291ns 2mb                                                                                     |L0.192|"
    - "L0.193[192000,193000] 292ns 2mb                                                                                      |L0.193|"
    - "L0.194[193000,194000] 293ns 2mb                                                                                      |L0.194|"
    - "L0.195[194000,195000] 294ns 2mb                                                                                       |L0.195|"
    - "L0.196[195000,196000] 295ns 2mb                                                                                       |L0.196|"
    - "L0.197[196000,197000] 296ns 2mb                                                                                        |L0.197|"
    - "L0.198[197000,198000] 297ns 2mb                                                                                        |L0.198|"
    - "L0.199[198000,199000] 298ns 2mb                                                                                         |L0.199|"
    - "L0.200[199000,200000] 299ns 2mb                                                                                         |L0.200|"
    - "L1                                                                                                                 "
    - "L1.201[0,1999] 0ns 100mb |L1.201|                                                                                  "
    - "L1.202[2000,3999] 1ns 100mb|L1.202|                                                                                  "
    - "L1.203[4000,5999] 2ns 100mb |L1.203|                                                                                 "
    - "L1.204[6000,7999] 3ns 100mb  |L1.204|                                                                                "
    - "L1.205[8000,9999] 4ns 100mb   |L1.205|                                                                               "
    - "L1.206[10000,11999] 5ns 100mb    |L1.206|                                                                              "
    - "L1.207[12000,13999] 6ns 100mb     |L1.207|                                                                             "
    - "L1.208[14000,15999] 7ns 100mb      |L1.208|                                                                            "
    - "L1.209[16000,17999] 8ns 100mb       |L1.209|                                                                           "
    - "L1.210[18000,19999] 9ns 100mb        |L1.210|                                                                          "
    - "L1.211[20000,21999] 10ns 100mb         |L1.211|                                                                         "
    - "L1.212[22000,23999] 11ns 100mb         |L1.212|                                                                         "
    - "L1.213[24000,25999] 12ns 100mb          |L1.213|                                                                        "
    - "L1.214[26000,27999] 13ns 100mb           |L1.214|                                                                       "
    - "L1.215[28000,29999] 14ns 100mb            |L1.215|                                                                      "
    - "L1.216[30000,31999] 15ns 100mb             |L1.216|                                                                     "
    - "L1.217[32000,33999] 16ns 100mb              |L1.217|                                                                    "
    - "L1.218[34000,35999] 17ns 100mb               |L1.218|                                                                   "
    - "L1.219[36000,37999] 18ns 100mb                |L1.219|                                                                  "
    - "L1.220[38000,39999] 19ns 100mb                 |L1.220|                                                                 "
    - "L1.221[40000,41999] 20ns 100mb                  |L1.221|                                                                "
    - "L1.222[42000,43999] 21ns 100mb                  |L1.222|                                                                "
    - "L1.223[44000,45999] 22ns 100mb                   |L1.223|                                                               "
    - "L1.224[46000,47999] 23ns 100mb                    |L1.224|                                                              "
    - "L1.225[48000,49999] 24ns 100mb                     |L1.225|                                                             "
    - "L1.226[50000,51999] 25ns 100mb                      |L1.226|                                                            "
    - "L1.227[52000,53999] 26ns 100mb                       |L1.227|                                                           "
    - "L1.228[54000,55999] 27ns 100mb                        |L1.228|                                                          "
    - "L1.229[56000,57999] 28ns 100mb                         |L1.229|                                                         "
    - "L1.230[58000,59999] 29ns 100mb                          |L1.230|                                                        "
    - "L1.231[60000,61999] 30ns 100mb                           |L1.231|                                                       "
    - "L1.232[62000,63999] 31ns 100mb                           |L1.232|                                                       "
    - "L1.233[64000,65999] 32ns 100mb                            |L1.233|                                                      "
    - "L1.234[66000,67999] 33ns 100mb                             |L1.234|                                                     "
    - "L1.235[68000,69999] 34ns 100mb                              |L1.235|                                                    "
    - "L1.236[70000,71999] 35ns 100mb                               |L1.236|                                                   "
    - "L1.237[72000,73999] 36ns 100mb                                |L1.237|                                                  "
    - "L1.238[74000,75999] 37ns 100mb                                 |L1.238|                                                 "
    - "L1.239[76000,77999] 38ns 100mb                                  |L1.239|                                                "
    - "L1.240[78000,79999] 39ns 100mb                                   |L1.240|                                               "
    - "L1.241[80000,81999] 40ns 100mb                                    |L1.241|                                              "
    - "L1.242[82000,83999] 41ns 100mb                                    |L1.242|                                              "
    - "L1.243[84000,85999] 42ns 100mb                                     |L1.243|                                             "
    - "L1.244[86000,87999] 43ns 100mb                                      |L1.244|                                            "
    - "L1.245[88000,89999] 44ns 100mb                                       |L1.245|                                           "
    - "L1.246[90000,91999] 45ns 100mb                                        |L1.246|                                          "
    - "L1.247[92000,93999] 46ns 100mb                                         |L1.247|                                         "
    - "L1.248[94000,95999] 47ns 100mb                                          |L1.248|                                        "
    - "L1.249[96000,97999] 48ns 100mb                                           |L1.249|                                       "
    - "L1.250[98000,99999] 49ns 100mb                                            |L1.250|                                      "
    - "L1.251[100000,101999] 50ns 100mb                                             |L1.251|                                     "
    - "L1.252[102000,103999] 51ns 100mb                                             |L1.252|                                     "
    - "L1.253[104000,105999] 52ns 100mb                                              |L1.253|                                    "
    - "L1.254[106000,107999] 53ns 100mb                                               |L1.254|                                   "
    - "L1.255[108000,109999] 54ns 100mb                                                |L1.255|                                  "
    - "L1.256[110000,111999] 55ns 100mb                                                 |L1.256|                                 "
    - "L1.257[112000,113999] 56ns 100mb                                                  |L1.257|                                "
    - "L1.258[114000,115999] 57ns 100mb                                                   |L1.258|                               "
    - "L1.259[116000,117999] 58ns 100mb                                                    |L1.259|                              "
    - "L1.260[118000,119999] 59ns 100mb                                                     |L1.260|                             "
    - "L1.261[120000,121999] 60ns 100mb                                                      |L1.261|                            "
    - "L1.262[122000,123999] 61ns 100mb                                                      |L1.262|                            "
    - "L1.263[124000,125999] 62ns 100mb                                                       |L1.263|                           "
    - "L1.264[126000,127999] 63ns 100mb                                                        |L1.264|                          "
    - "L1.265[128000,129999] 64ns 100mb                                                         |L1.265|                         "
    - "L1.266[130000,131999] 65ns 100mb                                                          |L1.266|                        "
    - "L1.267[132000,133999] 66ns 100mb                                                           |L1.267|                       "
    - "L1.268[134000,135999] 67ns 100mb                                                            |L1.268|                      "
    - "L1.269[136000,137999] 68ns 100mb                                                             |L1.269|                     "
    - "L1.270[138000,139999] 69ns 100mb                                                              |L1.270|                    "
    - "L1.271[140000,141999] 70ns 100mb                                                               |L1.271|                   "
    - "L1.272[142000,143999] 71ns 100mb                                                               |L1.272|                   "
    - "L1.273[144000,145999] 72ns 100mb                                                                |L1.273|                  "
    - "L1.274[146000,147999] 73ns 100mb                                                                 |L1.274|                 "
    - "L1.275[148000,149999] 74ns 100mb                                                                  |L1.275|                "
    - "L1.276[150000,151999] 75ns 100mb                                                                   |L1.276|               "
    - "L1.277[152000,153999] 76ns 100mb                                                                    |L1.277|              "
    - "L1.278[154000,155999] 77ns 100mb                                                                     |L1.278|             "
    - "L1.279[156000,157999] 78ns 100mb                                                                      |L1.279|            "
    - "L1.280[158000,159999] 79ns 100mb                                                                       |L1.280|           "
    - "L1.281[160000,161999] 80ns 100mb                                                                        |L1.281|          "
    - "L1.282[162000,163999] 81ns 100mb                                                                        |L1.282|          "
    - "L1.283[164000,165999] 82ns 100mb                                                                         |L1.283|         "
    - "L1.284[166000,167999] 83ns 100mb                                                                          |L1.284|        "
    - "L1.285[168000,169999] 84ns 100mb                                                                           |L1.285|       "
    - "L1.286[170000,171999] 85ns 100mb                                                                            |L1.286|      "
    - "L1.287[172000,173999] 86ns 100mb                                                                             |L1.287|     "
    - "L1.288[174000,175999] 87ns 100mb                                                                              |L1.288|    "
    - "L1.289[176000,177999] 88ns 100mb                                                                               |L1.289|   "
    - "L1.290[178000,179999] 89ns 100mb                                                                                |L1.290|  "
    - "L1.291[180000,181999] 90ns 100mb                                                                                 |L1.291| "
    - "L1.292[182000,183999] 91ns 100mb                                                                                 |L1.292| "
    - "L1.293[184000,185999] 92ns 100mb                                                                                  |L1.293|"
    - "L1.294[186000,187999] 93ns 100mb                                                                                   |L1.294|"
    - "L1.295[188000,189999] 94ns 100mb                                                                                    |L1.295|"
    - "L1.296[190000,191999] 95ns 100mb                                                                                     |L1.296|"
    - "L1.297[192000,193999] 96ns 100mb                                                                                      |L1.297|"
    - "L1.298[194000,195999] 97ns 100mb                                                                                       |L1.298|"
    - "L1.299[196000,197999] 98ns 100mb                                                                                        |L1.299|"
    - "L1.300[198000,199999] 99ns 100mb                                                                                         |L1.300|"
    - "L2                                                                                                                 "
    - "L2.301[0,1999] 0ns 100mb |L2.301|                                                                                  "
    - "L2.302[2000,3999] 1ns 100mb|L2.302|                                                                                  "
    - "L2.303[4000,5999] 2ns 100mb |L2.303|                                                                                 "
    - "L2.304[6000,7999] 3ns 100mb  |L2.304|                                                                                "
    - "L2.305[8000,9999] 4ns 100mb   |L2.305|                                                                               "
    - "L2.306[10000,11999] 5ns 100mb    |L2.306|                                                                              "
    - "L2.307[12000,13999] 6ns 100mb     |L2.307|                                                                             "
    - "L2.308[14000,15999] 7ns 100mb      |L2.308|                                                                            "
    - "L2.309[16000,17999] 8ns 100mb       |L2.309|                                                                           "
    - "L2.310[18000,19999] 9ns 100mb        |L2.310|                                                                          "
    - "L2.311[20000,21999] 10ns 100mb         |L2.311|                                                                         "
    - "L2.312[22000,23999] 11ns 100mb         |L2.312|                                                                         "
    - "L2.313[24000,25999] 12ns 100mb          |L2.313|                                                                        "
    - "L2.314[26000,27999] 13ns 100mb           |L2.314|                                                                       "
    - "L2.315[28000,29999] 14ns 100mb            |L2.315|                                                                      "
    - "L2.316[30000,31999] 15ns 100mb             |L2.316|                                                                     "
    - "L2.317[32000,33999] 16ns 100mb              |L2.317|                                                                    "
    - "L2.318[34000,35999] 17ns 100mb               |L2.318|                                                                   "
    - "L2.319[36000,37999] 18ns 100mb                |L2.319|                                                                  "
    - "L2.320[38000,39999] 19ns 100mb                 |L2.320|                                                                 "
    - "L2.321[40000,41999] 20ns 100mb                  |L2.321|                                                                "
    - "L2.322[42000,43999] 21ns 100mb                  |L2.322|                                                                "
    - "L2.323[44000,45999] 22ns 100mb                   |L2.323|                                                               "
    - "L2.324[46000,47999] 23ns 100mb                    |L2.324|                                                              "
    - "L2.325[48000,49999] 24ns 100mb                     |L2.325|                                                             "
    - "L2.326[50000,51999] 25ns 100mb                      |L2.326|                                                            "
    - "L2.327[52000,53999] 26ns 100mb                       |L2.327|                                                           "
    - "L2.328[54000,55999] 27ns 100mb                        |L2.328|                                                          "
    - "L2.329[56000,57999] 28ns 100mb                         |L2.329|                                                         "
    - "L2.330[58000,59999] 29ns 100mb                          |L2.330|                                                        "
    - "L2.331[60000,61999] 30ns 100mb                           |L2.331|                                                       "
    - "L2.332[62000,63999] 31ns 100mb                           |L2.332|                                                       "
    - "L2.333[64000,65999] 32ns 100mb                            |L2.333|                                                      "
    - "L2.334[66000,67999] 33ns 100mb                             |L2.334|                                                     "
    - "L2.335[68000,69999] 34ns 100mb                              |L2.335|                                                    "
    - "L2.336[70000,71999] 35ns 100mb                               |L2.336|                                                   "
    - "L2.337[72000,73999] 36ns 100mb                                |L2.337|                                                  "
    - "L2.338[74000,75999] 37ns 100mb                                 |L2.338|                                                 "
    - "L2.339[76000,77999] 38ns 100mb                                  |L2.339|                                                "
    - "L2.340[78000,79999] 39ns 100mb                                   |L2.340|                                               "
    - "L2.341[80000,81999] 40ns 100mb                                    |L2.341|                                              "
    - "L2.342[82000,83999] 41ns 100mb                                    |L2.342|                                              "
    - "L2.343[84000,85999] 42ns 100mb                                     |L2.343|                                             "
    - "L2.344[86000,87999] 43ns 100mb                                      |L2.344|                                            "
    - "L2.345[88000,89999] 44ns 100mb                                       |L2.345|                                           "
    - "L2.346[90000,91999] 45ns 100mb                                        |L2.346|                                          "
    - "L2.347[92000,93999] 46ns 100mb                                         |L2.347|                                         "
    - "L2.348[94000,95999] 47ns 100mb                                          |L2.348|                                        "
    - "L2.349[96000,97999] 48ns 100mb                                           |L2.349|                                       "
    - "L2.350[98000,99999] 49ns 100mb                                            |L2.350|                                      "
    - "L2.351[100000,101999] 50ns 100mb                                             |L2.351|                                     "
    - "L2.352[102000,103999] 51ns 100mb                                             |L2.352|                                     "
    - "L2.353[104000,105999] 52ns 100mb                                              |L2.353|                                    "
    - "L2.354[106000,107999] 53ns 100mb                                               |L2.354|                                   "
    - "L2.355[108000,109999] 54ns 100mb                                                |L2.355|                                  "
    - "L2.356[110000,111999] 55ns 100mb                                                 |L2.356|                                 "
    - "L2.357[112000,113999] 56ns 100mb                                                  |L2.357|                                "
    - "L2.358[114000,115999] 57ns 100mb                                                   |L2.358|                               "
    - "L2.359[116000,117999] 58ns 100mb                                                    |L2.359|                              "
    - "L2.360[118000,119999] 59ns 100mb                                                     |L2.360|                             "
    - "L2.361[120000,121999] 60ns 100mb                                                      |L2.361|                            "
    - "L2.362[122000,123999] 61ns 100mb                                                      |L2.362|                            "
    - "L2.363[124000,125999] 62ns 100mb                                                       |L2.363|                           "
    - "L2.364[126000,127999] 63ns 100mb                                                        |L2.364|                          "
    - "L2.365[128000,129999] 64ns 100mb                                                         |L2.365|                         "
    - "L2.366[130000,131999] 65ns 100mb                                                          |L2.366|                        "
    - "L2.367[132000,133999] 66ns 100mb                                                           |L2.367|                       "
    - "L2.368[134000,135999] 67ns 100mb                                                            |L2.368|                      "
    - "L2.369[136000,137999] 68ns 100mb                                                             |L2.369|                     "
    - "L2.370[138000,139999] 69ns 100mb                                                              |L2.370|                    "
    - "L2.371[140000,141999] 70ns 100mb                                                               |L2.371|                   "
    - "L2.372[142000,143999] 71ns 100mb                                                               |L2.372|                   "
    - "L2.373[144000,145999] 72ns 100mb                                                                |L2.373|                  "
    - "L2.374[146000,147999] 73ns 100mb                                                                 |L2.374|                 "
    - "L2.375[148000,149999] 74ns 100mb                                                                  |L2.375|                "
    - "L2.376[150000,151999] 75ns 100mb                                                                   |L2.376|               "
    - "L2.377[152000,153999] 76ns 100mb                                                                    |L2.377|              "
    - "L2.378[154000,155999] 77ns 100mb                                                                     |L2.378|             "
    - "L2.379[156000,157999] 78ns 100mb                                                                      |L2.379|            "
    - "L2.380[158000,159999] 79ns 100mb                                                                       |L2.380|           "
    - "L2.381[160000,161999] 80ns 100mb                                                                        |L2.381|          "
    - "L2.382[162000,163999] 81ns 100mb                                                                        |L2.382|          "
    - "L2.383[164000,165999] 82ns 100mb                                                                         |L2.383|         "
    - "L2.384[166000,167999] 83ns 100mb                                                                          |L2.384|        "
    - "L2.385[168000,169999] 84ns 100mb                                                                           |L2.385|       "
    - "L2.386[170000,171999] 85ns 100mb                                                                            |L2.386|      "
    - "L2.387[172000,173999] 86ns 100mb                                                                             |L2.387|     "
    - "L2.388[174000,175999] 87ns 100mb                                                                              |L2.388|    "
    - "L2.389[176000,177999] 88ns 100mb                                                                               |L2.389|   "
    - "L2.390[178000,179999] 89ns 100mb                                                                                |L2.390|  "
    - "L2.391[180000,181999] 90ns 100mb                                                                                 |L2.391| "
    - "L2.392[182000,183999] 91ns 100mb                                                                                 |L2.392| "
    - "L2.393[184000,185999] 92ns 100mb                                                                                  |L2.393|"
    - "L2.394[186000,187999] 93ns 100mb                                                                                   |L2.394|"
    - "L2.395[188000,189999] 94ns 100mb                                                                                    |L2.395|"
    - "L2.396[190000,191999] 95ns 100mb                                                                                     |L2.396|"
    - "L2.397[192000,193999] 96ns 100mb                                                                                      |L2.397|"
    - "L2.398[194000,195999] 97ns 100mb                                                                                       |L2.398|"
    - "L2.399[196000,197999] 98ns 100mb                                                                                        |L2.399|"
    - "L2.400[198000,199999] 99ns 100mb                                                                                         |L2.400|"
    - "**** Final Output Files (41.18gb written)"
    - "L2                                                                                                                 "
    - "L2.854[0,975] 299ns 100mb|L2.854|                                                                                  "
    - "L2.855[976,1950] 299ns 100mb|L2.855|                                                                                  "
    - "L2.856[1951,1999] 299ns 5mb|L2.856|                                                                                  "
    - "L2.857[2000,2975] 299ns 100mb|L2.857|                                                                                  "
    - "L2.858[2976,3950] 299ns 100mb |L2.858|                                                                                 "
    - "L2.859[3951,3999] 299ns 5mb |L2.859|                                                                                 "
    - "L2.860[4000,4975] 299ns 100mb |L2.860|                                                                                 "
    - "L2.861[4976,5950] 299ns 100mb  |L2.861|                                                                                "
    - "L2.862[5951,5999] 299ns 5mb  |L2.862|                                                                                "
    - "L2.863[6000,6975] 299ns 100mb  |L2.863|                                                                                "
    - "L2.864[6976,7950] 299ns 100mb   |L2.864|                                                                               "
    - "L2.865[7951,7999] 299ns 5mb   |L2.865|                                                                               "
    - "L2.866[8000,8975] 299ns 100mb   |L2.866|                                                                               "
    - "L2.867[8976,9950] 299ns 100mb    |L2.867|                                                                              "
    - "L2.868[9951,9999] 299ns 5mb    |L2.868|                                                                              "
    - "L2.869[10000,10975] 299ns 100mb    |L2.869|                                                                              "
    - "L2.870[10976,11950] 299ns 100mb    |L2.870|                                                                              "
    - "L2.871[11951,11999] 299ns 5mb     |L2.871|                                                                             "
    - "L2.872[12000,12975] 299ns 100mb     |L2.872|                                                                             "
    - "L2.873[12976,13950] 299ns 100mb     |L2.873|                                                                             "
    - "L2.874[13951,13999] 299ns 5mb      |L2.874|                                                                            "
    - "L2.875[14000,14975] 299ns 100mb      |L2.875|                                                                            "
    - "L2.876[14976,15950] 299ns 100mb      |L2.876|                                                                            "
    - "L2.877[15951,15999] 299ns 5mb       |L2.877|                                                                           "
    - "L2.878[16000,16975] 299ns 100mb       |L2.878|                                                                           "
    - "L2.879[16976,17950] 299ns 100mb       |L2.879|                                                                           "
    - "L2.880[17951,17999] 299ns 5mb        |L2.880|                                                                          "
    - "L2.881[18000,18975] 299ns 100mb        |L2.881|                                                                          "
    - "L2.882[18976,19950] 299ns 100mb        |L2.882|                                                                          "
    - "L2.883[19951,19999] 299ns 5mb        |L2.883|                                                                          "
    - "L2.884[20000,20975] 299ns 100mb         |L2.884|                                                                         "
    - "L2.885[20976,21950] 299ns 100mb         |L2.885|                                                                         "
    - "L2.886[21951,21999] 299ns 5mb         |L2.886|                                                                         "
    - "L2.887[22000,22975] 299ns 100mb         |L2.887|                                                                         "
    - "L2.888[22976,23950] 299ns 100mb          |L2.888|                                                                        "
    - "L2.889[23951,23999] 299ns 5mb          |L2.889|                                                                        "
    - "L2.890[24000,24975] 299ns 100mb          |L2.890|                                                                        "
    - "L2.891[24976,25950] 299ns 100mb           |L2.891|                                                                       "
    - "L2.892[25951,25999] 299ns 5mb           |L2.892|                                                                       "
    - "L2.893[26000,26975] 299ns 100mb           |L2.893|                                                                       "
    - "L2.894[26976,27950] 299ns 100mb            |L2.894|                                                                      "
    - "L2.895[27951,27999] 299ns 5mb            |L2.895|                                                                      "
    - "L2.896[28000,28975] 299ns 100mb            |L2.896|                                                                      "
    - "L2.897[28976,29950] 299ns 100mb             |L2.897|                                                                     "
    - "L2.898[29951,29999] 299ns 5mb             |L2.898|                                                                     "
    - "L2.899[30000,30975] 299ns 100mb             |L2.899|                                                                     "
    - "L2.900[30976,31950] 299ns 100mb             |L2.900|                                                                     "
    - "L2.901[31951,31999] 299ns 5mb              |L2.901|                                                                    "
    - "L2.902[32000,32975] 299ns 100mb              |L2.902|                                                                    "
    - "L2.903[32976,33950] 299ns 100mb              |L2.903|                                                                    "
    - "L2.904[33951,33999] 299ns 5mb               |L2.904|                                                                   "
    - "L2.905[34000,34975] 299ns 100mb               |L2.905|                                                                   "
    - "L2.906[34976,35950] 299ns 100mb               |L2.906|                                                                   "
    - "L2.907[35951,35999] 299ns 5mb                |L2.907|                                                                  "
    - "L2.908[36000,36975] 299ns 100mb                |L2.908|                                                                  "
    - "L2.909[36976,37950] 299ns 100mb                |L2.909|                                                                  "
    - "L2.910[37951,37999] 299ns 5mb                 |L2.910|                                                                 "
    - "L2.911[38000,38975] 299ns 100mb                 |L2.911|                                                                 "
    - "L2.912[38976,39950] 299ns 100mb                 |L2.912|                                                                 "
    - "L2.913[39951,39999] 299ns 5mb                 |L2.913|                                                                 "
    - "L2.914[40000,40975] 299ns 100mb                  |L2.914|                                                                "
    - "L2.915[40976,41950] 299ns 100mb                  |L2.915|                                                                "
    - "L2.916[41951,41999] 299ns 5mb                  |L2.916|                                                                "
    - "L2.917[42000,42975] 299ns 100mb                  |L2.917|                                                                "
    - "L2.918[42976,43950] 299ns 100mb                   |L2.918|                                                               "
    - "L2.919[43951,43999] 299ns 5mb                   |L2.919|                                                               "
    - "L2.920[44000,44975] 299ns 100mb                   |L2.920|                                                               "
    - "L2.921[44976,45950] 299ns 100mb                    |L2.921|                                                              "
    - "L2.922[45951,45999] 299ns 5mb                    |L2.922|                                                              "
    - "L2.923[46000,46975] 299ns 100mb                    |L2.923|                                                              "
    - "L2.924[46976,47950] 299ns 100mb                     |L2.924|                                                             "
    - "L2.925[47951,47999] 299ns 5mb                     |L2.925|                                                             "
    - "L2.926[48000,48975] 299ns 100mb                     |L2.926|                                                             "
    - "L2.927[48976,49950] 299ns 100mb                      |L2.927|                                                            "
    - "L2.928[49951,49999] 299ns 5mb                      |L2.928|                                                            "
    - "L2.929[50000,50975] 299ns 100mb                      |L2.929|                                                            "
    - "L2.930[50976,51950] 299ns 100mb                      |L2.930|                                                            "
    - "L2.931[51951,51999] 299ns 5mb                       |L2.931|                                                           "
    - "L2.932[52000,52975] 299ns 100mb                       |L2.932|                                                           "
    - "L2.933[52976,53950] 299ns 100mb                       |L2.933|                                                           "
    - "L2.934[53951,53999] 299ns 5mb                        |L2.934|                                                          "
    - "L2.935[54000,54975] 299ns 100mb                        |L2.935|                                                          "
    - "L2.936[54976,55950] 299ns 100mb                        |L2.936|                                                          "
    - "L2.937[55951,55999] 299ns 5mb                         |L2.937|                                                         "
    - "L2.938[56000,56975] 299ns 100mb                         |L2.938|                                                         "
    - "L2.939[56976,57950] 299ns 100mb                         |L2.939|                                                         "
    - "L2.940[57951,57999] 299ns 5mb                          |L2.940|                                                        "
    - "L2.941[58000,58975] 299ns 100mb                          |L2.941|                                                        "
    - "L2.942[58976,59950] 299ns 100mb                          |L2.942|                                                        "
    - "L2.943[59951,59999] 299ns 5mb                          |L2.943|                                                        "
    - "L2.944[60000,60975] 299ns 100mb                           |L2.944|                                                       "
    - "L2.945[60976,61950] 299ns 100mb                           |L2.945|                                                       "
    - "L2.946[61951,61999] 299ns 5mb                           |L2.946|                                                       "
    - "L2.947[62000,62975] 299ns 100mb                           |L2.947|                                                       "
    - "L2.948[62976,63950] 299ns 100mb                            |L2.948|                                                      "
    - "L2.949[63951,63999] 299ns 5mb                            |L2.949|                                                      "
    - "L2.950[64000,64975] 299ns 100mb                            |L2.950|                                                      "
    - "L2.951[64976,65950] 299ns 100mb                             |L2.951|                                                     "
    - "L2.952[65951,65999] 299ns 5mb                             |L2.952|                                                     "
    - "L2.953[66000,66975] 299ns 100mb                             |L2.953|                                                     "
    - "L2.954[66976,67950] 299ns 100mb                              |L2.954|                                                    "
    - "L2.955[67951,67999] 299ns 5mb                              |L2.955|                                                    "
    - "L2.956[68000,68975] 299ns 100mb                              |L2.956|                                                    "
    - "L2.957[68976,69950] 299ns 100mb                               |L2.957|                                                   "
    - "L2.958[69951,69999] 299ns 5mb                               |L2.958|                                                   "
    - "L2.959[70000,70975] 299ns 100mb                               |L2.959|                                                   "
    - "L2.960[70976,71950] 299ns 100mb                               |L2.960|                                                   "
    - "L2.961[71951,71999] 299ns 5mb                                |L2.961|                                                  "
    - "L2.962[72000,72975] 299ns 100mb                                |L2.962|                                                  "
    - "L2.963[72976,73950] 299ns 100mb                                |L2.963|                                                  "
    - "L2.964[73951,73999] 299ns 5mb                                 |L2.964|                                                 "
    - "L2.965[74000,74975] 299ns 100mb                                 |L2.965|                                                 "
    - "L2.966[74976,75950] 299ns 100mb                                 |L2.966|                                                 "
    - "L2.967[75951,75999] 299ns 5mb                                  |L2.967|                                                "
    - "L2.968[76000,76975] 299ns 100mb                                  |L2.968|                                                "
    - "L2.969[76976,77950] 299ns 100mb                                  |L2.969|                                                "
    - "L2.970[77951,77999] 299ns 5mb                                   |L2.970|                                               "
    - "L2.971[78000,78975] 299ns 100mb                                   |L2.971|                                               "
    - "L2.972[78976,79950] 299ns 100mb                                   |L2.972|                                               "
    - "L2.973[79951,79999] 299ns 5mb                                   |L2.973|                                               "
    - "L2.974[80000,80975] 299ns 100mb                                    |L2.974|                                              "
    - "L2.975[80976,81950] 299ns 100mb                                    |L2.975|                                              "
    - "L2.976[81951,81999] 299ns 5mb                                    |L2.976|                                              "
    - "L2.977[82000,82975] 299ns 100mb                                    |L2.977|                                              "
    - "L2.978[82976,83950] 299ns 100mb                                     |L2.978|                                             "
    - "L2.979[83951,83999] 299ns 5mb                                     |L2.979|                                             "
    - "L2.980[84000,84975] 299ns 100mb                                     |L2.980|                                             "
    - "L2.981[84976,85950] 299ns 100mb                                      |L2.981|                                            "
    - "L2.982[85951,85999] 299ns 5mb                                      |L2.982|                                            "
    - "L2.983[86000,86975] 299ns 100mb                                      |L2.983|                                            "
    - "L2.984[86976,87950] 299ns 100mb                                       |L2.984|                                           "
    - "L2.985[87951,87999] 299ns 5mb                                       |L2.985|                                           "
    - "L2.986[88000,88975] 299ns 100mb                                       |L2.986|                                           "
    - "L2.987[88976,89950] 299ns 100mb                                        |L2.987|                                          "
    - "L2.988[89951,89999] 299ns 5mb                                        |L2.988|                                          "
    - "L2.989[90000,90975] 299ns 100mb                                        |L2.989|                                          "
    - "L2.990[90976,91950] 299ns 100mb                                        |L2.990|                                          "
    - "L2.991[91951,91999] 299ns 5mb                                         |L2.991|                                         "
    - "L2.992[92000,92975] 299ns 100mb                                         |L2.992|                                         "
    - "L2.993[92976,93950] 299ns 100mb                                         |L2.993|                                         "
    - "L2.994[93951,93999] 299ns 5mb                                          |L2.994|                                        "
    - "L2.995[94000,94975] 299ns 100mb                                          |L2.995|                                        "
    - "L2.996[94976,95950] 299ns 100mb                                          |L2.996|                                        "
    - "L2.997[95951,95999] 299ns 5mb                                           |L2.997|                                       "
    - "L2.998[96000,96975] 299ns 100mb                                           |L2.998|                                       "
    - "L2.999[96976,97950] 299ns 100mb                                           |L2.999|                                       "
    - "L2.1000[97951,97999] 299ns 5mb                                            |L2.1000|                                     "
    - "L2.1001[98000,98975] 299ns 100mb                                            |L2.1001|                                     "
    - "L2.1002[98976,99950] 299ns 100mb                                            |L2.1002|                                     "
    - "L2.1003[99951,99999] 299ns 5mb                                            |L2.1003|                                     "
    - "L2.1004[100000,100975] 299ns 100mb                                             |L2.1004|                                    "
    - "L2.1005[100976,101950] 299ns 100mb                                             |L2.1005|                                    "
    - "L2.1006[101951,101999] 299ns 5mb                                             |L2.1006|                                    "
    - "L2.1007[102000,102975] 299ns 100mb                                             |L2.1007|                                    "
    - "L2.1008[102976,103950] 299ns 100mb                                              |L2.1008|                                   "
    - "L2.1009[103951,103999] 299ns 5mb                                              |L2.1009|                                   "
    - "L2.1010[104000,104975] 299ns 100mb                                              |L2.1010|                                   "
    - "L2.1011[104976,105950] 299ns 100mb                                               |L2.1011|                                  "
    - "L2.1012[105951,105999] 299ns 5mb                                               |L2.1012|                                  "
    - "L2.1013[106000,106975] 299ns 100mb                                               |L2.1013|                                  "
    - "L2.1014[106976,107950] 299ns 100mb                                                |L2.1014|                                 "
    - "L2.1015[107951,107999] 299ns 5mb                                                |L2.1015|                                 "
    - "L2.1016[108000,108975] 299ns 100mb                                                |L2.1016|                                 "
    - "L2.1017[108976,109950] 299ns 100mb                                                 |L2.1017|                                "
    - "L2.1018[109951,109999] 299ns 5mb                                                 |L2.1018|                                "
    - "L2.1019[110000,110975] 299ns 100mb                                                 |L2.1019|                                "
    - "L2.1020[110976,111950] 299ns 100mb                                                 |L2.1020|                                "
    - "L2.1021[111951,111999] 299ns 5mb                                                  |L2.1021|                               "
    - "L2.1022[112000,112975] 299ns 100mb                                                  |L2.1022|                               "
    - "L2.1023[112976,113950] 299ns 100mb                                                  |L2.1023|                               "
    - "L2.1024[113951,113999] 299ns 5mb                                                   |L2.1024|                              "
    - "L2.1025[114000,114975] 299ns 100mb                                                   |L2.1025|                              "
    - "L2.1026[114976,115950] 299ns 100mb                                                   |L2.1026|                              "
    - "L2.1027[115951,115999] 299ns 5mb                                                    |L2.1027|                             "
    - "L2.1028[116000,116975] 299ns 100mb                                                    |L2.1028|                             "
    - "L2.1029[116976,117950] 299ns 100mb                                                    |L2.1029|                             "
    - "L2.1030[117951,117999] 299ns 5mb                                                     |L2.1030|                            "
    - "L2.1031[118000,118975] 299ns 100mb                                                     |L2.1031|                            "
    - "L2.1032[118976,119950] 299ns 100mb                                                     |L2.1032|                            "
    - "L2.1033[119951,119999] 299ns 5mb                                                     |L2.1033|                            "
    - "L2.1034[120000,120975] 299ns 100mb                                                      |L2.1034|                           "
    - "L2.1035[120976,121950] 299ns 100mb                                                      |L2.1035|                           "
    - "L2.1036[121951,121999] 299ns 5mb                                                      |L2.1036|                           "
    - "L2.1037[122000,122975] 299ns 100mb                                                      |L2.1037|                           "
    - "L2.1038[122976,123950] 299ns 100mb                                                       |L2.1038|                          "
    - "L2.1039[123951,123999] 299ns 5mb                                                       |L2.1039|                          "
    - "L2.1040[124000,124975] 299ns 100mb                                                       |L2.1040|                          "
    - "L2.1041[124976,125950] 299ns 100mb                                                        |L2.1041|                         "
    - "L2.1042[125951,125999] 299ns 5mb                                                        |L2.1042|                         "
    - "L2.1043[126000,126975] 299ns 100mb                                                        |L2.1043|                         "
    - "L2.1044[126976,127950] 299ns 100mb                                                         |L2.1044|                        "
    - "L2.1045[127951,127999] 299ns 5mb                                                         |L2.1045|                        "
    - "L2.1046[128000,128975] 299ns 100mb                                                         |L2.1046|                        "
    - "L2.1047[128976,129950] 299ns 100mb                                                          |L2.1047|                       "
    - "L2.1048[129951,129999] 299ns 5mb                                                          |L2.1048|                       "
    - "L2.1049[130000,130975] 299ns 100mb                                                          |L2.1049|                       "
    - "L2.1050[130976,131950] 299ns 100mb                                                          |L2.1050|                       "
    - "L2.1051[131951,131999] 299ns 5mb                                                           |L2.1051|                      "
    - "L2.1052[132000,132975] 299ns 100mb                                                           |L2.1052|                      "
    - "L2.1053[132976,133950] 299ns 100mb                                                           |L2.1053|                      "
    - "L2.1054[133951,133999] 299ns 5mb                                                            |L2.1054|                     "
    - "L2.1055[134000,134975] 299ns 100mb                                                            |L2.1055|                     "
    - "L2.1056[134976,135950] 299ns 100mb                                                            |L2.1056|                     "
    - "L2.1057[135951,135999] 299ns 5mb                                                             |L2.1057|                    "
    - "L2.1058[136000,136975] 299ns 100mb                                                             |L2.1058|                    "
    - "L2.1059[136976,137950] 299ns 100mb                                                             |L2.1059|                    "
    - "L2.1060[137951,137999] 299ns 5mb                                                              |L2.1060|                   "
    - "L2.1061[138000,138975] 299ns 100mb                                                              |L2.1061|                   "
    - "L2.1062[138976,139950] 299ns 100mb                                                              |L2.1062|                   "
    - "L2.1063[139951,139999] 299ns 5mb                                                              |L2.1063|                   "
    - "L2.1064[140000,140975] 299ns 100mb                                                               |L2.1064|                  "
    - "L2.1065[140976,141950] 299ns 100mb                                                               |L2.1065|                  "
    - "L2.1066[141951,141999] 299ns 5mb                                                               |L2.1066|                  "
    - "L2.1067[142000,142975] 299ns 100mb                                                               |L2.1067|                  "
    - "L2.1068[142976,143950] 299ns 100mb                                                                |L2.1068|                 "
    - "L2.1069[143951,143999] 299ns 5mb                                                                |L2.1069|                 "
    - "L2.1070[144000,144975] 299ns 100mb                                                                |L2.1070|                 "
    - "L2.1071[144976,145950] 299ns 100mb                                                                 |L2.1071|                "
    - "L2.1072[145951,145999] 299ns 5mb                                                                 |L2.1072|                "
    - "L2.1073[146000,146975] 299ns 100mb                                                                 |L2.1073|                "
    - "L2.1074[146976,147950] 299ns 100mb                                                                  |L2.1074|               "
    - "L2.1075[147951,147999] 299ns 5mb                                                                  |L2.1075|               "
    - "L2.1076[148000,148975] 299ns 100mb                                                                  |L2.1076|               "
    - "L2.1077[148976,149950] 299ns 100mb                                                                   |L2.1077|              "
    - "L2.1078[149951,149999] 299ns 5mb                                                                   |L2.1078|              "
    - "L2.1079[150000,150975] 299ns 100mb                                                                   |L2.1079|              "
    - "L2.1080[150976,151950] 299ns 100mb                                                                   |L2.1080|              "
    - "L2.1081[151951,151999] 299ns 5mb                                                                    |L2.1081|             "
    - "L2.1082[152000,152975] 299ns 100mb                                                                    |L2.1082|             "
    - "L2.1083[152976,153950] 299ns 100mb                                                                    |L2.1083|             "
    - "L2.1084[153951,153999] 299ns 5mb                                                                     |L2.1084|            "
    - "L2.1085[154000,154975] 299ns 100mb                                                                     |L2.1085|            "
    - "L2.1086[154976,155950] 299ns 100mb                                                                     |L2.1086|            "
    - "L2.1087[155951,155999] 299ns 5mb                                                                      |L2.1087|           "
    - "L2.1088[156000,156975] 299ns 100mb                                                                      |L2.1088|           "
    - "L2.1089[156976,157950] 299ns 100mb                                                                      |L2.1089|           "
    - "L2.1090[157951,157999] 299ns 5mb                                                                       |L2.1090|          "
    - "L2.1091[158000,158975] 299ns 100mb                                                                       |L2.1091|          "
    - "L2.1092[158976,159950] 299ns 100mb                                                                       |L2.1092|          "
    - "L2.1093[159951,159999] 299ns 5mb                                                                       |L2.1093|          "
    - "L2.1094[160000,160975] 299ns 100mb                                                                        |L2.1094|         "
    - "L2.1095[160976,161950] 299ns 100mb                                                                        |L2.1095|         "
    - "L2.1096[161951,161999] 299ns 5mb                                                                        |L2.1096|         "
    - "L2.1097[162000,162975] 299ns 100mb                                                                        |L2.1097|         "
    - "L2.1098[162976,163950] 299ns 100mb                                                                         |L2.1098|        "
    - "L2.1099[163951,163999] 299ns 5mb                                                                         |L2.1099|        "
    - "L2.1100[164000,164975] 299ns 100mb                                                                         |L2.1100|        "
    - "L2.1101[164976,165950] 299ns 100mb                                                                          |L2.1101|       "
    - "L2.1102[165951,165999] 299ns 5mb                                                                          |L2.1102|       "
    - "L2.1103[166000,166975] 299ns 100mb                                                                          |L2.1103|       "
    - "L2.1104[166976,167950] 299ns 100mb                                                                           |L2.1104|      "
    - "L2.1105[167951,167999] 299ns 5mb                                                                           |L2.1105|      "
    - "L2.1106[168000,168975] 299ns 100mb                                                                           |L2.1106|      "
    - "L2.1107[168976,169950] 299ns 100mb                                                                            |L2.1107|     "
    - "L2.1108[169951,169999] 299ns 5mb                                                                            |L2.1108|     "
    - "L2.1109[170000,170975] 299ns 100mb                                                                            |L2.1109|     "
    - "L2.1110[170976,171950] 299ns 100mb                                                                            |L2.1110|     "
    - "L2.1111[171951,171999] 299ns 5mb                                                                             |L2.1111|    "
    - "L2.1112[172000,172975] 299ns 100mb                                                                             |L2.1112|    "
    - "L2.1113[172976,173950] 299ns 100mb                                                                             |L2.1113|    "
    - "L2.1114[173951,173999] 299ns 5mb                                                                              |L2.1114|   "
    - "L2.1115[174000,174975] 299ns 100mb                                                                              |L2.1115|   "
    - "L2.1116[174976,175950] 299ns 100mb                                                                              |L2.1116|   "
    - "L2.1117[175951,175999] 299ns 5mb                                                                               |L2.1117|  "
    - "L2.1118[176000,176975] 299ns 100mb                                                                               |L2.1118|  "
    - "L2.1119[176976,177950] 299ns 100mb                                                                               |L2.1119|  "
    - "L2.1120[177951,177999] 299ns 5mb                                                                                |L2.1120| "
    - "L2.1121[178000,178975] 299ns 100mb                                                                                |L2.1121| "
    - "L2.1122[178976,179950] 299ns 100mb                                                                                |L2.1122| "
    - "L2.1123[179951,179999] 299ns 5mb                                                                                |L2.1123| "
    - "L2.1124[180000,180975] 299ns 100mb                                                                                 |L2.1124|"
    - "L2.1125[180976,181950] 299ns 100mb                                                                                 |L2.1125|"
    - "L2.1126[181951,181999] 299ns 5mb                                                                                 |L2.1126|"
    - "L2.1127[182000,183599] 299ns 84mb                                                                                 |L2.1127|"
    - "L2.1128[183600,183999] 299ns 21mb                                                                                  |L2.1128|"
    - "L2.1129[184000,184980] 299ns 100mb                                                                                  |L2.1129|"
    - "L2.1130[184981,185960] 299ns 100mb                                                                                   |L2.1130|"
    - "L2.1131[185961,185999] 299ns 4mb                                                                                   |L2.1131|"
    - "L2.1132[186000,186980] 299ns 100mb                                                                                   |L2.1132|"
    - "L2.1133[186981,187960] 299ns 100mb                                                                                    |L2.1133|"
    - "L2.1134[187961,187999] 299ns 4mb                                                                                    |L2.1134|"
    - "L2.1135[188000,188980] 299ns 100mb                                                                                    |L2.1135|"
    - "L2.1136[188981,189960] 299ns 100mb                                                                                     |L2.1136|"
    - "L2.1137[189961,189999] 299ns 4mb                                                                                     |L2.1137|"
    - "L2.1138[190000,190980] 299ns 100mb                                                                                     |L2.1138|"
    - "L2.1139[190981,191960] 299ns 100mb                                                                                     |L2.1139|"
    - "L2.1140[191961,191999] 299ns 4mb                                                                                      |L2.1140|"
    - "L2.1141[192000,192980] 299ns 100mb                                                                                      |L2.1141|"
    - "L2.1142[192981,193960] 299ns 100mb                                                                                      |L2.1142|"
    - "L2.1143[193961,193999] 299ns 4mb                                                                                       |L2.1143|"
    - "L2.1144[194000,194980] 299ns 100mb                                                                                       |L2.1144|"
    - "L2.1145[194981,195960] 299ns 100mb                                                                                       |L2.1145|"
    - "L2.1146[195961,195999] 299ns 4mb                                                                                        |L2.1146|"
    - "L2.1147[196000,196980] 299ns 100mb                                                                                        |L2.1147|"
    - "L2.1148[196981,197960] 299ns 100mb                                                                                        |L2.1148|"
    - "L2.1150[197961,198942] 299ns 100mb                                                                                         |L2.1150|"
    - "L2.1151[198943,199923] 299ns 100mb                                                                                         |L2.1151|"
    - "L2.1152[199924,200000] 299ns 8mb                                                                                         |L2.1152|"
    - "**** Breakdown of where bytes were written"
    - 1000mb written by split(ReduceOverlap)
    - 38.21gb written by split(CompactAndSplitOutput(FoundSubsetLessThanMaxCompactSize))
    - 400mb written by compact(ManySmallFiles)
    - 780mb written by split(CompactAndSplitOutput(TotalSizeLessThanMaxCompactSize))
    - 860mb written by split(HighL0OverlapTotalBacklog)
    "###
    );
}
